@@ -5,15 +5,18 @@
 
 """A convenience wrapper for starting gsutil."""
 
+from __future__ import absolute_import
+from __future__ import unicode_literals
 import os
-import sys
 
 
 import bootstrapping
+from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.core import config
 from googlecloudsdk.core import metrics
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.credentials import gce as c_gce
+from googlecloudsdk.core.util import encoding
 
 
 def _MaybeAddBotoOption(args, section, name, value):
@@ -37,10 +40,11 @@ def main():
 
     # Allow gsutil to only check for the '1' string value, as is done
     # with regard to the 'CLOUDSDK_WRAPPER' environment variable.
-    os.environ['CLOUDSDK_CORE_PASS_CREDENTIALS_TO_GSUTIL'] = '1'
+    encoding.SetEncodedValue(
+        os.environ, 'CLOUDSDK_CORE_PASS_CREDENTIALS_TO_GSUTIL', '1')
 
-    boto_config = os.environ.get('BOTO_CONFIG', '')
-    boto_path = os.environ.get('BOTO_PATH', '')
+    boto_config = encoding.GetEncodedValue(os.environ, 'BOTO_CONFIG', '')
+    boto_path = encoding.GetEncodedValue(os.environ, 'BOTO_PATH', '')
 
     # We construct a BOTO_PATH that tacks the refresh token config
     # on the end.
@@ -54,12 +58,12 @@ def main():
                     gsutil_path]
       boto_path = os.pathsep.join(path_parts)
 
-    if 'BOTO_CONFIG' in os.environ:
-      del os.environ['BOTO_CONFIG']
-    os.environ['BOTO_PATH'] = boto_path
+    encoding.SetEncodedValue(os.environ, 'BOTO_CONFIG', None)
+    encoding.SetEncodedValue(os.environ, 'BOTO_PATH', boto_path)
 
   # Tell gsutil whether gcloud analytics collection is enabled.
-  os.environ['GA_CID'] = metrics.GetCIDIfMetricsEnabled()
+  encoding.SetEncodedValue(
+      os.environ, 'GA_CID', metrics.GetCIDIfMetricsEnabled())
 
   args = []
 
@@ -69,32 +73,39 @@ def main():
     _MaybeAddBotoOption(args, 'GoogleCompute', 'service_account', 'default')
 
   proxy_params = properties.VALUES.proxy
-  _MaybeAddBotoOption(args, 'Boto', 'proxy', proxy_params.address.Get())
-  _MaybeAddBotoOption(args, 'Boto', 'proxy_port', proxy_params.port.Get())
-  _MaybeAddBotoOption(args, 'Boto', 'proxy_user', proxy_params.username.Get())
-  _MaybeAddBotoOption(args, 'Boto', 'proxy_pass', proxy_params.password.Get())
+  proxy_address = proxy_params.address.Get()
+  if proxy_address:
+    _MaybeAddBotoOption(args, 'Boto', 'proxy', proxy_address)
+    _MaybeAddBotoOption(args, 'Boto', 'proxy_port', proxy_params.port.Get())
+    _MaybeAddBotoOption(args, 'Boto', 'proxy_rdns', proxy_params.rdns.GetBool())
+    _MaybeAddBotoOption(args, 'Boto', 'proxy_user', proxy_params.username.Get())
+    _MaybeAddBotoOption(args, 'Boto', 'proxy_pass', proxy_params.password.Get())
   disable_ssl = properties.VALUES.auth.disable_ssl_validation.GetBool()
   _MaybeAddBotoOption(args, 'Boto', 'https_validate_certificates',
                       None if disable_ssl is None else not disable_ssl)
   _MaybeAddBotoOption(args, 'Boto', 'ca_certificates_file',
                       properties.VALUES.core.custom_ca_certs_file.Get())
 
-  bootstrapping.ExecutePythonTool(
-      'platform/gsutil', 'gsutil', *args)
+  bootstrapping.ExecutePythonTool('platform/gsutil', 'gsutil', *args)
 
 
 if __name__ == '__main__':
-  version = bootstrapping.GetFileContents('platform/gsutil', 'VERSION')
-  bootstrapping.CommandStart('gsutil', version=version)
+  bootstrapping.DisallowPython3()
+  try:
+    version = bootstrapping.ReadFileContents('platform/gsutil', 'VERSION')
+    bootstrapping.CommandStart('gsutil', version=version)
 
-  blacklist = {
-      'update': 'To update, run: gcloud components update',
-  }
+    blacklist = {
+        'update': 'To update, run: gcloud components update',
+    }
 
-  bootstrapping.CheckForBlacklistedCommand(sys.argv, blacklist, warn=True,
-                                           die=True)
-  # Don't call bootstrapping.PreRunChecks because anonymous access is
-  # supported for some endpoints. gsutil will output the appropriate
-  # error message upon receiving an authentication error.
-  bootstrapping.CheckUpdates('gsutil')
-  main()
+    argv = bootstrapping.GetDecodedArgv()
+    bootstrapping.CheckForBlacklistedCommand(argv, blacklist, warn=True,
+                                             die=True)
+    # Don't call bootstrapping.PreRunChecks because anonymous access is
+    # supported for some endpoints. gsutil will output the appropriate
+    # error message upon receiving an authentication error.
+    bootstrapping.CheckUpdates('gsutil')
+    main()
+  except Exception as e:  # pylint: disable=broad-except
+    exceptions.HandleError(e, 'gsutil')
