@@ -2,9 +2,15 @@
 #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
-# This is script no longer needed as of release 133.0.0
-# It is here so that git repositories set before this version continue to work.
-
+# This Git credential helper (see gitcredentials(7)) can be used for accessing
+# Google Cloud Source Repositories:
+# https://cloud.google.com/source-repositories/docs/adding-repositories-as-remotes
+#
+# It is also possible to configure Git to use gcloud as a credential helper
+# directly, as "gcloud source repos clone" does:
+#
+# [credential "https://source.developers.google.com"]
+#   helper = !gcloud auth git-helper --ignore-unknown \"$@\"
 
 # <cloud-sdk-sh-preamble>
 #
@@ -16,6 +22,12 @@
 # (a) always defined by the preamble
 # (u) user definition overrides preamble
 
+# Wrapper around 'which' and 'command -v', tries which first, then falls back
+# to command -v
+_cloudsdk_which() {
+  which "$1" 2>/dev/null || command -v "$1" 2>/dev/null
+}
+
 # Determines the real cloud sdk root dir given the script path.
 # Would be easier with a portable "readlink -f".
 _cloudsdk_root_dir() {
@@ -24,7 +36,7 @@ _cloudsdk_root_dir() {
         ;;
   */*)  _cloudsdk_path=$PWD/$1
         ;;
-  *)    _cloudsdk_path=$(which "$1")
+  *)    _cloudsdk_path=$(_cloudsdk_which $1)
         case $_cloudsdk_path in
         /*) ;;
         *)  _cloudsdk_path=$PWD/$_cloudsdk_path ;;
@@ -44,11 +56,16 @@ _cloudsdk_root_dir() {
     case $_cloudsdk_dir in
     1)  break ;;
     esac
+    if [ -d "${_cloudsdk_path}" ]; then
+      break
+    fi
     _cloudsdk_dir=1
     _cloudsdk_path=$(dirname "$_cloudsdk_path")
   done
   while :
   do  case $_cloudsdk_path in
+      */)     _cloudsdk_path=$(dirname "$_cloudsdk_path/.")
+              ;;
       */.)    _cloudsdk_path=$(dirname "$_cloudsdk_path")
               ;;
       */bin)  dirname "$_cloudsdk_path"
@@ -62,28 +79,32 @@ _cloudsdk_root_dir() {
 }
 CLOUDSDK_ROOT_DIR=$(_cloudsdk_root_dir "$0")
 
-# Cloud SDK requires python 2.7
-CLOUDSDK_PYTHON=/usr/local/bin/python2.7
-case $CLOUDSDK_PYTHON in
-*python2*)
-  ;;
-*python[0-9]*)
-  CLOUDSDK_PYTHON=
-  ;;
-esac
 # if CLOUDSDK_PYTHON is empty
 if [ -z "$CLOUDSDK_PYTHON" ]; then
   # if python2 exists then plain python may point to a version != 2
-  if which python2 >/dev/null; then
+  if _cloudsdk_which python2 >/dev/null; then
     CLOUDSDK_PYTHON=python2
-  elif which python2.7 >/dev/null; then
+  elif _cloudsdk_which python2.7 >/dev/null; then
     # this is what some OS X versions call their built-in Python
     CLOUDSDK_PYTHON=python2.7
+  elif _cloudsdk_which python >/dev/null; then
+    # Use unversioned python if it exists.
+    CLOUDSDK_PYTHON=python
+  elif _cloudsdk_which python3 >/dev/null; then
+    # We support python3, but only want to default to it if nothing else is
+    # found.
+    CLOUDSDK_PYTHON=python3
   else
+    # This won't work because it wasn't found above, but at this point this
+    # is our best guess for the error message.
     CLOUDSDK_PYTHON=python
   fi
 fi
 
+# $PYTHONHOME can interfere with gcloud. Users should use
+# CLOUDSDK_PYTHON to configure which python gcloud uses.
+unset PYTHONHOME
+
 # if CLOUDSDK_PYTHON_SITEPACKAGES and VIRTUAL_ENV are empty
 case :$CLOUDSDK_PYTHON_SITEPACKAGES:$VIRTUAL_ENV: in
 :::)  # add -S to CLOUDSDK_PYTHON_ARGS if not already there
@@ -114,7 +135,6 @@ export CLOUDSDK_ROOT_DIR CLOUDSDK_PYTHON_ARGS
 
 # </cloud-sdk-sh-preamble>
 
-
 export CLOUDSDK_COMPONENT_MANAGER_DISABLE_UPDATE_CHECK=1
 
 "${CLOUDSDK_ROOT_DIR}/bin/gcloud" auth git-helper --ignore-unknown "$@"
