@@ -1,4 +1,4 @@
-
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,13 +15,17 @@
 
 """Utility methods used by the deploy_app command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+import datetime
 import hashlib
-import json
 import os
-import shutil
+
+from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.app import metric_names
-from googlecloudsdk.api_lib.app import util
 from googlecloudsdk.api_lib.storage import storage_api
 from googlecloudsdk.command_lib.storage import storage_parallel
 from googlecloudsdk.core import exceptions as core_exceptions
@@ -29,15 +33,17 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import metrics
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.util import files as file_utils
+from googlecloudsdk.core.util import times
 from googlecloudsdk.third_party.appengine.tools import context_util
-
-
-# Max App Engine file size; see https://cloud.google.com/appengine/docs/quotas
-_MAX_FILE_SIZE = 32 * 1024 * 1024
+from six.moves import map  # pylint: disable=redefined-builtin
 
 
 _DEFAULT_NUM_THREADS = 8
 
+# TTL expiry margin, to compensate for incorrect local time and timezone,
+# as well as deployment time.
+_TTL_MARGIN = datetime.timedelta(1)
+
 
 class LargeFileError(core_exceptions.Error):
 
@@ -61,25 +67,24 @@ class MultiError(core_exceptions.Error):
     self.errors = errors
 
 
-def _BuildDeploymentManifest(info, source_dir, bucket_ref, tmp_dir):
+def _BuildDeploymentManifest(upload_dir, source_files, bucket_ref, tmp_dir):
   """Builds a deployment manifest for use with the App Engine Admin API.
 
   Args:
-    info: An instance of yaml_parsing.ServiceInfo.
-    source_dir: str, path to the service's source directory
+    upload_dir: str, path to the service's upload directory
+    source_files: [str], relative paths to upload.
     bucket_ref: The reference to the bucket files will be placed in.
     tmp_dir: A temp directory for storing generated files (currently just source
         context files).
   Returns:
     A deployment manifest (dict) for use with the Admin API.
   """
-  excluded_files_regex = info.parsed.skip_files.regex
   manifest = {}
   bucket_url = 'https://storage.googleapis.com/{0}'.format(bucket_ref.bucket)
 
   # Normal application files.
-  for rel_path in util.FileIterator(source_dir, excluded_files_regex):
-    full_path = os.path.join(source_dir, rel_path)
+  for rel_path in source_files:
+    full_path = os.path.join(upload_dir, rel_path)
     sha1_hash = file_utils.Checksum.HashSingleFile(full_path,
                                                    algorithm=hashlib.sha1)
     manifest_path = '/'.join([bucket_url, sha1_hash])
@@ -91,7 +96,7 @@ def _BuildDeploymentManifest(info, source_dir, bucket_ref, tmp_dir):
   # Source context files. These are temporary files which indicate the current
   # state of the source repository (git, cloud repo, etc.)
   context_files = context_util.CreateContextFiles(
-      tmp_dir, None, source_dir=source_dir)
+      tmp_dir, None, source_dir=upload_dir)
   for context_file in context_files:
     rel_path = os.path.basename(context_file)
     if rel_path in manifest:
@@ -109,7 +114,58 @@ def _BuildDeploymentManifest(info, source_dir, bucket_ref, tmp_dir):
   return manifest
 
 
-def _BuildFileUploadMap(manifest, source_dir, bucket_ref, tmp_dir):
+def _GetLifecycleDeletePolicy(storage_client, bucket_ref):
+  """Get the TTL of objects in days as specified by the lifecycle policy.
+
+  Only "delete by age" policies are accounted for.
+
+  Args:
+    storage_client: storage_api.StorageClient, API client wrapper.
+    bucket_ref: The GCS bucket reference.
+
+  Returns:
+    datetime.timedelta, TTL of objects in days, or None if no deletion
+    policy on the bucket.
+  """
+  try:
+    bucket = storage_client.client.buckets.Get(
+        request=storage_client.messages.StorageBucketsGetRequest(
+            bucket=bucket_ref.bucket),
+        global_params=storage_client.messages.StandardQueryParameters(
+            fields='lifecycle'))
+  except apitools_exceptions.HttpForbiddenError:
+    return None
+  if not bucket.lifecycle:
+    return None
+  rules = bucket.lifecycle.rule
+  ages = [rule.condition.age for rule in rules
+          if rule.condition.age >= 0 and rule.action.type == 'Delete']
+  return datetime.timedelta(min(ages)) if ages else None
+
+
+def _IsTTLSafe(ttl, obj):
+  """Determines whether a GCS object is close to end-of-life.
+
+  In order to reduce false negative rate (objects that are close to deletion but
+  aren't marked as such) the returned filter is forward-adjusted with
+  _TTL_MARGIN.
+
+  Args:
+    ttl: datetime.timedelta, TTL of objects, or None if no TTL.
+    obj: storage object to check.
+
+  Returns:
+    True if the ojbect is safe or False if it is approaching end of life.
+  """
+  if ttl is None:
+    return True
+  now = times.Now(times.UTC)
+  delta = ttl - _TTL_MARGIN
+  return (now - obj.timeCreated) <= delta
+
+
+def _BuildFileUploadMap(manifest, source_dir, bucket_ref, tmp_dir,
+                        max_file_size):
   """Builds a map of files to upload, indexed by their hash.
 
   This skips already-uploaded files.
@@ -121,6 +177,7 @@ def _BuildFileUploadMap(manifest, source_dir, bucket_ref, tmp_dir):
     tmp_dir: The path to a temporary directory where generated files may be
       stored. If a file in the manifest is not found in the source directory,
       it will be retrieved from this directory instead.
+    max_file_size: int, File size limit per individual file or None if no limit.
 
   Raises:
     LargeFileError: if one of the files to upload exceeds the maximum App Engine
@@ -131,7 +188,10 @@ def _BuildFileUploadMap(manifest, source_dir, bucket_ref, tmp_dir):
   """
   files_to_upload = {}
   storage_client = storage_api.StorageClient()
-  existing_items = storage_client.ListBucket(bucket_ref)
+  ttl = _GetLifecycleDeletePolicy(storage_client, bucket_ref)
+  existing_items = set(o.name for o in storage_client.ListBucket(bucket_ref)
+                       if _IsTTLSafe(ttl, o))
+  skipped_size, total_size = 0, 0
   for rel_path in manifest:
     full_path = os.path.join(source_dir, rel_path)
     # For generated files, the relative path is based on the tmp_dir rather
@@ -142,14 +202,18 @@ def _BuildFileUploadMap(manifest, source_dir, bucket_ref, tmp_dir):
     # Perform this check when creating the upload map, so we catch too-large
     # files that have already been uploaded
     size = os.path.getsize(full_path)
-    if size > _MAX_FILE_SIZE:
-      raise LargeFileError(full_path, size, _MAX_FILE_SIZE)
+    if max_file_size and size > max_file_size:
+      raise LargeFileError(full_path, size, max_file_size)
 
     sha1_hash = manifest[rel_path]['sha1Sum']
+    total_size += size
     if sha1_hash in existing_items:
       log.debug('Skipping upload of [{f}]'.format(f=rel_path))
+      skipped_size += size
     else:
       files_to_upload[sha1_hash] = full_path
+  log.info('Incremental upload skipped {pct}% of data'.format(
+      pct=round(100.0*skipped_size/total_size, 2)))
   return files_to_upload
 
 
@@ -177,7 +241,7 @@ def _UploadFilesThreads(files_to_upload, bucket_ref):
   tasks = []
   # Have to sort files because the test framework requires a known order for
   # mocked API calls.
-  for sha1_hash, path in sorted(files_to_upload.iteritems()):
+  for sha1_hash, path in sorted(files_to_upload.items()):
     task = storage_parallel.FileUploadTask(path, bucket_ref.ToBucketUrl(),
                                            sha1_hash)
     tasks.append(task)
@@ -185,7 +249,8 @@ def _UploadFilesThreads(files_to_upload, bucket_ref):
                                show_progress_bar=True)
 
 
-def CopyFilesToCodeBucket(service, source_dir, bucket_ref):
+def CopyFilesToCodeBucket(upload_dir, source_files,
+                          bucket_ref, max_file_size=None):
   """Copies application files to the Google Cloud Storage code bucket.
 
   Use the Cloud Storage API using threads.
@@ -218,9 +283,10 @@ def CopyFilesToCodeBucket(service, source_dir, bucket_ref):
     values already present in the bucket will not be uploaded again.
 
   Args:
-    service: ServiceYamlInfo, The service being deployed.
-    source_dir: str, path to the service's source directory
+    upload_dir: str, path to the service's upload directory
+    source_files: [str], relative paths to upload.
     bucket_ref: The reference to the bucket files will be placed in.
+    max_file_size: int, File size limit per individual file or None if no limit.
 
   Returns:
     A dictionary representing the manifest.
@@ -229,10 +295,10 @@ def CopyFilesToCodeBucket(service, source_dir, bucket_ref):
   # Collect a list of files to upload, indexed by the SHA so uploads are
   # deduplicated.
   with file_utils.TemporaryDirectory() as tmp_dir:
-    manifest = _BuildDeploymentManifest(service, source_dir, bucket_ref,
-                                        tmp_dir)
+    manifest = _BuildDeploymentManifest(
+        upload_dir, source_files, bucket_ref, tmp_dir)
     files_to_upload = _BuildFileUploadMap(
-        manifest, source_dir, bucket_ref, tmp_dir)
+        manifest, upload_dir, bucket_ref, tmp_dir, max_file_size)
     _UploadFilesThreads(files_to_upload, bucket_ref)
   log.status.Print('File upload done.')
   log.info('Manifest: [{0}]'.format(manifest))
@@ -240,130 +306,6 @@ def CopyFilesToCodeBucket(service, source_dir, bucket_ref):
   return manifest
 
 
-def _BuildStagingDirectory(source_dir, staging_dir, bucket_ref,
-                           excluded_regexes):
-  """Creates a staging directory to be uploaded to Google Cloud Storage.
-
-  The staging directory will contain a symlink for each file in the original
-  directory. The source is a file whose name is the sha1 hash of the original
-  file and points to the original file.
-
-  Consider the following original structure:
-    app/
-      main.py
-      tools/
-        foo.py
-   Assume main.py has SHA1 hash 123 and foo.py has SHA1 hash 456. The resultant
-   staging directory will look like:
-     /tmp/staging/
-       123 -> app/main.py
-       456 -> app/tools/foo.py
-   (Note: "->" denotes a symlink)
-
-   If the staging directory is then copied to a GCS bucket at
-   gs://staging-bucket/ then the resulting manifest will be:
-     {
-       "app/main.py": {
-         "sourceUrl": "https://storage.googleapis.com/staging-bucket/123",
-         "sha1Sum": "123"
-       },
-       "app/tools/foo.py": {
-         "sourceUrl": "https://storage.googleapis.com/staging-bucket/456",
-         "sha1Sum": "456"
-       }
-     }
-
-  Args:
-    source_dir: The original directory containing the application's source
-      code.
-    staging_dir: The directory where the staged files will be created.
-    bucket_ref: A reference to the GCS bucket where the files will be uploaded.
-    excluded_regexes: List of file patterns to skip while building the staging
-      directory.
-
-  Raises:
-    LargeFileError: if one of the files to upload exceeds the maximum App Engine
-    file size.
-
-  Returns:
-    A dictionary which represents the file manifest.
-  """
-  manifest = {}
-  bucket_url = bucket_ref.GetPublicUrl()
-
-  def AddFileToManifest(manifest_path, input_path):
-    """Adds the given file to the current manifest.
-
-    Args:
-      manifest_path: The path to the file as it will be stored in the manifest.
-      input_path: The location of the file to be added to the manifest.
-    Returns:
-      If the target was already in the manifest with different contexts,
-      returns None. In all other cases, returns a target location to which the
-      caller must copy, move, or link the file.
-    """
-    file_ext = os.path.splitext(input_path)[1]
-    sha1_hash = (file_utils.Checksum(algorithm=hashlib.sha1)
-                 .AddFileContents(input_path).HexDigest())
-
-    target_filename = sha1_hash + file_ext
-    target_path = os.path.join(staging_dir, target_filename)
-
-    dest_path = '/'.join([bucket_url, target_filename])
-    old_url = manifest.get(manifest_path, {}).get('sourceUrl', '')
-    if old_url and old_url != dest_path:
-      return None
-    manifest[_FormatForManifest(manifest_path)] = {
-        'sourceUrl': dest_path,
-        'sha1Sum': sha1_hash,
-    }
-    return target_path
-
-  for relative_path in util.FileIterator(source_dir, excluded_regexes):
-    local_path = os.path.join(source_dir, relative_path)
-    size = os.path.getsize(local_path)
-    if size > _MAX_FILE_SIZE:
-      raise LargeFileError(local_path, size, _MAX_FILE_SIZE)
-    target_path = AddFileToManifest(_FormatForManifest(relative_path),
-                                    local_path)
-    if not os.path.exists(target_path):
-      _CopyOrSymlink(local_path, target_path)
-
-  context_files = context_util.CreateContextFiles(
-      staging_dir, None, overwrite=True, source_dir=source_dir)
-  for context_file in context_files:
-    manifest_path = os.path.basename(context_file)
-    target_path = AddFileToManifest(manifest_path, context_file)
-    if not target_path:
-      log.status.Print('Not generating {0} because a user-generated '
-                       'file with the same name exists.'.format(manifest_path))
-    if not target_path or os.path.exists(target_path):
-      # If we get here, it probably means that the user already generated the
-      # context file manually and put it either in the top directory or in some
-      # subdirectory. The new context file is useless and may confuse later
-      # stages of the upload (it is in the staging directory with a
-      # nonconformant name), so delete it. The entry in the manifest will point
-      # at the existing file.
-      os.remove(context_file)
-    else:
-      # Rename the source-context*.json file (which is in the staging directory)
-      # to the hash-based name in the same directory.
-      os.rename(context_file, target_path)
-
-  log.debug('Generated deployment manifest: "{0}"'.format(
-      json.dumps(manifest, indent=2, sort_keys=True)))
-  return manifest
-
-
-def _CopyOrSymlink(source, target):
-  try:
-    # If possible, create a symlink to save time and space.
-    os.symlink(source, target)
-  except AttributeError:
-    # The system does not support symlinks. Do a file copy instead.
-    shutil.copyfile(source, target)
-
-
 def _FormatForManifest(filename):
   """Reformat a filename for the deployment manifest if it is Windows format."""
   if os.path.sep == '\\':
