@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """This module holds exceptions raised by api lib."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.core import exceptions
 
 
@@ -21,31 +26,9 @@ class Error(exceptions.Error):
   """Base error for this module."""
 
 
-class NotFoundError(Error):
-  """Raised when the requested resource does not exist."""
-
-  def __init__(self):
-    super(exceptions.Error,
-          self).__init__('The requested resource was not found.')
-
-
-class ConflictError(Error):
-  """Raised when a new resource already exists."""
-
-  def __init__(self):
-    super(exceptions.Error,
-          self).__init__('The requested resource already exists.')
-
-
 class ConfigError(Error):
   """Raised when unable to parse a config file."""
 
   def __init__(self, message=None, **kwargs):
     message = message or 'Config Error.'
     super(ConfigError, self).__init__(message, **kwargs)
-
-
-STATUS_CODE_TO_ERROR = {
-    404: NotFoundError,
-    409: ConflictError
-}
