@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,11 +16,20 @@
 """Utilities for working with long running operations go/long-running-operation.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import json
+
 from apitools.base.py import encoding
+from apitools.base.py import exceptions as apitools_exceptions
+
 import enum
 
-from googlecloudsdk.api_lib.app.api import requests
+from googlecloudsdk.api_lib.app import exceptions as app_exceptions
+from googlecloudsdk.api_lib.util import exceptions as api_exceptions
+from googlecloudsdk.api_lib.util import requests
 from googlecloudsdk.api_lib.util import waiter
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
@@ -27,7 +37,43 @@ from googlecloudsdk.core import resources
 
 # Default is to retry every 5 seconds for 1 hour.
 DEFAULT_OPERATION_RETRY_INTERVAL = 5
-DEFAULT_OPERATION_MAX_TRIES = (60 / DEFAULT_OPERATION_RETRY_INTERVAL) * 60
+DEFAULT_OPERATION_MAX_TRIES = (60 // DEFAULT_OPERATION_RETRY_INTERVAL) * 60
+
+
+def CallAndCollectOpErrors(method, *args, **kwargs):
+  """Wrapper for method(...) which re-raises operation-style errors.
+
+  Args:
+    method: Original method to call.
+    *args: Positional arguments to method.
+    **kwargs: Keyword arguments to method.
+
+  Raises:
+    MiscOperationError: If the method call itself raises one of the exceptions
+      listed below. Otherwise, the original exception is raised. Preserves
+      stack trace. Re-uses the error string from original error or in the case
+      of HttpError, we synthesize human-friendly string from HttpException.
+      However, HttpException is neither raised nor part of the stack trace.
+
+  Returns:
+    Result of calling method(*args, **kwargs).
+  """
+  try:
+    return method(*args, **kwargs)
+  except apitools_exceptions.HttpError as http_err:
+    # Create HttpException locally only to get its human friendly string
+    _ReraiseMiscOperationError(api_exceptions.HttpException(http_err))
+  except (OperationError, OperationTimeoutError, app_exceptions.Error) as err:
+    _ReraiseMiscOperationError(err)
+
+
+def _ReraiseMiscOperationError(err):
+  """Transform and re-raise error helper."""
+  exceptions.reraise(MiscOperationError(str(err)))
+
+
+class MiscOperationError(exceptions.Error):
+  """Wrapper exception for errors treated as operation failures."""
 
 
 class OperationError(exceptions.Error):
@@ -83,12 +129,14 @@ def GetStatus(operation):
   Returns:
     The status of the operation in string form.
   """
+  # pytype: disable=attribute-error
   if not operation.done:
     return Status.PENDING.name
   elif operation.error:
     return Status.ERROR.name
   else:
     return Status.COMPLETED.name
+  # pytype: enable=attribute-error
 
 
 def _GetInsertTime(operation):
@@ -111,17 +159,22 @@ def _GetInsertTime(operation):
 class AppEngineOperationPoller(waiter.OperationPoller):
   """A poller for appengine operations."""
 
-  def __init__(self, operation_service):
+  def __init__(self, operation_service, operation_metadata_type=None):
     """Sets up poller for appengine operations.
 
     Args:
       operation_service: apitools.base.py.base_api.BaseApiService, api service
         for retrieving information about ongoing operation.
+      operation_metadata_type: Message class for the Operation metadata (for
+        instance, OperationMetadataV1, or OperationMetadataV1Beta).
     """
     self.operation_service = operation_service
+    self.operation_metadata_type = operation_metadata_type
+    self.warnings_seen = set()
 
   def IsDone(self, operation):
     """Overrides."""
+    self._LogNewWarnings(operation)
     if operation.done:
       log.debug('Operation [{0}] complete. Result: {1}'.format(
           operation.name,
@@ -145,7 +198,18 @@ class AppEngineOperationPoller(waiter.OperationPoller):
     """
     request_type = self.operation_service.GetRequestType('Get')
     request = request_type(name=operation_ref.RelativeName())
-    return requests.MakeRequest(self.operation_service.Get, request)
+    operation = self.operation_service.Get(request)
+    self._LogNewWarnings(operation)
+    return operation
+
+  def _LogNewWarnings(self, operation):
+    if self.operation_metadata_type:
+      # Log any new warnings to the end user.
+      new_warnings = GetWarningsFromOperation(
+          operation, self.operation_metadata_type) - self.warnings_seen
+      for warning in new_warnings:
+        log.warning(warning + '\n')
+        self.warnings_seen.add(warning)
 
   def GetResult(self, operation):
     """Simply returns the operation.
@@ -159,11 +223,55 @@ class AppEngineOperationPoller(waiter.OperationPoller):
     return operation
 
 
+class AppEngineOperationBuildPoller(AppEngineOperationPoller):
+  """Waits for a build to be present, or for the operation to finish."""
+
+  def __init__(self, operation_service, operation_metadata_type):
+    """Sets up poller for appengine operations.
+
+    Args:
+      operation_service: apitools.base.py.base_api.BaseApiService, api service
+        for retrieving information about ongoing operation.
+      operation_metadata_type: Message class for the Operation metadata (for
+        instance, OperationMetadataV1, or OperationMetadataV1Beta).
+    """
+    super(AppEngineOperationBuildPoller, self).__init__(operation_service,
+                                                        operation_metadata_type)
+
+  def IsDone(self, operation):
+    if GetBuildFromOperation(operation, self.operation_metadata_type):
+      return True
+    return super(AppEngineOperationBuildPoller, self).IsDone(operation)
+
+
+def GetMetadataFromOperation(operation, operation_metadata_type):
+  if not operation.metadata:
+    return None
+  return encoding.JsonToMessage(
+      operation_metadata_type,
+      encoding.MessageToJson(operation.metadata))
+
+
+def GetBuildFromOperation(operation, operation_metadata_type):
+  metadata = GetMetadataFromOperation(operation, operation_metadata_type)
+  if not metadata or not metadata.createVersionMetadata:
+    return None
+  return metadata.createVersionMetadata.cloudBuildId
+
+
+def GetWarningsFromOperation(operation, operation_metadata_type):
+  metadata = GetMetadataFromOperation(operation, operation_metadata_type)
+  if not metadata:
+    return set()
+  return set(warning for warning in metadata.warning)
+
+
 def WaitForOperation(operation_service, operation,
                      max_retries=None,
                      retry_interval=None,
                      operation_collection='appengine.apps.operations',
-                     message=None):
+                     message=None,
+                     poller=None):
   """Wait until the operation is complete or times out.
 
   Args:
@@ -173,6 +281,7 @@ def WaitForOperation(operation_service, operation,
     retry_interval: Frequency of polling in seconds
     operation_collection: The resource collection of the operation.
     message: str, the message to display while progress tracker displays.
+    poller: AppEngineOperationPoller to poll with, defaulting to done.
   Returns:
     The operation resource when it has completed
   Raises:
@@ -180,7 +289,7 @@ def WaitForOperation(operation_service, operation,
     OperationTimeoutError: when the operation polling times out
 
   """
-  poller = AppEngineOperationPoller(operation_service)
+  poller = poller or AppEngineOperationPoller(operation_service)
   if poller.IsDone(operation):
     return poller.GetResult(operation)
   operation_ref = resources.REGISTRY.ParseRelativeName(
