@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Manage and stream build logs from Cloud Builds."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import time
 
 from apitools.base.py import exceptions as api_exceptions
@@ -22,6 +27,7 @@ from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_attr_os
 from googlecloudsdk.core.credentials import http
+from googlecloudsdk.core.util import encoding
 
 
 class NoLogsBucketException(exceptions.Error):
@@ -49,6 +55,46 @@ class LogTailer(object):
     self.cursor = 0
     self.out = out
 
+  @classmethod
+  def FromBuild(cls, build):
+    """Build a LogTailer from a build resource.
+
+    Args:
+      build: Build resource, The build whose logs shall be streamed.
+
+    Raises:
+      NoLogsBucketException: If the build does not specify a logsBucket.
+
+    Returns:
+      LogTailer, the tailer of this build's logs.
+    """
+    if not build.logsBucket:
+      raise NoLogsBucketException()
+
+    # remove gs:// prefix from bucket
+    log_stripped = build.logsBucket
+    gcs_prefix = 'gs://'
+    if log_stripped.startswith(gcs_prefix):
+      log_stripped = log_stripped[len(gcs_prefix):]
+
+    if '/' not in log_stripped:
+      log_bucket = log_stripped
+      log_object_dir = ''
+    else:
+      [log_bucket, log_object_dir] = log_stripped.split('/', 1)
+      log_object_dir += '/'
+
+    log_object = '{object}log-{id}.txt'.format(
+        object=log_object_dir,
+        id=build.id,
+    )
+
+    return cls(
+        bucket=log_bucket,
+        obj=log_object,
+        out=log.out,
+        url_pattern='https://storage.googleapis.com/{bucket}/{obj}')
+
   def Poll(self, is_last=False):
     """Poll the GCS object and print any new bytes to the console.
 
@@ -82,7 +128,9 @@ class LogTailer(object):
       if self.cursor == 0:
         self._PrintFirstLine()
       self.cursor += len(body)
+      body = encoding.Decode(body)
       self._PrintLogLine(body.rstrip('\n'))
+
       if is_last:
         self._PrintLastLine()
       return
@@ -125,8 +173,8 @@ class CloudBuildClient(object):
   """Client for interacting with the Cloud Build API (and Cloud Build logs)."""
 
   def __init__(self, client=None, messages=None):
-    self.client = client or cloudbuild_util.GetClient()
-    self.messages = messages or cloudbuild_util.GetMessages()
+    self.client = client or cloudbuild_util.GetClientInstance()
+    self.messages = messages or cloudbuild_util.GetMessagesModule()
 
   def GetBuild(self, build_ref):
     """Get a Build message.
@@ -142,46 +190,6 @@ class CloudBuildClient(object):
             projectId=build_ref.projectId,
             id=build_ref.id))
 
-  def _MakeLogTailer(self, build):
-    """Stream the logs for a build.
-
-    Args:
-      build: Build resource, The build whose logs shall be streamed.
-
-    Raises:
-      NoLogsBucketException: If the build does not specify a logsBucket.
-
-    Returns:
-      LogTailer, the tailer of this build's logs.
-    """
-
-    if not build.logsBucket:
-      raise NoLogsBucketException()
-
-    # remove gs:// prefix from bucket
-    log_stripped = build.logsBucket
-    gcs_prefix = 'gs://'
-    if log_stripped.startswith(gcs_prefix):
-      log_stripped = log_stripped[len(gcs_prefix):]
-
-    if '/' not in log_stripped:
-      log_bucket = log_stripped
-      log_object_dir = ''
-    else:
-      [log_bucket, log_object_dir] = log_stripped.split('/', 1)
-      log_object_dir += '/'
-
-    log_object = '{object}log-{id}.txt'.format(
-        object=log_object_dir,
-        id=build.id,
-    )
-
-    return LogTailer(
-        bucket=log_bucket,
-        obj=log_object,
-        out=log.out,
-        url_pattern='https://storage.googleapis.com/{bucket}/{obj}')
-
   def Stream(self, build_ref):
     """Stream the logs for a build.
 
@@ -196,7 +204,7 @@ class CloudBuildClient(object):
       poll.
     """
     build = self.GetBuild(build_ref)
-    log_tailer = self._MakeLogTailer(build)
+    log_tailer = LogTailer.FromBuild(build)
 
     statuses = self.messages.Build.StatusValueValuesEnum
     working_statuses = [
@@ -211,7 +219,7 @@ class CloudBuildClient(object):
 
     # Poll the logs one final time to ensure we have everything. We know this
     # final poll will get the full log contents because GCS is strongly
-    # consistent and Container Builder waits for logs to finish pushing before
+    # consistent and Cloud Build waits for logs to finish pushing before
     # marking the build complete.
     log_tailer.Poll(is_last=True)
 
@@ -227,6 +235,6 @@ class CloudBuildClient(object):
       NoLogsBucketException: If the build does not specify a logsBucket.
     """
     build = self.GetBuild(build_ref)
-    log_tailer = self._MakeLogTailer(build)
+    log_tailer = LogTailer.FromBuild(build)
 
     log_tailer.Poll(is_last=True)
