@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,3 +14,5 @@
 # limitations under the License.
 
 """Cloudresourcemanager API lib helper files."""
+
+from __future__ import unicode_literals
