@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,11 +15,11 @@
 
 """Util for projects."""
 
-from apitools.base.py import exceptions
-from googlecloudsdk.api_lib.cloudresourcemanager import errors
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.util import apis
-from googlecloudsdk.api_lib.util import exceptions as api_exceptions
-from googlecloudsdk.core import log
 
 
 class DeletedResource(object):
@@ -52,26 +53,3 @@ def IsActive(project):
   """
   lifecycle_enum = GetMessages().Project.LifecycleStateValueValuesEnum
   return project.lifecycleState == lifecycle_enum.ACTIVE
-
-
-def ConvertHttpError(error):
-  """Raises a more specific ProjectAccessError from an HttpError.
-
-  Args:
-    error: HttpError resulting from unsuccessful call to API.
-
-  Returns:
-    ProjectAccessError or original error
-
-  First line will parse project ID out of error url.
-  Example:
-   URL = .../v1/projects/BAD_ID:method?prettyPrint=True&alt=json
-   project_id = 'BAD_ID'
-  """
-  log.debug('HttpError in projects API call.', exc_info=True)
-  if isinstance(error, exceptions.HttpError):
-    project_id = error.url.split('/')[-1].split('?')[0].split(':')[0]
-    if error.status_code == 403 or error.status_code == 404:
-      return errors.ProjectAccessError(project_id)
-    return api_exceptions.HttpException(error, '{message}{details?\n{?}}')
-  return error
