@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,31 +15,36 @@
 
 """Base classes for abstracting away common logic."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import abc
-import collections
+import argparse  # pylint: disable=unused-import
 import json
 import textwrap
 
+from apitools.base.py import base_api  # pylint: disable=unused-import
 import enum
 from googlecloudsdk.api_lib.compute import base_classes_resource_registry as resource_registry
 from googlecloudsdk.api_lib.compute import client_adapter
 from googlecloudsdk.api_lib.compute import constants
 from googlecloudsdk.api_lib.compute import lister
-from googlecloudsdk.api_lib.compute import managed_instance_groups_utils
 from googlecloudsdk.api_lib.compute import property_selector
 from googlecloudsdk.api_lib.compute import request_helper
 from googlecloudsdk.api_lib.compute import resource_specs
 from googlecloudsdk.api_lib.compute import scope_prompter
 from googlecloudsdk.api_lib.compute import utils
-from googlecloudsdk.api_lib.util import apis as core_apis
-from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions as calliope_exceptions
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
+from googlecloudsdk.core import yaml
 from googlecloudsdk.core.util import text
-import yaml
+from googlecloudsdk.core.util import typing  # pylint: disable=unused-import
+import six
 
 
 class ComputeApiHolder(object):
@@ -52,7 +58,7 @@ class ComputeApiHolder(object):
     else:
       self._api_version = 'v1'
     self._client = None
-    self._resources = None
+    self._resources = None  # type: resources.Registry
 
   @property
   def client(self):
@@ -70,34 +76,6 @@ class ComputeApiHolder(object):
     return self._resources
 
 
-class ComputeUserAccountsApiHolder(object):
-  """Convenience class to hold lazy initialized client and resources."""
-
-  def __init__(self, release_track):
-    if release_track == base.ReleaseTrack.ALPHA:
-      self._api_version = 'alpha'
-    else:
-      self._api_version = 'beta'
-    self._client = None
-    self._resources = None
-
-  @property
-  def client(self):
-    """Specifies the compute client."""
-    if self._client is None:
-      self._client = core_apis.GetClientInstance(
-          'clouduseraccounts', self._api_version)
-    return self._client
-
-  @property
-  def resources(self):
-    """Specifies the resources parser for compute resources."""
-    if self._resources is None:
-      self._resources = resources.REGISTRY.Clone()
-      self._resources.RegisterApiByName('clouduseraccounts', self._api_version)
-    return self._resources
-
-
 class BaseCommand(base.Command, scope_prompter.ScopePrompter):
   """Base class for all compute subcommands."""
 
@@ -105,10 +83,8 @@ class BaseCommand(base.Command, scope_prompter.ScopePrompter):
     super(BaseCommand, self).__init__(*args, **kwargs)
 
     self.__resource_spec = None
-    self._project = properties.VALUES.core.project.Get(required=True)
+    self._project = properties.VALUES.core.project.Get(required=True)  # type: str
     self._compute_holder = ComputeApiHolder(self.ReleaseTrack())
-    self._user_accounts_holder = ComputeUserAccountsApiHolder(
-        self.ReleaseTrack())
 
   @property
   def _resource_spec(self):
@@ -141,7 +117,7 @@ class BaseCommand(base.Command, scope_prompter.ScopePrompter):
   @property
   def project(self):
     """Specifies the user's project."""
-    return self._project
+    return self._project  # pytype: disable=attribute-error
 
   @property
   def batch_url(self):
@@ -161,15 +137,7 @@ class BaseCommand(base.Command, scope_prompter.ScopePrompter):
   @property
   def resources(self):
     """Specifies the resources parser for compute resources."""
-    return self._compute_holder.resources
-
-  @property
-  def clouduseraccounts(self):
-    return self._user_accounts_holder.client
-
-  @property
-  def clouduseraccounts_resources(self):
-    return self._user_accounts_holder.resources
+    return self._compute_holder.resources   # pytype: disable=attribute-error
 
   @property
   def messages(self):
@@ -184,6 +152,12 @@ class BaseCommand(base.Command, scope_prompter.ScopePrompter):
 class BaseLister(base.ListCommand, BaseCommand):
   """Base class for the list subcommands."""
 
+  # Define class attributes for pytype
+  self_links = None  # type: set
+  names = None  # type: set
+  resource_refs = None  # type: list
+  service = None  # type: base_api.BaseApiService
+
   @staticmethod
   def Args(parser):
     parser.add_argument(
@@ -191,14 +165,14 @@ class BaseLister(base.ListCommand, BaseCommand):
         metavar='NAME',
         nargs='*',
         default=[],
-        completion_resource='compute.instances',
+        completer=completers.InstancesCompleter,
         help=('If provided, show details for the specified names and/or URIs '
               'of resources.'))
 
     parser.add_argument(
         '--regexp', '-r',
         help="""\
-        A regular expression to filter the names of the results on. Any names
+        Regular expression to filter the names of the results on. Any names
         that do not match the entire regular expression will be filtered out.
         """)
 
@@ -242,6 +216,7 @@ class BaseLister(base.ListCommand, BaseCommand):
       self.names.add(name)
 
   def FilterResults(self, args, items):
+    # type: (typing.Any, list[dict[str, str]]) -> typing.Generator[dict[str,str], typing.Any, typing.Any]  # pylint: disable=line-too-long
     """Filters the list results by name and URI."""
     for item in items:
       # If no positional arguments were given, do no filtering.
@@ -250,7 +225,7 @@ class BaseLister(base.ListCommand, BaseCommand):
 
       # At this point, we have to do filtering because there was at
       # least one positional argument.
-      elif item.selfLink in self.self_links or item.name in self.names:
+      elif item['selfLink'] in self.self_links or item['name'] in self.names:
         yield item
 
   def ComputeDynamicProperties(self, args, items):
@@ -286,7 +261,8 @@ class BaseLister(base.ListCommand, BaseCommand):
     errors = []
 
     self.PopulateResourceFilteringStructures(args)
-    items = self.FilterResults(args, self.GetResources(args, errors))
+    items = self.FilterResults(
+        args, self.GetResources(args, errors))  # pytype: disable=wrong-arg-types
     items = lister.ProcessResults(
         resources=items,
         field_selector=field_selector)
@@ -303,7 +279,7 @@ class GlobalLister(BaseLister):
   """Base class for listing global resources."""
 
   def GetResources(self, args, errors):
-    return lister.GetGlobalResources(
+    return lister.GetGlobalResourcesDicts(
         service=self.service,
         project=self.project,
         filter_expr=self.GetFilterExpr(args),
@@ -314,21 +290,29 @@ class GlobalLister(BaseLister):
 
 def GetGlobalListerHelp(resource):
   """Returns the detailed help dict for a global list command."""
-  return {
+  detailed_help = {
       'brief': 'List Google Compute Engine ' + resource,
       'DESCRIPTION': """\
-          *{{command}}* displays all Google Compute Engine {0} in a project.
-          """.format(resource),
+*{{command}}* displays all Google Compute Engine {0} in a project.
+""".format(resource),
       'EXAMPLES': """\
-          To list all {0} in a project in table form, run:
+To list all {0} in a project in table form, run:
 
-            $ {{command}}
+  $ {{command}}
 
-          To list the URIs of all {0} in a project, run:
+To list the URIs of all {0} in a project, run:
 
-            $ {{command}} --uri
-            """.format(resource)
+  $ {{command}} --uri
+""".format(resource)
   }
+  if resource == 'images':
+    detailed_help['EXAMPLES'] += """
+To list the names of {0} older than one year from oldest to newest
+(`-P1Y` is an [ISO8601 duration](https://en.wikipedia.org/wiki/ISO_8601)):
+
+  $ {{command}} --format="value(NAME)" --filter="creationTimestamp < -P1Y"
+""".format(resource)
+  return detailed_help
 
 
 class RegionalLister(BaseLister):
@@ -349,7 +333,7 @@ class RegionalLister(BaseLister):
         self.CreateGlobalReference(region, resource_type='regions').Name()
         for region in args.regions]
 
-    return lister.GetRegionalResources(
+    return lister.GetRegionalResourcesDicts(
         service=self.service,
         project=self.project,
         requested_regions=region_names,
@@ -364,25 +348,25 @@ def GetRegionalListerHelp(resource):
   return {
       'brief': 'List Google Compute Engine ' + resource,
       'DESCRIPTION': """\
-          *{{command}}* displays all Google Compute Engine {0} in a project.
+*{{command}}* displays all Google Compute Engine {0} in a project.
 
-          By default, {0} from all regions are listed. The results can be
-          narrowed down by providing the ``--regions'' flag.
-          """.format(resource),
+By default, {0} from all regions are listed. The results can be
+narrowed down using a filter: `--filter="region:( REGION ... )"`.
+""".format(resource),
       'EXAMPLES': """\
-          To list all {0} in a project in table form, run:
+To list all {0} in a project in table form, run:
 
-            $ {{command}}
+  $ {{command}}
 
-          To list the URIs of all {0} in a project, run:
+To list the URIs of all {0} in a project, run:
 
-            $ {{command}} --uri
+  $ {{command}} --uri
 
-          To list all {0} in the ``us-central1'' and ``europe-west1'' regions,
-          run:
+To list all {0} in the ``us-central1'' and ``europe-west1'' regions,
+run:
 
-            $ {{command}} --regions us-central1 europe-west1
-            """.format(resource)
+  $ {{command}} --filter="region:( us-central1 europe-west1 )"
+""".format(resource)
   }
 
 
@@ -397,14 +381,14 @@ class ZonalLister(BaseLister):
         metavar='ZONE',
         help='If provided, only resources from the given zones are queried.',
         type=arg_parsers.ArgList(min_length=1),
-        completion_resource='compute.zones',
+        completer=completers.ZonesCompleter,
         default=[])
 
   def GetResources(self, args, errors):
     zone_names = [
         self.CreateGlobalReference(zone, resource_type='zones').Name()
         for zone in args.zones]
-    return lister.GetZonalResources(
+    return lister.GetZonalResourcesDicts(
         service=self.service,
         project=self.project,
         requested_zones=zone_names,
@@ -419,25 +403,25 @@ def GetZonalListerHelp(resource):
   return {
       'brief': 'List Google Compute Engine ' + resource,
       'DESCRIPTION': """\
-          *{{command}}* displays all Google Compute Engine {0} in a project.
+*{{command}}* displays all Google Compute Engine {0} in a project.
 
-          By default, {0} from all zones are listed. The results can be narrowed
-          down by providing the ``--zones'' flag.
-          """.format(resource),
+By default, {0} from all zones are listed. The results can be narrowed
+down using a filter: `--filter="zone:( ZONE ... )"`.
+""".format(resource),
       'EXAMPLES': """\
-          To list all {0} in a project in table form, run:
+To list all {0} in a project in table form, run:
 
-            $ {{command}}
+  $ {{command}}
 
-          To list the URIs of all {0} in a project, run:
+To list the URIs of all {0} in a project, run:
 
-            $ {{command}} --uri
+  $ {{command}} --uri
 
-          To list all {0} in the ``us-central1-b'' and ``europe-west1-d'' zones,
-          run:
+To list all {0} in the ``us-central1-b'' and ``europe-west1-d'' zones,
+run:
 
-            $ {{command}} --zones us-central1-b europe-west1-d
-            """.format(resource)
+  $ {{command}} --filter="zone:( us-central1-b europe-west1-d )"
+""".format(resource)
   }
 
 
@@ -497,7 +481,7 @@ class MultiScopeLister(BaseLister):
     """The service used to get aggregated list of resources."""
 
   def GetResources(self, args, errors):
-    """Yields zonal, regional and/or global resources.
+    """Returns zonal, regional and/or global resources.
 
     Args:
       args: argparse.Namespace, Parsed arguments
@@ -571,7 +555,7 @@ class MultiScopeLister(BaseLister):
                maxResults=max_results,
                project=project)))
 
-    return request_helper.MakeRequests(
+    return request_helper.ListJson(
         requests=requests,
         http=self.http,
         batch_url=self.batch_url,
@@ -583,24 +567,24 @@ def GetMultiScopeListerHelp(resource, scopes):
 
   zone_example_text = """\
 
-          To list all {0} in zones ``us-central1-b'' and ``europe-west1-d'',
-          run:
+To list all {0} in zones ``us-central1-b''
+and ``europe-west1-d'', given they are zonal resources, run:
 
-            $ {{command}} --zones us-central1,europe-west1
-  """
+  $ {{command}} --filter="zone:( europe-west1-d us-central1-b )"
+"""
   region_example_text = """\
 
-          To list all {0} in the ``us-central1'' and ``europe-west1'' regions,
-          run:
+To list all {0} in the ``us-central1'' and ``europe-west1'' regions,
+given they are regional resources, run:
 
-            $ {{command}} --regions us-central1,europe-west1
-  """
+  $ {{command}} --filter="region:( europe-west1 us-central1 )"
+"""
   global_example_text = """\
 
-          To list all global {0} in a project, run:
+To list all global {0} in a project, run:
 
-            $ {{command}} --global
-  """
+  $ {{command}} --global
+"""
 
   allowed_flags = []
   default_result = []
@@ -622,21 +606,22 @@ def GetMultiScopeListerHelp(resource, scopes):
   return {
       'brief': 'List Google Compute Engine ' + resource,
       'DESCRIPTION': """\
-          *{{command}}* displays all Google Compute Engine {0} in a project.
+*{{command}}* displays all Google Compute Engine {0} in a project.
 
-          By default, {1} are listed. The results can be narrowed down by
-          providing the {2} flag.
-          """.format(resource, default_result_text, allowed_flags_text),
+By default, {1} are listed. The results can be narrowed down by
+providing the {2} flag.
+""".format(resource, default_result_text, allowed_flags_text),
       'EXAMPLES': ("""\
-          To list all {0} in a project in table form, run:
+To list all {0} in a project in table form, run:
 
-            $ {{command}}
+  $ {{command}}
 
-          To list the URIs of all {0} in a project, run:
+To list the URIs of all {0} in a project, run:
 
-            $ {{command}} --uri
-          """ + (global_example_text
-                 if ScopeType.global_scope in scopes else '')
+  $ {{command}} --uri
+"""
+                   + (global_example_text
+                      if ScopeType.global_scope in scopes else '')
                    + (region_example_text
                       if ScopeType.regional_scope in scopes else '')
                    + (zone_example_text
@@ -669,17 +654,18 @@ def GetGlobalRegionalListerHelp(resource):
 class BaseDescriber(base.DescribeCommand, BaseCommand):
   """Base class for the describe subcommands."""
 
-  @staticmethod
-  def Args(parser, resource=None, list_command_path=None):
-    BaseDescriber.AddArgs(parser, resource, list_command_path)
+  # Define class attributes for pytype
+  service = None  # type: base_api.BaseApiService
 
   @staticmethod
-  def AddArgs(parser, resource=None, list_command_path=None):
+  def Args(parser, resource=None):
+    BaseDescriber.AddArgs(parser, resource)
+
+  @staticmethod
+  def AddArgs(parser, resource=None):
     parser.add_argument(
         'name',
         metavar='NAME',
-        completion_resource=resource,
-        list_command_path=list_command_path,
         help='The name of the resource to fetch.')
 
   @property
@@ -694,6 +680,7 @@ class BaseDescriber(base.DescribeCommand, BaseCommand):
     pass
 
   def SetNameField(self, ref, request):
+    # type: (resources.Resource, typing.Any) -> None
     """Sets the field in the request that corresponds to the object name."""
     name_field = self.service.GetMethodConfig(self.method).ordered_params[-1]
     setattr(request, name_field, ref.Name())
@@ -705,7 +692,7 @@ class BaseDescriber(base.DescribeCommand, BaseCommand):
 
   def Run(self, args):
     """Yields JSON-serializable dicts of resources."""
-    ref = self.CreateReference(args)
+    ref = self.CreateReference(args)  # type: resources.Resource
 
     get_request_class = self.service.GetRequestType(self.method)
 
@@ -732,141 +719,6 @@ class BaseDescriber(base.DescribeCommand, BaseCommand):
     return resource_list[0]
 
 
-class MultiScopeDescriber(BaseDescriber):
-  """Base class for describing global or regional resources."""
-
-  @staticmethod
-  def AddScopeArgs(parser, resource_type, scope_types, command=None):
-    resource = resource_type
-    BaseDescriber.AddArgs(parser, 'compute.' + resource, command)
-
-    scope = parser.add_mutually_exclusive_group()
-
-    if ScopeType.zonal_scope in scope_types:
-      scope.add_argument(
-          '--zone',
-          help='The zone of the resource to fetch.',
-          completion_resource='compute.zones',
-          action=actions.StoreProperty(properties.VALUES.compute.zone))
-    if ScopeType.regional_scope in scope_types:
-      scope.add_argument(
-          '--region',
-          help='The region of the resource to fetch.',
-          completion_resource='compute.regions',
-          action=actions.StoreProperty(properties.VALUES.compute.region))
-    if ScopeType.global_scope in scope_types:
-      scope.add_argument(
-          '--global',
-          action='store_true',
-          help=('If provided, it is assumed that the requested resource is '
-                'global.'))
-
-  @abc.abstractproperty
-  def global_service(self):
-    """The service used to list global resources."""
-
-  @abc.abstractproperty
-  def regional_service(self):
-    """The service used to list regional resources."""
-
-  @abc.abstractproperty
-  def zonal_service(self):
-    """The service used to list zonal resources."""
-
-  @abc.abstractproperty
-  def global_resource_type(self):
-    """The type of global resources."""
-
-  @abc.abstractproperty
-  def regional_resource_type(self):
-    """The type of regional resources."""
-
-  @abc.abstractproperty
-  def zonal_resource_type(self):
-    """The type of regional resources."""
-
-  @property
-  def service(self):
-    return self._service
-
-  def CreateReference(self, args, default=None):
-    # Check if scope was provided
-    has_region = bool(getattr(args, 'region', None))
-    has_zone = bool(getattr(args, 'zone', None))
-    has_global = bool(getattr(args, 'global', None))
-
-    null = object()
-
-    # Check if only one kind of scope can be provided
-    only_zone_prompt = ((getattr(args, 'zone', null) is not null) and
-                        (getattr(args, 'region', null) is null))
-    only_region_prompt = ((getattr(args, 'region', null) is not null) and
-                          (getattr(args, 'zone', null) is null))
-
-    if not (has_region or has_zone or has_global):
-      if default == ScopeType.global_scope:
-        has_global = True
-      elif default == ScopeType.regional_scope:
-        has_region = True
-      elif default == ScopeType.zonal_scope:
-        has_zone = True
-
-    ref = None
-    try:
-      params = {}
-      if has_region:
-        params['region'] = args.region
-      if has_zone:
-        params['zone'] = args.zone
-      ref = self.resources.Parse(args.name, params=params)
-    except resources.UnknownCollectionException:
-      ref = None
-
-    if ref is None:
-      if has_global:
-        ref = self.CreateGlobalReference(
-            args.name, resource_type=self.global_resource_type)
-      elif has_region or only_region_prompt:
-        ref = self.CreateRegionalReference(
-            args.name, args.region, resource_type=self.regional_resource_type)
-      elif has_zone or only_zone_prompt:
-        ref = self.CreateZonalReference(
-            args.name, args.zone, resource_type=self.zonal_resource_type)
-      else:
-        ref = self.PromptForMultiScopedReferences(
-            [args.name],
-            scope_names=['zone', 'region'],
-            scope_services=[self.compute.zones, self.compute.regions],
-            resource_types=[
-                self.zonal_resource_type, self.regional_resource_type],
-            flag_names=['--zone', '--region'])[0]
-
-    valid_collections = ['compute.{0}'.format(resource_type)
-                         for resource_type in [self.zonal_resource_type,
-                                               self.regional_resource_type,
-                                               self.global_resource_type]
-                         if resource_type is not None]
-
-    if ref.Collection() not in valid_collections:
-      raise calliope_exceptions.ToolException(
-          'You must pass in a reference to a global or regional resource.')
-
-    ref_resource_type = utils.CollectionToResourceType(ref.Collection())
-    if ref_resource_type == self.global_resource_type:
-      self._service = self.global_service
-    elif ref_resource_type == self.regional_resource_type:
-      self._service = self.regional_service
-    else:
-      self._service = self.zonal_service
-    return ref
-
-  def ScopeRequest(self, ref, request):
-    if ref.Collection() == 'compute.{0}'.format(self.regional_resource_type):
-      request.region = ref.region
-    if ref.Collection() == 'compute.{0}'.format(self.zonal_resource_type):
-      request.zone = ref.zone
-
-
 def GetMultiScopeDescriberHelp(resource, scopes):
   """Returns the detailed help dict for a multiscope describe command.
 
@@ -880,32 +732,33 @@ def GetMultiScopeDescriberHelp(resource, scopes):
   article = text.GetArticle(resource)
   zone_example_text = """\
 
-          To get details about a zonal {0} in the ``us-central1-b'' zone, run:
+To get details about a zonal {0} in the ``us-central1-b'' zone, run:
 
-            $ {{command}} --zone us-central1-b
-  """
+  $ {{command}} --zone us-central1-b
+"""
   region_example_text = """\
 
-          To get details about a regional {0} in the ``us-central1'' regions,
-          run:
+To get details about a regional {0} in the ``us-central1'' regions,
+run:
 
-            $ {{command}} --region us-central1
-  """
+  $ {{command}} --region us-central1
+"""
   global_example_text = """\
 
-          To get details about a global {0}, run:
+To get details about a global {0}, run:
 
-            $ {{command}} --global
-  """
+  $ {{command}} --global
+"""
   return {
       'brief': ('Display detailed information about {0} {1}'
                 .format(article, resource)),
       'DESCRIPTION': """\
-          *{{command}}* displays all data associated with {0} {1} in a project.
-          """.format(article, resource),
+*{{command}}* displays all data associated with {0} {1} in a project.
+""".format(article, resource),
       'EXAMPLES': ("""\
-          """ + (global_example_text
-                 if ScopeType.global_scope in scopes else '')
+"""
+                   + (global_example_text
+                      if ScopeType.global_scope in scopes else '')
                    + (region_example_text
                       if ScopeType.regional_scope in scopes else '')
                    + (zone_example_text
@@ -914,93 +767,6 @@ def GetMultiScopeDescriberHelp(resource, scopes):
   }
 
 
-# TODO(b/37764090) - Remove this mixin with refactoring of instance-groups
-# managed list command away from base_classes.BaseLister.
-class InstanceGroupManagerDynamicProperiesMixin(object):
-  """Mixin class to compute dynamic information for instance groups."""
-
-  def ComputeDynamicProperties(self, args, items):
-    """Add Autoscaler information if Autoscaler is defined for the item."""
-    _ = args
-    # Items are expected to be IGMs.
-    items = list(items)
-    for mig in managed_instance_groups_utils.AddAutoscalersToMigs(
-        migs_iterator=self.ComputeInstanceGroupSize(items=items),
-        client=self.compute_client,
-        resources=self.resources,
-        fail_when_api_not_supported=False):
-      if 'autoscaler' in mig and mig['autoscaler'] is not None:
-        # status is present in autoscaler iff Autoscaler message has embedded
-        # StatusValueValuesEnum defined.
-        if (getattr(mig['autoscaler'], 'status', False) and mig['autoscaler']
-            .status == self.messages.Autoscaler.StatusValueValuesEnum.ERROR):
-          mig['autoscaled'] = 'yes (*)'
-          self._had_errors = True
-        else:
-          mig['autoscaled'] = 'yes'
-      else:
-        mig['autoscaled'] = 'no'
-      yield mig
-
-  def ComputeInstanceGroupSize(self, items):
-    """Add information about Instance Group size."""
-    errors = []
-    items = list(items)
-    zone_refs = [
-        self.resources.Parse(
-            mig['zone'],
-            params={'project': properties.VALUES.core.project.GetOrFail},
-            collection='compute.zones')
-        for mig in items if 'zone' in mig
-    ]
-    region_refs = [
-        self.resources.Parse(
-            mig['region'],
-            params={'project': properties.VALUES.core.project.GetOrFail},
-            collection='compute.regions')
-        for mig in items if 'region' in mig
-    ]
-    group_by_project = managed_instance_groups_utils.GroupByProject
-
-    zonal_instance_groups = []
-    for project, zone_refs in group_by_project(zone_refs).iteritems():
-      zonal_instance_groups.extend(lister.GetZonalResources(
-          service=self.compute.instanceGroups,
-          project=project,
-          requested_zones=set([zone.zone for zone in zone_refs]),
-          filter_expr=None,
-          http=self.http,
-          batch_url=self.batch_url,
-          errors=errors))
-
-    regional_instance_groups = []
-    if getattr(self.compute, 'regionInstanceGroups', None):
-      for project, region_refs in group_by_project(region_refs).iteritems():
-        regional_instance_groups.extend(lister.GetRegionalResources(
-            service=self.compute.regionInstanceGroups,
-            project=project,
-            requested_regions=set([region.region for region in region_refs]),
-            filter_expr=None,
-            http=self.http,
-            batch_url=self.batch_url,
-            errors=errors))
-
-    instance_groups = zonal_instance_groups + regional_instance_groups
-    instance_group_uri_to_size = {ig.selfLink: ig.size
-                                  for ig in instance_groups}
-
-    if errors:
-      utils.RaiseToolException(errors)
-
-    for item in items:
-      self_link = item['selfLink']
-      gm_self_link = self_link.replace(
-          '/instanceGroupManagers/', '/instanceGroups/')
-
-      item['size'] = str(instance_group_uri_to_size.get(gm_self_link, ''))
-      yield item
-
-
 HELP = textwrap.dedent("""\
     You can edit the resource below. Lines beginning with "#" are
     ignored.
@@ -1020,21 +786,11 @@ HELP = textwrap.dedent("""\
 def SerializeDict(value, fmt):
   """Serializes value to either JSON or YAML."""
   if fmt == 'json':
-    return json.dumps(
-        value,
-        indent=2,
-        sort_keys=True,
-        separators=(',', ': '))
+    return six.text_type(
+        json.dumps(
+            value, indent=2, sort_keys=True, separators=(str(','), str(': '))))
   else:
-    yaml.add_representer(
-        collections.OrderedDict,
-        yaml.dumper.SafeRepresenter.represent_dict,
-        Dumper=yaml.dumper.SafeDumper)
-    return yaml.safe_dump(
-        value,
-        indent=2,
-        default_flow_style=False,
-        width=70)
+    return six.text_type(yaml.dump(value))
 
 
 def DeserializeValue(value, fmt):
