@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,7 +14,10 @@
 # limitations under the License.
 """Interconnect."""
 
-from googlecloudsdk.core.resource import resource_projector
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 
 class Interconnect(object):
@@ -29,7 +33,7 @@ class Interconnect(object):
 
   def _MakeCreateRequestTuple(self, description, location, interconnect_type,
                               requested_link_count, link_type, admin_enabled,
-                              noc_contact_email):
+                              noc_contact_email, customer_name):
     return (self._client.interconnects, 'Insert',
             self._messages.ComputeInterconnectsInsertRequest(
                 project=self.ref.project,
@@ -41,11 +45,17 @@ class Interconnect(object):
                     nocContactEmail=noc_contact_email,
                     requestedLinkCount=requested_link_count,
                     location=location,
-                    adminEnabled=admin_enabled)))
+                    adminEnabled=admin_enabled,
+                    customerName=customer_name)))
 
   def _MakePatchRequestTuple(self, description, location, interconnect_type,
                              requested_link_count, link_type, admin_enabled,
-                             noc_contact_email):
+                             noc_contact_email, labels, label_fingerprint):
+    kwargs = {}
+    if labels is not None:
+      kwargs['labels'] = labels
+    if label_fingerprint is not None:
+      kwargs['labelFingerprint'] = label_fingerprint
     return (self._client.interconnects, 'Patch',
             self._messages.ComputeInterconnectsPatchRequest(
                 interconnect=self.ref.Name(),
@@ -57,7 +67,8 @@ class Interconnect(object):
                     nocContactEmail=noc_contact_email,
                     requestedLinkCount=requested_link_count,
                     location=location,
-                    adminEnabled=admin_enabled),
+                    adminEnabled=admin_enabled,
+                    **kwargs),
                 project=self.ref.project))
 
   def _MakeDeleteRequestTuple(self):
@@ -70,6 +81,11 @@ class Interconnect(object):
             self._messages.ComputeInterconnectsGetRequest(
                 project=self.ref.project, interconnect=self.ref.Name()))
 
+  def _MakeGetDiagnosticsRequestTuple(self):
+    return (self._client.interconnects, 'GetDiagnostics',
+            self._messages.ComputeInterconnectsGetDiagnosticsRequest(
+                project=self.ref.project, interconnect=self.ref.Name()))
+
   @property
   def _messages(self):
     return self._compute_client.messages
@@ -82,12 +98,14 @@ class Interconnect(object):
              link_type=None,
              admin_enabled=False,
              noc_contact_email=None,
+             customer_name=None,
              only_generate_request=False):
     """Create an interconnect."""
     requests = [
         self._MakeCreateRequestTuple(description, location, interconnect_type,
                                      requested_link_count, link_type,
-                                     admin_enabled, noc_contact_email)
+                                     admin_enabled, noc_contact_email,
+                                     customer_name)
     ]
     if not only_generate_request:
       resources = self._compute_client.MakeRequests(requests)
@@ -104,7 +122,14 @@ class Interconnect(object):
     requests = [self._MakeDescribeRequestTuple()]
     if not only_generate_request:
       resources = self._compute_client.MakeRequests(requests)
-      return resource_projector.MakeSerializable(resources[0])
+      return resources[0]
+    return requests
+
+  def GetDiagnostics(self, only_generate_request=False):
+    requests = [self._MakeGetDiagnosticsRequestTuple()]
+    if not only_generate_request:
+      resources = self._compute_client.MakeRequests(requests)
+      return resources[0]
     return requests
 
   def Patch(self,
@@ -115,12 +140,15 @@ class Interconnect(object):
             link_type=None,
             admin_enabled=False,
             noc_contact_email=None,
-            only_generate_request=False):
+            only_generate_request=False,
+            labels=None,
+            label_fingerprint=None):
     """Patch an interconnect."""
     requests = [
         self._MakePatchRequestTuple(description, location, interconnect_type,
                                     requested_link_count, link_type,
-                                    admin_enabled, noc_contact_email)
+                                    admin_enabled, noc_contact_email, labels,
+                                    label_fingerprint)
     ]
     if not only_generate_request:
       resources = self._compute_client.MakeRequests(requests)
