@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,14 +13,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Annotates the resource types with extra information."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import collections
-import httplib
 
 from apitools.base.protorpclite import messages
 
 from googlecloudsdk.api_lib.compute import instance_utils
 from googlecloudsdk.api_lib.compute import path_simplifier
 from googlecloudsdk.api_lib.compute import property_selector
+import six
+import six.moves.http_client
 
 
 def _FirewallRulesToCell(firewall):
@@ -136,7 +143,7 @@ def _MachineTypeMemoryToCell(machine_type):
   """Returns the memory of the given machine type in GB."""
   memory = machine_type.get('memoryMb')
   if memory:
-    return '{0:5.2f}'.format(memory / 2.0 ** 10)
+    return '{0:5.2f}'.format(float(memory) / 2**10)
   else:
     return ''
 
@@ -156,7 +163,7 @@ def _FormatCustomMachineTypeName(mt):
   custom_cpu, custom_ram = instance_utils.GetCpuRamFromCustomName(mt)
   if custom_cpu and custom_ram:
     # Restricting output to 2 decimal places
-    custom_ram_gb = '{0:.2f}'.format(float(custom_ram) / (2 ** 10))
+    custom_ram_gb = '{0:.2f}'.format(custom_ram / (2**10))
     mt = 'custom ({0} vCPU, {1} GiB)'.format(custom_cpu, custom_ram_gb)
   return mt
 
@@ -221,7 +228,7 @@ def FormatDescribeMachineTypeName(resources, com_path):
 def _OperationHttpStatusToCell(operation):
   """Returns the HTTP response code of the given operation."""
   if operation.get('status') == 'DONE':
-    return operation.get('httpErrorStatusCode') or httplib.OK
+    return operation.get('httpErrorStatusCode') or six.moves.http_client.OK
   else:
     return ''
 
@@ -861,27 +868,6 @@ _SPECS_V1 = {
         editables=None
     ),
 
-    'urlMaps': _InternalSpec(
-        message_class_name='UrlMap',
-        table_cols=[
-            ('NAME', 'name'),
-            ('DEFAULT_SERVICE', 'defaultService'),
-        ],
-        transformations=[
-            ('defaultService', path_simplifier.TypeSuffix),
-            ('pathMatchers[].defaultService', path_simplifier.TypeSuffix),
-            ('pathMatchers[].pathRules[].service', path_simplifier.TypeSuffix),
-            ('tests[].service', path_simplifier.TypeSuffix),
-        ],
-        editables=[
-            'defaultService',
-            'description',
-            'hostRules',
-            'pathMatchers',
-            'tests',
-        ],
-    ),
-
     'users': _InternalSpec(
         message_class_name='User',
         table_cols=[
@@ -976,26 +962,6 @@ _SPECS_BETA['commitments'] = _InternalSpec(
 
 
 _SPECS_ALPHA = _SPECS_BETA.copy()
-_SPECS_ALPHA['hosts'] = _InternalSpec(
-    message_class_name='Host',
-    table_cols=[
-        ('NAME', 'name'),
-        ('REQUEST_PATH', 'requestPath'),
-    ],
-    transformations=[],
-    editables=None)
-_SPECS_ALPHA['hostTypes'] = _InternalSpec(
-    message_class_name='HostType',
-    table_cols=[
-        ('NAME', 'name'),
-        ('ZONE', 'zone'),
-        ('DEPRECATED', 'deprecated'),
-        ('CPUs', 'guestCpus'),
-        ('MEMORY(MB)', 'memoryMb'),
-        ('LOCAL SSD(GB)', 'localSsdGb'),
-    ],
-    transformations=[],
-    editables=None)
 
 _SPECS_ALPHA['instanceGroups'] = _InternalSpec(
     message_class_name='InstanceGroup',
@@ -1030,25 +996,6 @@ _SPECS_ALPHA['instanceGroupManagers'] = _InternalSpec(
     ],
     editables=None,
     )
-_SPECS_ALPHA['urlMaps'] = _InternalSpec(
-    message_class_name='UrlMap',
-    table_cols=[
-        ('NAME', 'name'),
-        ('DEFAULT_SERVICE', 'defaultService'),
-    ],
-    transformations=[
-        ('defaultService', path_simplifier.TypeSuffix),
-        ('pathMatchers[].defaultService', path_simplifier.TypeSuffix),
-        ('pathMatchers[].pathRules[].service', path_simplifier.TypeSuffix),
-        ('tests[].service', path_simplifier.TypeSuffix),
-    ],
-    editables=[
-        'defaultService',
-        'description',
-        'hostRules',
-        'pathMatchers',
-        'tests',
-    ])
 
 
 def _GetSpecsForVersion(api_version):
@@ -1087,7 +1034,7 @@ def GetSpec(resource_type, message_classes, api_version):
 
   table_cols = []
   for name, action in spec.table_cols:
-    if isinstance(action, basestring):
+    if isinstance(action, six.string_types):
       table_cols.append((name, property_selector.PropertyGetter(action)))
     elif callable(action):
       table_cols.append((name, action))
