@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,7 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Utilities for waiting on Compute Engine operations."""
-import httplib
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import batch_helper
 from googlecloudsdk.api_lib.compute import path_simplifier
 from googlecloudsdk.command_lib.util import time_util
@@ -121,9 +126,9 @@ class OperationData(object):
     return not self == o
 
 
-def WaitForOperations(operations_data,
-                      http, batch_url, warnings, errors,
-                      timeout=None):
+def WaitForOperations(
+    operations_data, http, batch_url, warnings, errors,
+    progress_tracker=None, timeout=None):
   """Blocks until the given operations are done or until a timeout is reached.
 
   Args:
@@ -132,6 +137,8 @@ def WaitForOperations(operations_data,
     batch_url: The URL to which batch requests should be sent.
     warnings: An output parameter for capturing warnings.
     errors: An output parameter for capturing errors.
+    progress_tracker: progress tracker to tick while waiting for operations to
+                      finish.
     timeout: The maximum amount of time, in seconds, to wait for the
       operations to reach the DONE state.
 
@@ -158,6 +165,8 @@ def WaitForOperations(operations_data,
   sleep_sec = 0
 
   while unfinished_operations:
+    if progress_tracker:
+      progress_tracker.Tick()
     resource_requests = []
     operation_requests = []
 
@@ -184,7 +193,7 @@ def WaitForOperations(operations_data,
         # httpErrorStatusCode is set only when the operation is not
         # successful.
         if (operation.httpErrorStatusCode and
-            operation.httpErrorStatusCode != httplib.OK):
+            operation.httpErrorStatusCode != 200):  # httplib.OK
           continue
 
         # Just in case the server did not set httpErrorStatusCode but
