@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,17 +14,22 @@
 # limitations under the License.
 
 """Utilities for loading and parsing kubeconfig."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import os
 
 from googlecloudsdk.core import config
 from googlecloudsdk.core import exceptions as core_exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
+from googlecloudsdk.core import yaml
+from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import files as file_utils
 from googlecloudsdk.core.util import platforms
 
-import yaml
-
 
 class Error(core_exceptions.Error):
   """Class for errors raised by kubeconfig utilities."""
@@ -66,21 +72,11 @@ class Kubeconfig(object):
     Raises:
       Error: don't have the permission to open kubeconfig file.
     """
-    self._data['clusters'] = self.clusters.values()
-    self._data['users'] = self.users.values()
-    self._data['contexts'] = self.contexts.values()
-    # We use os.open here to explicitly set file mode 0600.
-    # the flags passed should mimic behavior of open(self._filename, 'w'),
-    # which does write with truncate and creates file if not existing.
-    flags = os.O_WRONLY | os.O_TRUNC | os.O_CREAT
-    try:
-      fd = os.open(self._filename, flags, 0o600)
-    except OSError as error:
-      raise Error(
-          'don\'t have the permission to open kubeconfig file {0}: {1}'.format(
-              self._filename, error))
-    with os.fdopen(fd, 'w') as fp:
-      yaml.safe_dump(self._data, fp, default_flow_style=False)
+    self._data['clusters'] = list(self.clusters.values())
+    self._data['users'] = list(self.users.values())
+    self._data['contexts'] = list(self.contexts.values())
+    with file_utils.FileWriter(self._filename, private=True) as fp:
+      yaml.dump(self._data, fp)
 
   def SetCurrentContext(self, context):
     self._data['current-context'] = context
@@ -100,13 +96,12 @@ class Kubeconfig(object):
   @classmethod
   def LoadFromFile(cls, filename):
     try:
-      with open(filename, 'r') as fp:
-        data = yaml.load(fp)
-        cls._Validate(data)
-        return cls(data, filename)
-    except yaml.YAMLError as error:
+      data = yaml.load_path(filename)
+    except yaml.Error as error:
       raise Error('unable to load kubeconfig for {0}: {1}'.format(
-          filename, error))
+          filename, error.inner_error))
+    cls._Validate(data)
+    return cls(data, filename)
 
   @classmethod
   def LoadOrCreate(cls, filename):
@@ -128,18 +123,19 @@ class Kubeconfig(object):
   def DefaultPath():
     """Return default path for kubeconfig file."""
 
-    if os.environ.get('KUBECONFIG'):
-      return os.environ['KUBECONFIG']
+    kubeconfig = encoding.GetEncodedValue(os.environ, 'KUBECONFIG')
+    if kubeconfig:
+      return os.path.abspath(kubeconfig)
 
     # This follows the same resolution process as kubectl for the config file.
-    home_dir = os.environ.get('HOME')
+    home_dir = encoding.GetEncodedValue(os.environ, 'HOME')
     if not home_dir and platforms.OperatingSystem.IsWindows():
-      home_drive = os.environ.get('HOMEDRIVE')
-      home_path = os.environ.get('HOMEPATH')
+      home_drive = encoding.GetEncodedValue(os.environ, 'HOMEDRIVE')
+      home_path = encoding.GetEncodedValue(os.environ, 'HOMEPATH')
       if home_drive and home_path:
         home_dir = os.path.join(home_drive, home_path)
       if not home_dir:
-        home_dir = os.environ.get('USERPROFILE')
+        home_dir = encoding.GetEncodedValue(os.environ, 'USERPROFILE')
 
     if not home_dir:
       raise MissingEnvVarError(
@@ -169,15 +165,16 @@ def Cluster(name, server, ca_path=None, ca_data=None):
   }
 
 
-def User(name, token=None, username=None, password=None, auth_provider=None,
-         cert_path=None, cert_data=None, key_path=None, key_data=None):
+def User(name,
+         auth_provider=None,
+         cert_path=None,
+         cert_data=None,
+         key_path=None,
+         key_data=None):
   """Generate and return a user kubeconfig object.
 
   Args:
     name: str, nickname for this user entry.
-    token: str, bearer token.
-    username: str, basic auth user.
-    password: str, basic auth password.
     auth_provider: str, authentication provider.
     cert_path: str, path to client certificate file.
     cert_data: str, base64 encoded client certificate data.
@@ -187,19 +184,15 @@ def User(name, token=None, username=None, password=None, auth_provider=None,
     dict, valid kubeconfig user entry.
 
   Raises:
-    Error: if no auth info is provided (token or username AND password)
+    Error: if no auth info is provided (auth_provider or cert AND key)
   """
-  if not auth_provider and not token and (not username or not password):
-    raise Error('either auth_provider, token or username & password must be'
-                ' provided')
+  # TODO(b/70856999) Figure out what the correct behavior for client certs is.
+  if not (auth_provider or (cert_path and key_path) or
+          (cert_data and key_data)):
+    raise Error('either auth_provider or cert & key must be provided')
   user = {}
   if auth_provider:
     user['auth-provider'] = _AuthProvider(name=auth_provider)
-  elif token:
-    user['token'] = token
-  else:
-    user['username'] = username
-    user['password'] = password
 
   if cert_path and cert_data:
     raise Error('cannot specify both cert_path and cert_data')
@@ -298,4 +291,3 @@ def EmptyKubeconfig():
       'preferences': {},
       'users': [],
   }
-
