@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,6 +17,7 @@
 # TODO(b/36055865): Move defaults to the server
 
 # Path inside of GCS bucket, where Dataproc stores metadata.
+from __future__ import unicode_literals
 GCS_METADATA_PREFIX = 'google-cloud-dataproc-metainfo'
 
 # Beginning of driver output files.
@@ -24,7 +26,6 @@ JOB_OUTPUT_PREFIX = 'driveroutput'
 # The scopes that will be added to user-specified scopes. Used for
 # documentation only. Keep in sync with server specified list.
 MINIMUM_SCOPE_URIS = [
-    'https://www.googleapis.com/auth/cloud.useraccounts.readonly',
     'https://www.googleapis.com/auth/devstorage.read_write',
     'https://www.googleapis.com/auth/logging.write',
 ]
