@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,8 +19,11 @@ Bulk object uploads and downloads use methods that shell out to gsutil.
 Lightweight metadata / streaming operations use the StorageClient class.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import sys
-import urlparse
 
 from apitools.base.py import exceptions as apitools_exceptions
 from apitools.base.py import transfer
@@ -29,6 +33,7 @@ from googlecloudsdk.api_lib.util import apis as core_apis
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import resources
+import six.moves.urllib.parse
 
 
 # URI scheme for GCS.
@@ -39,8 +44,8 @@ HTTP_TIMEOUT = 60
 
 # Fix urlparse for storage paths.
 # This allows using urljoin in other files (that import this).
-urlparse.uses_relative.append(STORAGE_SCHEME)
-urlparse.uses_netloc.append(STORAGE_SCHEME)
+six.moves.urllib.parse.uses_relative.append(STORAGE_SCHEME)
+six.moves.urllib.parse.uses_netloc.append(STORAGE_SCHEME)
 
 
 def Upload(files, destination):
@@ -52,11 +57,11 @@ def Upload(files, destination):
   """
   args = files
   args += [destination]
-  exit_code = storage_util.RunGsutilCommand('cp', ' '.join(args))
+  exit_code = storage_util.RunGsutilCommand('cp', args)
   if exit_code != 0:
     raise exceptions.ToolException(
-        "Failed to upload files {0} to '{1}' using gsutil.".format(
-            files, destination))
+        "Failed to upload files ['{0}'] to '{1}' using gsutil.".format(
+            "', '".join(files), destination))
 
 
 def GetObjectRef(path, messages):
@@ -79,11 +84,9 @@ class StorageClient(object):
         bucket=object_ref.bucket, object=object_ref.name)
     try:
       return self.client.objects.Get(request=request, download=download)
-    except apitools_exceptions.HttpError as error:
+    except apitools_exceptions.HttpNotFoundError:
       # TODO(b/36052479): Clean up error handling. Handle 403s.
-      if error.status_code == 404:
-        return None
-      raise error
+      return None
 
   def GetObject(self, object_ref):
     """Get the object metadata of a GCS object.
@@ -197,7 +200,7 @@ class StorageObjectSeriesStream(object):
         try:
           object_info = self._GetObject(self._current_object_index)
         except apitools_exceptions.HttpError as error:
-          log.warn('Failed to fetch GCS output:\n%s', error)
+          log.warning('Failed to fetch GCS output:\n%s', error)
           break
         if not object_info:
           # Nothing to read yet.
