@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,9 +14,14 @@
 # limitations under the License.
 """A shared library for processing and validating test arguments."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.firebase.test import arg_file
 from googlecloudsdk.api_lib.firebase.test import arg_validate
 from googlecloudsdk.api_lib.firebase.test import exceptions
+from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.core import log
@@ -43,15 +49,17 @@ def AddCommonTestRunArgs(parser):
       'arg:value pairs to use. Run *$ gcloud topic arg-files* for more '
       'information and examples.')
 
-  parser.add_argument(
-      '--app',
-      category=base.COMMONLY_USED_FLAGS,
-      help='The path to the application binary file. The path may be in the '
-      'local filesystem or in Google Cloud Storage using gs:// notation.')
   parser.add_argument(
       '--async',
       action='store_true',
+      default=None,
       help='Invoke a test asynchronously without waiting for test results.')
+  parser.add_argument(
+      '--record-video',
+      action='store_true',
+      default=None,
+      help='Enable video recording during the test. Enabled by default, use '
+      '--no-record-video to disable.')
   parser.add_argument(
       '--results-bucket',
       help='The name of a Google Cloud Storage bucket where raw test results '
@@ -65,12 +73,6 @@ def AddCommonTestRunArgs(parser):
       'timestamp with a random suffix). Caution: if specified, this argument '
       '*must be unique* for each test matrix you create, otherwise results '
       'from multiple test matrices will be overwritten or intermingled.')
-  parser.add_argument(
-      '--results-history-name',
-      help='The history name for your test results (an arbitrary string label; '
-      'default: the application\'s label from the APK manifest). All tests '
-      'which use the same history name will have their results grouped '
-      'together in the Firebase console in a time-ordered test history list.')
   parser.add_argument(
       '--timeout',
       category=base.COMMONLY_USED_FLAGS,
@@ -93,6 +95,11 @@ def AddAndroidTestArgs(parser):
     parser: An argparse parser used to add arguments that follow a command in
         the CLI.
   """
+  parser.add_argument(
+      '--app',
+      category=base.COMMONLY_USED_FLAGS,
+      help='The path to the application binary file. The path may be in the '
+      'local filesystem or in Google Cloud Storage using gs:// notation.')
   parser.add_argument(
       '--app-package',
       help='The Java package of the application under test (default: extracted '
@@ -100,26 +107,58 @@ def AddAndroidTestArgs(parser):
   parser.add_argument(
       '--auto-google-login',
       action='store_true',
-      default=True,
+      default=None,
       help='Automatically log into the test device using a preconfigured '
-      'Google account before beginning the test.')
+      'Google account before beginning the test. Enabled by default, use '
+      '--no-auto-google-login to disable.')
   parser.add_argument(
       '--directories-to-pull',
       type=arg_parsers.ArgList(),
       metavar='DIR_TO_PULL',
       help='A list of paths that will be copied from the device\'s storage to '
-      'the designated results bucket after the test is complete. (For example '
-      '--directories-to-pull /sdcard/tempDir1,/data/tempDir2)')
+      'the designated results bucket after the test is complete. These must be '
+      'absolute paths under `/sdcard` or `/data/local/tmp` (for example, '
+      '`--directories-to-pull /sdcard/tempDir1,/data/local/tmp/tempDir2`). '
+      'Path names are restricted to the characters ```a-zA-Z0-9_-./+```. '
+      'The paths `/sdcard` and `/data` will be made available and treated as '
+      'implicit path substitutions. E.g. if `/sdcard` on a particular device '
+      'does not map to external storage, the system will replace it with the '
+      'external storage path prefix for that device.')
   parser.add_argument(
       '--environment-variables',
       type=arg_parsers.ArgDict(),
       metavar='KEY=VALUE',
-      help='A comma-separated, key=value, map of environment variables and '
-      'their desired values. The environment variables passed here will '
-      'be mirrored on to the adb run command. For example, specify '
-      '--environment-variables '
-      'coverage=true,coverageFile="/sdcard/tempDir/coverage.ec" to enable code '
-      'coverage and provide a file path to store the coverage results.')
+      help="""\
+      A comma-separated, key=value map of environment variables and their
+      desired values. This flag is repeatable. The environment variables are
+      mirrored as extra options to the `am instrument -e KEY1 VALUE1 ...`
+      command and passed to your test runner (typically AndroidJUnitRunner).
+      Examples:
+
+      Break test cases into four shards and run only the first shard:
+
+      ```
+      --environment-variables numShards=4,shardIndex=0
+      ```
+
+      Enable code coverage and provide a file path to store the coverage
+      results when using Android Test Orchestrator (`--use-orchestrator`):
+
+      ```
+      --environment-variables clearPackageData,coverage=true,coverageFile=/sdcard/coverage.ec
+      ```
+
+      Enable code coverage and provide a file path to store the coverage
+      results when *not* using Android Test Orchestrator
+      (`--no-use-orchestrator`):
+
+      ```
+      --environment-variables coverage=true,coverageFile=/sdcard/coverage.ec
+      ```
+
+      Note: If you need to embed a comma into a `VALUE` string, please refer to
+      `gcloud topic escaping` for ways to change the default list delimiter.
+      """)
   parser.add_argument(
       '--obb-files',
       type=arg_parsers.ArgList(min_length=1, max_length=2),
@@ -129,6 +168,24 @@ def AddAndroidTestArgs(parser):
       'OBB file name must conform to the format as specified by Android (e.g. '
       '[main|patch].0300110.com.example.android.obb) and will be installed '
       'into <shared-storage>/Android/obb/<package-name>/ on the test device.')
+  parser.add_argument(
+      '--performance-metrics',
+      action='store_true',
+      default=None,
+      help='Monitor and record performance metrics: CPU, memory, network usage,'
+      ' and FPS (game-loop only). Enabled by default, use '
+      '--no-performance-metrics to disable.')
+  parser.add_argument(
+      '--results-history-name',
+      help='The history name for your test results (an arbitrary string label; '
+      'default: the application\'s label from the APK manifest). All tests '
+      'which use the same history name will have their results grouped '
+      'together in the Firebase console in a time-ordered test history list.')
+  parser.add_argument(
+      '--type',
+      category=base.COMMONLY_USED_FLAGS,
+      choices=['instrumentation', 'robo', 'game-loop'],
+      help='The type of test to run.')
 
   # The following args are specific to Android instrumentation tests.
 
@@ -137,7 +194,7 @@ def AddAndroidTestArgs(parser):
       category=base.COMMONLY_USED_FLAGS,
       help='The path to the binary file containing instrumentation tests. The '
       'given path may be in the local filesystem or in Google Cloud Storage '
-      'using gs:// notation.')
+      'using a URL beginning with `gs://`.')
   parser.add_argument(
       '--test-package',
       category=ANDROID_INSTRUMENTATION_TEST,
@@ -153,34 +210,70 @@ def AddAndroidTestArgs(parser):
       category=ANDROID_INSTRUMENTATION_TEST,
       type=arg_parsers.ArgList(min_length=1),
       metavar='TEST_TARGET',
-      help='A list of one or more instrumentation test targets to be run '
-      '(default: all targets). Each target must be fully qualified with the '
-      'package name or class name, in one of these formats:\n'
-      '* "package package_name"\n'
-      '* "class package_name.class_name"\n'
-      '* "class package_name.class_name#method_name".')
+      help="""\
+      A list of one or more test target filters to apply (default: run all test
+      targets). Each target filter must be fully qualified with the package
+      name, class name, or test annotation desired. Any test filter supported by
+      `am instrument -e ...` is supported. See
+       https://developer.android.com/reference/android/support/test/runner/AndroidJUnitRunner
+       for more information. Examples:
+
+         * `--test-targets "package com.my.package.name"`
+         * `--test-targets "notPackage com.package.to.skip"`
+         * `--test-targets "class com.foo.ClassName"`
+         * `--test-targets "notClass com.foo.ClassName#testMethodToSkip"`
+         * `--test-targets "annotation com.foo.AnnotationToRun"`
+         * `--test-targets "size large notAnnotation com.foo.AnnotationToSkip"`
+      """)
+  parser.add_argument(
+      '--use-orchestrator',
+      category=ANDROID_INSTRUMENTATION_TEST,
+      action='store_true',
+      default=None,
+      help='Whether each test runs in its own Instrumentation instance with '
+      'the Android Test Orchestrator (default: Orchestrator is not used, same '
+      'as specifying --no-use-orchestrator). Orchestrator is only compatible '
+      'with AndroidJUnitRunner v1.0 or higher. See '
+      'https://developer.android.com/training/testing/junit-runner.html'
+      '#using-android-test-orchestrator for more information about Android '
+      'Test Orchestrator.')
 
   # The following args are specific to Android Robo tests.
 
   parser.add_argument(
       '--max-steps',
+      action=actions.DeprecationAction(
+          '--max-steps',
+          warn=('The `--max-steps` flag is deprecated and no longer has any '
+                'effect on the Robo crawler. The `--timeout` flag may be '
+                'optionally used to limit the maximum length of a Robo test.')),
       metavar='int',
       category=ANDROID_ROBO_TEST,
       type=arg_validate.NONNEGATIVE_INT_PARSER,
-      help='The maximum number of steps/actions a robo test can execute '
+      help='The maximum number of steps/actions a Robo test can execute '
       '(default: no limit).')
   parser.add_argument(
       '--max-depth',
+      action=actions.DeprecationAction(
+          '--max-depth',
+          warn=('The `--max-depth` flag is deprecated and no longer has any '
+                'effect on the actions of the Robo crawler.')),
       metavar='int',
       category=ANDROID_ROBO_TEST,
       type=arg_validate.POSITIVE_INT_PARSER,
-      help='The maximum depth of the traversal stack a robo test can explore. '
+      help='The maximum depth of the traversal stack a Robo test can explore. '
       'Needs to be at least 2 to make Robo explore the app beyond the first '
       'activity (default: 50).')
   parser.add_argument(
       '--app-initial-activity',
+      action=actions.DeprecationAction(
+          '--app-initial-activity',
+          warn=('The `--app-initial-activity` flag is deprecated and no longer '
+                'has any effect on the Robo crawler. Alternatively, the '
+                '`--robo-script` flag (in beta) can be used to guide Robo to a '
+                'specific part of your app before the Robo test begins.')),
       category=ANDROID_ROBO_TEST,
-      help='The initial activity used to start the app during a robo test.')
+      help='The initial activity used to start the app during a Robo test.')
   parser.add_argument(
       '--robo-directives',
       metavar='TYPE:RESOURCE_NAME=INPUT',
@@ -210,31 +303,7 @@ def AddAndroidTestArgs(parser):
       'Caution: You should only use credentials for test accounts that are not '
       'associated with real users.')
 
-
-def AddGaArgs(parser):
-  """Register args which are only available in the GA run command.
-
-  Args:
-    parser: An argparse parser used to add args that follow a command.
-  """
-  parser.add_argument(
-      '--type',
-      category=base.COMMONLY_USED_FLAGS,
-      choices=['instrumentation', 'robo'],
-      help='The type of test to run.')
-
-
-def AddBetaArgs(parser):
-  """Register args which are only available in the beta run command.
-
-  Args:
-    parser: An argparse parser used to add args that follow a command.
-  """
-  parser.add_argument(
-      '--type',
-      category=base.COMMONLY_USED_FLAGS,
-      choices=['instrumentation', 'robo', 'game-loop'],
-      help='The type of test to run.')
+  # The following args are specific to Android game-loop tests.
 
   parser.add_argument(
       '--scenario-numbers',
@@ -260,20 +329,134 @@ def AddBetaArgs(parser):
       'expand each given scenario label into a list of scenario numbers marked '
       'with that label, and execute those scenarios.')
 
-  # TODO(b/36366322): use {grandparent_command} once available
+
+def AddIosTestArgs(parser):
+  """Register args which are specific to iOS test commands.
+
+  Args:
+    parser: An argparse parser used to add arguments that follow a command in
+        the CLI.
+  """
+  parser.add_argument(
+      '--type',
+      category=base.COMMONLY_USED_FLAGS,
+      hidden=True,
+      choices=['xctest'],
+      help='The type of iOS test to run.')
+  parser.add_argument(
+      '--test',
+      category=base.COMMONLY_USED_FLAGS,
+      metavar='XCTEST_ZIP',
+      help='The path to the test package (a zip file containing the iOS app '
+      'and XCTest files). The given path may be in the local filesystem or in '
+      'Google Cloud Storage using a URL beginning with `gs://`. Note: any '
+      '.xctestrun file in this zip file will be ignored if *--xctestrun-file* '
+      'is specified.')
+  parser.add_argument(
+      '--xctestrun-file',
+      category=base.COMMONLY_USED_FLAGS,
+      metavar='XCTESTRUN_FILE',
+      help='The path to an .xctestrun file that will override any .xctestrun '
+      'file contained in the *--test* package. Because the .xctestrun file '
+      'contains environment variables along with test methods to run and/or '
+      'ignore, this can be useful for customizing or sharding test suites. The '
+      'given path may be in the local filesystem or in Google Cloud Storage '
+      'using a URL beginning with `gs://`.')
+  parser.add_argument(
+      '--device',
+      category=base.COMMONLY_USED_FLAGS,
+      type=arg_parsers.ArgDict(min_length=1),
+      action='append',
+      metavar='DIMENSION=VALUE',
+      help="""\
+      A list of ``DIMENSION=VALUE'' pairs which specify a target device to test
+      against. This flag may be repeated to specify multiple devices. The device
+      dimensions are: *model*, *version*, *locale*, and *orientation*. If any
+      dimensions are omitted, they will use a default value. The default value,
+      and all possible values, for each dimension can be found with the
+      ``list'' command for that dimension, such as `$ {parent_command} models
+      list`. Omitting this flag entirely will run tests against a single device
+      using defaults for every dimension.
+
+      Examples:\n
+      ```
+      --device model=iphone8plus
+      --device version=11.2
+      --device model=ipadmini4,version=11.2,locale=zh_CN,orientation=landscape
+      ```
+      """)
+  parser.add_argument(
+      '--results-history-name',
+      help='The history name for your test results (an arbitrary string label; '
+      'default: the bundle ID for the iOS application). All tests '
+      'which use the same history name will have their results grouped '
+      'together in the Firebase console in a time-ordered test history list.')
+
+
+def AddGaArgs(parser):
+  """Register args which are only available in the GA run command.
+
+  Args:
+    parser: An argparse parser used to add args that follow a command.
+  """
+  del parser  # Unused by AddGaArgs
+
+
+def AddBetaArgs(parser):
+  """Register args which are only available in the beta run command.
+
+  Args:
+    parser: An argparse parser used to add args that follow a command.
+  """
   parser.add_argument(
       '--network-profile',
       metavar='PROFILE_ID',
+      # TODO(b/36366322): use {grandparent_command} once available
       help='The name of the network traffic profile, for example '
       '--network-profile=LTE, which consists of a set of parameters to emulate '
       'network conditions when running the test (default: no network shaping; '
-      'see available profiles listed by the `$ gcloud beta firebase test '
+      'see available profiles listed by the `$ gcloud firebase test '
       'network-profiles list` command). This feature only works on physical '
       'devices.')
+  parser.add_argument(
+      '--robo-script',
+      category=ANDROID_ROBO_TEST,
+      help='The path to a Robo Script JSON file. The path may be in the local '
+      'filesystem or in Google Cloud Storage using gs:// notation. You can '
+      'guide the Robo test to perform specific actions by recording a Robo '
+      'Script in Android Studio and then specifying this argument. Learn more '
+      'at https://firebase.google.com/docs/test-lab/robo-ux-test#scripting.')
+  parser.add_argument(
+      '--additional-apks',
+      type=arg_parsers.ArgList(min_length=1, max_length=100),
+      metavar='APK',
+      help='A list of up to 100 additional APKs to install, in addition to '
+      'those being directly tested. The path may be in the local filesystem or '
+      'in Google Cloud Storage using gs:// notation.')
+  parser.add_argument(
+      '--other-files',
+      type=arg_parsers.ArgDict(min_length=1),
+      action='append',
+      metavar='FILE=DEVICE_DIR',
+      help="""\
+      A list of file=device-directory pairs that indicate paths of files to push
+      to the device before starting tests, and the device directory to push them
+      to.\n
+      Source file paths may be in the local filesystem or in Google Cloud
+      Storage (gs://...). Device directories must be absolute, whitelisted paths
+      (${EXTERNAL_STORAGE}, or ${ANDROID_DATA}/local/tmp).\n
+      Examples:\n
+      ```
+      --other-files local/file1=/sdcard/dir1/
+      --other-files gs://bucket/file2=/sdcard/dir2
+      ```\n
+      This flag only copies files to the device. To install files, like OBB or
+      APK files, see --obb-files and --additional-apks.
+      """)
 
 
 def AddMatrixArgs(parser):
-  """Register the repeatable args which define the the axes for a test matrix.
+  """Register the repeatable args which define the axes for a test matrix.
 
   Args:
     parser: An argparse parser used to add arguments that follow a command
@@ -288,12 +471,12 @@ def AddMatrixArgs(parser):
       help="""\
       A list of ``DIMENSION=VALUE'' pairs which specify a target device to test
       against. This flag may be repeated to specify multiple devices. The four
-      device dimensions are: *model*, *version*, *locale*, and
-      *orientation*. If any dimensions are omitted, they will use a default
-      value. The default value can be found with the list command for each
-      dimension, `$ {parent_command} <dimension> list`.
-      *--device* is now the preferred way to specify test devices and may not
-      be used in conjunction with *--devices-ids*, *--os-version-ids*,
+      device dimensions are: *model*, *version*, *locale*, and *orientation*. If
+      any dimensions are omitted, they will use a default value. The default
+      value, and all possible values, for each dimension can be found with the
+      ``list'' command for that dimension, such as `$ {parent_command} models
+      list`. *--device* is now the preferred way to specify test devices and may
+      not be used in conjunction with *--devices-ids*, *--os-version-ids*,
       *--locales*, or *--orientations*. Omitting all of the preceding
       dimension-related flags will run tests against a single device using
       defaults for all four device dimensions.
@@ -302,7 +485,7 @@ def AddMatrixArgs(parser):
       ```
       --device model=Nexus6
       --device version=23,orientation=portrait
-      --device model=shamu,version=22,locale=zh_CN,orientation=landscape
+      --device model=shamu,version=22,locale=zh_CN,orientation=default
       ```
       """)
   parser.add_argument(
@@ -338,7 +521,9 @@ def AddMatrixArgs(parser):
           min_length=1, max_length=2, choices=arg_validate.ORIENTATION_LIST),
       completer=arg_parsers.GetMultiCompleter(OrientationsCompleter),
       metavar='ORIENTATION',
-      help='The device orientation(s) to test against (default: portrait).')
+      help='The device orientation(s) to test against (default: portrait). '
+      'Specifying \'default\' will pick the preferred orientation '
+      'for the app.')
 
 
 def OrientationsCompleter(prefix, unused_parsed_args, unused_kwargs):
@@ -361,13 +546,13 @@ def GetSetOfAllTestArgs(type_rules, shared_rules):
   Returns:
     A set of strings for every gcloud-test argument.
   """
-  all_test_args_list = (shared_rules['required'] +
-                        shared_rules['optional'] +
-                        shared_rules['defaults'].keys())
+  all_test_args_list = (
+      shared_rules['required'] + shared_rules['optional'] + list(
+          shared_rules['defaults'].keys()))
   for type_dict in type_rules.values():
-    all_test_args_list += (type_dict['required'] +
-                           type_dict['optional'] +
-                           type_dict['defaults'].keys())
+    all_test_args_list += (
+        type_dict['required'] + type_dict['optional'] + list(
+            type_dict['defaults'].keys()))
   return set(all_test_args_list)
 
 
