@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +15,13 @@
 
 """A shared library to validate 'gcloud test' CLI argument values."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import datetime
+import os
+import posixpath
 import random
 import re
 import string
@@ -25,6 +32,7 @@ from googlecloudsdk.api_lib.firebase.test import util as util
 from googlecloudsdk.api_lib.storage import storage_util
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import exceptions
+import six
 
 
 def ValidateArgFromFile(arg_internal_name, arg_value):
@@ -64,10 +72,10 @@ def ValidateArgFromFile(arg_internal_name, arg_value):
 
 
 # Constants shared between arg-file validation and CLI flag validation.
-POSITIVE_INT_PARSER = arg_parsers.BoundedInt(1, sys.maxint)
-NONNEGATIVE_INT_PARSER = arg_parsers.BoundedInt(0, sys.maxint)
+POSITIVE_INT_PARSER = arg_parsers.BoundedInt(1, sys.maxsize)
+NONNEGATIVE_INT_PARSER = arg_parsers.BoundedInt(0, sys.maxsize)
 TIMEOUT_PARSER = arg_parsers.Duration(lower_bound='1m', upper_bound='6h')
-ORIENTATION_LIST = ['portrait', 'landscape']
+ORIENTATION_LIST = ['portrait', 'landscape', 'default']
 
 
 def ValidateStringList(arg_internal_name, arg_value):
@@ -83,7 +91,8 @@ def ValidateStringList(arg_internal_name, arg_value):
   Raises:
     InvalidArgException: the argument's value is not valid.
   """
-  if isinstance(arg_value, basestring):  # convert single str to a str list
+  # convert single str to a str list
+  if isinstance(arg_value, six.string_types):
     return [arg_value]
   if isinstance(arg_value, int):  # convert single int to a str list
     return [str(arg_value)]
@@ -94,7 +103,7 @@ def ValidateStringList(arg_internal_name, arg_value):
 
 def _ValidateString(arg_internal_name, arg_value):
   """Validates an arg whose value should be a simple string."""
-  if isinstance(arg_value, basestring):
+  if isinstance(arg_value, six.string_types):
     return arg_value
   if isinstance(arg_value, int):  # convert int->str if str is really expected
     return str(arg_value)
@@ -113,12 +122,13 @@ def _ValidateBool(arg_internal_name, arg_value):
 def _ValidateDuration(arg_internal_name, arg_value):
   """Validates an argument which should have a Duration value."""
   try:
-    if isinstance(arg_value, basestring):
+    if isinstance(arg_value, six.string_types):
       return TIMEOUT_PARSER(arg_value)
     elif isinstance(arg_value, int):
       return TIMEOUT_PARSER(str(arg_value))
   except arg_parsers.ArgumentTypeError as e:
-    raise test_exceptions.InvalidArgException(arg_internal_name, e.message)
+    raise test_exceptions.InvalidArgException(arg_internal_name,
+                                              six.text_type(e))
   raise test_exceptions.InvalidArgException(arg_internal_name, arg_value)
 
 
@@ -135,7 +145,8 @@ def _ValidatePositiveInteger(arg_internal_name, arg_value):
     if isinstance(arg_value, int):
       return POSITIVE_INT_PARSER(str(arg_value))
   except arg_parsers.ArgumentTypeError as e:
-    raise test_exceptions.InvalidArgException(arg_internal_name, e.message)
+    raise test_exceptions.InvalidArgException(arg_internal_name,
+                                              six.text_type(e))
   raise test_exceptions.InvalidArgException(arg_internal_name, arg_value)
 
 
@@ -145,7 +156,8 @@ def _ValidateNonNegativeInteger(arg_internal_name, arg_value):
     if isinstance(arg_value, int):
       return NONNEGATIVE_INT_PARSER(str(arg_value))
   except arg_parsers.ArgumentTypeError as e:
-    raise test_exceptions.InvalidArgException(arg_internal_name, e.message)
+    raise test_exceptions.InvalidArgException(arg_internal_name,
+                                              six.text_type(e))
   raise test_exceptions.InvalidArgException(arg_internal_name, arg_value)
 
 
@@ -170,7 +182,7 @@ def _ValidatePositiveIntList(arg_internal_name, arg_value):
 
 
 def _ValidateOrientationList(arg_internal_name, arg_value):
-  """Validates that 'orientations' only contains 'portrait' and 'landscape'."""
+  """Validates that 'orientations' only contains allowable values."""
   arg_value = ValidateStringList(arg_internal_name, arg_value)
   for orientation in arg_value:
     _ValidateOrientation(orientation)
@@ -194,6 +206,18 @@ def _ValidateObbFileList(arg_internal_name, arg_value):
   return arg_value
 
 
+def _ValidateAdditionalApksList(arg_internal_name, arg_value):
+  """Validates that 'additional-apks' contains [1, 100] entries."""
+  arg_value = ValidateStringList(arg_internal_name, arg_value)
+  if len(arg_value) < 1:
+    raise test_exceptions.InvalidArgException(
+        arg_internal_name, 'At least 1 additional apk must be specified.')
+  if len(arg_value) > 100:
+    raise test_exceptions.InvalidArgException(
+        arg_internal_name, 'At most 100 additional apks may be specified.')
+  return arg_value
+
+
 def _ValidateKeyValueStringPairs(arg_internal_name, arg_value):
   """Validates that an argument is a dict of string-type key-value pairs."""
   if isinstance(arg_value, dict):
@@ -228,26 +252,31 @@ def _ValidateListOfStringToStringDicts(arg_internal_name, arg_value):
 # Map of internal arg names to their appropriate validation functions.
 # Any arg not appearing in this map is assumed to be a simple string.
 _FILE_ARG_VALIDATORS = {
+    'additional_apks': _ValidateAdditionalApksList,
     'async': _ValidateBool,
     'auto_google_login': _ValidateBool,
-    'timeout': _ValidateDuration,
     'device': _ValidateListOfStringToStringDicts,
     'device_ids': ValidateStringList,
-    'os_version_ids': ValidateStringList,
+    'directories_to_pull': ValidateStringList,
+    'environment_variables': _ValidateKeyValueStringPairs,
+    'event_count': _ValidatePositiveInteger,
+    'event_delay': _ValidateNonNegativeInteger,
     'locales': ValidateStringList,
     'orientations': _ValidateOrientationList,
     'obb_files': _ValidateObbFileList,
-    'test_targets': ValidateStringList,
-    'event_count': _ValidatePositiveInteger,
-    'event_delay': _ValidateNonNegativeInteger,
     'random_seed': _ValidateInteger,
     'max_steps': _ValidateNonNegativeInteger,
     'max_depth': _ValidatePositiveInteger,
+    'os_version_ids': ValidateStringList,
+    'other_files': _ValidateKeyValueStringPairs,
+    'performance_metrics': _ValidateBool,
+    'record_video': _ValidateBool,
     'robo_directives': _ValidateKeyValueStringPairs,
     'scenario_labels': ValidateStringList,
     'scenario_numbers': _ValidatePositiveIntList,
-    'environment_variables': _ValidateKeyValueStringPairs,
-    'directories_to_pull': ValidateStringList,
+    'test_targets': ValidateStringList,
+    'timeout': _ValidateDuration,
+    'use_orchestrator': _ValidateBool,
 }
 
 
@@ -353,8 +382,10 @@ def _GenerateUniqueGcsObjectName():
   Returns:
     A string with the unique GCS object name.
   """
-  return '{0}_{1}'.format(datetime.datetime.now().isoformat('_'),
-                          ''.join(random.sample(string.letters, 4)))
+  # In PY2, isoformat() argument 1 needs a char. But in PY3 it needs unicode.
+  return '{0}_{1}'.format(
+      datetime.datetime.now().isoformat(b'_' if six.PY2 else '_'), ''.join(
+          random.sample(string.ascii_letters, 4)))
 
 
 def ValidateOsVersions(args, catalog_mgr):
@@ -383,8 +414,21 @@ _OBB_FILE_REGEX = re.compile(
     r'(.*[\\/:])?(main|patch)\.\d+(\.[a-zA-Z]\w*)+\.obb$')
 
 
-def ValidateObbFileNames(obb_files):
-  """Confirm that any OBB file names follow the required Android pattern."""
+def NormalizeAndValidateObbFileNames(obb_files):
+  """Confirm that any OBB file names follow the required Android pattern.
+
+  Also expand local paths with "~"
+
+  Args:
+    obb_files: list of obb file references. Each one is either a filename on the
+      local FS or a gs:// reference.
+  """
+  if obb_files:
+    obb_files[:] = [
+        obb_file if not obb_file or
+        obb_file.startswith(storage_util.GSUTIL_BUCKET_PREFIX) else
+        os.path.expanduser(obb_file) for obb_file in obb_files
+    ]
   for obb_file in (obb_files or []):
     if not _OBB_FILE_REGEX.match(obb_file):
       raise test_exceptions.InvalidArgException(
@@ -397,7 +441,7 @@ def ValidateRoboDirectivesList(args):
   """Validates key-value pairs for 'robo_directives' flag."""
   resource_names = set()
   duplicates = set()
-  for key, value in (args.robo_directives or {}).iteritems():
+  for key, value in six.iteritems((args.robo_directives or {})):
     (action_type, resource_name) = util.ParseRoboDirectiveKey(key)
     if action_type == 'click' and value:
       raise test_exceptions.InvalidArgException(
@@ -436,12 +480,23 @@ def ValidateEnvironmentVariablesList(args):
           'Invalid environment variable [{0}]'.format(key))
 
 
-_DIRECTORIES_TO_PULL_PATH_REGEX = re.compile(r'^(/.*)+/?$')
+_DIRECTORIES_TO_PULL_PATH_REGEX = re.compile(
+    r'^/?/(?:sdcard|data/local/tmp)(?:/[\w\-\.\+ /]+)*$')
 
 
-def ValidateDirectoriesToPullList(args):
-  """Validates list of file paths for 'directories-to-pull' flag."""
-  for file_path in (args.directories_to_pull or []):
+def NormalizeAndValidateDirectoriesToPullList(dirs):
+  """Validate list of file paths for 'directories-to-pull' flag.
+
+  Also collapse paths to remove "." ".." and "//".
+
+  Args:
+    dirs: list of directory names to pull from the device.
+  """
+  if dirs:
+    # Expand posix paths (Note: blank entries will fail in the next loop)
+    dirs[:] = [posixpath.abspath(path) if path else path for path in dirs]
+
+  for file_path in (dirs or []):
     # Check for correct file path format.
     if not _DIRECTORIES_TO_PULL_PATH_REGEX.match(file_path):
       raise test_exceptions.InvalidArgException(
