@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Spanner instance API helper."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import list_pager
 from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.core import properties
@@ -36,8 +41,10 @@ def Create(instance, config, description, nodes):
       config,
       params={'projectsId': properties.VALUES.core.project.GetOrFail},
       collection='spanner.projects.instanceConfigs')
+  project_ref = resources.REGISTRY.Create(
+      'spanner.projects', projectsId=properties.VALUES.core.project.GetOrFail)
   req = msgs.SpannerProjectsInstancesCreateRequest(
-      parent='projects/' + properties.VALUES.core.project.Get(required=True),
+      parent=project_ref.RelativeName(),
       createInstanceRequest=msgs.CreateInstanceRequest(
           instanceId=instance,
           instance=msgs.Instance(
@@ -47,13 +54,14 @@ def Create(instance, config, description, nodes):
   return client.projects_instances.Create(req)
 
 
-def SetPolicy(instance_ref, policy):
+def SetPolicy(instance_ref, policy, field_mask=None):
   """Saves the given policy on the instance, overwriting whatever exists."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
   req = msgs.SpannerProjectsInstancesSetIamPolicyRequest(
       resource=instance_ref.RelativeName(),
-      setIamPolicyRequest=msgs.SetIamPolicyRequest(policy=policy))
+      setIamPolicyRequest=msgs.SetIamPolicyRequest(policy=policy,
+                                                   updateMask=field_mask))
   return client.projects_instances.SetIamPolicy(req)
 
 
@@ -94,8 +102,10 @@ def List():
   """List instances in the project."""
   client = apis.GetClientInstance('spanner', 'v1')
   msgs = apis.GetMessagesModule('spanner', 'v1')
+  project_ref = resources.REGISTRY.Create(
+      'spanner.projects', projectsId=properties.VALUES.core.project.GetOrFail)
   req = msgs.SpannerProjectsInstancesListRequest(
-      parent='projects/' + properties.VALUES.core.project.Get(required=True))
+      parent=project_ref.RelativeName())
   return list_pager.YieldFromList(
       client.projects_instances,
       req,
