@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,10 +14,14 @@
 # limitations under the License.
 """Defines tool-wide constants."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 # Defaults for instance creation.
 DEFAULT_MACHINE_TYPE = 'db-n1-standard-1'
 
-# Determining what executables and flags to use for sql connect.
+# Determining what executables, flags, and defaults to use for sql connect.
 DB_EXE = {'MYSQL': 'mysql', 'POSTGRES': 'psql'}
 
 EXE_FLAGS = {
@@ -28,6 +33,12 @@ EXE_FLAGS = {
     'psql': {
         'user': '-U',
         'password': '-W',
-        'hostname': '-h'
+        'hostname': '-h',
+        'database': '-d'
     }
 }
+
+DEFAULT_SQL_USER = {'mysql': 'root', 'psql': 'postgres'}
+
+# Size conversions.
+BYTES_TO_GB = 1 << 30
