@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,7 +20,11 @@ tool. We use the command-line tool for syncing the contents of buckets as well
 as listing the contents. We use the API for checking ACLs.
 """
 
-import cStringIO
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+import io
 import mimetypes
 import os
 
@@ -28,9 +33,12 @@ from apitools.base.py import list_pager
 from apitools.base.py import transfer
 
 from googlecloudsdk.api_lib.storage import storage_util
+from googlecloudsdk.api_lib.util import exceptions as http_exc
 from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.core import exceptions as core_exc
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
+from googlecloudsdk.core.credentials import http
 
 
 GSUTIL_BUCKET_REGEX = r'^gs://.*$'
@@ -42,10 +50,18 @@ GCS_URL_PATTERN = (
     'https://www.googleapis.com/storage/v1/b/{bucket}/o/{obj}?alt=media')
 
 
-class Error(Exception):
+class Error(core_exc.Error):
   """Base exception for storage API module."""
 
 
+class BucketNotFoundError(Error):
+  """Error raised when the bucket specified does not exist."""
+
+
+class ListBucketError(Error):
+  """Error raised when there are problems listing the contents of a bucket."""
+
+
 class UploadError(Error):
   """Error raised when there are problems uploading files."""
 
@@ -70,6 +86,20 @@ class StorageClient(object):
     self.client = client or storage_util.GetClient()
     self.messages = messages or storage_util.GetMessages()
 
+  def _GetChunkSize(self):
+    """Returns the property defined chunksize corrected for server granularity.
+
+    Chunk size for GCS must be a multiple of 256 KiB. This functions rounds up
+    the property defined chunk size to the nearest chunk size interval.
+    """
+    gcs_chunk_granularity = 256 * 1024  # 256 KiB
+    chunksize = properties.VALUES.storage.chunk_size.GetInt()
+    if chunksize == 0:
+      chunksize = None  # Use apitools default (1048576 B)
+    elif chunksize % gcs_chunk_granularity != 0:
+      chunksize += gcs_chunk_granularity - (chunksize % gcs_chunk_granularity)
+    return chunksize
+
   def Copy(self, src, dst):
     """Copy one GCS object to another.
 
@@ -143,26 +173,44 @@ class StorageClient(object):
       Object, the storage object that was copied to.
 
     Raises:
-      BadFileException if the file upload is not successful.
+      BucketNotFoundError if the user-specified bucket does not exist.
+      UploadError if the file upload is not successful.
+      exceptions.BadFileException if the uploaded file size does not match the
+          size of the local file.
     """
     file_size = _GetFileSize(local_path)
     src_obj = self.messages.Object(size=file_size)
     mime_type = _GetMimetype(local_path)
 
-    upload = transfer.Upload.FromFile(local_path, mime_type=mime_type)
+    chunksize = self._GetChunkSize()
+    upload = transfer.Upload.FromFile(
+        local_path, mime_type=mime_type, chunksize=chunksize)
     insert_req = self.messages.StorageObjectsInsertRequest(
         bucket=bucket_ref.bucket,
         name=target_path,
         object=src_obj)
 
+    gsc_path = '{bucket}/{target_path}'.format(
+        bucket=bucket_ref.bucket, target_path=target_path,
+    )
+
     log.info('Uploading [{local_file}] to [{gcs}]'.format(local_file=local_path,
-                                                          gcs=target_path))
+                                                          gcs=gsc_path))
     try:
       response = self.client.objects.Insert(insert_req, upload=upload)
+    except api_exceptions.HttpNotFoundError:
+      raise BucketNotFoundError(
+          'Could not upload file: [{bucket}] bucket does not exist.'
+          .format(bucket=bucket_ref.bucket))
     except api_exceptions.HttpError as err:
-      raise exceptions.BadFileException(
-          'Could not copy [{local_file}] to [{gcs}]: {err}. Please retry.'
-          .format(local_file=local_path, gcs=target_path, err=err))
+      log.debug('Could not upload file [{local_file}] to [{gcs}]: {e}'.format(
+          local_file=local_path, gcs=gsc_path,
+          e=http_exc.HttpException(err)))
+      raise UploadError(
+          '{code} Could not upload file [{local_file}] to [{gcs}]: {message}'
+          .format(code=err.status_code, local_file=local_path, gcs=gsc_path,
+                  message=http_exc.HttpException(
+                      err, error_format='{status_message}')))
 
     if response.size != file_size:
       log.debug('Response size: {0} bytes, but local file is {1} bytes.'.format(
@@ -185,13 +233,19 @@ class StorageClient(object):
     Raises:
       BadFileException if the file download is not successful.
     """
-    download = transfer.Download.FromFile(local_path)
+    chunksize = self._GetChunkSize()
+    download = transfer.Download.FromFile(local_path, chunksize=chunksize)
+    download.bytes_http = http.Http(response_encoding=None)
     get_req = self.messages.StorageObjectsGetRequest(
         bucket=bucket_ref.bucket,
         object=object_path)
 
+    gsc_path = '{bucket}/{object_path}'.format(
+        bucket=bucket_ref.bucket, object_path=object_path,
+    )
+
     log.info('Downloading [{gcs}] to [{local_file}]'.format(
-        local_file=local_path, gcs=object_path))
+        local_file=local_path, gcs=gsc_path))
     try:
       self.client.objects.Get(get_req, download=download)
       # Close the stream to release the file handle so we can check its contents
@@ -201,8 +255,9 @@ class StorageClient(object):
       response = self.client.objects.Get(get_req)
     except api_exceptions.HttpError as err:
       raise exceptions.BadFileException(
-          'Could not copy [{gcs}] to [{local_file}]: {err}. Please retry.'
-          .format(local_file=local_path, gcs=object_path, err=err))
+          'Could not copy [{gcs}] to [{local_file}]. Please retry: {err}'
+          .format(local_file=local_path, gcs=gsc_path,
+                  err=http_exc.HttpException(err)))
 
     file_size = _GetFileSize(local_path)
     if response.size != file_size:
@@ -224,8 +279,10 @@ class StorageClient(object):
     Returns:
       file-like object containing the data read.
     """
-    data = cStringIO.StringIO()
-    download = transfer.Download.FromStream(data)
+    data = io.BytesIO()
+    chunksize = self._GetChunkSize()
+    download = transfer.Download.FromStream(data, chunksize=chunksize)
+    download.bytes_http = http.Http(response_encoding=None)
     get_req = self.messages.StorageObjectsGetRequest(
         bucket=object_ref.bucket,
         object=object_ref.name)
@@ -235,8 +292,8 @@ class StorageClient(object):
       self.client.objects.Get(get_req, download=download)
     except api_exceptions.HttpError as err:
       raise exceptions.BadFileException(
-          'Could not read [{object_}]: {err}. Please retry.'.format(
-              object_=object_ref, err=err))
+          'Could not read [{object_}]. Please retry: {err}'.format(
+              object_=object_ref, err=http_exc.HttpException(err)))
 
     data.seek(0)
     return data
@@ -263,10 +320,8 @@ class StorageClient(object):
               bucket=self.messages.Bucket(
                   name=bucket,
               )))
-    except api_exceptions.HttpError as e:
+    except api_exceptions.HttpConflictError:
       # It's ok if the error was 409, which means the resource already exists.
-      if e.status_code != 409:
-        raise
       # Make sure we have access to the bucket.  Storage returns a 409 whether
       # the already-existing bucket is owned by you or by someone else, so we
       # do a quick test to figure out which it was.
@@ -274,24 +329,40 @@ class StorageClient(object):
           bucket=bucket,
       ))
 
-  def ListBucket(self, bucket_ref):
+  def ListBucket(self, bucket_ref, prefix=None):
     """Lists the contents of a cloud storage bucket.
 
     Args:
       bucket_ref: The reference to the bucket.
-    Returns:
-      A set of names of items.
+      prefix: str, Filter results to those whose names begin with this prefix.
+
+    Yields:
+      Object messages.
+
+    Raises:
+      BucketNotFoundError if the user-specified bucket does not exist.
+      ListBucketError if there was an error listing the bucket.
     """
-    request = self.messages.StorageObjectsListRequest(bucket=bucket_ref.bucket)
-    items = set()
+    request = self.messages.StorageObjectsListRequest(
+        bucket=bucket_ref.bucket, prefix=prefix)
+
     try:
       # batch_size=None gives us the API default
-      for item in list_pager.YieldFromList(self.client.objects, request,
-                                           batch_size=None):
-        items.add(item.name)
+      for obj in list_pager.YieldFromList(self.client.objects,
+                                          request, batch_size=None):
+        yield obj
+    except api_exceptions.HttpNotFoundError:
+      raise BucketNotFoundError(
+          'Could not list bucket: [{bucket}] bucket does not exist.'
+          .format(bucket=bucket_ref.bucket))
     except api_exceptions.HttpError as e:
-      raise UploadError('Error uploading files: {e}'.format(e=e))
-    return items
+      log.debug('Could not list bucket [{bucket}]: {e}'.format(
+          bucket=bucket_ref.bucket, e=http_exc.HttpException(e)))
+      raise ListBucketError(
+          '{code} Could not list bucket [{bucket}]: {message}'
+          .format(code=e.status_code, bucket=bucket_ref.bucket,
+                  message=http_exc.HttpException(
+                      e, error_format='{status_message}')))
 
   def DeleteObject(self, bucket_ref, object_path):
     """Delete the specified object.
@@ -312,29 +383,3 @@ class StorageClient(object):
     """
     self.client.buckets.Delete(
         self.messages.StorageBucketsDeleteRequest(bucket=bucket_ref.bucket))
-
-
-def Rsync(source_dir, dest_dir, exclude_pattern=None):
-  """Copies files from the specified file system directory to a GCS bucket.
-
-  Args:
-    source_dir: The source directory for the rsync.
-    dest_dir: The destination directory for the rsync.
-    exclude_pattern: A string representation of a Python regular expression.
-      If provided, this is passed as the '-x' argument for the rsync command.
-
-  Returns:
-    The exit code of the call to "gsutil rsync".
-  """
-
-  # -m Allows concurrent uploads
-  # -c Causes gsutil to compute checksums when comparing files.
-  # -R recursively copy all files
-  # -x Ignore files using the specified pattern.
-  command_arg_str = '-R -c '
-  if exclude_pattern:
-    command_arg_str += '-x \'{0}\' '.format(exclude_pattern)
-
-  command_arg_str += ' '.join([source_dir, dest_dir])
-  return storage_util.RunGsutilCommand('rsync', command_arg_str,
-                                       run_concurrent=True)
