@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +15,18 @@
 
 """Utilities for interacting with Google Cloud Storage."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import argparse
 import os
 import re
 import string
 
 from googlecloudsdk.api_lib.util import apis as core_apis
-from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.core import config
+from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import execution_utils
 from googlecloudsdk.core import log
 from googlecloudsdk.core import resources
@@ -41,14 +46,22 @@ GCS_URL_PATTERN = (
     'https://www.googleapis.com/storage/v1/b/{bucket}/o/{obj}?alt=media')
 
 
+class Error(exceptions.Error):
+  """Base class for exceptions in this module."""
+
+
+class GsutilError(Error):
+  """Exception raised when gsutil cannot be found."""
+
+
 class InvalidNameError(ValueError):
   """Error indicating that a given name is invalid."""
 
-  def __init__(self, name, reason):
+  def __init__(self, name, reason, type_name, url):
     super(InvalidNameError, self).__init__(
         ('Invalid {type} name [{name}]: {reason}\n\n'
          'See {url} for details.').format(name=name, reason=reason,
-                                          type=self.TYPE, url=self.URL))
+                                          type=type_name, url=url))
 
 
 class InvalidBucketNameError(InvalidNameError):
@@ -56,12 +69,20 @@ class InvalidBucketNameError(InvalidNameError):
   TYPE = 'bucket'
   URL = 'https://cloud.google.com/storage/docs/naming#requirements'
 
+  def __init__(self, name, reason):
+    super(InvalidBucketNameError, self).__init__(
+        name, reason, self.TYPE, self.URL)
+
 
 class InvalidObjectNameError(InvalidNameError):
   """Error indicating that a given object name is invalid."""
   TYPE = 'object'
   URL = 'https://cloud.google.com/storage/docs/naming#objectnames'
 
+  def __init__(self, name, reason):
+    super(InvalidObjectNameError, self).__init__(
+        name, reason, self.TYPE, self.URL)
+
 
 VALID_BUCKET_CHARS_MESSAGE = """\
 Bucket names must contain only lowercase letters, numbers, dashes (-), \
@@ -179,6 +200,9 @@ class BucketReference(object):
   def __ne__(self, other):
     return not self.__eq__(other)
 
+  def __hash__(self):
+    return hash(self.ToBucketUrl())
+
 
 class ObjectReference(object):
   """Wrapper class to make working with Cloud Storage bucket/objects easier."""
@@ -247,6 +271,9 @@ class ObjectReference(object):
   def __ne__(self, other):
     return not self.__eq__(other)
 
+  def __hash__(self):
+    return hash(self.ToUrl())
+
 
 def GetMessages():
   """Import and return the appropriate storage messages module."""
@@ -268,42 +295,44 @@ def _GetGsutilPath():
       log.debug('Using gsutil found at [{path}]'.format(path=gsutil_path))
       return gsutil_path
     else:
-      raise exceptions.ToolException(('A SDK root could not be found. Please '
-                                      'check your installation.'))
+      raise GsutilError('A path to the storage client `gsutil` could not be '
+                        'found. Please check your SDK installation.')
   return os.path.join(sdk_bin_path, 'gsutil')
 
 
-def RunGsutilCommand(command_name, command_arg_str, run_concurrent=False):
+def RunGsutilCommand(command_name,
+                     command_args=None,
+                     run_concurrent=False,
+                     out_func=log.file_only_logger.debug,
+                     err_func=log.file_only_logger.debug):
   """Runs the specified gsutil command and returns the command's exit code.
 
-  This is more reliable than storage_api.StorageClient.CopyFilesToGcs especially
-  for large files.
+  WARNING: This is not compatible with python 3 and should no longer be used.
 
   Args:
     command_name: The gsutil command to run.
-    command_arg_str: Arguments to pass to the command.
+    command_args: List of arguments to pass to the command.
     run_concurrent: Whether concurrent uploads should be enabled while running
       the command.
+    out_func: str->None, a function to call with the stdout of the gsutil
+        command.
+    err_func: str->None, a function to call with the stderr of the gsutil
+        command.
 
   Returns:
     The exit code of the call to the gsutil command.
   """
   command_path = _GetGsutilPath()
 
-  if run_concurrent:
-    command_args = ['-m', command_name]
-  else:
-    command_args = [command_name]
-
-  command_args += command_arg_str.split(' ')
+  args = ['-m', command_name] if run_concurrent else [command_name]
+  if command_args is not None:
+    args += command_args
 
   if platforms.OperatingSystem.Current() == platforms.OperatingSystem.WINDOWS:
-    gsutil_args = execution_utils.ArgsForCMDTool(command_path + '.cmd',
-                                                 *command_args)
+    gsutil_args = execution_utils.ArgsForCMDTool(command_path + '.cmd', *args)
   else:
-    gsutil_args = execution_utils.ArgsForExecutableTool(command_path,
-                                                        *command_args)
+    gsutil_args = execution_utils.ArgsForExecutableTool(command_path, *args)
   log.debug('Running command: [{args}]]'.format(args=' '.join(gsutil_args)))
   return execution_utils.Exec(gsutil_args, no_exit=True,
-                              out_func=log.file_only_logger.debug,
-                              err_func=log.file_only_logger.debug)
+                              out_func=out_func,
+                              err_func=err_func)
