@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -46,14 +47,24 @@ Example usage:
 
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import argparse
 import copy
 import re
 
 from googlecloudsdk.calliope import parser_errors
 from googlecloudsdk.core import log
+from googlecloudsdk.core.console import console_attr
+from googlecloudsdk.core.console import console_io
+from googlecloudsdk.core.util import files
 from googlecloudsdk.core.util import times
 
+import six
+from six.moves import zip  # pylint: disable=redefined-builtin
+
 
 __all__ = ['Duration', 'BinarySize']
 
@@ -100,10 +111,10 @@ def _GenerateErrorMessage(error, user_input=None, error_idx=None):
 
 
 _VALUE_PATTERN = r"""
-    ^                       # Beginning of input marker.
-    (?P<amount>\d+)         # Amount.
-    ((?P<unit>[a-zA-Z]+))?  # Optional unit.
-    $                       # End of input marker.
+    ^                           # Beginning of input marker.
+    (?P<amount>\d+)             # Amount.
+    ((?P<suffix>[-/a-zA-Z]+))?  # Optional scale and type abbr.
+    $                           # End of input marker.
 """
 
 _RANGE_PATTERN = r'^(?P<start>[0-9]+)(-(?P<end>[0-9]+))?$'
@@ -123,17 +134,17 @@ _DURATION_SCALES = {
 }
 
 _BINARY_SIZE_SCALES = {
-    'B': 1,
-    'KB': 1 << 10,
-    'MB': 1 << 20,
-    'GB': 1 << 30,
-    'TB': 1 << 40,
-    'PB': 1 << 50,
-    'KiB': 1 << 10,
-    'MiB': 1 << 20,
-    'GiB': 1 << 30,
-    'TiB': 1 << 40,
-    'PiB': 1 << 50,
+    '': 1,
+    'K': 1 << 10,
+    'M': 1 << 20,
+    'G': 1 << 30,
+    'T': 1 << 40,
+    'P': 1 << 50,
+    'Ki': 1 << 10,
+    'Mi': 1 << 20,
+    'Gi': 1 << 30,
+    'Ti': 1 << 40,
+    'Pi': 1 << 50,
 }
 
 
@@ -157,8 +168,42 @@ def GetMultiCompleter(individual_completer):
   return MultiCompleter
 
 
+def _DeleteTypeAbbr(suffix, type_abbr='B'):
+  """Returns suffix with trailing type abbreviation deleted."""
+  if not suffix:
+    return suffix
+  s = suffix.upper()
+  i = len(s)
+  for c in reversed(type_abbr.upper()):
+    if not i:
+      break
+    if s[i - 1] == c:
+      i -= 1
+  return suffix[:i]
+
+
+def GetBinarySizePerUnit(suffix, type_abbr='B'):
+  """Returns the binary size per unit for binary suffix string.
+
+  Args:
+    suffix: str, A case insensitive unit suffix string with optional type
+      abbreviation.
+    type_abbr: str, The optional case insensitive type abbreviation following
+      the suffix.
+
+  Raises:
+    ValueError for unknown units.
+
+  Returns:
+    The binary size per unit for a unit+type_abbr suffix.
+  """
+  unit = _DeleteTypeAbbr(suffix.upper(), type_abbr)
+  return _BINARY_SIZE_SCALES.get(unit)
+
+
 def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None,
-                 strict_case=True, suggested_binary_size_scales=None):
+                 strict_case=True, type_abbr='B',
+                 suggested_binary_size_scales=None):
   """A helper that returns a function that can parse values with units.
 
   Casing for all units matters.
@@ -171,6 +216,8 @@ def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None,
     lower_bound: str, An inclusive lower bound.
     upper_bound: str, An inclusive upper bound.
     strict_case: bool, whether to be strict on case-checking
+    type_abbr: str, the type suffix abbreviation, e.g., B for bytes, b/s for
+      bits/sec.
     suggested_binary_size_scales: list, A list of strings with units that will
                                     be recommended to user.
 
@@ -180,15 +227,15 @@ def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None,
 
   def UnitsByMagnitude(suggested_binary_size_scales=None):
     """Returns a list of the units in scales sorted by magnitude."""
-    scale_items = sorted(scales.iteritems(),
+    scale_items = sorted(six.iteritems(scales),
                          key=lambda value: (value[1], value[0]))
     if suggested_binary_size_scales is None:
-      return [key for key, _ in scale_items]
-    return [key for key, _ in scale_items
-            if key in suggested_binary_size_scales]
+      return [key + type_abbr for key, _ in scale_items]
+    return [key + type_abbr for key, _ in scale_items
+            if key + type_abbr in suggested_binary_size_scales]
 
   def Parse(value):
-    """Parses value that can contain a unit."""
+    """Parses value that can contain a unit and type avvreviation."""
     match = re.match(_VALUE_PATTERN, value, re.VERBOSE)
     if not match:
       raise ArgumentTypeError(_GenerateErrorMessage(
@@ -198,17 +245,18 @@ def _ValueParser(scales, default_unit, lower_bound=None, upper_bound=None,
           user_input=value))
 
     amount = int(match.group('amount'))
-    unit = match.group('unit')
+    suffix = match.group('suffix') or ''
+    unit = _DeleteTypeAbbr(suffix, type_abbr)
     if strict_case:
       unit_case = unit
-      default_unit_case = default_unit
+      default_unit_case = _DeleteTypeAbbr(default_unit, type_abbr)
       scales_case = scales
     else:
-      unit_case = unit and unit.upper()
-      default_unit_case = default_unit.upper()
+      unit_case = unit.upper()
+      default_unit_case = _DeleteTypeAbbr(default_unit.upper(), type_abbr)
       scales_case = dict([(k.upper(), v) for k, v in scales.items()])
 
-    if unit_case is None:
+    if not unit and unit == suffix:
       return amount * scales_case[default_unit_case]
     elif unit_case in scales_case:
       return amount * scales_case[unit_case]
@@ -274,25 +322,57 @@ def RegexpValidator(pattern, description):
   return Parse
 
 
-def Duration(lower_bound=None, upper_bound=None):
+def CustomFunctionValidator(fn, description, parser=None):
+  """Returns a function that validates the input by running it through fn.
+
+  For example:
+
+  >>> def isEven(val):
+  ...   return val % 2 == 0
+  >>> even_number_parser = arg_parsers.CustomFunctionValidator(
+        isEven, 'This is not even!', parser=arg_parsers.BoundedInt(0))
+  >>> parser.add_argument('--foo', type=even_number_parser)
+  >>> parser.parse_args(['--foo', '3'])
+  >>> # SystemExit raised and the error "error: argument foo: Bad value [3]:
+  >>> # This is not even!" is displayed
+
+  Args:
+    fn: str -> boolean
+    description: an error message to show if boolean function returns False
+    parser: an arg_parser that is applied to to value before validation. The
+      value is also returned by this parser.
+
+  Returns:
+    function: str -> str, usable as an argparse type
+  """
+
+  def Parse(value):
+    """Validates and returns a custom object from an argument string value."""
+    try:
+      parsed_value = parser(value) if parser else value
+    except ArgumentTypeError:
+      pass
+    else:
+      if fn(parsed_value):
+        return parsed_value
+    encoded_value = console_attr.SafeText(value)
+    formatted_err = 'Bad value [{0}]: {1}'.format(encoded_value, description)
+    raise ArgumentTypeError(formatted_err)
+
+  return Parse
+
+
+def Duration(default_unit='s', lower_bound='0', upper_bound=None):
   """Returns a function that can parse time durations.
 
-  Input to the parsing function must be a string of the form:
-
-    INTEGER[UNIT]
-
-  The integer must be non-negative. Valid units are "s", "m", "h", and
-  "d" for seconds, seconds, minutes, hours, and days,
-  respectively. The casing of the units matters.
-
-  If the unit is omitted, seconds is assumed.
-
-  The result is parsed in seconds. For example:
+  See times.ParseDuration() for details. If the unit is omitted, seconds is
+  assumed. The parsed result is in seconds. For example:
 
     parser = Duration()
     assert parser('10s') == 10
 
   Args:
+    default_unit: str, The default duration unit.
     lower_bound: str, An inclusive lower bound for values.
     upper_bound: str, An inclusive upper bound for values.
 
@@ -307,12 +387,45 @@ def Duration(lower_bound=None, upper_bound=None):
     A function that accepts a single time duration as input to be
       parsed.
   """
-  return _ValueParser(_DURATION_SCALES, default_unit='s',
-                      lower_bound=lower_bound, upper_bound=upper_bound)
+
+  def Parse(value):
+    """Parses a duration from value and returns integer seconds."""
+    try:
+      return int(
+          times.ParseDuration(value, default_suffix=default_unit).total_seconds)
+    except times.Error as e:
+      message = six.text_type(e).rstrip('.')
+      raise ArgumentTypeError(_GenerateErrorMessage(
+          'Failed to parse duration: {0}'.format(message, user_input=value)))
+
+  parsed_lower_bound = Parse(lower_bound)
+
+  if upper_bound is None:
+    parsed_upper_bound = None
+  else:
+    parsed_upper_bound = Parse(upper_bound)
+
+  def ParseWithBoundsChecking(value):
+    """Same as Parse except bound checking is performed."""
+    if value is None:
+      return None
+    parsed_value = Parse(value)
+    if parsed_lower_bound is not None and parsed_value < parsed_lower_bound:
+      raise ArgumentTypeError(_GenerateErrorMessage(
+          'value must be greater than or equal to {0}'.format(lower_bound),
+          user_input=value))
+    if parsed_upper_bound is not None and parsed_value > parsed_upper_bound:
+      raise ArgumentTypeError(_GenerateErrorMessage(
+          'value must be less than or equal to {0}'.format(upper_bound),
+          user_input=value))
+    return parsed_value
+
+  return ParseWithBoundsChecking
 
 
 def BinarySize(lower_bound=None, upper_bound=None,
-               suggested_binary_size_scales=None, default_unit='GB'):
+               suggested_binary_size_scales=None, default_unit='G',
+               type_abbr='B'):
   """Returns a function that can parse binary sizes.
 
   Binary sizes are defined as base-2 values representing number of
@@ -324,7 +437,7 @@ def BinarySize(lower_bound=None, upper_bound=None,
 
   The integer must be non-negative. Valid units are "B", "KB", "MB",
   "GB", "TB", "KiB", "MiB", "GiB", "TiB", "PiB".  If the unit is
-  omitted, GB is assumed.
+  omitted then default_unit is assumed.
 
   The result is parsed in bytes. For example:
 
@@ -337,6 +450,8 @@ def BinarySize(lower_bound=None, upper_bound=None,
     suggested_binary_size_scales: list, A list of strings with units that will
                                     be recommended to user.
     default_unit: str, unit used when user did not specify unit.
+    type_abbr: str, the type suffix abbreviation, e.g., B for bytes, b/s for
+      bits/sec.
 
   Raises:
     ArgumentTypeError: If either the lower_bound or upper_bound
@@ -350,9 +465,8 @@ def BinarySize(lower_bound=None, upper_bound=None,
       parsed.
   """
   return _ValueParser(
-      _BINARY_SIZE_SCALES, default_unit=default_unit,
-      lower_bound=lower_bound, upper_bound=upper_bound,
-      strict_case=False,
+      _BINARY_SIZE_SCALES, default_unit=default_unit, lower_bound=lower_bound,
+      upper_bound=upper_bound, strict_case=False, type_abbr=type_abbr,
       suggested_binary_size_scales=suggested_binary_size_scales)
 
 
@@ -474,8 +588,9 @@ class Day(object):
       return times.ParseDateTime(s, '%Y-%m-%d').date()
     except times.Error as e:
       raise ArgumentTypeError(
-          _GenerateErrorMessage(u'Failed to parse date: {0}'.format(unicode(e)),
-                                user_input=s))
+          _GenerateErrorMessage(
+              'Failed to parse date: {0}'.format(six.text_type(e)),
+              user_input=s))
 
 
 class Datetime(object):
@@ -491,7 +606,7 @@ class Datetime(object):
     except times.Error as e:
       raise ArgumentTypeError(
           _GenerateErrorMessage(
-              u'Failed to parse date/time: {0}'.format(unicode(e)),
+              'Failed to parse date/time: {0}'.format(six.text_type(e)),
               user_input=s))
 
 
@@ -602,6 +717,38 @@ class ArgType(object):
   """Base class for arg types."""
 
 
+class ArgBoolean(ArgType):
+  """Interpret an argument value as a bool."""
+
+  def __init__(
+      self, truthy_strings=None, falsey_strings=None, case_sensitive=False):
+    self._case_sensitive = case_sensitive
+    if truthy_strings:
+      self._truthy_strings = truthy_strings
+    else:
+      self._truthy_strings = ['true', 'yes']
+    if falsey_strings:
+      self._falsey_strings = falsey_strings
+    else:
+      self._falsey_strings = ['false', 'no']
+
+  def __call__(self, arg_value):
+    if not self._case_sensitive:
+      normalized_arg_value = arg_value.lower()
+    else:
+      normalized_arg_value = arg_value
+    if normalized_arg_value in self._truthy_strings:
+      return True
+    if normalized_arg_value in self._falsey_strings:
+      return False
+    raise ArgumentTypeError(
+        'Invalid flag value [{0}], expected one of [{1}]'.format(
+            arg_value,
+            ', '.join(self._truthy_strings + self._falsey_strings)
+        )
+    )
+
+
 class ArgList(ArgType):
   """Interpret an argument value as a list.
 
@@ -678,12 +825,64 @@ class ArgList(ArgType):
 
     return arg_list
 
+  _MAX_METAVAR_LENGTH = 30  # arbitrary, but this is pretty long
+
   def GetUsageMsg(self, is_custom_metavar, metavar):
-    is_custom_metavar = is_custom_metavar
-    msg = '[{metavar},...]'.format(metavar=metavar)
-    if self.min_length:
-      msg = ','.join([metavar]*self.min_length+[msg])
-    return msg
+    """Get a specially-formatted metavar for the ArgList to use in help.
+
+    An example is worth 1,000 words:
+
+    >>> ArgList().GetUsageMetavar('FOO')
+    '[FOO,...]'
+    >>> ArgList(min_length=1).GetUsageMetavar('FOO')
+    'FOO,[FOO,...]'
+    >>> ArgList(max_length=2).GetUsageMetavar('FOO')
+    'FOO,[FOO]'
+    >>> ArgList(max_length=3).GetUsageMetavar('FOO')  # One, two, many...
+    'FOO,[FOO,...]'
+    >>> ArgList(min_length=2, max_length=2).GetUsageMetavar('FOO')
+    'FOO,FOO'
+    >>> ArgList().GetUsageMetavar('REALLY_VERY_QUITE_LONG_METAVAR')
+    'REALLY_VERY_QUITE_LONG_METAVAR,[...]'
+
+    Args:
+      is_custom_metavar: unused in GetUsageMsg
+      metavar: string, the base metavar to turn into an ArgList metavar
+
+    Returns:
+      string, the ArgList usage metavar
+    """
+    del is_custom_metavar  # Unused in GetUsageMsg
+
+    required = ','.join([metavar] * self.min_length)
+
+    if self.max_length:
+      num_optional = self.max_length - self.min_length
+    else:
+      num_optional = None
+
+    # Use the "1, 2, many" approach to counting
+    if num_optional == 0:
+      optional = ''
+    elif num_optional == 1:
+      optional = '[{}]'.format(metavar)
+    elif num_optional == 2:
+      optional = '[{0},[{0}]]'.format(metavar)
+    else:
+      optional = '[{0},...]'.format(metavar)
+
+    msg = ','.join([x for x in [required, optional] if x])
+
+    if len(msg) < self._MAX_METAVAR_LENGTH:
+      return msg
+
+    # With long metavars, only put it in once.
+    if self.min_length == 0:
+      return '[{},...]'.format(metavar)
+    if self.min_length == 1:
+      return '{},[...]'.format(metavar)
+    else:
+      return '{},...,[...]'.format(metavar)
 
 
 class ArgDict(ArgList):
@@ -694,11 +893,13 @@ class ArgDict(ArgList):
   key-value pairs to get a dict.
   """
 
-  def __init__(self, value_type=None, spec=None, min_length=0, max_length=None,
-               allow_key_only=False, required_keys=None, operators=None):
+  def __init__(self, key_type=None, value_type=None, spec=None, min_length=0,
+               max_length=None, allow_key_only=False, required_keys=None,
+               operators=None):
     """Initialize an ArgDict.
 
     Args:
+      key_type: (str)->str, A function to apply to each of the dict keys.
       value_type: (str)->str, A function to apply to each of the dict values.
       spec: {str: (str)->str}, A mapping of expected keys to functions.
         The functions are applied to the values. If None, an arbitrary
@@ -723,6 +924,7 @@ class ArgDict(ArgList):
     super(ArgDict, self).__init__(min_length=min_length, max_length=max_length)
     if spec and value_type:
       raise ValueError('cannot have both spec and sub_type')
+    self.key_type = key_type
     self.spec = spec
     self.allow_key_only = allow_key_only
     self.required_keys = required_keys or []
@@ -732,7 +934,7 @@ class ArgDict(ArgList):
       if len(op) != 1:
         raise ArgumentTypeError(
             'Operator [{}] must be one character.'.format(op))
-    ops = ''.join(operators.keys())
+    ops = ''.join(six.iterkeys(operators))
     key_op_value_pattern = '([^{ops}]+)([{ops}]?)(.*)'.format(
         ops=re.escape(ops))
     self.key_op_value = re.compile(key_op_value_pattern, re.DOTALL)
@@ -767,9 +969,17 @@ class ArgDict(ArgList):
             ('Bad syntax for dict arg: [{0}]. Please see `gcloud topic '
              'escaping` if you would like information on escaping list or '
              'dictionary flag values.').format(arg))
+      if self.key_type:
+        try:
+          key = self.key_type(key)
+        except ValueError:
+          raise ArgumentTypeError('Invalid key [{0}]'.format(key))
       convert_value = self.operators.get(op, None)
       if convert_value:
-        value = convert_value(value)
+        try:
+          value = convert_value(value)
+        except ValueError:
+          raise ArgumentTypeError('Invalid value [{0}]'.format(value))
       if self.spec:
         value = self._ApplySpec(key, value)
       arg_dict[key] = value
@@ -789,7 +999,7 @@ class ArgDict(ArgList):
       return super(ArgDict, self).GetUsageMsg(is_custom_metavar, metavar)
 
     msg_list = []
-    spec_list = sorted(self.spec.iteritems())
+    spec_list = sorted(six.iteritems(self.spec))
 
     # First put the spec keys with no value followed by those that expect a
     # value
@@ -902,7 +1112,7 @@ class UpdateAction(argparse.Action):
       # Get the existing arg value (if any)
       items = copy.copy(argparse._ensure_value(namespace, self.dest, {}))
       # Merge the new key/value pair(s) in
-      for k, v in values.iteritems():
+      for k, v in six.iteritems(values):
         if k in items:
           v = self.onduplicatekey_handler(self, k, items[k], v)
         items[k] = v
@@ -979,11 +1189,11 @@ class RemainderAction(argparse._StoreAction):  # pylint: disable=protected-acces
   Primarily, this Action provides two utility parsers to help a modified
   ArgumentParser parse -- properly.
 
-  There is one additional property:
+  There is one additional property kwarg:
     example: A usage statement used to construct nice additional help.
   """
 
-  def __init__(self, example=None, *args, **kwargs):
+  def __init__(self, *args, **kwargs):
     if kwargs['nargs'] is not argparse.REMAINDER:
       raise ValueError(
           'The RemainderAction should only be used when '
@@ -995,9 +1205,10 @@ class RemainderAction(argparse._StoreAction):  # pylint: disable=protected-acces
         'the left and {metavar} on the right.'
     ).format(metavar=kwargs['metavar'])
     if 'help' in kwargs:
-      kwargs['help'] += '\n\n' + self.explanation
-      if example:
-        kwargs['help'] += ' Example:\n\n' + example
+      kwargs['help'] += '\n+\n' + self.explanation
+      if 'example' in kwargs:
+        kwargs['help'] += ' Example:\n\n' + kwargs['example']
+        del kwargs['example']
     super(RemainderAction, self).__init__(*args, **kwargs)
 
   def _SplitOnDash(self, args):
@@ -1039,7 +1250,7 @@ class RemainderAction(argparse._StoreAction):  # pylint: disable=protected-acces
     # Strip out everything after '--'
     if '--' in original_args:
       original_args, _ = self._SplitOnDash(original_args)
-    # Find common suffix between remaining_args and orginal_args
+    # Find common suffix between remaining_args and original_args
     split_index = 0
     for i, (arg1, arg2) in enumerate(
         zip(reversed(remaining_args), reversed(original_args))):
@@ -1050,8 +1261,8 @@ class RemainderAction(argparse._StoreAction):  # pylint: disable=protected-acces
     remaining_args = remaining_args[:split_index]
 
     if pass_through_args:
-      msg = (u'unrecognized args: {args}\n' + self.explanation).format(
-          args=u' '.join(pass_through_args))
+      msg = ('unrecognized args: {args}\n' + self.explanation).format(
+          args=' '.join(pass_through_args))
       raise parser_errors.UnrecognizedArgumentsError(msg)
     self(None, namespace, pass_through_args)
     return namespace, remaining_args
@@ -1114,7 +1325,7 @@ class _HandleNoArgAction(argparse.Action):
 
   def __call__(self, parser, namespace, value, option_string=None):
     if value is None:
-      log.warn(self.deprecation_message)
+      log.warning(self.deprecation_message)
       if self.none_arg:
         setattr(namespace, self.none_arg, True)
 
@@ -1126,7 +1337,7 @@ def HandleNoArgAction(none_arg, deprecation_message):
 
   This function creates an argparse action which can be used to gracefully
   deprecate a flag using nargs=?. When a flag is created with this action, it
-  simply log.warn()s the given deprecation_message and then sets the value of
+  simply log.warning()s the given deprecation_message and then sets the value of
   the none_arg to True.
 
   This means if you use the none_arg no_foo and attach this action to foo,
@@ -1144,3 +1355,70 @@ def HandleNoArgAction(none_arg, deprecation_message):
     return _HandleNoArgAction(none_arg, deprecation_message, **kwargs)
 
   return HandleNoArgActionInit
+
+
+class BufferedFileInput(object):
+  """Creates an argparse type that reads and buffers file or stdin contents.
+
+  This is similar to argparse.FileType, but unlike FileType it does not leave
+  a dangling file handle open. The argument stored in the argparse Namespace
+  is the file's contents.
+
+  Args:
+    max_bytes: int, The maximum file size in bytes, or None to specify no
+        maximum.
+    chunk_size: int, When max_bytes is not None, the buffer size to use when
+        reading chunks from the input file.
+    binary: bool, If True, the contents of the file will be returned as bytes.
+
+  Returns:
+    A function that accepts a filename, or "-" representing that stdin should be
+    used as input.
+  """
+
+  def __init__(self, binary=False):
+    self.binary = binary
+
+  def __call__(self, name):
+    """Return the contents of the file with the specified name.
+
+    If name is "-", stdin is read until EOF. Otherwise, the named file is read.
+    If max_bytes is provided when calling BufferedFileInput, this function will
+    raise an ArgumentTypeError if the specified file is too large.
+
+    Args:
+      name: str, The file name, or '-' to indicate stdin.
+
+    Returns:
+      The contents of the file.
+
+    Raises:
+      ArgumentTypeError: If the file cannot be read or is too large.
+    """
+    try:
+      return console_io.ReadFromFileOrStdin(name, binary=self.binary)
+    except files.Error as e:
+      raise ArgumentTypeError(e)
+
+
+class StoreTrueFalseAction(argparse._StoreTrueAction):  # pylint: disable=protected-access
+  """Argparse action that acts as a combination of store_true and store_false.
+
+  Calliope already gives any bool-type arguments the standard and `--no-`
+  variants. In most cases we only want to document the option that does
+  something---if we have `default=False`, we don't want to show `--no-foo`,
+  since it won't do anything.
+
+  But in some cases we *do* want to show both variants: one example is when
+  `--foo` means "enable," `--no-foo` means "disable," and neither means "do
+  nothing." The obvious way to represent this is `default=None`; however, (1)
+  the default value of `default` is already None, so most boolean actions would
+  have this setting by default (not what we want), and (2) we still want an
+  option to have this True/False/None behavior *without* the flag documentation.
+
+  To get around this, we have an opt-in version of the same thing that documents
+  both the flag and its inverse.
+  """
+
+  def __init__(self, *args, **kwargs):
+    super(StoreTrueFalseAction, self).__init__(*args, default=None, **kwargs)
