@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -36,8 +37,10 @@ There is a tapper for each of the --filter, --flatten, --limit, and --page
 flags.
 """
 
-from googlecloudsdk.core import remote_completion
-from googlecloudsdk.core.cache import cache_update_ops
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.core.resource import resource_filter
 from googlecloudsdk.core.resource import resource_printer_base
 from googlecloudsdk.core.resource import resource_projector
@@ -230,17 +233,7 @@ class UriCacher(peek_iterable.Tap):
 
   def Done(self):
     if self._uris is not None:
-      if isinstance(self._update_cache_op, cache_update_ops.AddToCacheOp):
-        update_cache_op = remote_completion.AddToCacheOp
-      elif isinstance(self._update_cache_op,
-                      cache_update_ops.DeleteFromCacheOp):
-        update_cache_op = remote_completion.DeleteFromCacheOp
-      elif isinstance(self._update_cache_op, cache_update_ops.ReplaceCacheOp):
-        update_cache_op = remote_completion.ReplaceCacheOp
-      else:
-        return
-      remote_completion.RemoteCompletion().UpdateCache(update_cache_op,
-                                                       self._uris)
+      self._update_cache_op.Update(self._uris)
 
 
 class UriReplacer(peek_iterable.Tap):
