@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,15 +15,19 @@
 
 """The Calliope command help document markdown generator."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import abc
-import argparse
+import io
 import re
-import StringIO
 import textwrap
 
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import usage_text
 from googlecloudsdk.core.console import console_io
+import six
 
 
 _SPLIT = 78  # Split lines longer than this.
@@ -167,7 +172,7 @@ class ExampleCommandLineSplitter(object):
     return ''.join(lines)
 
 
-class MarkdownGenerator(object):
+class MarkdownGenerator(six.with_metaclass(abc.ABCMeta, object)):
   """Command help markdown document generator base class.
 
   Attributes:
@@ -179,19 +184,13 @@ class MarkdownGenerator(object):
     _docstring: The command docstring.
     _file_name: The command path name (used to name documents).
     _final_sections: The list of PrintFinalSections section names.
-    _flag_sections: The flag sections generated by _SetFlagSections().
     _is_hidden: The command is hidden.
     _is_topic: True if the command is a help topic.
     _out: Output writer.
     _printed_sections: The set of already printed sections.
     _release_track: The calliope.base.ReleaseTrack.
-    _sections: Command SECTION dict indexed by section name.
-    _subcommand: The list of subcommand instances or None.
-    _subgroup: The list of subgroup instances or None.
   """
 
-  __metaclass__ = abc.ABCMeta
-
   def __init__(self, command_path, release_track, is_hidden):
     """Constructor.
 
@@ -202,18 +201,20 @@ class MarkdownGenerator(object):
     """
     self._command_path = command_path
     self._command_name = ' '.join(self._command_path)
+    self._subcommands = None  # type: dict
+    self._subgroups = None  # type: dict
     self._top = self._command_path[0] if self._command_path else ''
-    self._buf = StringIO.StringIO()
+    self._buf = io.StringIO()
     self._out = self._buf.write
     self._capsule = ''
     self._docstring = ''
     self._final_sections = ['EXAMPLES', 'SEE ALSO']
+    self._arg_sections = None  # type: list
     self._sections = {}
     self._file_name = '_'.join(self._command_path)
-    self._flag_sections = None
-    self._global_flags = []
+    self._global_flags = set()
     self._is_hidden = is_hidden
-    self._is_root = len(self._command_path) == 1
+    self._is_root = self._IsRoot()
     self._release_track = release_track
     if (len(self._command_path) >= 3 and
         self._command_path[1] == release_track.prefix):
@@ -224,27 +225,6 @@ class MarkdownGenerator(object):
                       self._command_path[command_index] == 'topic')
     self._printed_sections = set()
 
-  @staticmethod
-  @abc.abstractmethod
-  def FlagGroupSortKey(flags):
-    """Returns a flag group sort key for sorted().
-
-    This orders individual flags before mutually exclusive groups.
-
-    Args:
-      flags: A list of flags in the group.
-
-    Returns:
-      A sort key for the sorted() builtin where singletons sort before groups.
-    """
-    pass
-
-  @staticmethod
-  @abc.abstractmethod
-  def IsHidden(arg):
-    """Returns True if arg is suppressed."""
-    pass
-
   @abc.abstractmethod
   def IsValidSubPath(self, sub_command_path):
     """Determines if the given sub command path is valid from this node.
@@ -259,18 +239,49 @@ class MarkdownGenerator(object):
     pass
 
   @abc.abstractmethod
-  def GetPositionalArgs(self):
-    """Returns the command positional args."""
+  def GetArguments(self):
+    """Returns the command arguments."""
     pass
 
-  @abc.abstractmethod
-  def GetFlagGroups(self):
-    """Returns (group, group_attr, global_flags)."""
-    pass
+  def FormatExample(self, cmd, args, with_args):
+    """Creates a link to the command reference from a command example.
 
-  def _FilterOutHidden(self, args):
-    """Returns a copy of args containing only non-hidden arguments."""
-    return [a for a in args if not self.IsHidden(a)]
+    If with_args is False and the provided command includes args,
+    returns None.
+
+    Args:
+      cmd: [str], a command.
+      args: [str], args with the command.
+      with_args: bool, whether the example is valid if it has args.
+
+    Returns:
+      (str) a representation of the command with a link to the reference, plus
+      any args. | None, if the command isn't valid.
+    """
+    if args and not with_args:
+      return None
+    ref = '/'.join(cmd)
+    command_link = 'link:' + ref + '[' + ' '.join(cmd) + ']'
+    if args:
+      command_link += ' ' + ' '.join(args)
+    return command_link
+
+  def _IsRoot(self):
+    """Determine if this node should be treated as a "root" of the CLI tree.
+
+    The top element is the root, but we also treat any additional release tracks
+    as a root so that global flags are shown there as well.
+
+    Returns:
+      True if this node should be treated as a root, False otherwise.
+    """
+    if len(self._command_path) == 1:
+      return True
+    elif len(self._command_path) == 2:
+      tracks = [t.prefix for t in base.ReleaseTrack.AllValues()]
+      if self._command_path[-1] in tracks:
+        return True
+    return False
 
   def _ExpandHelpText(self, text):
     """Expand command {...} references in text.
@@ -291,89 +302,11 @@ class MarkdownGenerator(object):
         **self._sections
     )
 
-  def _SetFlagSections(self):
-    """Sets self._flag_sections in document order and self._global_flags.
-
-    Returns:
-      ([section], global_flags)
-        section - (heading, is_priority, flags)
-          heading - The section heading.
-          is_priority - True if this is a priority section. Priority sections
-            are grouped first. The first 2 priority sections appear in short
-            help.
-          flags - The list of flags in the section.
-          attrs - A dict of calliope.backend.ArgumentGroupAttr objects indexed
-            by group_id.
-        global_flags - The list of global flags not included in the section
-          .list
-    """
-    if self._flag_sections is not None:
-      return
-    groups, group_attr, self._global_flags = self.GetFlagGroups()
-    # Partition the non-GLOBAL flag groups dict into categorized sections. A
-    # group is REQUIRED if any flag in it is required, categorized if any flag
-    # in it is categorized, otherwise its OTHER.  REQUIRED takes precedence
-    # over categorized.
-    categorized_groups = {}
-    attrs = {}
-    for group_id, group in groups.iteritems():
-      if groups and not group_id:
-        continue
-      flags = self._FilterOutHidden(group)
-      if not flags:
-        continue
-
-      attr = group_attr.get(group_id)
-      if attr and attr.is_mutex and attr.is_required:
-        category = 'REQUIRED'
-      else:
-        category = 'OTHER'
-        for f in flags:
-          if f.required:
-            category = 'REQUIRED'
-            break
-          elif f.category:
-            category = f.category
-            break
-
-      if category not in categorized_groups:
-        categorized_groups[category] = {}
-      categorized_groups[category][group_id] = flags
-      if category not in attrs:
-        attrs[category] = {}
-      attrs[category][group_id] = attr
-
-    # Collect the priority sections first in order:
-    #   REQUIRED, COMMON, OTHER, and categorized.
-    self._flag_sections = []
-    other_flags_heading = 'FLAGS'
-    for category, other in (('REQUIRED', 'OPTIONAL'),
-                            (base.COMMONLY_USED_FLAGS, 'OTHER'),
-                            ('OTHER', None)):
-      if category in categorized_groups:
-        if other:
-          other_flags_heading = other
-          heading = category
-        elif len(categorized_groups) > 1:
-          heading = 'FLAGS'
-        else:
-          heading = other_flags_heading
-        if heading == base.COMMONLY_USED_FLAGS and self._is_root:
-          # The root command COMMON flags are "<self._top> WIDE".
-          heading = '{} WIDE'.format(self._top.upper())
-        self._flag_sections.append((GetFlagHeading(heading),
-                                    other is not None,
-                                    categorized_groups[category],
-                                    attrs[category]))
-        # This prevents the category from being re-added in the loop below.
-        del categorized_groups[category]
-
-    # Add the remaining categories in sorted order.
-    for category, groups in sorted(categorized_groups.iteritems()):
-      self._flag_sections.append((GetFlagHeading(category),
-                                  False,
-                                  groups,
-                                  attrs[category]))
+  def _SetArgSections(self):
+    """Sets self._arg_sections in document order."""
+    if self._arg_sections is None:
+      self._arg_sections, self._global_flags = usage_text.GetArgSections(
+          self.GetArguments(), self._is_root)
 
   def _SplitCommandFromArgs(self, cmd):
     """Splits cmd into command and args lists.
@@ -413,6 +346,10 @@ class MarkdownGenerator(object):
             msg +
             base.MARKDOWN_ITALIC + base.MARKDOWN_CODE)
 
+  def _ArgTypeName(self, arg):
+    """Returns the argument type name for arg."""
+    return 'positional' if arg.is_positional else 'flag'
+
   def PrintSectionHeader(self, name, sep=True):
     """Prints the section header markdown for name.
 
@@ -445,7 +382,7 @@ class MarkdownGenerator(object):
     """
     if self._is_topic:
       return
-    self._SetFlagSections()
+    self._SetArgSections()
     # MARKDOWN_CODE is the default SYNOPSIS font style.
     code = base.MARKDOWN_CODE
     em = base.MARKDOWN_ITALIC
@@ -454,18 +391,6 @@ class MarkdownGenerator(object):
     self._out('{code}{command}{code}'.format(code=code,
                                              command=self._command_name))
 
-    # Output the positional args up to the first REMAINDER or '-- *' args. The
-    # rest will be picked up after the flag args are output. argparse does not
-    # have an explicit '--' arg intercept, so we use the metavar value as a '--'
-    # sentinel. Any suppressed args are ingnored by a pre-pass.
-    positional_args = self._FilterOutHidden(self.GetPositionalArgs())
-    while positional_args:
-      arg = positional_args[0]
-      if arg.nargs == argparse.REMAINDER or arg.metavar.startswith('-- '):
-        break
-      positional_args.pop(0)
-      self._out(' ' + usage_text.PositionalDisplayString(arg, markdown=True))
-
     if self._subcommands and self._subgroups:
       self._out(' ' + em + 'GROUP' + em + ' | ' + em + 'COMMAND' + em)
     elif self._subcommands:
@@ -473,51 +398,78 @@ class MarkdownGenerator(object):
     elif self._subgroups:
       self._out(' ' + em + 'GROUP' + em)
 
-    # Generate the flag usage string with flags in section order.
-    for _, _, groups, attrs in self._flag_sections:
-      for group_id, group in sorted(
-          groups.iteritems(), key=lambda x: self.FlagGroupSortKey(x[1])):
-        flag = group[0]
-        if len(group) == 1:
-          msg = usage_text.FlagDisplayString(
-              flag,
-              markdown=True,
-              inverted=getattr(flag, 'inverted_synopsis', False))
-          if not msg:
-            continue
-          if flag.required:
-            self._out(' {msg}'.format(msg=msg))
-          else:
-            self._out(' [{msg}]'.format(msg=msg))
-        else:
-          group.sort(key=lambda f: f.option_strings)
-          attr = attrs.get(group_id)
-          if not attr or not attr.is_mutex:
-            for flag in group:
-              self._out(' [{0}]'.format(
-                  usage_text.FlagDisplayString(flag, markdown=True)))
-          else:
-            msg = ' | '.join(usage_text.FlagDisplayString(flag, markdown=True)
-                             for flag in group)
-            if not msg:
-              continue
-            if attr.is_required:
-              self._out(' ({msg})'.format(msg=msg))
-            else:
-              self._out(' [{msg}]'.format(msg=msg))
-
+    # Generate the arg usage string with flags in section order.
+    remainder_usage = []
+    for section in self._arg_sections:
+      self._out(' ')
+      self._out(usage_text.GetArgUsage(section.args, markdown=True, top=True,
+                                       remainder_usage=remainder_usage))
     if self._global_flags:
       self._out(' [' + em + self._top.upper() + '_WIDE_FLAG ...' + em + ']')
-
-    # positional_args will only be non-empty if we had -- ... or REMAINDER left.
-    for arg in self._FilterOutHidden(positional_args):
-      self._out(' ' + usage_text.PositionalDisplayString(arg, markdown=True))
+    if remainder_usage:
+      self._out(' ')
+      self._out(' '.join(remainder_usage))
 
     self._out('\n')
 
+  def _PrintArgDefinition(self, arg, depth=0, single=False):
+    """Prints a positional or flag arg definition list item at depth."""
+    usage = usage_text.GetArgUsage(arg, definition=True, markdown=True)
+    if not usage:
+      return
+    self._out('\n{usage}{depth}\n'.format(
+        usage=usage, depth=':' * (depth + 2)))
+    if arg.is_required and depth and not single:
+      modal = (' This {arg_type} must be specified if any of the other '
+               'arguments in this group are specified.').format(
+                   arg_type=self._ArgTypeName(arg))
+    else:
+      modal = ''
+    self._out('\n{details}{modal}\n'.format(
+        details=self.GetArgDetails(arg), modal=modal))
+
+  def _PrintArgGroup(self, arg, depth=0, single=False):
+    """Prints an arg group definition list at depth."""
+
+    args = sorted(arg.arguments, key=usage_text.GetArgSortKey)
+    heading = []
+    if arg.help or arg.is_mutex or arg.is_required:
+      if arg.help:
+        heading.append(arg.help)
+      if len(args) == 1 or args[0].is_required:
+        if arg.is_required:
+          # TODO (b/77314072): Put this before (NOTE) section in resource args.
+          heading.append('This must be specified.')
+      elif arg.is_mutex:
+        if arg.is_required:
+          heading.append('Exactly one of these must be specified:')
+        else:
+          heading.append('At most one of these may be specified:')
+      elif arg.is_required:
+        heading.append('At least one of these must be specified:')
+    for a in args:
+      if a.is_hidden:
+        continue
+      if heading:
+        self._out('\n{0} {1}\n\n'.format(':' * (depth + 2), ' '.join(heading)))
+        heading = None
+        depth += 1
+      if a.is_group:
+        single = False
+        singleton = usage_text.GetSingleton(a)
+        if singleton:
+          if not a.help:
+            a = singleton
+          else:
+            single = True
+      if a.is_group:
+        self._PrintArgGroup(a, depth=depth, single=single)
+      else:
+        self._PrintArgDefinition(a, depth=depth, single=single)
+
   def PrintPositionalDefinition(self, arg):
     self._out('\n{0}::\n'.format(
-        usage_text.PositionalDisplayString(arg, markdown=True)))
+        usage_text.GetPositionalUsage(arg, markdown=True)))
     self._out('\n{arghelp}\n'.format(arghelp=self.GetArgDetails(arg)))
 
   def PrintFlagDefinition(self, flag, disable_header=False):
@@ -530,32 +482,20 @@ class MarkdownGenerator(object):
     if not disable_header:
       self._out('\n')
     self._out('{0}::\n'.format(
-        usage_text.FlagDisplayString(flag, markdown=True)))
+        usage_text.GetFlagUsage(flag, markdown=True)))
     self._out('\n{arghelp}\n'.format(arghelp=self.GetArgDetails(flag)))
 
-  def PrintFlagSection(self, heading, groups, attrs, disable_header=False):
+  def PrintFlagSection(self, heading, arg, disable_header=False):
     """Prints a flag section.
 
     Args:
       heading: The flag section heading name.
-      groups: The flag group ids.
-      attrs: The flag group attributes.
+      arg: The flag args / group.
       disable_header: Disable printing the section header if True.
     """
     if not disable_header:
       self.PrintSectionHeader(heading, sep=False)
-    for group_id, group in sorted(
-        groups.iteritems(), key=lambda x: self.FlagGroupSortKey(x[1])):
-      if len(group) == 1 or any([getattr(f, 'inverted_synopsis', None)
-                                 for f in group]):
-        self.PrintFlagDefinition(group[0], disable_header=disable_header)
-      else:
-        if len(group) > 1:
-          attr = attrs.get(group_id)
-          if attr and attr.description:
-            self._out('\n' + attr.description + '\n')
-        for flag in sorted(group, key=lambda f: f.option_strings):
-          self.PrintFlagDefinition(flag, disable_header=disable_header)
+    self._PrintArgGroup(arg)
 
   def PrintPositionalsAndFlagsSections(self, disable_header=False):
     """Prints the positionals and flags sections.
@@ -565,18 +505,12 @@ class MarkdownGenerator(object):
     """
     if self._is_topic:
       return
-    self._SetFlagSections()
-    visible_positionals = self._FilterOutHidden(self.GetPositionalArgs())
-    if visible_positionals:
-      if not disable_header:
-        self.PrintSectionHeader('POSITIONAL ARGUMENTS', sep=False)
-      for arg in visible_positionals:
-        self.PrintPositionalDefinition(arg)
+    self._SetArgSections()
 
     # List the sections in order.
-    for heading, _, groups, attrs in self._flag_sections:
-      self.PrintFlagSection(heading, groups, attrs,
-                            disable_header=disable_header)
+    for section in self._arg_sections:
+      self.PrintFlagSection(
+          section.heading, section.args, disable_header=disable_header)
 
     if self._global_flags:
       if not disable_header:
@@ -669,7 +603,7 @@ class MarkdownGenerator(object):
     """
     # Determine if the section has any content.
     content = ''
-    for subcommand, help_info in sorted(subcommands.iteritems()):
+    for subcommand, help_info in sorted(six.iteritems(subcommands)):
       if self._is_hidden or not help_info.is_hidden:
         # If this group is already hidden, we can safely include hidden
         # sub-items.  Else, only include them if they are not hidden.
@@ -732,16 +666,21 @@ class MarkdownGenerator(object):
       doc = rep + doc[pos:]
     return doc
 
-  def _AddCommandLinkMarkdown(self, doc):
-    r"""Add ([`*])command ...\1 link markdown to doc."""
-    if not self._command_path:
-      return doc
-    # This pattern matches "([`*]){top} {arg}*\1" where {top}...{arg} is a
-    # known command. The negative lookbehind prefix prevents hyperlinks in
-    # SYNOPSIS sections and as the first line in a paragraph.
-    pat = re.compile(r'(?<!\n\n)(?<!\*\(ALPHA\)\* )(?<!\*\(BETA\)\* )'
-                     r'([`*])(?P<command>{top}( [a-z][-a-z0-9]*)*)\1'.format(
-                         top=self._top))
+  def _LinkMarkdown(self, doc, pat, with_args=True):
+    """Build a representation of a doc, finding all command examples.
+
+    Finds examples of both inline commands and commands on their own line.
+
+    Args:
+      doc: str, the doc to find examples in.
+      pat: the compiled regexp pattern to match against (the "command" match
+          group).
+      with_args: bool, whether the examples are valid if they also have
+          args.
+
+    Returns:
+      (str) The final representation of the doc.
+    """
     pos = 0
     rep = ''
     while True:
@@ -749,46 +688,71 @@ class MarkdownGenerator(object):
       if not match:
         break
       cmd, args = self._SplitCommandFromArgs(match.group('command').split(' '))
-      if args:
-        # Skip invalid commands.
-        rep += doc[pos:match.end(0)]
+      lnk = self.FormatExample(cmd, args, with_args=with_args)
+      if lnk:
+        rep += doc[pos:match.start('command')] + lnk
       else:
-        ref = '/'.join(cmd)
-        lnk = 'link:' + ref + '[' + ' '.join(cmd) + ']'
-        rep += (doc[pos:match.start('command')] + lnk +
-                doc[match.end('command'):match.end(0)])
-      pos = match.end(0)
+        # Skip invalid commands.
+        rep += doc[pos:match.end('command')]
+      rep += doc[match.end('command'):match.end('end')]
+      pos = match.end('end')
     if rep:
       doc = rep + doc[pos:]
     return doc
 
-  def _AddCommandLineLinkMarkdown(self, doc):
-    """Add $ command ... link markdown to doc."""
+  def InlineCommandExamplePattern(self):
+    """Regex to search for inline command examples enclosed in ` or *.
+
+    Contains a 'command' group and an 'end' group which will be used
+    by the regexp search later.
+
+    Returns:
+      (str) the regex pattern, including a format string for the 'top'
+      command.
+    """
+    # This pattern matches "([`*]){top} {arg}*\1" where {top}...{arg} is a
+    # known command. The negative lookbehind prefix prevents hyperlinks in
+    # SYNOPSIS sections and as the first line in a paragraph.
+    return (
+        r'(?<!\n\n)(?<!\*\(ALPHA\)\* )(?<!\*\(BETA\)\* )'
+        r'([`*])(?P<command>{top}( [a-z][-a-z0-9]*)*)(?P<end>\1)'
+        .format(top=re.escape(self._top)))
+
+  def _AddCommandLinkMarkdown(self, doc):
+    r"""Add ([`*])command ...\1 link markdown to doc."""
     if not self._command_path:
       return doc
+    pat = re.compile(self.InlineCommandExamplePattern())
+    doc = self._LinkMarkdown(doc, pat, with_args=False)
+    return doc
+
+  def CommandLineExamplePattern(self):
+    """Regex to search for command examples starting with '$ '.
+
+    Contains a 'command' group and an 'end' group which will be used
+    by the regexp search later.
+
+    Returns:
+      (str) the regex pattern, including a format string for the 'top'
+      command.
+    """
     # This pattern matches "$ {top} {arg}*" where each arg is lower case and
     # does not start with example-, my-, or sample-. This follows the style
     # guide rule that user-supplied args to example commands contain upper case
     # chars or start with example-, my-, or sample-. The trailing .? allows for
     # an optional punctuation character before end of line. This handles cases
     # like ``... run $ <top> foo bar.'' at the end of a sentence.
-    pat = re.compile(r'\$ (' + self._top +
-                     '((?: (?!(example|my|sample)-)[a-z][-a-z0-9]*)*)).?[ `\n]')
-    pos = 0
-    rep = ''
-    while True:
-      match = pat.search(doc, pos)
-      if not match:
-        break
-      cmd, args = self._SplitCommandFromArgs(match.group(1).split(' '))
-      ref = '/'.join(cmd)
-      lnk = 'link:' + ref + '[' + ' '.join(cmd) + ']'
-      if args:
-        lnk += ' ' + ' '.join(args)
-      rep += doc[pos:match.start(1)] + lnk
-      pos = match.end(1)
-    if rep:
-      doc = rep + doc[pos:]
+    # The <end> group ends at the same place as the command group, without
+    # the punctuation or newlines.
+    return (r'\$ (?P<end>(?P<command>{top}((?: (?!(example|my|sample)-)'
+            r'[a-z][-a-z0-9]*)*))).?[ `\n]'.format(top=re.escape(self._top)))
+
+  def _AddCommandLineLinkMarkdown(self, doc):
+    """Add $ command ... link markdown to doc."""
+    if not self._command_path:
+      return doc
+    pat = re.compile(self.CommandLineExamplePattern())
+    doc = self._LinkMarkdown(doc, pat, with_args=True)
     return doc
 
   def _AddManPageLinkMarkdown(self, doc):
@@ -819,14 +783,15 @@ class MarkdownGenerator(object):
     # are converted to unquoted strings with the _UserInput() font
     # embellishment. This is a subjective choice for aesthetically pleasing
     # renderings.
-    pat = re.compile(r"[^`](``([^`]*\w{2,}[^`']*)'')")
+    pat = re.compile(r"[^`](``([^`']*)'')")
     pos = 0
     rep = ''
-    while True:
-      match = pat.search(doc, pos)
-      if not match:
-        break
-      rep += doc[pos:match.start(1)] + self._UserInput(match.group(2))
+    for match in pat.finditer(doc):
+      if re.search(r'\w\w', match.group(2)):
+        quoted_string = self._UserInput(match.group(2))
+      else:
+        quoted_string = match.group(1)
+      rep += doc[pos:match.start(1)] + quoted_string
       pos = match.end(1)
     if rep:
       doc = rep + doc[pos:]
@@ -888,8 +853,6 @@ class CommandMarkdownGenerator(MarkdownGenerator):
     command.LoadAllSubElements()
     # pylint: disable=protected-access
     self._root_command = command._TopCLIElement()
-    self._subcommands = command.GetSubCommandHelps()
-    self._subgroups = command.GetSubGroupHelps()
     super(CommandMarkdownGenerator, self).__init__(
         command.GetPath(),
         command.ReleaseTrack(),
@@ -932,46 +895,13 @@ class CommandMarkdownGenerator(MarkdownGenerator):
         lines.append(line)
     self._SetSectionHelp(name, lines)
 
-  @staticmethod
-  def FlagGroupSortKey(flags):
-    """Returns a flag group sort key for sorted()."""
-    return [len(flags) > 1] + sorted([flag.option_strings for flag in flags])
-
-  @staticmethod
-  def IsHidden(arg):
-    """Returns True if arg is hidden."""
-    return arg.help == argparse.SUPPRESS
-
   def IsValidSubPath(self, sub_command_path):
     """Returns True if the given sub command path is valid from this node."""
     return self._root_command.IsValidSubPath(sub_command_path)
 
-  def GetPositionalArgs(self):
-    """Returns the command positional args."""
-    return self._command.ai.positional_args
-
-  def GetFlagGroups(self):
-    """Returns (group, group_attr, global_flags)."""
-    # Place all flag groups into a dict. Flags that are in a mutually
-    # exclusive group are mapped group_id -> [flags]. All other flags
-    # are mapped dest -> [flag].
-    global_flags = []
-    groups = {}
-    for flag in (self._command.ai.flag_args +
-                 self._command.ai.ancestor_flag_args):
-      if flag.is_global and not self._is_root:
-        if (flag.help != argparse.SUPPRESS and
-            flag.option_strings and
-            flag.option_strings[0].startswith('--')):
-          global_flags.append(flag.option_strings[0])
-      else:
-        group_id = self._command.ai.mutex_groups.get(
-            flag.dest,
-            self._command.ai.argument_groups.get(flag.dest, flag.dest))
-        if group_id not in groups:
-          groups[group_id] = []
-        groups[group_id].append(flag)
-    return groups, self._command.ai.group_attr, global_flags
+  def GetArguments(self):
+    """Returns the command arguments."""
+    return self._command.ai.arguments
 
   def GetNotes(self):
     """Returns the explicit and auto-generated NOTES section contents."""
