@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,30 +18,71 @@
 Refer to the calliope.parser_extensions module for a detailed overview.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import argparse
 
+from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import display_info
 from googlecloudsdk.calliope import parser_completer
 from googlecloudsdk.calliope import parser_errors
-from googlecloudsdk.command_lib.util import deprecated_completers
+from googlecloudsdk.command_lib.concepts import concept_managers
 from googlecloudsdk.core.cache import completion_cache
-from googlecloudsdk.core.resource import resource_property
 
 
-_MUTEX_GROUP_REQUIRED_DESCRIPTION = 'Exactly one of these must be specified:'
-_MUTEX_GROUP_OPTIONAL_DESCRIPTION = 'At most one of these may be specified:'
+# pylint: disable=protected-access
+def _IsStoreTrueAction(action):  # pylint: disable=invalid-name
+  return (action == 'store_true' or
+          isinstance(action, argparse._StoreTrueAction) or
+          (isinstance(action, type) and
+           issubclass(action, argparse._StoreTrueAction)))
 
 
-class ArgumentGroupAttr(object):
-  """Argument group attributes."""
-
-  def __init__(self, description=None, is_mutex=False, is_required=False):
-    self.description = description
-    self.is_mutex = is_mutex
-    self.is_required = is_required
+# pylint: disable=protected-access
+def _IsStoreFalseAction(action):  # pylint: disable=invalid-name
+  return (action == 'store_false' or
+          isinstance(action, argparse._StoreFalseAction) or
+          (isinstance(action, type) and
+           issubclass(action, argparse._StoreFalseAction)))
 
 
-class ArgumentInterceptor(object):
+def _IsStoreBoolAction(action):  # pylint: disable=invalid-name
+  return _IsStoreTrueAction(action) or _IsStoreFalseAction(action)
+
+
+class Argument(object):
+  """Parsed argument base class with help generation attributess.
+
+  Attributes:
+    arguments: [ArgumentInterceptor], The group argument list if is_group is
+      true.
+    category: str, The argument help category name.
+    help: str, The argument help text.
+    is_global: bool, The argument is global to all commands.
+    is_group: bool, The argument is a group with arguments in self.arguments.
+    is_mutex: bool, This is a mutex argument group; at most one argument in
+      arguments may be specified.
+    is_required: bool, The argument is required.
+  """
+
+  # pylint: disable=redefined-builtin, Python can't keep help to itself
+  def __init__(self, arguments=None, hidden=False, is_group=False,
+               is_global=False, mutex=False, required=False,
+               help=None, category=None):
+    self.arguments = arguments or []
+    self.is_group = is_group or arguments
+    self.is_global = is_global
+    self.is_hidden = hidden
+    self.is_mutex = mutex
+    self.is_positional = False
+    self.is_required = required
+    self.help = help
+    self.category = category
+
+
+class ArgumentInterceptor(Argument):
   """ArgumentInterceptor intercepts calls to argparse parsers.
 
   The argparse module provides no public way to access the arguments that were
@@ -68,47 +110,47 @@ class ArgumentInterceptor(object):
       allow_positional: bool, Allow positional arguments if True.
       ancestor_flag_args: [argparse.Action], The flags for all ancestor groups
         in the cli tree.
-      argument_groups: {dest: group-id}, Maps dests to argument group ids.
       cli_generator: cli.CLILoader, The builder used to generate this CLI.
-      command_name: str, The dotted command name path.
+      command_name: [str], The parts of the command name path.
+      concept_handler: calliope.concepts.handlers.RuntimeHandler, a handler
+        for concept args.
       defaults: {dest: default}, For all registered arguments.
       dests: [str], A list of the dests for all arguments.
       display_info: [display_info.DisplayInfo], The command display info object.
       flag_args: [ArgumentInterceptor], The flag arguments.
-      group_attr: {dest: ArgumentGroupAttr}, Maps dests to ArgumentGroupAttr.
-      groups: [ArgumentInterceptor], The arg groups.
-      mutex_groups: {dest: mutex_group_id}, Maps dests to mutex group ids.
       positional_args: [ArgumentInterceptor], The positional args.
       positional_completers: {Completer}, The set of completers for positionals.
       required: [str], The dests for all required arguments.
-      required_mutex_groups: set(id), Set of mutex group ids that are required.
     """
 
-    def __init__(self, command_name, is_root, cli_generator, allow_positional):
+    def __init__(self, command_name, cli_generator, allow_positional):
       self.command_name = command_name
-      self.is_root = is_root
       self.cli_generator = cli_generator
       self.allow_positional = allow_positional
-      self.is_root = is_root
 
       self.ancestor_flag_args = []
-      self.argument_groups = {}
+      self.concept_handler = None
+      # Concepts v2
+      self.concepts = None
       self.defaults = {}
       self.dests = []
       self.display_info = display_info.DisplayInfo()
       self.flag_args = []
-      self.group_attr = {}
-      self.groups = {}
-      self.mutex_groups = {}
       self.positional_args = []
       self.positional_completers = set()
       self.required = []
-      self.required_mutex_groups = set()
 
-  def __init__(self, parser, is_root=False, cli_generator=None,
-               allow_positional=True, data=None, mutex_group_id=None,
-               argument_group_id=None):
+  def __init__(self, parser, cli_generator=None, allow_positional=True,
+               data=None, **kwargs):
+    super(ArgumentInterceptor, self).__init__(is_group=True, **kwargs)
+    self.is_mutex = kwargs.pop('mutex', False)
+    self.help = kwargs.pop('help', None)
     self.parser = parser
+    if parser:
+      # validate_specified_args() needs access to this argument interceptor,
+      # but it is called from parse_args() by argparse which passes the argparse
+      # internal "parser". We add parser.ai here to make it available.
+      parser.ai = self
     # If this is an argument group within a command, use the data from the
     # parser for the entire command.  If it is the command itself, create a new
     # data object and extract the command name from the parser.
@@ -118,11 +160,8 @@ class ArgumentInterceptor(object):
       self.data = ArgumentInterceptor.ParserData(
           # pylint: disable=protected-access
           command_name=self.parser._calliope_command.GetPath(),
-          is_root=is_root,
           cli_generator=cli_generator,
           allow_positional=allow_positional)
-    self.mutex_group_id = mutex_group_id
-    self.argument_group_id = argument_group_id
 
   @property
   def allow_positional(self):
@@ -132,6 +171,10 @@ class ArgumentInterceptor(object):
   def cli_generator(self):
     return self.data.cli_generator
 
+  @property
+  def command_name(self):
+    return self.data.command_name
+
   @property
   def defaults(self):
     return self.data.defaults
@@ -144,30 +187,10 @@ class ArgumentInterceptor(object):
   def required(self):
     return self.data.required
 
-  @property
-  def group_attr(self):
-    return self.data.group_attr
-
-  @property
-  def is_root(self):
-    return self.data.is_root
-
   @property
   def dests(self):
     return self.data.dests
 
-  @property
-  def argument_groups(self):
-    return self.data.argument_groups
-
-  @property
-  def mutex_groups(self):
-    return self.data.mutex_groups
-
-  @property
-  def required_mutex_groups(self):
-    return self.data.required_mutex_groups
-
   @property
   def positional_args(self):
     return self.data.positional_args
@@ -184,51 +207,72 @@ class ArgumentInterceptor(object):
   def ancestor_flag_args(self):
     return self.data.ancestor_flag_args
 
+  @property
+  def concept_handler(self):
+    return self.data.concept_handler
+
+  @property
+  def concepts(self):
+    return self.data.concepts
+
+  def add_concepts(self, handler):
+    # RuntimeParser is the v2 concepts handler.
+    if isinstance(handler, concept_managers.RuntimeParser):
+      self.data.concepts = handler
+      return
+    if self.data.concept_handler:
+      raise AttributeError(
+          'It is not permitted to add two runtime handlers to a command class.')
+    self.data.concept_handler = handler
+
   # pylint: disable=g-bad-name
   def add_argument(self, *args, **kwargs):
     """add_argument intercepts calls to the parser to track arguments."""
-    # TODO(b/36050238): do not allow short-options without long-options.
-
-    # we will choose the first option as the name
     name = args[0]
+
+    # The flag category name, None for no category. This is also used for help
+    # printing. Flags in the same category are grouped together in a section
+    # named "{category} FLAGS".
+    category = kwargs.pop('category', None)
+    # The unbound completer object or raw argcomplete completer function).
+    completer = kwargs.pop('completer', None)
+    # The default value.
+    default = kwargs.get('default')
+    # The namespace destination attribute name.
     dest = kwargs.get('dest')
     if not dest:
-      # this is exactly what happens in argparse
       dest = name.lstrip(self.parser.prefix_chars).replace('-', '_')
-
-    default = kwargs.get('default')
-    required = kwargs.get('required', False)
-
     # A flag that can only be supplied where it is defined and not propagated to
     # subcommands.
     do_not_propagate = kwargs.pop('do_not_propagate', False)
+    # hidden=True retains help but does not display it.
+    hidden = kwargs.pop('hidden', False)
+    help_text = kwargs.get('help')
+    if not help_text:
+      raise ValueError('Argument {} requires help text [hidden={}]'.format(
+          name, hidden))
+    if help_text == argparse.SUPPRESS:
+      raise ValueError('Argument {} needs hidden=True instead of '
+                       'help=argparse.SUPPRESS.'.format(name))
     # A global flag that is added at each level explicitly because each command
     # has a different behavior (like -h).
     is_replicated = kwargs.pop('is_replicated', False)
     # This is used for help printing.  A flag is considered global if it is
     # added at the root of the CLI tree, or if it is explicitly added to every
     # command level.
-    is_global = self.is_root or is_replicated
-    # The flag category name, None for no category. This is also used for help
-    # printing. Flags in the same category are grouped together in a section
-    # named "{category} FLAGS".
-    category = kwargs.pop('category', None)
+    is_global = self.is_global or is_replicated
+    # The number positional args.
+    nargs = kwargs.get('nargs')
+    # The argument is required if True.
+    required = kwargs.get('required', False)
     # Any alias this flag has for the purposes of the "did you mean"
     # suggestions.
-    suggestion_aliases = kwargs.pop('suggestion_aliases', [])
-    # The unbound completer object or raw argcomplete completer function).
-    completer = kwargs.pop('completer', None)
-    # The resource name for the purposes of doing remote completion.
-    completion_resource = kwargs.pop('completion_resource', None)
-    # An explicit command to run for remote completion instead of the default
-    # for this resource type.
-    list_command_path = kwargs.pop('list_command_path', None)
-    # Callback function that receives the currently entered args at the time of
-    # remote completion processing, and returns the command to run.
-    list_command_callback_fn = kwargs.pop('list_command_callback_fn', None)
-    # hidden=True => help=argparse.SUPPRESS, but retains help in the source.
-    if kwargs.pop('hidden', False):
-      kwargs['help'] = argparse.SUPPRESS
+    suggestion_aliases = kwargs.pop('suggestion_aliases', None)
+    if suggestion_aliases is None:
+      suggestion_aliases = []
+
+    if self.is_global and category == base.COMMONLY_USED_FLAGS:
+      category = 'GLOBAL'
 
     positional = not name.startswith('-')
     if positional:
@@ -237,49 +281,21 @@ class ArgumentInterceptor(object):
         # group the problem is in.
         raise parser_errors.ArgumentException(
             'Illegal positional argument [{0}] for command [{1}]'.format(
-                name, self.data.command_name))
+                name, '.'.join(self.data.command_name)))
       if '-' in name:
         raise parser_errors.ArgumentException(
             "Positional arguments cannot contain a '-'. Illegal argument [{0}] "
-            'for command [{1}]'.format(name, self.data.command_name))
+            'for command [{1}]'.format(name, '.'.join(self.data.command_name)))
       if category:
         raise parser_errors.ArgumentException(
             'Positional argument [{0}] cannot have a category in '
-            'command [{1}]'.format(name, self.data.command_name))
+            'command [{1}]'.format(name, '.'.join(self.data.command_name)))
       if suggestion_aliases:
         raise parser_errors.ArgumentException(
             'Positional argument [{0}] cannot have suggestion aliases in '
-            'command [{1}]'.format(name, self.data.command_name))
+            'command [{1}]'.format(name, '.'.join(self.data.command_name)))
 
     self.defaults[dest] = default
-    if self.mutex_group_id:
-      self.mutex_groups[dest] = self.mutex_group_id
-      if self.parser.required:
-        self.required_mutex_groups.add(self.mutex_group_id)
-        self.group_attr[self.mutex_group_id] = ArgumentGroupAttr(
-            description=_MUTEX_GROUP_REQUIRED_DESCRIPTION,
-            is_mutex=True,
-            is_required=True,
-        )
-      else:
-        self.group_attr[self.mutex_group_id] = ArgumentGroupAttr(
-            description=_MUTEX_GROUP_OPTIONAL_DESCRIPTION,
-            is_mutex=True,
-            is_required=False,
-        )
-    elif self.argument_group_id:
-      self.argument_groups[dest] = self.argument_group_id
-      if self.parser.description:
-        description = self.parser.description
-      elif self.parser.title:
-        description = self.parser.title.rstrip('.') + ':'
-      else:
-        description = None
-      self.group_attr[self.argument_group_id] = ArgumentGroupAttr(
-          description=description,
-          is_mutex=False,
-          is_required=False,
-      )
     if required:
       self.required.append(dest)
     self.dests.append(dest)
@@ -290,34 +306,38 @@ class ArgumentInterceptor(object):
       added_argument = self.parser.AddRemainderArgument(*args, **kwargs)
     else:
       added_argument = self.parser.add_argument(*args, **kwargs)
-    self._AttachCompleter(
-        added_argument,
-        name,
-        completer,
-        positional,
-        deprecated_collection=completion_resource,
-        deprecated_list_command=list_command_path,
-        deprecated_list_command_callback=list_command_callback_fn)
+    self._AttachCompleter(added_argument, completer, positional)
+    added_argument.is_global = is_global
+    added_argument.is_group = False
+    added_argument.is_hidden = hidden
+    added_argument.is_required = required
+    added_argument.is_positional = positional
+    if hidden:
+      # argparse uses SUPPRESS -- cli_tree uses hidden_help to work around
+      added_argument.hidden_help = added_argument.help
+      added_argument.help = argparse.SUPPRESS
     if positional:
       if category:
         raise parser_errors.ArgumentException(
             'Positional argument [{0}] cannot have a category in '
-            'command [{1}]'.format(name, self.data.command_name))
+            'command [{1}]'.format(name, '.'.join(self.data.command_name)))
+      if (nargs is None or
+          nargs == '+' or
+          isinstance(nargs, int) and nargs > 0):
+        added_argument.is_required = True
       self.positional_args.append(added_argument)
     else:
       if category and required:
         raise parser_errors.ArgumentException(
             'Required flag [{0}] cannot have a category in '
-            'command [{1}]'.format(name, self.data.command_name))
+            'command [{1}]'.format(name, '.'.join(self.data.command_name)))
       if category == 'REQUIRED':
         raise parser_errors.ArgumentException(
             "Flag [{0}] cannot have category='REQUIRED' in "
-            'command [{1}]'.format(name, self.data.command_name))
+            'command [{1}]'.format(name, '.'.join(self.data.command_name)))
       added_argument.category = category
       added_argument.do_not_propagate = do_not_propagate
       added_argument.is_replicated = is_replicated
-      added_argument.is_global = is_global
-      added_argument.required = required
       added_argument.suggestion_aliases = suggestion_aliases
       if isinstance(added_argument.choices, dict):
         # choices is a name: description dict. Set the choices attribute to the
@@ -338,6 +358,9 @@ class ArgumentInterceptor(object):
         # to one or the other.
         self.flag_args.append(inverted_flag)
 
+    if (not getattr(added_argument, 'is_replicated', False) or
+        len(self.command_name) == 1):
+      self.arguments.append(added_argument)
     return added_argument
 
   # pylint: disable=redefined-builtin
@@ -354,17 +377,45 @@ class ArgumentInterceptor(object):
     """Hooks ArgumentInterceptor into the argcomplete monkeypatch."""
     return self.parser.parse_known_args(args=args, namespace=namespace)
 
-  def add_argument_group(self, *args, **kwargs):
-    new_parser = self.parser.add_argument_group(*args, **kwargs)
-    return ArgumentInterceptor(parser=new_parser,
-                               data=self.data,
-                               argument_group_id=id(new_parser))
+  def add_group(self, help=None, category=None, mutex=False, required=False,
+                hidden=False, **kwargs):
+    """Adds an argument group with mutex/required attributes to the parser.
 
-  def add_mutually_exclusive_group(self, **kwargs):
-    new_parser = self.parser.add_mutually_exclusive_group(**kwargs)
-    return ArgumentInterceptor(parser=new_parser,
-                               data=self.data,
-                               mutex_group_id=id(new_parser))
+    Args:
+      help: str, The group help text description.
+      category: str, The group flag category name, None for no category.
+      mutex: bool, A mutually exclusive group if True.
+      required: bool, A required group if True.
+      hidden: bool, A hidden group if True.
+      **kwargs: Passed verbatim to ArgumentInterceptor().
+
+    Returns:
+      The added argument object.
+    """
+    if 'description' in kwargs or 'title' in kwargs:
+      raise parser_errors.ArgumentException(
+          'parser.add_group(): description or title kwargs not supported '
+          '-- use help=... instead.')
+    new_parser = self.parser.add_argument_group()
+    group = ArgumentInterceptor(parser=new_parser,
+                                is_global=self.is_global,
+                                cli_generator=self.cli_generator,
+                                allow_positional=self.allow_positional,
+                                data=self.data,
+                                help=help,
+                                category=category,
+                                mutex=mutex,
+                                required=required,
+                                hidden=hidden,
+                                **kwargs)
+    self.arguments.append(group)
+    return group
+
+  def add_argument_group(self, help=None, **kwargs):
+    return self.add_group(help=help, **kwargs)
+
+  def add_mutually_exclusive_group(self, help=None, **kwargs):
+    return self.add_group(help=help, mutex=True, **kwargs)
 
   def AddDynamicPositional(self, name, action, **kwargs):
     """Add a positional argument that adds new args on the fly when called.
@@ -386,7 +437,12 @@ class ArgumentInterceptor(object):
 
     action = self.parser.add_subparsers(action=action, **kwargs)
     action.completer = action.Completions
+    action.is_group = False
+    action.is_hidden = kwargs.get('hidden', False)
+    action.is_positional = True
+    action.is_required = True
     self.positional_args.append(action)
+    self.arguments.append(action)
     return action
 
   def AddFlagActionFromAncestors(self, action):
@@ -397,7 +453,6 @@ class ArgumentInterceptor(object):
 
     Args:
       action: argparse.Action, The action for the flag being added.
-
     """
     # pylint:disable=protected-access, simply no other way to do this.
     self.parser._add_action(action)
@@ -420,6 +475,14 @@ class ArgumentInterceptor(object):
       necessary to create a new argument.
     """
     action = original_kwargs.get('action')
+    wrapped_action = getattr(action, 'wrapped_action', None)
+    if wrapped_action is not None:
+      # If action is a wrapper, then we save the wrapper to subclass below and
+      # set action to the wrapped action so function can correctly interpret
+      # the what the intended action is.
+      action_wrapper = action
+      action = wrapped_action
+
     # There are a few legitimate explicit --no-foo flags.
     should_invert, prop = self._ShouldInvertBooleanFlag(name, action)
     if not should_invert:
@@ -439,20 +502,37 @@ class ArgumentInterceptor(object):
       inverted_synopsis = False
 
     kwargs = dict(original_kwargs)
-    if action == 'store_true':
+    if _IsStoreTrueAction(action):
       action = 'store_false'
-    elif action == 'store_false':
+    elif _IsStoreFalseAction(action):
       action = 'store_true'
+
+    # This is a hacky workaround to get actions.DeprecationAction to properly
+    # generate an inverted boolean flag. The Action returned from
+    # actions.DeprecationAction will have a wrapped_action attribute which it
+    # uses at initialization to create an argparse Action. We check if this
+    # attribute exists, and if it does we create a new Action that subclasses
+    # from the action wrapper and use SetWrappedAction to change wrapped_action.
+    if wrapped_action is not None:
+
+      class NewAction(action_wrapper):
+        pass
+
+      NewAction.SetWrappedAction(action)
+      action = NewAction
+
     kwargs['action'] = action
     if not kwargs.get('dest'):
       kwargs['dest'] = dest
-    kwargs['help'] = argparse.SUPPRESS
 
     inverted_argument = self.parser.add_argument(
         name.replace('--', '--no-', 1), **kwargs)
+    inverted_argument.hidden = True
     if inverted_synopsis:
       # flag.inverted_synopsis means display the inverted flag in the SYNOPSIS.
       setattr(added_argument, 'inverted_synopsis', True)
+    inverted_argument.is_hidden = True
+    inverted_argument.is_required = added_argument.is_required
     return inverted_argument
 
   def _ShouldInvertBooleanFlag(self, name, action):
@@ -476,7 +556,7 @@ class ArgumentInterceptor(object):
     if '--no-' + name[2:] in self.parser._option_string_actions:  # pylint: disable=protected-access
       # Don't override explicit --no-* inverted flag.
       return False, None
-    if action in ('store_true', 'store_false'):
+    if _IsStoreBoolAction(action):
       return True, None
     prop, kind, _ = getattr(action, 'store_property', (None, None, None))
     if prop:
@@ -485,57 +565,16 @@ class ArgumentInterceptor(object):
     return False, None
 
   # TODO(b/38374705): Drop the deprecated code.
-  def _AttachCompleter(self, arg, name, completer, positional,
-                       deprecated_collection=None,
-                       deprecated_list_command=None,
-                       deprecated_list_command_callback=None):
+  def _AttachCompleter(self, arg, completer, positional):
     """Attaches a completer to arg if one is specified.
 
     Args:
       arg: The argument to attach the completer to.
-      name: The arg name for messaging.
       completer: The completer Completer class or argcomplete function object.
       positional: True if argument is a positional.
-      deprecated_collection: The collection name for the resource to complete.
-      deprecated_list_command: The command whose Run() method returns the
-        current resource list.
-      deprecated_list_command_callback: A callback function that returns the
-        list command to run.
     """
     if not completer:
-      if not deprecated_collection:
-        if deprecated_list_command or deprecated_list_command_callback:
-          raise parser_errors.ArgumentException(
-              'Command [{}] argument [{}] does not have completion_resource '
-              'set but has one or more of the deprecated list_command_path '
-              'and list_command_callback_fn attributes.'.format(
-                  ' '.join(self.data.command_name), name))
-        return
-
-      class DeprecatedCompleter(
-          deprecated_completers.DeprecatedListCommandCompleter):
-
-        def __init__(self, **kwargs):
-          super(DeprecatedCompleter, self).__init__(
-              collection=deprecated_collection,
-              list_command=deprecated_list_command,
-              list_command_callback=deprecated_list_command_callback,
-              **kwargs)
-
-      DeprecatedCompleter.__name__ = (
-          resource_property.ConvertToCamelCase(
-              deprecated_collection.capitalize().replace('.', '_')) +
-          'DeprecatedCompleter')
-
-      completer = DeprecatedCompleter
-    elif (deprecated_collection or
-          deprecated_list_command or
-          deprecated_list_command_callback):
-      raise parser_errors.ArgumentException(
-          'Command [{}] argument [{}] has a completer set with one or more '
-          'of the deprecated completion_resource, list_command_path, and '
-          'list_command_callback_fn attributes.'.format(
-              ' '.join(self.data.command_name), name))
+      return
     if isinstance(completer, type):
       # A completer class that will be instantiated at completion time.
       if positional and issubclass(completer, completion_cache.Completer):
