@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +15,11 @@
 
 """Utilities for app creation."""
 
-from googlecloudsdk.api_lib.app import exceptions as api_lib_exceptions
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from apitools.base.py import exceptions as apitools_exceptions
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_io
@@ -23,7 +28,7 @@ from googlecloudsdk.core.console import console_io
 APP_CREATE_WARNING = """\
 Creating an App Engine application for a project is irreversible and the region
 cannot be changed. More information about regions is at
-https://cloud.google.com/appengine/docs/locations.
+<https://cloud.google.com/appengine/docs/locations>.
 """
 
 
@@ -47,7 +52,7 @@ def CheckAppNotExists(api_client, project):
   """
   try:
     app = api_client.GetApplication()  # Should raise NotFoundError
-  except api_lib_exceptions.NotFoundError:
+  except apitools_exceptions.HttpNotFoundError:
     pass
   else:
     region = ' in region [{}]'.format(app.locationId) if app.locationId else ''
@@ -75,17 +80,17 @@ def CreateApp(api_client, project, region, suppress_warning=False):
   if not suppress_warning:
     log.status.Print('You are creating an app for project [{project}].'.format(
         project=project))
-    log.warn(APP_CREATE_WARNING)
+    log.warning(APP_CREATE_WARNING)
   try:
     api_client.CreateApp(region)
-  except api_lib_exceptions.ConflictError:
+  except apitools_exceptions.HttpConflictError:
     raise AppAlreadyExistsError(
         'The project [{project}] already contains an App Engine application. '
         'You can deploy your application using `gcloud app deploy`.'.format(
             project=project))
 
 
-def CreateAppInteractively(api_client, project):
+def CreateAppInteractively(api_client, project, regions=None, extra_warning=''):
   """Interactively choose a region and create an App Engine app.
 
   The caller is responsible for calling this method only when the user can be
@@ -93,7 +98,9 @@ def CreateAppInteractively(api_client, project):
 
   Example interaction:
 
-      Which region?
+      Please choose the region where you want your App Engine application
+      located:
+
         [1] us-east1      (supports standard and flexible)
         [2] europe-west   (supports standard)
         [3] us-central    (supports standard and flexible)
@@ -103,19 +110,24 @@ def CreateAppInteractively(api_client, project):
   Args:
     api_client: The App Engine Admin API client
     project: The GCP project
+    regions: The list of regions to choose from; if None, all possible regions
+             are listed
+    extra_warning: An additional warning to print before listing regions.
 
   Raises:
     AppAlreadyExistsError if app already exists
   """
   log.status.Print('You are creating an app for project [{}].'.format(project))
-  log.warn(APP_CREATE_WARNING)
+  log.warning(APP_CREATE_WARNING)
 
-  all_regions = sorted(set(api_client.ListRegions()))
+  regions = regions or sorted(set(api_client.ListRegions()), key=str)
+  if extra_warning:
+    log.warning(extra_warning)
   idx = console_io.PromptChoice(
-      all_regions,
+      regions,
       message=('Please choose the region where you want your App Engine '
                'application located:\n\n'),
       cancel_option=True)
-  region = all_regions[idx]
+  region = regions[idx]
   CreateApp(api_client, project, region.region, suppress_warning=True)
 
