@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,11 +14,26 @@
 # limitations under the License.
 """Flags and helpers for the compute accelerator-types commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
+
+class AcceleratorTypesCompleter(completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(AcceleratorTypesCompleter, self).__init__(
+        collection='compute.acceleratorTypes',
+        list_command='compute accelerator-types list --uri',
+        **kwargs)
+
+
 ACCELERATOR_TYPES_ARG = compute_flags.ResourceArgument(
     resource_name='accelerator type',
-    completion_resource_id='compute.acceleratorTypes',
+    completer=AcceleratorTypesCompleter,
     plural=False,
     required=True,
     zonal_collection='compute.acceleratorTypes')
