@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,23 +15,33 @@
 
 """Flags and helpers for the compute related commands."""
 
-import argparse
-import functools
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
+import functools
+import enum  # pylint: disable=unused-import, for pytype
+
+from googlecloudsdk.api_lib.compute import filter_rewrite
 from googlecloudsdk.api_lib.compute.regions import service as regions_service
 from googlecloudsdk.api_lib.compute.zones import service as zones_service
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute import scope_prompter
 from googlecloudsdk.core import exceptions
+from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
+from googlecloudsdk.core.resource import resource_projection_spec
 from googlecloudsdk.core.util import text
+import six
 
 ZONE_PROPERTY_EXPLANATION = """\
-If not specified, you may be prompted to select a zone.
+If not specified and the ``compute/zone'' property isn't set, you
+may be prompted to select a zone.
 
 To avoid prompting when this flag is omitted, you can set the
 ``compute/zone'' property:
@@ -58,7 +69,7 @@ A list of zones can be fetched by running:
 """
 
 REGION_PROPERTY_EXPLANATION = """\
-If not specified, you will be prompted to select a region.
+If not specified, you may be prompted to select a region.
 
 To avoid prompting when this flag is omitted, you can set the
 ``compute/region'' property:
@@ -78,7 +89,7 @@ variable ``CLOUDSDK_COMPUTE_REGION''.
 """
 
 REGION_PROPERTY_EXPLANATION_NO_DEFAULT = """\
-If not specified, you will be prompted to select a region.
+If not specified, you may be prompted to select a region.
 
 A list of regions can be fetched by running:
 
@@ -114,7 +125,7 @@ def AddZoneFlag(parser, resource_type, operation_type, flag_prefix=None,
                         resource types, otherwise resource_types will be
                         pluralized by appending 's'.
   """
-  short_help = 'The zone of the {0} to {1}.'.format(
+  short_help = 'Zone of the {0} to {1}.'.format(
       text.Pluralize(
           int(plural) + 1, resource_type or '', custom_plural), operation_type)
   flag_name = 'zone'
@@ -123,7 +134,7 @@ def AddZoneFlag(parser, resource_type, operation_type, flag_prefix=None,
   parser.add_argument(
       '--' + flag_name,
       hidden=hidden,
-      completion_resource='compute.zones',
+      completer=completers.ZonesCompleter,
       action=actions.StoreProperty(properties.VALUES.compute.zone),
       help='{0} {1}'.format(short_help, explanation))
 
@@ -148,7 +159,7 @@ def AddRegionFlag(parser, resource_type, operation_type,
                         resource types, otherwise resource_types will be
                         pluralized by appending 's'.
   """
-  short_help = 'The region of the {0} to {1}.'.format(
+  short_help = 'Region of the {0} to {1}.'.format(
       text.Pluralize(
           int(plural) + 1, resource_type or '', custom_plural), operation_type)
   flag_name = 'region'
@@ -156,7 +167,7 @@ def AddRegionFlag(parser, resource_type, operation_type,
     flag_name = flag_prefix + '-' + flag_name
   parser.add_argument(
       '--' + flag_name,
-      completion_resource='compute.regions',
+      completer=completers.RegionsCompleter,
       action=actions.StoreProperty(properties.VALUES.compute.region),
       hidden=hidden,
       help='{0} {1}'.format(short_help, explanation))
@@ -226,11 +237,12 @@ class ResourceArgScopes(object):
     self.scopes = {}
 
   def AddScope(self, scope, collection):
+    # type: (enum.Enum, str) -> None
     self.scopes[scope] = ResourceArgScope(scope, self.flag_prefix, collection)
 
   def SpecifiedByArgs(self, args):
     """Given argparse args return selected scope and its value."""
-    for resource_scope in self.scopes.itervalues():
+    for resource_scope in six.itervalues(self.scopes):
       scope_value = getattr(args, resource_scope.flag_name, None)
       if scope_value is not None:
         return resource_scope, scope_value
@@ -239,11 +251,11 @@ class ResourceArgScopes(object):
   def GetImplicitScope(self, default_scope=None):
     """See if there is no ambiguity even if scope is not known from args."""
     if len(self.scopes) == 1:
-      return next(self.scopes.itervalues())
+      return next(six.itervalues(self.scopes))
     return default_scope
 
   def __iter__(self):
-    return iter(self.scopes.itervalues())
+    return iter(six.itervalues(self.scopes))
 
   def __contains__(self, scope):
     return scope in self.scopes
@@ -320,10 +332,114 @@ class ResourceResolver(object):
       New instance of ResourceResolver.
     """
     scopes = ResourceArgScopes(flag_prefix=scope_flag_prefix)
-    for scope, resource in scopes_map.iteritems():
+    for scope, resource in six.iteritems(scopes_map):
       scopes.AddScope(scope, resource)
     return ResourceResolver(scopes, resource_name)
 
+  def _ValidateNames(self, names):
+    if not isinstance(names, list):
+      raise BadArgumentException(
+          "Expected names to be a list but it is '{0}'".format(names))
+
+  def _ValidateDefaultScope(self, default_scope):
+    if default_scope is not None and default_scope not in self.scopes:
+      raise BadArgumentException(
+          'Unexpected value for default_scope {0}, expected None or {1}'
+          .format(default_scope,
+                  ' or '.join([s.scope_enum.name for s in self.scopes])))
+
+  def _GetResourceScopeParam(
+      self, resource_scope, scope_value, project, api_resource_registry):
+    if scope_value is not None:
+      if resource_scope.scope_enum == compute_scope.ScopeEnum.GLOBAL:
+        return None
+      else:
+        collection = compute_scope.ScopeEnum.CollectionForScope(
+            resource_scope.scope_enum)
+        return api_resource_registry.Parse(
+            scope_value,
+            params={'project': project},
+            collection=collection).Name()
+    else:
+      if resource_scope and (
+          resource_scope.scope_enum != compute_scope.ScopeEnum.GLOBAL):
+        return resource_scope.scope_enum.property_func
+
+  def _GetRefsAndUnderspecifiedNames(
+      self, names, params, collection, scope_defined, api_resource_registry):
+    """Returns pair of lists: resolved references and unresolved names.
+
+    Args:
+      names: list of names to attempt resolving
+      params: params given when attempting to resolve references
+      collection: collection for the names
+      scope_defined: bool, whether scope is known
+      api_resource_registry: Registry object
+    """
+    refs = []
+    underspecified_names = []
+    for name in names:
+      try:
+        # Make each element an array so that we can do in place updates.
+        ref = [api_resource_registry.Parse(name, params=params,
+                                           collection=collection,
+                                           enforce_collection=False)]
+      except (resources.UnknownCollectionException,
+              resources.RequiredFieldOmittedException,
+              properties.RequiredPropertyError):
+        if scope_defined:
+          raise
+        ref = [name]
+        underspecified_names.append(ref)
+      refs.append(ref)
+    return refs, underspecified_names
+
+  def _ResolveUnderspecifiedNames(
+      self, underspecified_names, default_scope, scope_lister, project,
+      api_resource_registry):
+    """Attempt to resolve scope for unresolved names.
+
+    If unresolved_names was generated with _GetRefsAndUnderspecifiedNames
+    changing them will change corresponding elements of refs list.
+
+    Args:
+      underspecified_names: list of one-items lists containing str
+      default_scope: default scope for the resources
+      scope_lister: callback used to list potential scopes for the resources
+      project: str, id of the project
+      api_resource_registry: resources Registry
+
+    Raises:
+      UnderSpecifiedResourceError: when resource scope can't be resolved.
+    """
+    if not underspecified_names:
+      return
+
+    names = [n[0] for n in underspecified_names]
+
+    if not console_io.CanPrompt():
+      raise UnderSpecifiedResourceError(names, [s.flag for s in self.scopes])
+
+    resource_scope_enum, scope_value = scope_prompter.PromptForScope(
+        self.resource_name, names, [s.scope_enum for s in self.scopes],
+        default_scope.scope_enum if default_scope is not None else None,
+        scope_lister)
+    if resource_scope_enum is None:
+      raise UnderSpecifiedResourceError(names, [s.flag for s in self.scopes])
+
+    resource_scope = self.scopes[resource_scope_enum]
+    params = {'project': project,}
+
+    if resource_scope.scope_enum != compute_scope.ScopeEnum.GLOBAL:
+      params[resource_scope.scope_enum.param_name] = scope_value
+
+    for name in underspecified_names:
+      name[0] = api_resource_registry.Parse(
+          name[0],
+          params=params,
+          collection=resource_scope.collection,
+          enforce_collection=True)
+
   def ResolveResources(self,
                        names,
                        resource_scope,
@@ -362,85 +478,39 @@ class ResourceResolver(object):
       UnderSpecifiedResourceError: if it was not possible to resolve given names
           as resources references.
     """
-    if not isinstance(names, list):
-      raise BadArgumentException(
-          'Expected names to be a list but it is {0!r}'.format(names))
+    self._ValidateNames(names)
+    self._ValidateDefaultScope(default_scope)
     if resource_scope is not None:
       resource_scope = self.scopes[resource_scope]
     if default_scope is not None:
-      if default_scope not in self.scopes:
-        raise BadArgumentException(
-            'Unexpected value for default_scope {0}, expected None or {1}'
-            .format(default_scope,
-                    ' or '.join([s.scope_enum.name for s in self.scopes])))
       default_scope = self.scopes[default_scope]
+    project = properties.VALUES.core.project.GetOrFail
     params = {
-        'project': properties.VALUES.core.project.GetOrFail,
+        'project': project,
     }
-    if scope_value is not None:
-      if resource_scope.scope_enum == compute_scope.ScopeEnum.GLOBAL:
-        stored_value = scope_value
-      else:
-        collection = compute_scope.ScopeEnum.CollectionForScope(
-            resource_scope.scope_enum)
-        stored_value = api_resource_registry.Parse(
-            scope_value,
-            params=params,
-            collection=collection).Name()
-        params[resource_scope.scope_enum.param_name] = stored_value
-    else:
+    if scope_value is None:
       resource_scope = self.scopes.GetImplicitScope(default_scope)
-      if resource_scope and (
-          resource_scope.scope_enum != compute_scope.ScopeEnum.GLOBAL):
-        params[resource_scope.scope_enum.param_name] = (
-            resource_scope.scope_enum.property_func)
+
+    resource_scope_param = self._GetResourceScopeParam(
+        resource_scope, scope_value, project, api_resource_registry)
+    if resource_scope_param is not None:
+      params[resource_scope.scope_enum.param_name] = resource_scope_param
 
     collection = resource_scope and resource_scope.collection
 
     # See if we can resolve names with so far deduced scope and its value.
-    refs = []
-    underspecified_names = []
-    for name in names:
-      try:
-        # Make each element an array so that we can do in place updates.
-        ref = [api_resource_registry.Parse(name, params=params,
-                                           collection=collection,
-                                           enforce_collection=False)]
-      except (resources.UnknownCollectionException,
-              resources.RequiredFieldOmittedException,
-              properties.RequiredPropertyError):
-        if scope_value:
-          raise
-        ref = [name]
-        underspecified_names.append(ref)
-      refs.append(ref)
+    refs, underspecified_names = self._GetRefsAndUnderspecifiedNames(
+        names, params, collection, scope_value is not None,
+        api_resource_registry)
 
     # If we still have some resources which need to be resolve see if we can
     # prompt the user and try to resolve these again.
-    if underspecified_names:
-      names = [n[0] for n in underspecified_names]
-      if not console_io.CanPrompt():
-        raise UnderSpecifiedResourceError(names, [s.flag for s in self.scopes])
-      resource_scope_enum, scope_value = scope_prompter.PromptForScope(
-          self.resource_name, names, [s.scope_enum for s in self.scopes],
-          default_scope.scope_enum if default_scope is not None else None,
-          scope_lister)
-      if resource_scope_enum is None:
-        raise UnderSpecifiedResourceError(names, [s.flag for s in self.scopes])
-      resource_scope = self.scopes[resource_scope_enum]
-      params = {
-          'project': properties.VALUES.core.project.GetOrFail,
-      }
-      if resource_scope.scope_enum != compute_scope.ScopeEnum.GLOBAL:
-        params[resource_scope.scope_enum.param_name] = scope_value
-      for name in underspecified_names:
-        name[0] = api_resource_registry.Parse(
-            name[0],
-            params=params,
-            collection=resource_scope.collection,
-            enforce_collection=True)
+    self._ResolveUnderspecifiedNames(
+        underspecified_names, default_scope, scope_lister, project,
+        api_resource_registry)
+
     # Now unpack each element.
-    refs = [ref[0] for ref in refs]
+    refs = [ref[0] for ref in refs]  # type: list[resources.Resource]
 
     # Make sure correct collection was given for each resource, for example
     # URLs have implicit collections.
@@ -461,12 +531,12 @@ class ResourceArgument(object):
     class MyCommand(base.Command):
       _BACKEND_SERVICE_ARG = flags.ResourceArgument(
           resource_name='backend service',
-          completion_resource_id='compute.backendService',
+          completer=compute_completers.BackendServiceCompleter,
           regional_collection='compute.regionBackendServices',
           global_collection='compute.backendServices')
       _INSTANCE_GROUP_ARG = flags.ResourceArgument(
           resource_name='instance group',
-          completion_resource_id='compute.InstanceGroup',
+          completer=compute_completers.InstanceGroupsCompleter,
           zonal_collection='compute.instanceGroups',)
 
       @staticmethod
@@ -494,22 +564,19 @@ class ResourceArgument(object):
     and zonal qualifiers (or any combination of) for each resource.
   """
 
-  def __init__(self, name=None,
-               resource_name=None,
-               completion_resource_id=None,
+  def __init__(self, name=None, resource_name=None, completer=None,
                plural=False, required=True, zonal_collection=None,
                regional_collection=None, global_collection=None,
-               region_explanation=None, zone_explanation=None,
-               short_help=None, detailed_help=None,
-               custom_plural=None,
-               list_command_path=None):
+               region_explanation=None, region_hidden=False,
+               zone_explanation=None, zone_hidden=False,
+               short_help=None, detailed_help=None, custom_plural=None):
 
     """Constructor.
 
     Args:
       name: str, argument name.
       resource_name: str, human readable name for resources eg "instance group".
-      completion_resource_id: str, id of registered resource.
+      completer: completion_cache.Completer, The completer class type.
       plural: bool, whether to accept multiple values.
       required: bool, whether this argument is required.
       zonal_collection: str, include zone flag and use this collection
@@ -520,18 +587,18 @@ class ResourceArgument(object):
                               and uses this collection to resolve as
                               global resource.
       region_explanation: str, long help that will be given for region flag,
-                               empty by default. Provide argparse.SUPPRESS to
-                               hide in help.
+                               empty by default.
+      region_hidden: bool, Hide region in help if True.
       zone_explanation: str, long help that will be given for zone flag, empty
-                             by default. Provide argparse.SUPPRESS to hide in
-                             help.
+                             by default.
+      zone_hidden: bool, Hide region in help if True.
       short_help: str, help for the flag being added, if not provided help text
                        will be 'The name[s] of the ${resource_name}[s].'.
       detailed_help: str, detailed help for the flag being added, if not
                           provided there will be no detailed help for the flag.
       custom_plural: str, If plural is True then this string will be used as
                           plural resource name.
-      list_command_path: the list_command_path for the main argument
+
     Raises:
       exceptions.Error: if there some inconsistency in arguments.
     """
@@ -548,7 +615,7 @@ class ResourceArgument(object):
       self.scopes = ResourceArgScopes(flag_prefix=None)
       self.name = self.name_arg  # arg name is same as its spec.
     self.resource_name = resource_name
-    self.completion_resource_id = completion_resource_id
+    self.completer = completer
     self.plural = plural
     self.custom_plural = custom_plural
     self.required = required
@@ -565,9 +632,10 @@ class ResourceArgument(object):
       self.scopes.AddScope(compute_scope.ScopeEnum.GLOBAL,
                            collection=global_collection)
     self._region_explanation = region_explanation or ''
+    self._region_hidden = region_hidden
     self._zone_explanation = zone_explanation or ''
+    self._zone_hidden = zone_hidden
     self._resource_resolver = ResourceResolver(self.scopes, resource_name)
-    self._list_command_path = list_command_path
 
   # TODO(b/31933786) remove cust_metavar once surface supports metavars for
   # plural flags.
@@ -581,7 +649,7 @@ class ResourceArgument(object):
 
     params = dict(
         metavar=cust_metavar if cust_metavar else self.name.upper(),
-        completion_resource=self.completion_resource_id,
+        completer=self.completer,
     )
 
     if self._detailed_help:
@@ -589,7 +657,7 @@ class ResourceArgument(object):
     elif self._short_help:
       params['help'] = self._short_help
     else:
-      params['help'] = 'The name{} of the {} to {}.'.format(
+      params['help'] = 'Name{} of the {} to {}.'.format(
           's' if self.plural else '',
           text.Pluralize(
               int(self.plural) + 1, self.resource_name or '',
@@ -607,9 +675,6 @@ class ResourceArgument(object):
       else:
         params['nargs'] = '*' if self.plural else '?'
 
-    if self._list_command_path:
-      params['list_command_path'] = self._list_command_path
-
     (mutex_group or parser).add_argument(self.name_arg, **params)
 
     if len(self.scopes) > 1:
@@ -624,7 +689,7 @@ class ResourceArgument(object):
           resource_type=self.resource_name,
           operation_type=operation_type,
           explanation=self._zone_explanation,
-          hidden=self._zone_explanation is argparse.SUPPRESS,
+          hidden=self._zone_hidden,
           plural=self.plural,
           custom_plural=self.custom_plural)
 
@@ -635,7 +700,7 @@ class ResourceArgument(object):
           resource_type=self.resource_name,
           operation_type=operation_type,
           explanation=self._region_explanation,
-          hidden=self._region_explanation is argparse.SUPPRESS,
+          hidden=self._region_hidden,
           plural=self.plural,
           custom_plural=self.custom_plural)
 
@@ -650,7 +715,7 @@ class ResourceArgument(object):
           self.scopes[compute_scope.ScopeEnum.GLOBAL].flag,
           action='store_true',
           default=None,
-          help='If provided, it is assumed the {0} global.'
+          help='If set, the {0} global.'
           .format(resource_mention))
 
   def ResolveAsResource(self, args,
@@ -712,3 +777,50 @@ def AddRegexArg(parser):
       A regular expression to filter the names of the results on. Any names
       that do not match the entire regular expression will be filtered out.
       """)
+
+
+def AddPolicyFileFlag(parser):
+  parser.add_argument('policy_file', help="""\
+      JSON or YAML file containing the IAM policy.""")
+
+
+def AddStorageLocationFlag(parser, resource):
+  parser.add_argument(
+      '--storage-location',
+      metavar='LOCATION',
+      help="""\
+      Google Cloud Storage location, either regional or multi-regional, where
+      {} content is to be stored. If absent, a nearby regional or
+      multi-regional location is chosen automatically.
+      """.format(resource))
+
+
+def RewriteFilter(args, message=None, frontend_fields=None):
+  """Rewrites args.filter into client and server filter expression strings.
+
+  Usage:
+
+    args.filter, request_filter = flags.RewriteFilter(args)
+
+  Args:
+    args: The parsed args namespace containing the filter expression args.filter
+      and display_info.
+    message: The response resource message proto for the request.
+    frontend_fields: A set of dotted key names supported client side only.
+
+  Returns:
+    A (client_filter, server_filter) tuple of filter expression strings.
+    None means the filter does not need to applied on the respective
+    client/server side.
+  """
+  if not args.filter:
+    return None, None
+  display_info = args.GetDisplayInfo()
+  defaults = resource_projection_spec.ProjectionSpec(
+      symbols=display_info.transforms,
+      aliases=display_info.aliases)
+  client_filter, server_filter = filter_rewrite.Rewriter(
+      message=message, frontend_fields=frontend_fields).Rewrite(
+          args.filter, defaults=defaults)
+  log.info('client_filter=%r server_filter=%r', client_filter, server_filter)
+  return client_filter, server_filter
