@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,11 @@
 
 """Flags and helpers for the compute health-checks commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 DEFAULT_LIST_FORMAT = """\
@@ -23,12 +29,19 @@ DEFAULT_LIST_FORMAT = """\
     )"""
 
 
-def HealthCheckArgument(
-    protocol_string, name=None, required=True, plural=False):
+def HealthCheckArgument(protocol_string,
+                        name=None,
+                        required=True,
+                        plural=False,
+                        include_alpha=False):
   return compute_flags.ResourceArgument(
       name=name,
       resource_name='{} health check'.format(protocol_string),
-      completion_resource_id='compute.healthChecks',
+      completer=compute_completers.HealthChecksCompleter,
       plural=plural,
       required=required,
-      global_collection='compute.healthChecks')
+      global_collection='compute.healthChecks',
+      regional_collection='compute.regionHealthChecks'
+      if include_alpha else None,
+      region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION
+      if include_alpha else None)
