@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,11 @@
 # limitations under the License.
 """Flags and helpers for the compute instance groups commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags
 
 DEFAULT_LIST_FORMAT = """\
@@ -27,6 +33,20 @@ DEFAULT_LIST_FORMAT = """\
 def MakeInstanceTemplateArg(plural=False):
   return flags.ResourceArgument(
       resource_name='instance template',
-      completion_resource_id='compute.instanceTemplates',
+      completer=completers.InstanceTemplatesCompleter,
       plural=plural,
       global_collection='compute.instanceTemplates')
+
+
+def MakeSourceInstanceArg():
+  return flags.ResourceArgument(
+      name='--source-instance',
+      resource_name='instance',
+      completer=completers.InstancesCompleter,
+      required=False,
+      zonal_collection='compute.instances',
+      short_help=('The name of the source instance that the instance template '
+                  'will be created from.\n\nYou can override machine type and '
+                  'labels. Values of other flags will be ignored and values '
+                  'from the source instance will be used instead.')
+  )
