@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,12 +15,17 @@
 
 """Flags for the compute machine-types commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 
 def MakeMachineTypeArg():
   return compute_flags.ResourceArgument(
       resource_name='machine type',
-      completion_resource_id='compute.machineTypes',
+      completer=completers.MachineTypesCompleter,
       zonal_collection='compute.machineTypes',
       zone_explanation=compute_flags.ZONE_PROPERTY_EXPLANATION)
