@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,11 @@
 # limitations under the License.
 """Flags and helpers for the compute snapshots commands."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 
 
@@ -20,6 +26,6 @@ def MakeSnapshotArg(plural=False):
   return compute_flags.ResourceArgument(
       resource_name='snapshot',
       name='snapshot_name',
-      completion_resource_id='compute.snapshots',
+      completer=compute_completers.RoutesCompleter,
       plural=plural,
       global_collection='compute.snapshots')
