@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,7 +14,9 @@
 # limitations under the License.
 """Helper methods for constructing messages for the container CLI."""
 
-from googlecloudsdk.calliope import exceptions
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 
 def AutoUpdateUpgradeRepairMessage(value, flag_name):
@@ -28,13 +31,9 @@ def AutoUpdateUpgradeRepairMessage(value, flag_name):
     the formatted message string.
   """
   action = 'enable' if value else 'disable'
-  if flag_name != 'autoupgrade' and flag_name != 'autorepair':
-    raise exceptions.InvalidArgumentException(
-        'Invalid value for flag_name: ' + flag_name +
-        '. Must be either autorepair or autoupgrade')
   plural = flag_name + 's'
   link = 'node-management' if flag_name == 'autoupgrade' else 'node-auto-repair'
-  return ('This will {0} the {1} feature for nodes. Please see\n'
-          'https://cloud.google.com/container-engine/docs/'
-          '{2} for more\n'
-          'information on node {3}.\n').format(action, flag_name, link, plural)
+  return ('This will {0} the {1} feature for nodes. Please see '
+          'https://cloud.google.com/kubernetes-engine/docs/'
+          '{2} for more '
+          'information on node {3}.').format(action, flag_name, link, plural)
