@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,3 +15,5 @@
 
 
 """Utilities for debug surface commands."""
+
+from __future__ import unicode_literals
