@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,10 +18,15 @@ The main entry point is UploadPythonPackages, which takes in parameters derived
 from the command line arguments and returns a list of URLs to be given to the
 Cloud ML Engine API. See its docstring for details.
 """
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import abc
 import collections
 import contextlib
-import cStringIO
+import io
 import os
 import sys
 import textwrap
@@ -32,6 +38,8 @@ from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import execution_utils
 from googlecloudsdk.core import log
 from googlecloudsdk.core.util import files
+import six
+from six.moves import map
 
 
 DEFAULT_SETUP_FILE = """\
@@ -107,6 +115,14 @@ class NoStagingLocationError(UploadFailureError):
   """No staging location was provided but one was required."""
 
 
+class InvalidSourceDirError(UploadFailureError):
+  """Error indicating that the source directory is invalid."""
+
+  def __init__(self, source_dir):
+    super(InvalidSourceDirError, self).__init__(
+        'Source directory [{}] is not a valid directory.'.format(source_dir))
+
+
 def _CopyIfNotWritable(source_dir, temp_dir):
   """Returns a writable directory with the same contents as source_dir.
 
@@ -124,8 +140,18 @@ def _CopyIfNotWritable(source_dir, temp_dir):
 
   Raises:
     UploadFailureError: if the command exits non-zero.
+    InvalidSourceDirError: if the source directory is not valid.
   """
-  if files.HasWriteAccessInDir(source_dir):
+  if not os.path.isdir(source_dir):
+    raise InvalidSourceDirError(source_dir)
+  # A race condition may cause a ValueError while checking for write access
+  # even if the directory was valid before.
+  try:
+    writable = files.HasWriteAccessInDir(source_dir)
+  except ValueError:
+    raise InvalidSourceDirError(source_dir)
+
+  if writable:
     return source_dir
 
   if files.IsDirAncestorOf(source_dir, temp_dir):
@@ -161,8 +187,7 @@ def _GenerateSetupPyIfNeeded(setup_py_path, package_name):
 
   setup_contents = DEFAULT_SETUP_FILE.format(package_name=package_name)
   log.info('Generating temporary setup.py file:\n%s', setup_contents)
-  with open(setup_py_path, 'w') as setup_file:
-    setup_file.write(setup_contents)
+  files.WriteFileContents(setup_py_path, setup_contents)
   return True
 
 
@@ -199,7 +224,7 @@ def _TempDirOrBackup(default_dir):
       temp_dir.__exit__(*sys.exc_info())
 
 
-class _SetupPyCommand(object):
+class _SetupPyCommand(six.with_metaclass(abc.ABCMeta, object)):
   """A command to run setup.py in a given environment.
 
   Includes the Python version to use and the arguments with which to run
@@ -212,8 +237,6 @@ class _SetupPyCommand(object):
       (must be writable, or setuptools will fail)
   """
 
-  __metaclass__ = abc.ABCMeta
-
   def __init__(self, setup_py_path, setup_py_args, package_root):
     self.setup_py_path = setup_py_path
     self.setup_py_args = setup_py_args
@@ -253,15 +276,9 @@ class _CloudSdkPythonSetupPyCommand(_SetupPyCommand):
   """
 
   def GetArgs(self):
-    try:
-      return execution_utils.ArgsForPythonTool(self.setup_py_path,
-                                               *self.setup_py_args)
-    except ValueError:
-      # The most common cause of ValueError for ArgsForPythonTool is a missing
-      # executable; we want to display a more specific error in this case.
-      if not sys.executable:
-        raise SysExecutableMissingError()
-    raise
+    return execution_utils.ArgsForPythonTool(self.setup_py_path,
+                                             *self.setup_py_args,
+                                             python=GetPythonExecutable())
 
   def GetEnv(self):
     exec_env = os.environ.copy()
@@ -278,14 +295,21 @@ class _SystemPythonSetupPyCommand(_SetupPyCommand):
   """
 
   def GetArgs(self):
-    if not sys.executable:
-      raise SysExecutableMissingError()
-    return [sys.executable, self.setup_py_path] + self.setup_py_args
+    return [GetPythonExecutable(), self.setup_py_path] + self.setup_py_args
 
   def GetEnv(self):
     return None
 
 
+def GetPythonExecutable():
+  python_executable = None
+  try:
+    python_executable = execution_utils.GetPythonExecutable()
+  except ValueError:
+    raise SysExecutableMissingError()
+  return python_executable
+
+
 def _RunSetupTools(package_root, setup_py_path, output_dir):
   """Executes the setuptools `sdist` command.
 
@@ -362,7 +386,7 @@ def _RunSetupTools(package_root, setup_py_path, output_dir):
           setup_py_path, setup_py_args, package_root))
 
     for setup_py_command in setup_py_commands:
-      out = cStringIO.StringIO()
+      out = io.StringIO()
       return_code = setup_py_command.Execute(out)
       if not return_code:
         break
@@ -406,6 +430,7 @@ def BuildPackages(package_path, output_dir):
   Raises:
     SetuptoolsFailedError: If the setup.py file fails to successfully build.
     MissingInitError: If the package doesn't contain an `__init__.py` file.
+    InvalidSourceDirError: if the source directory is not valid.
   """
   package_path = os.path.abspath(package_path)
   package_root = os.path.dirname(package_path)
@@ -447,8 +472,8 @@ def _UploadFilesByPath(paths, staging_location):
   """Uploads files after validating and transforming input type."""
   if not staging_location:
     raise NoStagingLocationError()
-  counter = collections.Counter(map(os.path.basename, paths))
-  duplicates = [name for name, count in counter.iteritems() if count > 1]
+  counter = collections.Counter(list(map(os.path.basename, paths)))
+  duplicates = [name for name, count in six.iteritems(counter) if count > 1]
   if duplicates:
     raise DuplicateEntriesError(duplicates)
 
@@ -543,6 +568,6 @@ def GetStagingLocation(job_id=None, staging_bucket=None, job_dir=None):
                                                     job_id)
   elif job_dir:
     staging_location = storage_util.ObjectReference(
-        job_dir.bucket_ref, '/'.join(filter(None, [job_dir.name.rstrip('/'),
-                                                   'packages'])))
+        job_dir.bucket_ref, '/'.join([f for f in [job_dir.name.rstrip('/'),
+                                                  'packages'] if f]))
   return staging_location
