@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,26 +15,37 @@
 
 """Stores lookup keys for help search table."""
 
-COMMAND = 'command'
-COMMANDS = 'commands'
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.calliope import cli_tree
+
+
+CAPSULE = cli_tree.LOOKUP_CAPSULE
+CHOICES = cli_tree.LOOKUP_CHOICES
+COMMANDS = cli_tree.LOOKUP_COMMANDS
+DEFAULT = cli_tree.LOOKUP_DEFAULT
+DESCRIPTION = cli_tree.LOOKUP_DESCRIPTION
+FLAGS = cli_tree.LOOKUP_FLAGS
+IS_GLOBAL = cli_tree.LOOKUP_IS_GLOBAL
+IS_HIDDEN = cli_tree.LOOKUP_IS_HIDDEN
+NAME = cli_tree.LOOKUP_NAME
+PATH = cli_tree.LOOKUP_PATH
+POSITIONALS = cli_tree.LOOKUP_POSITIONALS
+RELEASE = cli_tree.LOOKUP_RELEASE
+SECTIONS = cli_tree.LOOKUP_SECTIONS
+
 FLAG = 'flag'
-POSITIONAL = 'positional'
-RELEASE = 'release'
-MARKDOWN = 'markdown'
-TEXT = 'text'
+COMMAND = 'command'
 GENERATED = 'generated'
+MARKDOWN = 'markdown'
+POSITIONAL = 'positional'
 SUBSECTIONS = 'subsections'
-NAME = 'name'
-FLAGS = 'flags'
-POSITIONALS = 'positionals'
-SECTIONS = 'sections'
-DESCRIPTION = 'description'
-CAPSULE = 'capsule'
-CHOICES = 'choices'
-RELEASE = 'release'
+SUMMARY = 'summary'
+TEXT = 'text'
+RESULTS = 'results'
+
 ALPHA = 'ALPHA'
 BETA = 'BETA'
 GA = 'GA'
-PATH = 'path'
-DEFAULT = 'default'
-SUMMARY = 'summary'
