@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,13 +15,15 @@
 
 """gcloud search-help command resources."""
 
-import json
-import os
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
+from googlecloudsdk.calliope import cli_tree
 from googlecloudsdk.command_lib.search_help import lookup
 from googlecloudsdk.command_lib.search_help import search_util
-from googlecloudsdk.command_lib.search_help import table
-from googlecloudsdk.core.console import progress_tracker
+
+from six.moves import zip
 
 
 def RunSearch(terms, cli):
@@ -33,22 +36,7 @@ def RunSearch(terms, cli):
   Returns:
     a list of json objects representing gcloud commands.
   """
-  try:
-    table_path = table.IndexPath()
-  # If the help table file can't be found, load the help tree directly from
-  # the passed cli.
-  except table.NoSdkRootException:
-    with progress_tracker.ProgressTracker('Command help index not found, '
-                                          'loading gcloud commands...'):
-      parent = table.GetSerializedHelpIndex(cli)
-  else:
-    if not os.path.exists(table_path):
-      with progress_tracker.ProgressTracker('Updating command help index...'):
-        table.Update(cli)
-
-    with open(table_path, 'r') as table_file:
-      parent = json.loads(table_file.read())
-
+  parent = cli_tree.Load(cli=cli, one_time_use_ok=True)
   searcher = Searcher(parent, terms)
   return searcher.Search()
 
