@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +15,21 @@
 
 """utils for search-help command resources."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import copy
+import io
 import re
-import StringIO
 
 from googlecloudsdk.command_lib.search_help import lookup
 from googlecloudsdk.core import log
 from googlecloudsdk.core.document_renderers import render_document
 
+import six
+from six.moves import filter
+
 DEFAULT_SNIPPET_LENGTH = 200
 DOT = '.'
 
@@ -198,7 +206,7 @@ def _Snip(text, length_per_snippet, terms):
 def _FormatHeader(header):
   """Helper function to reformat header string in markdown."""
   if header == lookup.CAPSULE:
-    header = 'summary description'
+    return None
   return '# {}'.format(header.upper())
 
 
@@ -218,27 +226,32 @@ def _AddFlagToSummary(command, summary, length_per_snippet, location, terms):
   # Add specific flag if given.
   if len(location) > 1:
     # Add flag name and description of flag if not added yet.
-    if _FormatItem(location[1]) not in summary:
-      lines.append(_FormatItem(location[1]))
-      desc_line = flags.get(location[1], {}).get(lookup.DESCRIPTION, '')
-      desc_line = _Snip(desc_line, length_per_snippet, terms)
-      if desc_line:
-        line = desc_line
-      else:
-        log.warn('Attempted to look up a location [{}] that was not '
-                 'found.'.format(location[1]))
+    flag = flags.get(location[1])
+    if flag and not flag[lookup.IS_HIDDEN]:
+      if _FormatItem(location[1]) not in summary:
+        lines.append(_FormatItem(location[1]))
+        desc_line = flag.get(lookup.DESCRIPTION, '')
+        desc_line = _Snip(desc_line, length_per_snippet, terms)
+        if desc_line:
+          line = desc_line
+        else:
+          log.warning('Attempted to look up a location [{}] that was not '
+                      'found.'.format(location[1]))
 
-    # Add subsections of flag if given.
-    if len(location) > 2:
-      if location[2] == lookup.DEFAULT:
-        default = flags.get(location[1]).get(lookup.DEFAULT)
-        if default:
-          lines.append(line)
-          line = 'Default: {}.'.format(
-              flags.get(location[1]).get(lookup.DEFAULT))
-      else:
-        log.warn('Attempted to look up a location [{}] that was not '
-                 'found.'.format(location[-1]))
+      # Add subsections of flag if given.
+      if len(location) > 2:
+        if location[2] == lookup.DEFAULT:
+          default = flags.get(location[1]).get(lookup.DEFAULT)
+          if default:
+            lines.append(line)
+            if isinstance(default, dict):
+              default = ', '.join([x for x in sorted(default.keys())])
+            elif isinstance(default, list):
+              default = ', '.join([x for x in default])
+            line = 'Default: {}.'.format(default)
+        else:
+          log.warning('Attempted to look up a location [{}] that was not '
+                      'found.'.format(location[-1]))
 
   # If no specific flag given, get list of all flags.
   else:
@@ -268,8 +281,8 @@ def _AddPositionalToSummary(command, summary, length_per_snippet,
       line = positional.get(lookup.DESCRIPTION, '')
       line = _Snip(line, length_per_snippet, terms)
     else:
-      log.warn('Attempted to look up a location [{}] that was not '
-               'found.'.format(location[1]))
+      log.warning('Attempted to look up a location [{}] that was not '
+                  'found.'.format(location[1]))
 
   # If no specific positional given, just add list of all available.
   else:
@@ -295,13 +308,15 @@ def _AddGenericSectionToSummary(command, summary, length_per_snippet,
     else:
       line = str(section)
   if line:
-    summary.append(_FormatHeader(location[-1]))
+    header = _FormatHeader(location[-1])
+    if header:
+      summary.append(header)
     loc = '.'.join(location)
     summary.append(
         _Snip(line, length_per_snippet, terms))
   else:
-    log.warn('Attempted to look up a location [{}] that was not '
-             'found.'.format(location[-1]))
+    log.warning('Attempted to look up a location [{}] that was not found.'
+                .format(location[-1]))
 
 
 def _Priority(x):
@@ -309,7 +324,7 @@ def _Priority(x):
   return PRIORITIES.get(x[0], len(PRIORITIES))
 
 
-def GetSummary(command, found_terms, length_per_snippet=DEFAULT_SNIPPET_LENGTH):
+def GetSummary(command, results, length_per_snippet=DEFAULT_SNIPPET_LENGTH):
   """Gets a summary of certain attributes of a command.
 
   This will summarize a json representation of a command using
@@ -325,7 +340,7 @@ def GetSummary(command, found_terms, length_per_snippet=DEFAULT_SNIPPET_LENGTH):
   GetSummary(command, {'alligator': 'capsule'},
              length_per_snippet=200)
 
-  # SUMMARY DESCRIPTION
+  [no heading]
   {200-char excerpt of command['capsule'] with first appearance of 'alligator'}
 
   2) To get a summary with a section (can be first-level or inside 'sections',
@@ -355,63 +370,56 @@ def GetSummary(command, found_terms, length_per_snippet=DEFAULT_SNIPPET_LENGTH):
 
   Args:
     command: dict, a json representation of a command.
-    found_terms: dict, mapping of terms to the locations where they are
-        found. If no term is relevant, a '' is used.
+    results: dict, mapping of terms to the locations where they are found. If a
+      '' is used as the key, a snippet of the location starting from the
+      beginning will be used.
     length_per_snippet: int, length of desired substrings to get from text.
 
   Returns:
     str, a markdown summary
   """
   # Always include capsule.
-  found_terms.update({'': lookup.CAPSULE})
+  results.update({'': lookup.CAPSULE})
   summary = []
   locations = [location.split('.')
-               for location in sorted(set(found_terms.values()))]
+               for location in sorted(set(results.values()))]
 
   for location in sorted(locations, key=_Priority):
-    terms = {t for t, l in found_terms.iteritems()
+    terms = {t for t, l in six.iteritems(results)
              if l == '.'.join(location) and t}
     if location[0] == lookup.FLAGS:
       _AddFlagToSummary(command, summary, length_per_snippet, location, terms)
     elif location[0] == lookup.POSITIONALS:
       _AddPositionalToSummary(command, summary, length_per_snippet, location,
                               terms)
-    # Add any other sections of help. path and name are ignored.
-    elif lookup.PATH not in location and lookup.NAME not in location:
+    # path and name are ignored.
+    elif lookup.PATH in location or lookup.NAME in location:
+      continue
+    else:
       _AddGenericSectionToSummary(command, summary, length_per_snippet,
                                   location, terms)
   return '\n'.join(summary)
 
 
-def ProcessResult(command, found_terms):
+def ProcessResult(command, results):
   """Helper function to create help text resource for listing results.
 
   Args:
     command: dict, json representation of command.
-    found_terms: {str: str}, lookup from terms to where they were
-      found.
+    results: {str: str}, lookup from terms to where they were found.
 
   Returns:
     A modified copy of the json command with a summary, and with the dict
         of subcommands replaced with just a list of available subcommands.
   """
   new_command = copy.deepcopy(command)
-  if lookup.COMMANDS in new_command.keys():
+  if lookup.COMMANDS in six.iterkeys(new_command):
     new_command[lookup.COMMANDS] = sorted([
-        c[lookup.NAME] for c in new_command[lookup.COMMANDS].values()])
-  summary = GetSummary(command, found_terms)
-  # Render the summary for console printing, but ignoring console width.
-  md = StringIO.StringIO(summary)
-  rendered_summary = StringIO.StringIO()
-  render_document.RenderDocument('text',
-                                 md,
-                                 out=rendered_summary,
-                                 width=len(summary))
-  # Remove indents and blank lines so summary can be easily
-  # printed in a table.
-  new_command[lookup.SUMMARY] = '\n'.join([
-      l.lstrip() for l in rendered_summary.getvalue().splitlines()
-      if l.lstrip()])
+        c[lookup.NAME]
+        for c in new_command[lookup.COMMANDS].values()
+        if not c[lookup.IS_HIDDEN]
+    ])
+  new_command[lookup.RESULTS] = results
   return new_command
 
 
@@ -430,6 +438,10 @@ def LocateTerm(command, term):
   Returns:
     str, lookup for where to find the term when building summary of command.
   """
+  # Skip hidden commands.
+  if command[lookup.IS_HIDDEN]:
+    return ''
+
   # Look in name/capsule
   regexp = re.compile(re.escape(term), re.IGNORECASE)
   if (regexp.search(command[lookup.NAME])
@@ -438,12 +450,15 @@ def LocateTerm(command, term):
 
   # Look in detailed help sections
   for section_name, section_desc in sorted(
-      command[lookup.SECTIONS].iteritems()):
+      six.iteritems(command[lookup.SECTIONS])):
     if regexp.search(section_desc):
       return DOT.join([lookup.SECTIONS, section_name])
 
   # Look in flags
-  for flag_name, flag in sorted(command[lookup.FLAGS].iteritems()):
+  for flag_name, flag in sorted(six.iteritems(command[lookup.FLAGS])):
+    if (flag[lookup.IS_HIDDEN] or
+        flag[lookup.IS_GLOBAL] and not command[lookup.IS_GLOBAL]):
+      continue
     if (regexp.search(flag[lookup.NAME])
         or regexp.search(flag[lookup.DESCRIPTION])):
       return DOT.join([lookup.FLAGS, flag_name])
@@ -459,8 +474,37 @@ def LocateTerm(command, term):
       return DOT.join([lookup.POSITIONALS, positional[lookup.NAME]])
 
   # Look in subcommands & path
-  if regexp.search(str(command[lookup.COMMANDS].keys())):
+  if regexp.search(str([n
+                        for n, c in six.iteritems(command[lookup.COMMANDS])
+                        if not c[lookup.IS_HIDDEN]
+                       ])):
     return lookup.COMMANDS
   if regexp.search(' '.join(command[lookup.PATH])):
     return lookup.PATH
   return ''
+
+
+def SummaryTransform(r):
+  """Get summary of command with desired terms included."""
+  summary = GetSummary(r, r[lookup.RESULTS])
+  md = io.StringIO(summary)
+  rendered_summary = io.StringIO()
+  # Render summary as markdown, ignoring console width.
+  render_document.RenderDocument('text',
+                                 md,
+                                 out=rendered_summary,
+                                 # Increase length in case of indentation.
+                                 width=len(summary) * 2)
+  final_summary = '\n'.join(
+      [l.lstrip() for l in rendered_summary.getvalue().splitlines()
+       if l.lstrip()])
+  return final_summary
+
+
+_TRANSFORMS = {
+    'summary': SummaryTransform,
+}
+
+
+def GetTransforms():
+  return _TRANSFORMS
