@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Provides common arguments for the Spanner command surface."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import itertools
 
 from googlecloudsdk.calliope import base
@@ -67,6 +72,16 @@ class OperationCompleter(completers.ListCommandCompleter):
         **kwargs)
 
 
+class DatabaseSessionCompleter(completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(DatabaseSessionCompleter, self).__init__(
+        collection='spanner.projects.instances.databases.sessions',
+        list_command='beta spanner databases sessions list --uri',
+        flags=['database', 'instance'],
+        **kwargs)
+
+
 def Database(positional=True,
              required=True,
              text='Cloud Spanner database ID.'):
@@ -144,3 +159,16 @@ def OperationId(database=False):
       metavar='OPERATION-ID',
       completer=DatabaseOperationCompleter if database else OperationCompleter,
       help='ID of the operation')
+
+
+def Session(positional=True, required=True, text='Cloud Spanner session ID'):
+  if positional:
+    return base.Argument(
+        'session', completer=DatabaseSessionCompleter, help=text)
+
+  else:
+    return base.Argument(
+        '--session',
+        required=required,
+        completer=DatabaseSessionCompleter,
+        help=text)
