@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -40,14 +41,14 @@ error. The row is simply replaced by the new data.
 
 A Table object can be restricted and hidden from cache users. These tables
 must be instantiated when the Cache object is instantiated, before the first
-user access to the the cache. This allows a cache implementation layer to have
+user access to the cache. This allows a cache implementation layer to have
 tables that are hidden from the layers above it.
 
 The table select and delete methods match against a row template. A template may
 have fewer columns than the number of columns in the table. Omitted template
-columns or columns with with value None match all values for that column. '*'
-and '?' matching operators are supported for string columns. It is not an error
-to select or delete a row that does not exist.
+columns or columns with value None match all values for that column. '*' and '?'
+matching operators are supported for string columns. It is not an error to
+select or delete a row that does not exist.
 
 HINTS for IMPLEMENTERS
 
@@ -58,19 +59,26 @@ disables expiry check. These can be used by meta commands/functions to view
 and debug cache data without modifying the underlying persistent data.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import abc
 import time
-import urllib
 
 from googlecloudsdk.core.cache import exceptions
 
+import six
+import six.moves.urllib.parse
+
 
 def Now():
   """Returns the current time in seconds since the epoch."""
   return time.time()
 
 
-class Table(object):
+@six.add_metaclass(abc.ABCMeta)
+class Table(object):  # pytype: disable=ignored-abstractmethod
   """A persistent cache table object.
 
   This object should only be instantiated by a Cache object.
@@ -90,8 +98,6 @@ class Table(object):
       are invalid. 0 means no timeout.
   """
 
-  __metaclass__ = abc.ABCMeta
-
   def __init__(self, cache, name, columns=1, keys=1, timeout=0, modified=0,
                restricted=False):
     self._cache = cache
@@ -140,7 +146,7 @@ class Table(object):
     if not name:
       raise exceptions.CacheTableNameInvalid(
           'Cache table name [{}] is invalid.'.format(name))
-    return urllib.quote(name, '!@+,')
+    return six.moves.urllib.parse.quote(name, '!@+,')
 
   def _CheckRows(self, rows):
     """Raise an exception if the size of any row in rows is invalid.
@@ -244,7 +250,8 @@ class Table(object):
     pass
 
 
-class Cache(object):
+@six.add_metaclass(abc.ABCMeta)
+class Cache(object):  # pytype: disable=ignored-abstractmethod
   r"""A persistent cache object.
 
   This class is also a context manager. Changes are automaticaly committed if
@@ -261,8 +268,6 @@ class Cache(object):
       stored when the persistent object was created.
   """
 
-  __metaclass__ = abc.ABCMeta
-
   def __init__(self, name, create=True, timeout=None, version=None):
     self.name = Cache.EncodeName(name)
     del create  # Unused in __init__. Subclass constructors may use this.
@@ -314,14 +319,16 @@ class Cache(object):
 
   @abc.abstractmethod
   def Close(self, commit=True):
-    """Closes the cache, optionally comitting any changes.
+    """Closes the cache, optionally committing any changes.
 
     Args:
       commit: Commits any changes before closing if True.
     """
     pass
 
+  @abc.abstractmethod
   def Table(self, name, create=True, columns=1, keys=1, timeout=None):
+    # type: (str, bool, int, int, int) -> Table
     """Returns the Table object for existing table name.
 
     Args:
