@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Config for Google Cloud Platform CLIs."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import json
 import os
 import time
@@ -29,7 +34,6 @@ from googlecloudsdk.core.util import platforms
 class Error(exceptions.Error):
   """Exceptions for the cli module."""
 
-
 # Environment variable for the directory containing Cloud SDK configuration.
 CLOUDSDK_CONFIG = 'CLOUDSDK_CONFIG'
 
@@ -75,14 +79,9 @@ class InstallationConfig(object):
     Returns:
       InstallationSpecificData: The loaded data.
     """
-    data = json.loads(pkg_resources.GetResource(__name__, 'config.json'))
-
-    # Python versions prior to 2.6.5 do not allow the keys in a kwargs dict to
-    # be unicode strings.  The json module parses files as unicode.  We are not
-    # using any unicode in the keys for our config properties so a simple
-    # conversion to str is safe.
-    non_unicode_data = dict([(str(k), v) for k, v in data.iteritems()])
-    return InstallationConfig(**non_unicode_data)
+    data = json.loads(
+        encoding.Decode(pkg_resources.GetResource(__name__, 'config.json')))
+    return InstallationConfig(**data)
 
   @staticmethod
   def FormatRevision(time_struct):
@@ -92,12 +91,12 @@ class InstallationConfig(object):
       time_struct: time.struct_time, The time you want to format.
 
     Returns:
-      long, A revision number from a component snapshot.  This is a long int but
+      int, A revision number from a component snapshot.  This is a int but
       formatted as an actual date in seconds (i.e 20151009132504).  It is *NOT*
       seconds since the epoch.
     """
-    return long(time.strftime(InstallationConfig.REVISION_FORMAT_STRING,
-                              time_struct))
+    return int(time.strftime(
+        InstallationConfig.REVISION_FORMAT_STRING, time_struct))  # pytype: disable=wrong-arg-types
 
   @staticmethod
   def ParseRevision(revision):
@@ -112,7 +111,7 @@ class InstallationConfig(object):
       time.struct_time, The parsed time.
     """
     return time.strptime(str(revision),
-                         InstallationConfig.REVISION_FORMAT_STRING)
+                         InstallationConfig.REVISION_FORMAT_STRING)  # pytype: disable=wrong-arg-types
 
   @staticmethod
   def ParseRevisionAsSeconds(revision):
@@ -226,7 +225,7 @@ def _GetGlobalConfigDir():
   if appdata:
     return os.path.join(appdata, _CLOUDSDK_GLOBAL_CONFIG_DIR_NAME)
   # This shouldn't happen unless someone is really messing with things.
-  drive = os.environ.get('SystemDrive', 'C:')
+  drive = encoding.GetEncodedValue(os.environ, 'SystemDrive', 'C:')
   return os.path.join(drive, os.path.sep, _CLOUDSDK_GLOBAL_CONFIG_DIR_NAME)
 
 
