@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,22 +14,40 @@
 # limitations under the License.
 
 """General console printing utilities used by the Cloud SDK."""
+# TODO(b/71388306): Unskip pytype on this file.
+# pytype: skip-file
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from collections import OrderedDict
 import contextlib
+
+import io
+import json
 import os
 import re
 import subprocess
 import sys
 import textwrap
+import threading
 
+import enum
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.console import console_attr
 from googlecloudsdk.core.console import console_pager
+from googlecloudsdk.core.console import prompt_completer
+from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import files
+from googlecloudsdk.core.util import platforms
 
-FLOAT_COMPARE_EPSILON = 1e-6
+import six
+from six.moves import input  # pylint: disable=redefined-builtin
+from six.moves import map  # pylint: disable=redefined-builtin
+from six.moves import range  # pylint: disable=redefined-builtin
 
 
 class Error(exceptions.Error):
@@ -84,26 +103,54 @@ def _NarrowWrap(narrow_by):
   TEXTWRAP.width += narrow_by
 
 
-def _RawInput(prompt=None):
-  """A simple redirect to the built-in raw_input function.
-
-  If the prompt is given, it is correctly line wrapped.
-
-  Args:
-    prompt: str, An optional prompt.
-
-  Returns:
-    The input from stdin.
-  """
-  if prompt:
-    sys.stderr.write(_DoWrap(prompt))
-
+def _GetInput():
   try:
-    return raw_input()
+    return input()
   except EOFError:
     return None
 
 
+def ReadFromFileOrStdin(path, binary):
+  """Returns the contents of the specified file or stdin if path is '-'.
+
+  Args:
+    path: str, The path of the file to read.
+    binary: bool, True to open the file in binary mode.
+
+  Raises:
+    Error: If the file cannot be read or is larger than max_bytes.
+
+  Returns:
+    The contents of the file.
+  """
+  if path == '-':
+    return ReadStdin(binary=binary)
+  if binary:
+    return files.ReadBinaryFileContents(path)
+  return files.ReadFileContents(path)
+
+
+def ReadStdin(binary=False):
+  """Reads data from stdin, correctly accounting for encoding.
+
+  Anything that needs to read sys.stdin must go through this method.
+
+  Args:
+    binary: bool, True to read raw bytes, False to read text.
+
+  Returns:
+    A text string if binary is False, otherwise a byte string.
+  """
+  if binary:
+    return files.ReadStdinBytes()
+  else:
+    data = sys.stdin.read()
+    if six.PY2:
+      # On Python 2, stdin comes in a a byte string. Convert it to text.
+      data = console_attr.Decode(data)
+    return data
+
+
 def IsInteractive(output=False, error=False, heuristic=False):
   """Determines if the current terminal session is interactive.
 
@@ -124,6 +171,7 @@ def IsInteractive(output=False, error=False, heuristic=False):
     return False
   if error and not sys.stderr.isatty():
     return False
+
   if heuristic:
     # Check the home path. Most startup scripts for example are executed by
     # users that don't have a home path set. Home is OS dependent though, so
@@ -142,6 +190,15 @@ def IsInteractive(output=False, error=False, heuristic=False):
   return True
 
 
+def IsRunFromShellScript():
+  """Check if command is being run from command line or a script."""
+  # Commands run from a shell script typically have getppid() == getpgrp()
+  if platforms.OperatingSystem.Current() != platforms.OperatingSystem.WINDOWS:
+    if os.getppid() == os.getpgrp():
+      return True
+  return False
+
+
 def CanPrompt():
   """Returns true if we can prompt the user for information.
 
@@ -189,8 +246,62 @@ def PromptContinue(message=None, prompt_string=None, default=True,
       raise OperationCancelledError()
     return default
 
+  style = properties.VALUES.core.interactive_ux_style.Get()
+  prompt_generator = (
+      _TestPromptContinuePromptGenerator
+      if style == properties.VALUES.core.InteractiveUXStyles.TESTING.name
+      else _NormalPromptContinuePromptGenerator)
+
+  prompt, reprompt, ending = prompt_generator(
+      message=message, prompt_string=prompt_string, default=default,
+      throw_if_unattended=throw_if_unattended, cancel_on_no=cancel_on_no,
+      cancel_string=cancel_string)
+  sys.stderr.write(prompt)
+
+  def GetAnswer(reprompt):
+    """Get answer to input prompt."""
+    while True:
+      answer = _GetInput()
+      # pylint:disable=g-explicit-bool-comparison, We explicitly want to
+      # distinguish between empty string and None.
+      if answer == '':
+        # User just hit enter, return default.
+        return default
+      elif answer is None:
+        # This means we hit EOF, no input or user closed the stream.
+        if throw_if_unattended and not IsInteractive():
+          raise UnattendedPromptError()
+        else:
+          return default
+      elif answer.strip().lower() in ['y', 'yes']:
+        return True
+      elif answer.strip().lower() in ['n', 'no']:
+        return False
+      elif reprompt:
+        sys.stderr.write(reprompt)
+
+  try:
+    answer = GetAnswer(reprompt)
+  finally:
+    if ending:
+      sys.stderr.write(ending)
+
+  if not answer and cancel_on_no:
+    raise OperationCancelledError(cancel_string)
+  return answer
+
+
+def _NormalPromptContinuePromptGenerator(
+    message, prompt_string, default, throw_if_unattended, cancel_on_no,
+    cancel_string):
+  """Generates prompts for prompt continue under normal conditions."""
+  del throw_if_unattended
+  del cancel_on_no
+  del cancel_string
+
+  buf = io.StringIO()
   if message:
-    sys.stderr.write(_DoWrap(message) + '\n\n')
+    buf.write(_DoWrap(message) + '\n\n')
 
   if not prompt_string:
     prompt_string = 'Do you want to continue'
@@ -198,56 +309,44 @@ def PromptContinue(message=None, prompt_string=None, default=True,
     prompt_string += ' (Y/n)?  '
   else:
     prompt_string += ' (y/N)?  '
-  sys.stderr.write(_DoWrap(prompt_string))
+  buf.write(_DoWrap(prompt_string))
 
-  def GetAnswer():
-    while True:
-      answer = _RawInput()
-      # pylint:disable=g-explicit-bool-comparison, We explicitly want to
-      # distinguish between empty string and None.
-      if answer == '':
-        # User just hit enter, return default.
-        sys.stderr.write('\n')
-        return default
-      elif answer is None:
-        # This means we hit EOF, no input or user closed the stream.
-        if throw_if_unattended and not IsInteractive():
-          sys.stderr.write('\n')
-          raise UnattendedPromptError()
-        else:
-          sys.stderr.write('\n')
-          return default
-      elif answer.strip().lower() in ['y', 'yes']:
-        sys.stderr.write('\n')
-        return True
-      elif answer.strip().lower() in ['n', 'no']:
-        sys.stderr.write('\n')
-        return False
-      else:
-        sys.stderr.write("Please enter 'y' or 'n':  ")
-
-  answer = GetAnswer()
-  if not answer and cancel_on_no:
-    raise OperationCancelledError(cancel_string)
-  return answer
+  return (buf.getvalue(), "Please enter 'y' or 'n':  ", '\n')
 
 
-def PromptResponse(message):
+def _TestPromptContinuePromptGenerator(
+    message, prompt_string, default, throw_if_unattended, cancel_on_no,
+    cancel_string):
+  """Generates prompts for prompt continue under test."""
+  del default
+  del throw_if_unattended
+  del cancel_on_no
+  return (JsonUXStub(
+      UXElementType.PROMPT_CONTINUE, message=message,
+      prompt_string=prompt_string, cancel_string=cancel_string) + '\n',
+          None, None)
+
+
+def PromptResponse(message=None, choices=None):
   """Prompts the user for a string.
 
   Args:
     message: str, The prompt to print before the question.
+    choices: callable or list, A callable with no arguments that returns the
+      list of all choices, or the list of choices.
 
   Returns:
     str, The string entered by the user, or None if prompts are disabled.
   """
   if properties.VALUES.core.disable_prompts.GetBool():
     return None
-  response = _RawInput(message)
-  return response
+  if choices and IsInteractive(error=True):
+    return prompt_completer.PromptCompleter(message, choices=choices).Input()
+  sys.stderr.write(_DoWrap(message))
+  return _GetInput()
 
 
-def PromptWithDefault(message, default=None):
+def PromptWithDefault(message=None, default=None, choices=None):
   """Prompts the user for a string, allowing a default.
 
   Unlike PromptResponse, this also appends a ':  ' to the prompt.  If 'default'
@@ -261,21 +360,21 @@ def PromptWithDefault(message, default=None):
   Args:
     message: str, The prompt to print before the question.
     default: str, The default value (if any).
+    choices: callable or list, A callable with no arguments that returns the
+      list of all choices, or the list of choices.
 
   Returns:
     str, The string entered by the user, or the default if no value was
     entered or prompts are disabled.
   """
-  if properties.VALUES.core.disable_prompts.GetBool():
-    return default
+  message = message or ''
   if default:
-    message += ' ({default}):  '.format(default=default)
+    if message:
+      message += ' '
+    message += '({default}):  '.format(default=default)
   else:
     message += ':  '
-  response = _RawInput(message)
-  if not response:
-    response = default
-  return response
+  return PromptResponse(message, choices=choices) or default
 
 
 def _ParseAnswer(answer, options, allow_freeform):
@@ -308,7 +407,7 @@ def _ParseAnswer(answer, options, allow_freeform):
     return None
 
   try:
-    return map(str, options).index(answer) + 1
+    return list(map(str, options)).index(answer) + 1
   except ValueError:
     # Answer not an entry in the options list
     pass
@@ -336,19 +435,19 @@ def _SuggestFreeformAnswer(suggester, answer, options):
   return suggester.GetSuggestion(answer)
 
 
-def _PrintOptions(options, limit=None):
+def _PrintOptions(options, write, limit=None):
   """Prints the options provided to stderr.
 
   Args:
     options:  [object], A list of objects to print as choices.  Their str()
       method will be used to display them.
+    write: f(x)->None, A function to call to write the data.
     limit: int, If set, will only print the first number of options equal
       to the given limit.
   """
   limited_options = options if limit is None else options[:limit]
   for i, option in enumerate(limited_options):
-    sys.stderr.write(' [{index}] {option}\n'.format(
-        index=i + 1, option=str(option)))
+    write(' [{index}] {option}\n'.format(index=i + 1, option=str(option)))
 
 
 # This defines the point at which, in a PromptChoice, the options
@@ -400,24 +499,32 @@ def PromptChoice(options, default=None, message=None,
   if properties.VALUES.core.disable_prompts.GetBool():
     return default
 
+  style = properties.VALUES.core.interactive_ux_style.Get()
+  if style == properties.VALUES.core.InteractiveUXStyles.TESTING.name:
+    write = lambda x: None
+    sys.stderr.write(JsonUXStub(
+        UXElementType.PROMPT_CHOICE, message=message,
+        prompt_string=prompt_string,
+        choices=[six.text_type(o) for o in options]) + '\n')
+  else:
+    write = sys.stderr.write
+
   if message:
-    sys.stderr.write(_DoWrap(message) + '\n')
+    write(_DoWrap(message) + '\n')
 
   if maximum > PROMPT_OPTIONS_OVERFLOW:
-    _PrintOptions(options, limit=PROMPT_OPTIONS_OVERFLOW)
+    _PrintOptions(options, write, limit=PROMPT_OPTIONS_OVERFLOW)
     truncated = maximum - PROMPT_OPTIONS_OVERFLOW
-    sys.stderr.write('Did not print [{truncated}] options.\n'
-                     .format(truncated=truncated))
-    sys.stderr.write(('Too many options [{maximum}]. Enter "list" at '
-                      'prompt to print choices fully.\n').format(
-                          maximum=maximum))
+    write('Did not print [{truncated}] options.\n'.format(truncated=truncated))
+    write('Too many options [{maximum}]. Enter "list" at prompt to print '
+          'choices fully.\n'.format(maximum=maximum))
   else:
-    _PrintOptions(options)
+    _PrintOptions(options, write)
 
   if not prompt_string:
     if allow_freeform:
-      prompt_string = ('Please enter numeric choice or text value '
-                       '(must exactly match list item)')
+      prompt_string = ('Please enter numeric choice or text value (must exactly'
+                       ' match list item)')
     else:
       prompt_string = 'Please enter your numeric choice'
   if default is None:
@@ -426,29 +533,29 @@ def PromptChoice(options, default=None, message=None,
     suffix_string = ' ({default}):  '.format(default=default + 1)
 
   def _PrintPrompt():
-    sys.stderr.write(_DoWrap(prompt_string + suffix_string))
+    write(_DoWrap(prompt_string + suffix_string))
   _PrintPrompt()
 
   while True:
-    answer = _RawInput()
+    answer = _GetInput()
     if answer is None or (answer is '' and default is not None):
       # Return default if we failed to read from stdin.
       # Return default if the user hit enter and there is a valid default,
       # or raise OperationCancelledError if default is the cancel option.
       # Prompt again otherwise
-      sys.stderr.write('\n')
+      write('\n')
       if cancel_option and default == maximum - 1:
         raise OperationCancelledError()
       return default
     if answer == 'list':
-      _PrintOptions(options)
+      _PrintOptions(options, write)
       _PrintPrompt()
       continue
     num_choice = _ParseAnswer(answer, options, allow_freeform)
     if cancel_option and num_choice == maximum:
       raise OperationCancelledError()
     if num_choice is not None and num_choice >= 1 and num_choice <= maximum:
-      sys.stderr.write('\n')
+      write('\n')
       return num_choice - 1
 
     # Arriving here means that there is no choice matching the answer that
@@ -458,17 +565,48 @@ def PromptChoice(options, default=None, message=None,
                                           answer,
                                           options)
       if suggestion is not None:
-        sys.stderr.write('[{answer}] not in list. Did you mean [{suggestion}]?'
-                         .format(answer=answer, suggestion=suggestion))
-        sys.stderr.write('\n')
+        write('[{answer}] not in list. Did you mean [{suggestion}]?'
+              .format(answer=answer, suggestion=suggestion))
+        write('\n')
 
     if allow_freeform:
-      sys.stderr.write(('Please enter a value between 1 and {maximum}, '
-                        'or a value present in the list:  ')
-                       .format(maximum=maximum))
+      write('Please enter a value between 1 and {maximum}, or a value present '
+            'in the list:  '.format(maximum=maximum))
     else:
-      sys.stderr.write('Please enter a value between 1 and {maximum}:  '
-                       .format(maximum=maximum))
+      write('Please enter a value between 1 and {maximum}:  '
+            .format(maximum=maximum))
+
+
+def PromptWithValidator(validator, error_message, prompt_string,
+                        message=None):
+  """Prompts the user for a string that must pass a validator.
+
+  Args:
+    validator: function, A validation function that accepts a string and returns
+      a boolean value indicating whether or not the user input is valid.
+    error_message: str, Error message to display when user input does not pass
+      in a valid string.
+    prompt_string: str, A string to print when prompting the user to enter a
+      choice.  If not given, a default prompt is used.
+    message: str, An optional message to print before prompting.
+
+  Returns:
+    str, The string entered by the user, or the default if no value was
+    entered or prompts are disabled.
+  """
+  if properties.VALUES.core.disable_prompts.GetBool():
+    return None
+
+  if message:
+    sys.stderr.write(_DoWrap(message) + '\n')
+
+  while True:
+    sys.stderr.write(_DoWrap(prompt_string))
+    answer = _GetInput()
+    if validator(answer):
+      return answer
+    else:
+      sys.stderr.write(_DoWrap(error_message) + '\n')
 
 
 def LazyFormat(s, **kwargs):
@@ -528,7 +666,84 @@ def FormatRequiredUserAction(s):
     return '\n==> ' + '\n==> '.join(wrapper.wrap(s)) + '\n'
 
 
-class ProgressBar(object):
+def ProgressBar(label, stream=log.status, total_ticks=60, first=True,
+                last=True):
+  """A simple progress bar for tracking completion of an action.
+
+  This progress bar works without having to use any control characters.  It
+  prints the action that is being done, and then fills a progress bar below it.
+  You should not print anything else on the output stream during this time as it
+  will cause the progress bar to break on lines.
+
+  Progress bars can be stacked into a group. first=True marks the first bar in
+  the group and last=True marks the last bar in the group. The default assumes
+  a singleton bar with first=True and last=True.
+
+  This class can also be used in a context manager.
+
+  Args:
+    label: str, The action that is being performed.
+    stream: The output stream to write to, stderr by default.
+    total_ticks: int, The number of ticks wide to make the progress bar.
+    first: bool, True if this is the first bar in a stacked group.
+    last: bool, True if this is the last bar in a stacked group.
+
+  Returns:
+    The progress bar.
+  """
+  style = properties.VALUES.core.interactive_ux_style.Get()
+  if style == properties.VALUES.core.InteractiveUXStyles.OFF.name:
+    return NoOpProgressBar()
+  elif style == properties.VALUES.core.InteractiveUXStyles.TESTING.name:
+    return _StubProgressBar(label, stream)
+  else:
+    return _NormalProgressBar(label, stream, total_ticks, first, last)
+
+
+def SplitProgressBar(original_callback, weights):
+  """Splits a progress bar into logical sections.
+
+  Wraps the original callback so that each of the subsections can use the full
+  range of 0 to 1 to indicate its progress.  The overall progress bar will
+  display total progress based on the weights of the tasks.
+
+  Args:
+    original_callback: f(float), The original callback for the progress bar.
+    weights: [float], The weights of the tasks to create.  These can be any
+      numbers you want and the split will be based on their proportions to
+      each other.
+
+  Raises:
+    ValueError: If the weights don't add up to 1.
+
+  Returns:
+    (f(float), ), A tuple of callback functions, in order, for the subtasks.
+  """
+  if (original_callback is None or
+      original_callback == DefaultProgressBarCallback):
+    return tuple([DefaultProgressBarCallback for _ in range(len(weights))])
+
+  def MakeCallback(already_done, weight):
+    def Callback(done_fraction):
+      original_callback(already_done + (done_fraction * weight))
+    return Callback
+
+  total = sum(weights)
+  callbacks = []
+  already_done = 0
+  for weight in weights:
+    normalized_weight = weight / total
+    callbacks.append(MakeCallback(already_done, normalized_weight))
+    already_done += normalized_weight
+
+  return tuple(callbacks)
+
+
+def DefaultProgressBarCallback(progress_factor):
+  del progress_factor
+
+
+class _NormalProgressBar(object):
   """A simple progress bar for tracking completion of an action.
 
   This progress bar works without having to use any control characters.  It
@@ -543,53 +758,7 @@ class ProgressBar(object):
   This class can also be used in a context manager.
   """
 
-  @staticmethod
-  def _DefaultCallback(progress_factor):
-    pass
-
-  DEFAULT_CALLBACK = _DefaultCallback
-
-  @staticmethod
-  def SplitProgressBar(original_callback, weights):
-    """Splits a progress bar into logical sections.
-
-    Wraps the original callback so that each of the subsections can use the full
-    range of 0 to 1 to indicate its progress.  The overall progress bar will
-    display total progress based on the weights of the tasks.
-
-    Args:
-      original_callback: f(float), The original callback for the progress bar.
-      weights: [float], The weights of the tasks to create.  These can be any
-        numbers you want and the split will be based on their proportions to
-        each other.
-
-    Raises:
-      ValueError: If the weights don't add up to 1.
-
-    Returns:
-      (f(float), ), A tuple of callback functions, in order, for the subtasks.
-    """
-    if (original_callback is None or
-        original_callback == ProgressBar.DEFAULT_CALLBACK):
-      return tuple([ProgressBar.DEFAULT_CALLBACK for _ in range(len(weights))])
-
-    def MakeCallback(already_done, weight):
-      def Callback(done_fraction):
-        original_callback(already_done + (done_fraction * weight))
-      return Callback
-
-    total = float(sum(weights))
-    callbacks = []
-    already_done = 0
-    for weight in weights:
-      normalized_weight = weight / total
-      callbacks.append(MakeCallback(already_done, normalized_weight))
-      already_done += normalized_weight
-
-    return tuple(callbacks)
-
-  def __init__(self, label, stream=log.status, total_ticks=60, first=True,
-               last=True):
+  def __init__(self, label, stream, total_ticks, first, last):
     """Creates a progress bar for the given action.
 
     Args:
@@ -599,12 +768,13 @@ class ProgressBar(object):
       first: bool, True if this is the first bar in a stacked group.
       last: bool, True if this is the last bar in a stacked group.
     """
+    self._raw_label = label
     self._stream = stream
     self._ticks_written = 0
     self._total_ticks = total_ticks
     self._first = first
     self._last = last
-    attr = console_attr.ConsoleAttr(out=stream)
+    attr = console_attr.ConsoleAttr()
     self._box = attr.GetBoxLineCharacters()
     self._redraw = (self._box.d_dr != self._box.d_vr or
                     self._box.d_dl != self._box.d_vl)
@@ -625,15 +795,15 @@ class ProgressBar(object):
       label += ' ' * diff
     left = self._box.d_vr + self._box.d_h
     right = self._box.d_h + self._box.d_vl
-    self._label = u'{left} {label} {right}'.format(left=left, label=label,
-                                                   right=right)
+    self._label = '{left} {label} {right}'.format(
+        left=left, label=label, right=right)
 
   def Start(self):
     """Starts the progress bar by writing the top rule and label."""
     if self._first or self._redraw:
       left = self._box.d_dr if self._first else self._box.d_vr
       right = self._box.d_dl if self._first else self._box.d_vl
-      rule = u'{left}{middle}{right}\n'.format(
+      rule = '{left}{middle}{right}\n'.format(
           left=left, middle=self._box.d_h * self._total_ticks, right=right)
       self._Write(rule)
     self._Write(self._label + '\n')
@@ -677,6 +847,61 @@ class ProgressBar(object):
     self.Finish()
 
 
+class NoOpProgressBar(object):
+  """A progress bar that outputs nothing at all."""
+
+  def __init__(self):
+    pass
+
+  def Start(self):
+    pass
+
+  def SetProgress(self, progress_factor):
+    pass
+
+  def Finish(self):
+    """Mark the progress as done."""
+    self.SetProgress(1)
+
+  def __enter__(self):
+    self.Start()
+    return self
+
+  def __exit__(self, *args):
+    self.Finish()
+
+
+class _StubProgressBar(object):
+  """A progress bar that only prints deterministic start and end points.
+
+  No UX about progress should be exposed here. This is strictly for being able
+  to tell that the progress bar was invoked, not what it actually looks like.
+  """
+
+  def __init__(self, label, stream):
+    self._raw_label = label
+    self._stream = stream
+
+  def Start(self):
+    self._stream.write(
+        JsonUXStub(UXElementType.PROGRESS_BAR, message=self._raw_label))
+
+  def SetProgress(self, progress_factor):
+    pass
+
+  def Finish(self):
+    """Mark the progress as done."""
+    self.SetProgress(1)
+    self._stream.write('\n')
+
+  def __enter__(self):
+    self.Start()
+    return self
+
+  def __exit__(self, *args):
+    self.Finish()
+
+
 def More(contents, out=None, prompt=None, check_pager=True):
   """Run a user specified pager or fall back to the internal pager.
 
@@ -697,7 +922,7 @@ def More(contents, out=None, prompt=None, check_pager=True):
     # Paging shenanigans to stdout.
     out = sys.stdout
   if check_pager:
-    pager = os.environ.get('PAGER', None)
+    pager = encoding.GetEncodedValue(os.environ, 'PAGER', None)
     if pager == '-':
       # Use the fallback Pager.
       pager = None
@@ -708,27 +933,30 @@ def More(contents, out=None, prompt=None, check_pager=True):
           pager = command
           break
     if pager:
-      less = os.environ.get('LESS', None)
-      if less is None:
-        os.environ['LESS'] = '-R'
+      # If the pager is less(1) then instruct it to display raw ANSI escape
+      # sequences to enable colors and font embellishments.
+      less_orig = encoding.GetEncodedValue(os.environ, 'LESS', None)
+      less = '-R' + (less_orig or '')
+      encoding.SetEncodedValue(os.environ, 'LESS', less)
       p = subprocess.Popen(pager, stdin=subprocess.PIPE, shell=True)
-      encoding = console_attr.GetConsoleAttr().GetEncoding()
-      p.communicate(input=contents.encode(encoding))
+      enc = console_attr.GetConsoleAttr().GetEncoding()
+      p.communicate(input=contents.encode(enc))
       p.wait()
-      if less is None:
-        os.environ.pop('LESS')
+      if less_orig is None:
+        encoding.SetEncodedValue(os.environ, 'LESS', None)
       return
   # Fall back to the internal pager.
   console_pager.Pager(contents, out, prompt).Run()
 
 
 class TickableProgressBar(object):
-  """A progress bar with a discrete number of tasks."""
+  """A thread safe progress bar with a discrete number of tasks."""
 
   def __init__(self, total, *args, **kwargs):
     self.completed = 0
     self.total = total
     self._progress_bar = ProgressBar(*args, **kwargs)
+    self._lock = threading.Lock()
 
   def __enter__(self):
     self._progress_bar.__enter__()
@@ -738,5 +966,39 @@ class TickableProgressBar(object):
     self._progress_bar.__exit__(exc_type, exc_value, traceback)
 
   def Tick(self):
-    self.completed += 1
-    self._progress_bar.SetProgress(float(self.completed) / self.total)
+    with self._lock:
+      self.completed += 1
+      self._progress_bar.SetProgress(self.completed / self.total)
+
+
+def JsonUXStub(ux_type, **kwargs):
+  """Generates a stub message for UX console output."""
+  output = OrderedDict()
+  output['ux'] = ux_type.name
+  extra_args = list(set(kwargs) - set(ux_type.GetDataFields()))
+  if extra_args:
+    raise ValueError('Extraneous args for Ux Element {}: {}'.format(
+        ux_type.name, extra_args))
+  for field in ux_type.GetDataFields():
+    val = kwargs.get(field, None)
+    if val:
+      output[field] = val
+  return json.dumps(output)
+
+
+class UXElementType(enum.Enum):
+  """Describes the type of a ux element."""
+  PROGRESS_BAR = (['message'])
+  PROGRESS_TRACKER = (['message', 'aborted_message', 'status'])
+  STAGED_PROGRESS_TRACKER = (['message', 'status',
+                              'succeeded_stages', 'failed_stage'])
+  PROMPT_CONTINUE = (['message', 'prompt_string', 'cancel_string'])
+  PROMPT_RESPONSE = (['choices'])
+  PROMPT_CHOICE = (['message', 'prompt_string', 'choices'])
+
+  def __init__(self, data_fields):
+    self._data_fields = data_fields
+
+  def GetDataFields(self):
+    """Returns the ordered list of additional fields in the UX Element."""
+    return self._data_fields
