@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,13 +17,18 @@
 
 """
 
-from httplib import ResponseNotReady
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.core import log
 from googlecloudsdk.core.util import pkg_resources
 from googlecloudsdk.core.util import platforms
 
 from oauth2client import client
 from oauth2client import tools
+from six.moves import input  # pylint: disable=redefined-builtin
+from six.moves.http_client import ResponseNotReady
 
 
 try:
@@ -61,7 +67,7 @@ class ClientRedirectHandler(tools.ClientRedirectHandler):
     if an error occurred.
     """
     self.send_response(200)
-    self.send_header('Content-type', 'text/html')
+    self.send_header('Content-type', 'text/html')  # pytype: disable=wrong-arg-types
     self.end_headers()
     query = self.path.split('?', 1)[-1]
     query = dict(parse_qsl(query))
@@ -155,13 +161,13 @@ def Run(flow, launch_browser=True, http=None,
                  'between {start_port} and {end_port}. Please check your '
                  'firewall settings or locally running programs that may be '
                  'blocking or using those ports.')
-      log.warn(message.format(
+      log.warning(message.format(
           start_port=auth_host_port_start,
           end_port=port_number,
       ))
 
       launch_browser = False
-      log.warn('Defaulting to URL copy/paste mode.')
+      log.warning('Defaulting to URL copy/paste mode.')
 
   if not launch_browser:
     flow.redirect_uri = client.OOB_CALLBACK_URN
@@ -172,7 +178,7 @@ def Run(flow, launch_browser=True, http=None,
         url=authorize_url,
     ))
     try:
-      code = raw_input('Enter verification code: ').strip()
+      code = input('Enter verification code: ').strip()
     except EOFError as e:
       raise AuthRequestRejectedException(e)
 
