@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,6 +18,10 @@
 A detailed description of auth.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import datetime
 import json
 import os
@@ -31,10 +36,12 @@ from googlecloudsdk.core.credentials import creds
 from googlecloudsdk.core.credentials import devshell as c_devshell
 from googlecloudsdk.core.credentials import gce as c_gce
 from googlecloudsdk.core.util import files
+
 import httplib2
 from oauth2client import client
 from oauth2client.contrib import gce as oauth2client_gce
 from oauth2client.contrib import reauth_errors
+import six
 
 
 GOOGLE_OAUTH2_PROVIDER_AUTHORIZATION_URI = (
@@ -329,7 +336,9 @@ def Load(account=None, scopes=None, prevent_refresh=False):
       if cred_type in (creds.CredentialType.SERVICE_ACCOUNT,
                        creds.CredentialType.P12_SERVICE_ACCOUNT):
         cred.token_uri = token_uri_override
-    return cred
+    # The credential override is not stored in credential store, but we still
+    # want to cache access tokens between invocations.
+    return creds.MaybeAttachAccessTokenCacheStore(cred)
 
   if not account:
     account = properties.VALUES.core.account.Get()
@@ -368,10 +377,12 @@ def Refresh(credentials, http_client=None):
     TokenRefreshError: If the credentials fail to refresh.
     TokenRefreshReauthError: If the credentials fail to refresh due to reauth.
   """
+  response_encoding = None if six.PY2 else 'utf-8'
   try:
-    credentials.refresh(http_client or http.Http())
+    credentials.refresh(http_client or
+                        http.Http(response_encoding=response_encoding))
   except (client.AccessTokenRefreshError, httplib2.ServerNotFoundError) as e:
-    raise TokenRefreshError(e.message)
+    raise TokenRefreshError(six.text_type(e))
   except reauth_errors.ReauthError as e:
     raise TokenRefreshReauthError(e.message)
 
@@ -624,10 +635,10 @@ def SaveCredentialsAsADC(credentials, file_path):
         credentials.user_agent,
         credentials.revoke_uri)
   try:
-    with files.OpenForWritingPrivate(file_path) as f:
-      json.dump(credentials.serialization_data, f, sort_keys=True,
-                indent=2, separators=(',', ': '))
-  except IOError as e:
+    contents = json.dumps(credentials.serialization_data, sort_keys=True,
+                          indent=2, separators=(',', ': '))  # pytype: disable=wrong-arg-types
+    files.WriteFileContents(file_path, contents, private=True)
+  except files.Error as e:
     log.debug(e, exc_info=True)
     raise CredentialFileSaveError(
         'Error saving Application Default Credentials: ' + str(e))
@@ -707,9 +718,7 @@ class _LegacyGenerator(object):
       cred = self.credentials
       key = cred._private_key_pkcs12  # pylint: disable=protected-access
       password = cred._private_key_password  # pylint: disable=protected-access
-
-      with files.OpenForWritingPrivate(self._p12_key_path, binary=True) as pk:
-        pk.write(key)
+      files.WriteBinaryFileContents(self._p12_key_path, key, private=True)
 
       # the .boto file gets some different fields
       self._WriteFileContents(
@@ -731,9 +740,4 @@ class _LegacyGenerator(object):
     """
 
     full_path = os.path.realpath(os.path.expanduser(filepath))
-
-    try:
-      with files.OpenForWritingPrivate(full_path) as cred_file:
-        cred_file.write(contents)
-    except (OSError, IOError) as e:
-      raise Exception('Failed to open %s for writing: %s' % (filepath, e))
+    files.WriteFileContents(full_path, contents, private=True)
