@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,14 +14,18 @@
 # limitations under the License.
 
 """A module to get an unauthenticated http object."""
+# TODO(b/71388306): Unskip pytype on this file.
+# pytype: skip-file
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
-import copy
 import platform
+import re
 import time
-import urllib
-import urlparse
 import uuid
+import enum
 
 from googlecloudsdk.core import config
 from googlecloudsdk.core import http_proxy
@@ -28,11 +33,16 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import metrics
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.console import console_io
+from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import platforms
+
 import httplib2
+import six
+from six.moves import urllib
+from six.moves import zip  # pylint: disable=redefined-builtin
 
 
-def Http(timeout='unset'):
+def Http(timeout='unset', response_encoding=None, ca_certs=None):
   """Get an httplib2.Http client that is properly configured for use by gcloud.
 
   This method does not add credentials to the client.  For an Http client that
@@ -42,36 +52,52 @@ def Http(timeout='unset'):
     timeout: double, The timeout in seconds to pass to httplib2.  This is the
         socket level timeout.  If timeout is None, timeout is infinite.  If
         default argument 'unset' is given, a sensible default is selected.
+    response_encoding: str, the encoding to use to decode the response.
+    ca_certs: str, absolute filename of a ca_certs file that overrides the
+        default. The gcloud config property for ca_certs, in turn, overrides
+        this argument.
 
   Returns:
     An httplib2.Http client object configured with all the required settings
     for gcloud.
   """
-  # Compared with setting the default timeout in the function signature (i.e.
-  # timeout=300), this lets you test with short default timeouts by mocking
-  # GetDefaultTimeout.
-  effective_timeout = timeout if timeout != 'unset' else GetDefaultTimeout()
-  no_validate = properties.VALUES.auth.disable_ssl_validation.GetBool()
-  ca_certs = properties.VALUES.core.custom_ca_certs_file.Get()
-  http_client = httplib2.Http(timeout=effective_timeout,
-                              proxy_info=http_proxy.GetHttpProxyInfo(),
-                              ca_certs=ca_certs,
-                              disable_ssl_certificate_validation=no_validate)
-
   # Wrap the request method to put in our own user-agent, and trace reporting.
   gcloud_ua = MakeUserAgentString(properties.VALUES.metrics.command_name.Get())
+  http_client = _CreateRawHttpClient(timeout, ca_certs)
   http_client = _Wrap(
       http_client,
       properties.VALUES.core.trace_token.Get(),
       properties.VALUES.core.trace_email.Get(),
       properties.VALUES.core.trace_log.GetBool(),
+      properties.VALUES.core.request_reason.Get(),
       gcloud_ua,
-      properties.VALUES.core.log_http.GetBool()
+      properties.VALUES.core.log_http.GetBool(),
+      properties.VALUES.core.log_http_redact_token.GetBool(),
+      response_encoding
   )
 
   return http_client
 
 
+def _CreateRawHttpClient(timeout='unset', ca_certs=None):
+  """Create an HTTP client matching the appropriate gcloud properties."""
+  # Compared with setting the default timeout in the function signature (i.e.
+  # timeout=300), this lets you test with short default timeouts by mocking
+  # GetDefaultTimeout.
+  effective_timeout = timeout if timeout != 'unset' else GetDefaultTimeout()
+  no_validate = properties.VALUES.auth.disable_ssl_validation.GetBool() or False
+  ca_certs_property = properties.VALUES.core.custom_ca_certs_file.Get()
+  # Believe an explicitly-set ca_certs property over anything we added.
+  if ca_certs_property:
+    ca_certs = ca_certs_property
+  if no_validate:
+    ca_certs = None
+  return httplib2.Http(timeout=effective_timeout,
+                       proxy_info=http_proxy.GetHttpProxyInfo(),
+                       ca_certs=ca_certs,
+                       disable_ssl_certificate_validation=no_validate)
+
+
 def MakeUserAgentString(cmd_path=None):
   """Return a user-agent string for this request.
 
@@ -84,22 +110,25 @@ def MakeUserAgentString(cmd_path=None):
   Returns:
     str, User Agent string.
   """
-  return ('gcloud/{0}'
-          ' command/{1}'
-          ' invocation-id/{2}'
-          ' environment/{3}'
-          ' environment-version/{4}'
-          ' interactive/{5}'
-          ' python/{6}'
-          ' {7}').format(
-              config.CLOUD_SDK_VERSION.replace(' ', '_'),
-              cmd_path or properties.VALUES.metrics.command_name.Get(),
-              uuid.uuid4().hex,
-              properties.GetMetricsEnvironment(),
-              properties.VALUES.metrics.environment_version.Get(),
-              console_io.IsInteractive(error=True, heuristic=True),
-              platform.python_version(),
-              platforms.Platform.Current().UserAgentFragment())
+  return ('gcloud/{version}'
+          ' command/{cmd}'
+          ' invocation-id/{inv_id}'
+          ' environment/{environment}'
+          ' environment-version/{env_version}'
+          ' interactive/{is_interactive}'
+          ' from-script/{from_script}'
+          ' python/{py_version}'
+          ' {ua_fragment}').format(
+              version=config.CLOUD_SDK_VERSION.replace(' ', '_'),
+              cmd=(cmd_path or properties.VALUES.metrics.command_name.Get()),
+              inv_id=uuid.uuid4().hex,
+              environment=properties.GetMetricsEnvironment(),
+              env_version=properties.VALUES.metrics.environment_version.Get(),
+              is_interactive=console_io.IsInteractive(error=True,
+                                                      heuristic=True),
+              py_version=platform.python_version(),
+              ua_fragment=platforms.Platform.Current().UserAgentFragment(),
+              from_script=console_io.IsRunFromShellScript())
 
 
 def GetDefaultTimeout():
@@ -107,16 +136,21 @@ def GetDefaultTimeout():
 
 
 def _Wrap(
-    http_client, trace_token, trace_email, trace_log, gcloud_ua, log_http):
+    http_client, trace_token, trace_email, trace_log, request_reason, gcloud_ua,
+    log_http, log_http_redact_token, response_encoding):
   """Wrap request with user-agent, and trace reporting.
 
   Args:
     http_client: The original http object.
     trace_token: str, Token to be used to route service request traces.
     trace_email: str, username to which service request traces should be sent.
-    trace_log: bool, Enable/diable server side logging of service requests.
+    trace_log: bool, Enable/disable server side logging of service requests.
+    request_reason: str, Justification for access.
     gcloud_ua: str, User agent string to be included in the request.
     log_http: bool, True to enable request/response logging.
+    log_http_redact_token: bool, True to avoid logging access tokens if log_http
+                           is set.
+    response_encoding: str, the encoding to use to decode the response.
 
   Returns:
     http, The same http object but with the request method wrapped.
@@ -142,13 +176,18 @@ def _Wrap(
     handlers.append(Modifiers.Handler(
         Modifiers.AddQueryParam('trace', trace_value)))
 
+  if request_reason:
+    handlers.append(Modifiers.Handler(
+        Modifiers.SetHeader('X-Goog-Request-Reason', request_reason)))
+
   # Do this one last so that it sees the affects of the other modifiers.
   if log_http:
     handlers.append(Modifiers.Handler(
-        Modifiers.LogRequest(),
+        Modifiers.LogRequest(log_http_redact_token),
         Modifiers.LogResponse()))
 
-  return Modifiers.WrapRequest(http_client, handlers)
+  return Modifiers.WrapRequest(http_client, handlers,
+                               response_encoding=response_encoding)
 
 
 class Modifiers(object):
@@ -209,8 +248,8 @@ class Modifiers(object):
       self.data = data
 
   @classmethod
-  def WrapRequest(cls, http_client, handlers,
-                  exc_handler=None, exc_type=Exception):
+  def WrapRequest(cls, http_client, handlers, exc_handler=None,
+                  exc_type=Exception, response_encoding=None):
     """Wraps an http client with request modifiers.
 
     Args:
@@ -221,6 +260,7 @@ class Modifiers(object):
         should also throw an exception if you don't want it to be swallowed.
       exc_type: The type of exception that should be caught and given to the
         handler.
+      response_encoding: str, the encoding to use to decode the response.
 
     Returns:
       The wrapped http client.
@@ -229,13 +269,26 @@ class Modifiers(object):
 
     def WrappedRequest(*args, **kwargs):
       """Replacement http.request() method."""
-      modified_args = args
+      modified_args = list(args)
+
+      if not six.PY2:
+        # httplib2 needs text under Python 3.
+        RequestParam.URI.Set(
+            modified_args, kwargs,
+            encoding.Decode(RequestParam.URI.Get(modified_args, kwargs)))
+
       # We need to make a copy here because if we don't we will be modifying the
       # dictionary that people pass in.
       # TODO(b/37281703): Copy the entire dictionary. This is blocked on making
       # sure anything that comes through is actually copyable.
-      if 'headers' in kwargs:
-        kwargs['headers'] = copy.copy(kwargs['headers'])
+      headers = RequestParam.HEADERS.Get(modified_args, kwargs) or {}
+      # httplib2 adds headers and body together to POST messages. If headers are
+      # text, it prevents POSTing bytes because you will get a decode error when
+      # it combines them. Normalize everything to bytes here so we call request
+      # consistently.
+      headers = Modifiers._EncodeHeaders(headers)
+      RequestParam.HEADERS.Set(modified_args, kwargs, headers)
+
       modifier_data = []
 
       for handler in handlers:
@@ -253,6 +306,9 @@ class Modifiers(object):
         else:
           raise
 
+      if response_encoding is not None:
+        response = Modifiers._DecodeResponse(response, response_encoding)
+
       for handler, data in zip(handlers, modifier_data):
         if handler.response:
           handler.response(response, data)
@@ -268,6 +324,26 @@ class Modifiers(object):
 
     return http_client
 
+  @classmethod
+  def _EncodeHeaders(cls, headers):
+    return dict(
+        Modifiers._EncodeHeader(h, v) for h, v in six.iteritems(headers))
+
+  @classmethod
+  def _EncodeHeader(cls, header, value):
+    if isinstance(header, six.text_type):
+      header = header.encode('utf8')
+    if isinstance(value, six.text_type):
+      value = value.encode('utf8')
+    return header, value
+
+  @classmethod
+  def _DecodeResponse(cls, response, response_encoding):
+    """Decodes the response content if an encoding is given."""
+    response, content = response
+    content = content.decode(response_encoding)
+    return response, content
+
   @classmethod
   def AppendToHeader(cls, header, value):
     """Appends the given value to the existing value in the http request.
@@ -279,11 +355,15 @@ class Modifiers(object):
     Returns:
       A function that can be used in a Handler.request.
     """
+    header, value = Modifiers._EncodeHeader(header, value)
     def _AppendToHeader(args, kwargs):
       """Replacement http.request() method."""
-      current_value = Modifiers._GetHeader(args, kwargs, header, '')
-      new_value = '{0} {1}'.format(current_value, value).strip()
-      modified_args = Modifiers._SetHeader(args, kwargs, header, new_value)
+      modified_args = list(args)
+      headers = RequestParam.HEADERS.Get(args, kwargs) or {}
+      current_value = headers.get(header, b'')
+      headers[header] = (
+          (current_value + b' ' + value).strip() if current_value else value)
+      RequestParam.HEADERS.Set(modified_args, kwargs, headers)
       return Modifiers.Result(args=modified_args)
     return _AppendToHeader
 
@@ -298,9 +378,13 @@ class Modifiers(object):
     Returns:
       A function that can be used in a Handler.request.
     """
+    header, value = Modifiers._EncodeHeader(header, value)
     def _SetHeader(args, kwargs):
       """Replacement http.request() method."""
-      modified_args = Modifiers._SetHeader(args, kwargs, header, value)
+      modified_args = list(args)
+      headers = RequestParam.HEADERS.Get(args, kwargs) or {}
+      headers[header] = value
+      RequestParam.HEADERS.Set(modified_args, kwargs, headers)
       return Modifiers.Result(args=modified_args)
     return _SetHeader
 
@@ -315,66 +399,99 @@ class Modifiers(object):
     Returns:
       A function that can be used in a Handler.request.
     """
-    def _AddQueryParam(args, unused_kwargs):
+    def _AddQueryParam(args, kwargs):
       """Replacement http.request() method."""
-      url_parts = urlparse.urlsplit(args[0])
-      query_params = urlparse.parse_qs(url_parts.query)
+      url_parts = urllib.parse.urlsplit(
+          RequestParam.URI.Get(args, kwargs))
+      query_params = urllib.parse.parse_qs(url_parts.query)
       query_params[param] = value
       # Need to do this to convert a SplitResult into a list so it can be
       # modified.
       url_parts = list(url_parts)
-      url_parts[3] = urllib.urlencode(query_params, doseq=True)
+      url_parts[3] = urllib.parse.urlencode(query_params, doseq=True)
+
       modified_args = list(args)
-      modified_args[0] = urlparse.urlunsplit(url_parts)
+      # pylint:disable=too-many-function-args, This is just bogus.
+      new_url = urllib.parse.urlunsplit(url_parts)
+      RequestParam.URI.Set(modified_args, kwargs, new_url)
       return Modifiers.Result(args=modified_args)
     return _AddQueryParam
 
   @classmethod
-  def LogRequest(cls):
+  def LogRequest(cls, redact_token=True):
     """Logs the contents of the http request.
 
+    Args:
+      redact_token: bool, True to redact auth tokens.
+
     Returns:
       A function that can be used in a Handler.request.
     """
     def _LogRequest(args, kwargs):
       """Replacement http.request() method."""
-      # http.request has the following signature:
-      # request(self, uri, method="GET", body=None, headers=None,
-      #         redirections=DEFAULT_MAX_REDIRECTS, connection_type=None)
-      uri = args[0]
-      method = 'GET'
-      body = ''
-      headers = {}
 
-      if len(args) > 1:
-        method = args[1]
-      elif 'method' in kwargs:
-        method = kwargs['method']
-      if len(args) > 2:
-        body = args[2]
-        if len(args) > 3:
-          headers = args[3]
-      if 'body' in kwargs:
-        body = kwargs['body']
-      if 'headers' in kwargs:
-        headers = kwargs['headers']
+      uri, method, body, headers = Modifiers._GetRequest(args, kwargs)
+
+      # If set, these prevent the printing of the http body and replace it with
+      # the reason the body is not being printed.
+      redact_req_body_reason = None
+      redact_resp_body_reason = None
+
+      if redact_token and IsTokenUri(uri):
+        redact_req_body_reason = (
+            'Contains oauth token. Set log_http_redact_token property to false '
+            'to print the body of this request.'
+        )
+        redact_resp_body_reason = (
+            'Contains oauth token. Set log_http_redact_token property to false '
+            'to print the body of this response.'
+        )
 
       log.status.Print('=======================')
       log.status.Print('==== request start ====')
       log.status.Print('uri: {uri}'.format(uri=uri))
       log.status.Print('method: {method}'.format(method=method))
       log.status.Print('== headers start ==')
-      for h, v in sorted(headers.iteritems()):
+      for h, v in sorted(six.iteritems(headers)):
+        if redact_token and (h == b'Authorization' or
+                             h == b'x-goog-iam-authorization-token'):
+          v = '--- Token Redacted ---'
         log.status.Print('{0}: {1}'.format(h, v))
       log.status.Print('== headers end ==')
       log.status.Print('== body start ==')
-      log.status.Print(body)
+      if redact_req_body_reason is None:
+        log.status.Print(body)
+      else:
+        log.status.Print('Body redacted: {}'.format(redact_req_body_reason))
       log.status.Print('== body end ==')
       log.status.Print('==== request end ====')
 
-      return Modifiers.Result(data=time.time())
+      return Modifiers.Result(data={
+          'start_time': time.time(),
+          'redact_resp_body_reason': redact_resp_body_reason,
+      })
     return _LogRequest
 
+  @classmethod
+  def DumpRequest(cls, capturer):
+    """Dumps the contents of the http request to capturer.
+
+    Args:
+      capturer: SessionCapturer, instance to pass request to
+
+    Returns:
+      A function that can be used in a Handler.request.
+    """
+
+    def _DumpRequest(args, kwargs):
+      """Replacement http.request() method."""
+
+      capturer.CaptureHttpRequest(*Modifiers._GetRequest(args, kwargs))
+
+      return Modifiers.Result()
+
+    return _DumpRequest
+
   @classmethod
   def LogResponse(cls):
     """Logs the contents of the http response.
@@ -382,17 +499,22 @@ class Modifiers(object):
     Returns:
       A function that can be used in a Handler.response.
     """
-    def _LogResponse(response, start_time):
+    def _LogResponse(response, data):
       """Response handler."""
-      time_taken = time.time() - start_time
+      redact_resp_body_reason = data['redact_resp_body_reason']
+
+      time_taken = time.time() - data['start_time']
       headers, content = response
       log.status.Print('---- response start ----')
       log.status.Print('-- headers start --')
-      for h, v in sorted(headers.iteritems()):
+      for h, v in sorted(six.iteritems(headers)):
         log.status.Print('{0}: {1}'.format(h, v))
       log.status.Print('-- headers end --')
       log.status.Print('-- body start --')
-      log.status.Print(content)
+      if redact_resp_body_reason is None:
+        log.status.Print(content)
+      else:
+        log.status.Print('Body redacted: {}'.format(redact_resp_body_reason))
       log.status.Print('-- body end --')
       log.status.Print('total round trip time (request+response): {0:.3f} secs'
                        .format(time_taken))
@@ -400,6 +522,23 @@ class Modifiers(object):
       log.status.Print('----------------------')
     return _LogResponse
 
+  @classmethod
+  def DumpResponse(cls, capturer):
+    """Dumps the contents of the http response to capturer.
+
+    Args:
+      capturer: SessionCapturer, instance to pass response to
+
+    Returns:
+      A function that can be used in a Handler.request.
+    """
+
+    def _DumpResponse(response, unused_args):
+      """Response handler."""
+      capturer.CaptureHttpResponse(response[0], response[1])
+
+    return _DumpResponse
+
   @classmethod
   def RecordStartTime(cls):
     """Records the time at which the request was started.
@@ -427,35 +566,55 @@ class Modifiers(object):
     return _ReportDuration
 
   @classmethod
-  def _GetHeader(cls, args, kwargs, header, default=None):
-    """Get a header given the args and kwargs of an Http Request call."""
+  def _GetRequest(cls, args, kwargs):
+    """Parse args and kwargs to get uri, method, body, headers."""
+    # http.request has the following signature:
+    # request(self, uri, method="GET", body=None, headers=None,
+    #         redirections=DEFAULT_MAX_REDIRECTS, connection_type=None)
 
-    if 'headers' in kwargs:
-      return kwargs['headers'].get(header, default)
-    elif len(args) > 3 and args[3]:
-      return args[3].get(header, default)
+    uri = RequestParam.URI.Get(args, kwargs)
+    method = RequestParam.METHOD.Get(args, kwargs) or 'GET'
+    body = RequestParam.BODY.Get(args, kwargs) or ''
+    headers = RequestParam.HEADERS.Get(args, kwargs) or {}
+    return uri, method, body, headers
+
+
+class RequestParam(enum.Enum):
+  """Encapsulates the parameters to a request() call and how to extract them.
+  """
+  URI = ('uri', 0)
+  METHOD = ('method', 1)
+  BODY = ('body', 2)
+  HEADERS = ('headers', 3)
+
+  def __init__(self, arg_name, index):
+    self.arg_name = arg_name
+    self.index = index
+
+  def Get(self, args, kwargs):
+    if len(args) > self.index:
+      return args[self.index]
+    if self.arg_name in kwargs:
+      return kwargs[self.arg_name]
+    return None
+
+  def Set(self, args, kwargs, value):
+    if len(args) > self.index:
+      args[self.index] = value
     else:
-      return default
+      kwargs[self.arg_name] = value
 
-  @classmethod
-  def _SetHeader(cls, args, kwargs, header, value):
-    """Set a header given the args and kwargs of an Http Request call."""
 
-    modified_args = list(args)
+def IsTokenUri(uri):
+  """Determine if the given URI is for requesting an access token."""
+  if uri in ['https://accounts.google.com/o/oauth2/token',
+             'https://www.googleapis.com/oauth2/v3/token',
+             'https://www.googleapis.com/oauth2/v4/token',
+             'https://oauth2.googleapis.com/token',
+             'https://oauth2.googleapis.com/oauth2/v4/token']:
+    return True
 
-    if 'headers' in kwargs:
-      # Headers was given to request() as a kwarg, we can just update it.
-      kwargs['headers'][header] = value
-    elif len(modified_args) > 3:
-      # Headers was given as a positional, we need to update that arg.
-      if modified_args[3] is not None:
-        # Headers were actually provided, update that dictionary.
-        modified_args[3][header] = value
-      else:
-        # Headers were explicitly provided as a positional but None, copy the
-        # args because the tuple is immutable and insert the header.
-        modified_args[3] = {header: value}
-    else:
-      kwargs['headers'] = {header: value}
+  metadata_regexp = ('metadata.google.internal/computeMetadata/.*?/instance/'
+                     'service-accounts/.*?/token')
 
-    return modified_args
+  return re.search(metadata_regexp, uri) is not None
