@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -40,16 +41,24 @@ Pythonicness of the Transform*() methods:
       Exceptions for arguments explicitly under the caller's control are OK.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+import base64
 import datetime
+import io
 import re
-import StringIO
-import urllib2
 
 from googlecloudsdk.core.console import console_attr
 from googlecloudsdk.core.resource import resource_exceptions
 from googlecloudsdk.core.resource import resource_property
 from googlecloudsdk.core.util import times
 
+import six
+from six.moves import map  # pylint: disable=redefined-builtin
+from six.moves import urllib
+
 
 def GetBooleanArgValue(arg):
   """Returns the Boolean value for arg."""
@@ -123,7 +132,7 @@ def TransformBaseName(r, undefined=''):
   """
   if not r:
     return undefined
-  s = unicode(r)
+  s = six.text_type(r)
   for separator in ('/', '\\'):
     i = s.rfind(separator)
     if i >= 0:
@@ -169,7 +178,7 @@ def TransformColor(r, red=None, yellow=None, green=None, blue=None, **kwargs):
     For the resource string "CAUTION means GO FASTER" displays the
     substring "CAUTION" in yellow.
   """
-  string = unicode(r)
+  string = six.text_type(r)
   for color, pattern in (('red', red), ('yellow', yellow), ('green', green),
                          ('blue', blue)):
     if pattern and re.search(pattern, string):
@@ -247,7 +256,7 @@ def TransformDate(r, format='%Y-%m-%dT%H:%M:%S', unit=1, undefined='',
 
   # Check if r is a serialized datetime object.
   original_repr = resource_property.Get(r, ['datetime'], None)
-  if original_repr and isinstance(original_repr, basestring):
+  if original_repr and isinstance(original_repr, six.string_types):
     r = original_repr
 
   tz_out = times.GetTimeZone(tz) if tz else None
@@ -314,6 +323,25 @@ def TransformDecode(r, encoding, undefined=''):
   Returns:
     The decoded resource.
   """
+  # Apitools will encode bytefields using URL-safe base64 when translating
+  # from message to JSON. The built in base64 codec uses the standard
+  # implementation which will fail to decode some URL-safe base64 encoded
+  # strings. So if a encoded string contains '-' or '_' characters, the
+  # built-in decode function will fail to decode the string.
+  # This solution attempts to first use URL-safe base64 decoding and will fall
+  # through to the built-in decode implementation. This was deemed better than
+  # registering a new codec which would mess with the global state.
+  # TODO(b/69855177): See if this can be done by registering a base64 codec
+  # instead.
+  if encoding == 'base64':
+    try:
+      # This uses str.translate, so we must cast unicode to str here.
+      return base64.urlsafe_b64decode(str(r))
+    except:  # pylint: disable=bare-except
+      # This shouldn't happen because the URL-safe implementation can handle
+      # both standard and URL-safe encoded base64 strings.
+      pass
+
   # Some codecs support 'replace', all support 'strict' (the default).
   for errors in ('replace', 'strict'):
     try:
@@ -347,7 +375,7 @@ def TransformDuration(r, start='', end='', parts=3, precision=3, calendar=True,
       non-zero part.
     precision: Format the last duration part with precision digits after the
       decimal point. Trailing "0" and "." are always stripped.
-    calendar: Allow time units larger than hours in formated durations if true.
+    calendar: Allow time units larger than hours in formatted durations if true.
       Durations specifying hours or smaller units are exact across daylight
       savings time boundaries. On by default. Use calendar=false to disable.
       For example, if `calendar=true` then at the daylight savings boundary
@@ -437,13 +465,16 @@ def TransformEncode(r, encoding, undefined=''):
   Returns:
     The encoded resource.
   """
-  # Some codecs support 'replace', all support 'strict' (the default).
-  for errors in ('replace', 'strict'):
+  if encoding == 'base64':
     try:
-      return r.encode(encoding, errors).rstrip('\n')
+      b = base64.b64encode(console_attr.EncodeToBytes(r))
+      return console_attr.SafeText(b).rstrip('\n')
     except:  # pylint: disable=bare-except, undefined for any exception
-      pass
-  return undefined
+      return undefined
+  try:
+    return console_attr.SafeText(r, encoding)
+  except:  # pylint: disable=bare-except, undefined for any exception
+    return undefined
 
 
 def TransformEnum(r, projection, enums, inverse=False, undefined=''):
@@ -467,7 +498,7 @@ def TransformEnum(r, projection, enums, inverse=False, undefined=''):
     if normal:
       # Create the inverse dict and memoize it in projection.symbols.
       descriptions = {}
-      for k, v in normal.iteritems():
+      for k, v in six.iteritems(normal):
         descriptions[v] = k
       projection.symbols[type_name] = descriptions
   return descriptions.get(r, undefined) if descriptions else undefined
@@ -485,7 +516,7 @@ def TransformError(r, message=None):
     Error: This will not generate a stack trace.
   """
   if message is None:
-    message = unicode(r)
+    message = six.text_type(r)
   raise resource_exceptions.Error(message)
 
 
@@ -519,8 +550,38 @@ def TransformFatal(r, message=None):
   Raises:
     InternalError: This generates a stack trace.
   """
-  raise resource_exceptions.InternalError(message if message is not None
-                                          else str(r))
+  raise resource_exceptions.InternalError(
+      message if message is not None else six.text_type(r))
+
+
+def TransformFilter(r, expression):
+  """Selects elements of r that match the filter expression.
+
+  Args:
+    r: A JSON-serializable object.
+    expression: The filter expression to apply to r.
+
+  Returns:
+    The elements of r that match the filter expression.
+
+  Example:
+    x.filter("key:val") selects elements of r that have 'key' fields containing
+    'val'.
+  """
+
+  # import loop
+  from googlecloudsdk.core.resource import resource_filter  # pylint: disable=g-import-not-at-top
+
+  if not r:
+    return r
+  select = resource_filter.Compile(expression).Evaluate
+  if not resource_property.IsListLike(r):
+    return r if select(r) else ''
+  transformed = []
+  for item in r:
+    if select(item):
+      transformed.append(item)
+  return transformed
 
 
 def TransformFirstOf(r, *keys):
@@ -545,6 +606,50 @@ def TransformFirstOf(r, *keys):
   return ''
 
 
+def TransformFlatten(r, show='', undefined='', separator=','):
+  """Formats nested dicts and/or lists into a compact comma separated list.
+
+  Args:
+    r: A JSON-serializable object.
+    show: If show=*keys* then list dict keys; if show=*values* then list dict
+      values; otherwise list dict key=value pairs.
+    undefined: Return this if the resource is empty.
+    separator: The list item separator string.
+
+  Returns:
+    The key=value pairs for a dict or list values for a list, separated by
+    separator. Returns undefined if r is empty, or r if it is not a dict or
+    list.
+
+  Example:
+    `--format="table(field.map(2).list().map().list().list()"`:::
+    Expression with explicit flattening.
+    `--format="table(field.flatten()"`:::
+    Equivalent expression using .flatten().
+  """
+
+  def Flatten(x):
+    return TransformFlatten(
+        x, show=show, undefined=undefined, separator=separator)
+
+  if isinstance(r, dict):
+    if show == 'keys':
+      r = separator.join(
+          [six.text_type(k) for k in sorted(r)])
+    elif show == 'values':
+      r = separator.join(
+          [Flatten(v) for _, v in sorted(six.iteritems(r))])
+    else:
+      r = separator.join(
+          ['{k}={v}'.format(k=k, v=Flatten(v))
+           for k, v in sorted(six.iteritems(r))])
+  if r and isinstance(r, list):
+    if isinstance(r[0], (dict, list)):
+      r = [Flatten(v) for v in r]
+    return separator.join(map(six.text_type, r))
+  return r or undefined
+
+
 def TransformFloat(r, precision=6, spec=None, undefined=''):
   """Returns the string representation of a floating point number.
 
@@ -642,7 +747,7 @@ def TransformGroup(r, *keys):
   """
   if not r:
     return '[]'
-  buf = StringIO.StringIO()
+  buf = io.StringIO()
   sep = None
   parsed_keys = [_GetParsedKey(key) for key in keys]
   for item in r:
@@ -651,7 +756,7 @@ def TransformGroup(r, *keys):
     else:
       sep = ' '
     if not parsed_keys:
-      buf.write('[{0}]'.format(unicode(item)))
+      buf.write('[{0}]'.format(six.text_type(item)))
     else:
       buf.write('[')
       sub = None
@@ -663,7 +768,7 @@ def TransformGroup(r, *keys):
           sub = ': '
         value = resource_property.Get(item, key, None)
         if value is not None:
-          buf.write(unicode(value))
+          buf.write(six.text_type(value))
       buf.write(']')
   return buf.getvalue()
 
@@ -722,7 +827,7 @@ def TransformJoin(r, sep='/', undefined=''):
     Returns "a!b!c!d".
   """
   try:
-    parts = [unicode(i) for i in r]
+    parts = [six.text_type(i) for i in r]
     return sep.join(parts) or undefined
   except (AttributeError, TypeError):
     return undefined
@@ -760,14 +865,15 @@ def TransformList(r, show='', undefined='', separator=','):
   """
   if isinstance(r, dict):
     if show == 'keys':
-      return separator.join([unicode(k) for k in sorted(r)])
+      return separator.join([six.text_type(k) for k in sorted(r)])
     elif show == 'values':
-      return separator.join([unicode(v) for _, v in sorted(r.iteritems())])
+      return separator.join(
+          [six.text_type(v) for _, v in sorted(six.iteritems(r))])
     else:
-      return separator.join([u'{k}={v}'.format(k=k, v=v)
-                             for k, v in sorted(r.iteritems())])
+      return separator.join(['{k}={v}'.format(k=k, v=v)
+                             for k, v in sorted(six.iteritems(r))])
   if isinstance(r, list):
-    return separator.join(map(unicode, r))
+    return separator.join(map(six.text_type, r))
   return r or undefined
 
 
@@ -778,11 +884,15 @@ def TransformMap(r, depth=1):
     `list_field.map().foo().list()`:::
     Applies foo() to each item in list_field and then list() to the resulting
     value to return a compact comma-separated list.
+    `list_field.*foo().list()`:::
+    ```*``` is shorthand for map().
     `list_field.map().foo().map().bar()`:::
     Applies foo() to each item in list_field and then bar() to each item in the
     resulting list.
     `abc.xyz.map(2).foo()`:::
     Applies foo() to each item in xyz[] for all items in abc[].
+    `abc.xyz.**foo()`:::
+    ```**``` is shorthand for map(2).
 
   Args:
     r: A resource.
@@ -880,14 +990,16 @@ def TransformScope(r, *args):
       component in r if none found.
 
   Example:
-    `"https://abc/foo/projects/bar/xyz".scope("projects")`:::
+    `"http://abc/foo/projects/bar/xyz".scope("projects")`:::
     Returns "bar/xyz".
-    `"https://xyz/foo/regions/abc".scope()`:::
+    `"http://xyz/foo/regions/abc".scope()`:::
     Returns "abc".
   """
   if not r:
     return ''
-  r = urllib2.unquote(unicode(r))
+  # pylint: disable=too-many-function-args
+  r = urllib.parse.unquote(six.text_type(r))  # pytype: disable=module-attr
+  # pylint: enable=too-many-function-args
   if '/' not in r:
     return r
   # Checking for regions and/or zones is the most common use case.
@@ -913,7 +1025,9 @@ def TransformSegment(r, index=-1, undefined=''):
   """
   if not r:
     return undefined
-  r = urllib2.unquote(unicode(r))
+  # pylint: disable=too-many-function-args
+  r = urllib.parse.unquote(six.text_type(r))  # pytype: disable=module-attr
+  # pylint: enable=too-many-function-args
   segments = r.split('/')
   try:
     return segments[int(index)] or undefined
@@ -1225,9 +1339,9 @@ def TransformUri(r, undefined='.'):
       attr = attr()
     except TypeError:
       pass
-    return attr if isinstance(attr, (basestring, buffer)) else None
+    return attr if isinstance(attr, six.string_types) else None
 
-  if isinstance(r, (basestring, buffer)):
+  if isinstance(r, six.string_types):
     if r.startswith('https://'):
       return r
   elif r:
@@ -1268,7 +1382,9 @@ _BUILTIN_TRANSFORMS = {
     'error': TransformError,
     'extract': TransformExtract,
     'fatal': TransformFatal,
+    'filter': TransformFilter,
     'firstof': TransformFirstOf,
+    'flatten': TransformFlatten,
     'float': TransformFloat,
     'format': TransformFormat,
     'group': TransformGroup,
@@ -1307,6 +1423,7 @@ _API_TO_TRANSFORMS = {
                   'GetTransforms'),
     'runtimeconfig': ('googlecloudsdk.api_lib.runtime_config.transforms',
                       'GetTransforms'),
+    'dns': ('googlecloudsdk.command_lib.dns.dns_keys', 'GetTransforms'),
 }
 
 
