@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,11 +19,18 @@ This tracks the installed modules along with the files they created.  It also
 provides functionality like extracting tar files into the installation and
 tracking when we check for updates.
 """
+# TODO(b/70520907): remove the attribute-error disable.
+# pytype: disable=attribute-error
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 import compileall
 import errno
 import logging
 import os
+import posixpath
 import re
 import shutil
 import sys
@@ -35,6 +43,8 @@ from googlecloudsdk.core.updater import snapshots
 from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import files as file_utils
 
+import six
+
 
 class Error(exceptions.Error):
   """Base exception for the local_state module."""
@@ -46,10 +56,13 @@ class InvalidSDKRootError(Error):
 
   def __init__(self):
     super(InvalidSDKRootError, self).__init__(
-        'You cannot perform this action because this Cloud SDK installation '
-        'is managed by an external package manager.  If you would like to get'
-        'the latest version, please see our main download page at:\n  '
-        + config.INSTALLATION_CONFIG.documentation_url)
+        'You cannot perform this action because this Cloud SDK '
+        'installation is managed by an external package manager.\n'
+        'Please consider using a separate installation of the Cloud '
+        'SDK created through the default mechanism described at: '
+        '{doc_url}\n'.format(
+            doc_url=config.INSTALLATION_CONFIG.documentation_url))
+
 
 
 class InvalidDownloadError(Error):
@@ -72,8 +85,8 @@ class PermissionsError(Error):
           operated on, but can't because of insufficient permissions.
     """
     super(PermissionsError, self).__init__(
-        u'{message}: [{path}]\n\nEnsure you have the permissions to access the '
-        u'file and that the file is not in use.'
+        '{message}: [{path}]\n\nEnsure you have the permissions to access the '
+        'file and that the file is not in use.'
         .format(message=message, path=path))
 
 
@@ -93,24 +106,22 @@ def _RaisesPermissionsError(func):
 
   def _TryFunc(*args, **kwargs):
     try:
-      return func(*args, **kwargs)
-    except (OSError, IOError) as e:
-      if e.errno == errno.EACCES:
-        new_exc = PermissionsError(
-            message=e.strerror, path=os.path.abspath(e.filename))
-        # Maintain original stack trace.
-        raise new_exc, None, sys.exc_info()[2]
-      raise
+      return func(*args, **kwargs)  # pytype: disable=missing-parameter
     except shutil.Error as e:
       args = e.args[0][0]
       # unfortunately shutil.Error *only* has formatted strings to inspect.
       # Looking for this substring is looking for errno.EACCES, which has
       # a numeric value of 13.
       if args[2].startswith('[Errno 13]'):
-        new_exc = PermissionsError(
-            message=args[2], path=os.path.abspath(args[0]))
-        # Maintain original stack trace.
-        raise new_exc, None, sys.exc_info()[2]
+        exceptions.reraise(
+            PermissionsError(message=args[2],
+                             path=os.path.abspath(args[0])))
+      raise
+    except (OSError, IOError) as e:
+      if e.errno == errno.EACCES:
+        exceptions.reraise(
+            PermissionsError(message=e.strerror,
+                             path=os.path.abspath(e.filename)))
       raise
   return _TryFunc
 
@@ -189,7 +200,7 @@ class InstallationState(object):
       ValueError: If the given SDK root does not exist.
     """
     if not os.path.isdir(sdk_root):
-      raise ValueError(u'The given Cloud SDK root does not exist: [{0}]'
+      raise ValueError('The given Cloud SDK root does not exist: [{0}]'
                        .format(sdk_root))
 
     self.__sdk_root = encoding.Decode(sdk_root)
@@ -290,8 +301,7 @@ class InstallationState(object):
     """
     self._CreateStateDir()
     (rm_staging_cb, rm_backup_cb, rm_trash_cb, copy_cb) = (
-        console_io.ProgressBar.SplitProgressBar(progress_callback,
-                                                [1, 1, 1, 7]))
+        console_io.SplitProgressBar(progress_callback, [1, 1, 1, 7]))
 
     self._ClearStaging(progress_callback=rm_staging_cb)
     self.ClearBackup(progress_callback=rm_backup_cb)
@@ -302,7 +312,7 @@ class InstallationState(object):
       def __init__(self, progress_callback, total):
         self.count = 0
         self.progress_callback = progress_callback
-        self.total = float(total)
+        self.total = total
 
       # This function must match the signature that shutil expects for the
       # ignore function.
@@ -315,7 +325,7 @@ class InstallationState(object):
       # This takes a little time, so only do it if we are going to report
       # progress.
       dirs = set()
-      for _, manifest in self.InstalledComponents().iteritems():
+      for _, manifest in six.iteritems(self.InstalledComponents()):
         dirs.update(manifest.InstalledDirectories())
       # There is always the root directory itself and the .install directory.
       # In general, there could be in the SDK (if people just put stuff in there
@@ -539,7 +549,7 @@ class InstallationState(object):
     """
     manifest = InstallationManifest(self._state_directory, component_id)
     paths = manifest.InstalledPaths()
-    total_paths = float(len(paths))
+    total_paths = len(paths)
     root = self.__sdk_root
 
     dirs_to_remove = set()
@@ -552,9 +562,10 @@ class InstallationState(object):
           pyc_path = path + 'c'
           if os.path.isfile(pyc_path):
             os.remove(pyc_path)
-        dir_path = os.path.dirname(path)
-        if dir_path:
-          dirs_to_remove.add(os.path.normpath(dir_path))
+        dir_path = os.path.dirname(os.path.normpath(p))
+        while dir_path:
+          dirs_to_remove.add(os.path.join(root, dir_path))
+          dir_path = os.path.dirname(dir_path)
       elif os.path.isdir(path):
         dirs_to_remove.add(os.path.normpath(path))
       if progress_callback:
@@ -597,11 +608,11 @@ class InstallationState(object):
     with file_utils.ChDir(self.sdk_root):
       to_compile = [
           os.path.join('bin', 'bootstrapping'),
+          os.path.join('data', 'cli'),
           'lib',
           'platform',
       ]
       for d in to_compile:
-        d = encoding.Decode(d)
         # Using rx to skip unused Python3 directory vendored with gsutil's copy
         # of httplib2.
         compileall.compile_dir(d, rx=re.compile('python3'), quiet=True)
@@ -639,10 +650,10 @@ class InstallationManifest(object):
         of the install.
       files: list of str, The files that were created by the installation.
     """
-    with open(self.manifest_file, 'w') as fp:
-      for f in files:
+    with file_utils.FileWriter(self.manifest_file) as fp:
+      for f in _NormalizeFileList(files):
         fp.write(f + '\n')
-    snapshot.WriteToFile(self.snapshot_file)
+    snapshot.WriteToFile(self.snapshot_file, component_id=self.id)
 
   def MarkUninstalled(self):
     """Marks this component as no longer being installed.
@@ -684,7 +695,7 @@ class InstallationManifest(object):
     Returns:
       list of str, The files and directories installed by this component.
     """
-    with open(self.manifest_file) as f:
+    with file_utils.FileReader(self.manifest_file) as f:
       files = [line.rstrip() for line in f]
     return files
 
@@ -694,10 +705,34 @@ class InstallationManifest(object):
     Returns:
       set(str), The directories installed by this component.
     """
-    with open(self.manifest_file) as f:
+    with file_utils.FileReader(self.manifest_file) as f:
       dirs = set()
       for line in f:
-        fixed = line.rstrip()
-        if fixed.endswith('/'):
-          dirs.add(fixed)
+        norm_file_path = os.path.dirname(line.rstrip())
+        prev_file = norm_file_path + '/'
+        while len(prev_file) > len(norm_file_path) and norm_file_path:
+          dirs.add(norm_file_path)
+          prev_file = norm_file_path
+          norm_file_path = os.path.dirname(norm_file_path)
+
     return dirs
+
+
+def _NormalizeFileList(file_list):
+  """Removes non-empty directory entries and sorts resulting list."""
+  parent_directories = set([])
+  directories = set([])
+  files = set([])
+  for f in file_list:
+    # Drops any trailing /.
+    norm_file_path = posixpath.normpath(f)
+    if f.endswith('/'):
+      directories.add(norm_file_path + '/')
+    else:
+      files.add(norm_file_path)
+    norm_file_path = os.path.dirname(norm_file_path)
+    while norm_file_path:
+      parent_directories.add(norm_file_path + '/')
+      norm_file_path = os.path.dirname(norm_file_path)
+
+  return sorted((directories - parent_directories) | files)
