@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -34,13 +35,23 @@ The general usage is as follows:
 Errors are raised at the time of the Get() call on the future (which is implicit
 for Apply() and Map()).
 """
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import abc
 import pickle
-import Queue
 import sys
 import threading
 import time
 
+from googlecloudsdk.core import exceptions
+import six
+from six.moves import map  # pylint: disable=redefined-builtin
+from six.moves import queue   # pylint: disable=redefined-builtin
+from six.moves import range  # pylint: disable=redefined-builtin
+
 
 ################################################################################
 # Common Pool Infrastructure
@@ -62,7 +73,8 @@ class InvalidStateException(Exception):
     super(InvalidStateException, self).__init__(msg)
 
 
-class BasePool(object):
+@six.add_metaclass(abc.ABCMeta)
+class BasePool(object):  # pytype: disable=ignored-abstractmethod
   """Base class for parallel pools.
 
   Provides a limited subset of the multiprocessing.Pool API.
@@ -73,8 +85,6 @@ class BasePool(object):
   ...  assert pool.Map(str, [1, 2, 3]) == ['1', '2', '3']
   """
 
-  __metaclass__ = abc.ABCMeta
-
   @abc.abstractmethod
   def Start(self):
     """Initialize non-trivial infrastructure (e.g. processes/threads/queues)."""
@@ -110,11 +120,10 @@ class BasePool(object):
     self.Join()
 
 
-class BaseFuture(object):
+@six.add_metaclass(abc.ABCMeta)
+class BaseFuture(object):  # pytype: disable=ignored-abstractmethod
   """A future object containing a value that may not be available yet."""
 
-  __metaclass__ = abc.ABCMeta
-
   def Get(self):
     return self.GetResult().GetOrRaise()
 
@@ -155,7 +164,7 @@ class _Result(object):
     elif self.error:
       raise self.error  # pylint: disable=raising-bad-type
     else:
-      raise self.exc_info[0], self.exc_info[1], self.exc_info[2]
+      exceptions.reraise(self.exc_info[1], tb=self.exc_info[2])
 
   def ToPickleableResult(self):
     """Return a pickleable version of this _Result.
@@ -190,8 +199,10 @@ class MultiError(Exception):
 
   def __init__(self, errors):
     self.errors = errors
+    fn = lambda e: '{}: {}'.format(type(e).__name__, six.text_type(e))
     super(MultiError, self).__init__(
-        'One or more errors occurred:\n' + '\n\n'.join(map(unicode, errors)))
+        'One or more errors occurred:\n' +
+        '\n\n'.join(map(fn, errors)))
 
 
 class _MultiFuture(BaseFuture):
@@ -342,7 +353,7 @@ class ThreadPool(BasePool):
 
   def __init__(self, num_threads):
     self.num_threads = num_threads
-    self._task_queue = Queue.Queue()
+    self._task_queue = queue.Queue()
     self.worker_threads = []
     self._results_map = {}
 
