@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -42,18 +43,26 @@ naiive datetimes specify tzinfo=None in all calls that have a timezone kwarg.
 The datetime and/or dateutil modules should have covered all of this.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import datetime
 import re
 
 from dateutil import parser
 from dateutil import tz
 from dateutil.tz import _common as tz_common
+import enum
 
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core.util import encoding
 from googlecloudsdk.core.util import iso_duration
 from googlecloudsdk.core.util import times_data
 
+import six
+
+
 try:
   from dateutil import tzwin  # pylint: disable=g-import-not-at-top, Windows
 except ImportError:
@@ -86,12 +95,20 @@ LOCAL = tz.tzlocal()  # The local timezone.
 UTC = tz.tzutc()  # The UTC timezone.
 
 
+_MICROSECOND_PRECISION = 6
+
+
 def _StrFtime(dt, fmt):
   """Convert strftime exceptions to Datetime Errors."""
   try:
     return dt.strftime(fmt)
-  except (AttributeError, OverflowError, TypeError, ValueError) as e:
-    raise DateTimeValueError(unicode(e))
+  except TypeError as e:
+    if '%Z' not in fmt:
+      raise DateTimeValueError(six.text_type(e))
+    # Most likely a non-ascii tzname() in python2. Fall back to +-HH:MM.
+    return FormatDateTime(dt, fmt.replace('%Z', '%Ez'))
+  except (AttributeError, OverflowError, ValueError) as e:
+    raise DateTimeValueError(six.text_type(e))
 
 
 def _StrPtime(string, fmt):
@@ -99,9 +116,9 @@ def _StrPtime(string, fmt):
   try:
     return datetime.datetime.strptime(string, fmt)
   except (AttributeError, OverflowError, TypeError) as e:
-    raise DateTimeValueError(unicode(e))
+    raise DateTimeValueError(six.text_type(e))
   except ValueError as e:
-    raise DateTimeSyntaxError(unicode(e))
+    raise DateTimeSyntaxError(six.text_type(e))
 
 
 def FormatDuration(duration, parts=3, precision=3):
@@ -128,7 +145,40 @@ def FormatDuration(duration, parts=3, precision=3):
   return duration.Format(parts=parts, precision=precision)
 
 
-def ParseDuration(string, calendar=False):
+def FormatDurationForJson(duration):
+  """Returns a string representation of the duration, ending in 's'.
+
+  See the section of
+  <https://github.com/google/protobuf/blob/master/src/google/protobuf/duration.proto>
+  on JSON formats.
+
+  For example:
+
+    >>> FormatDurationForJson(iso_duration.Duration(seconds=10))
+    10s
+    >>> FormatDurationForJson(iso_duration.Duration(hours=1))
+    3600s
+    >>> FormatDurationForJson(iso_duration.Duration(seconds=1, microseconds=5))
+    1.000005s
+
+  Args:
+    duration: An iso_duration.Duration object.
+
+  Raises:
+    DurationValueError: A Duration numeric constant exceeded its range.
+
+  Returns:
+    An string representation of the duration.
+  """
+  # Caution: the default precision for formatting floats is also 6, so when
+  # introducing adjustable precision, make sure to account for that.
+  num = '{}'.format(round(duration.total_seconds, _MICROSECOND_PRECISION))
+  if num.endswith('.0'):
+    num = num[:-len('.0')]
+  return num + 's'
+
+
+def ParseDuration(string, calendar=False, default_suffix=None):
   """Parses a duration string and returns a Duration object.
 
   Durations using only hours, miniutes, seconds and microseconds are exact.
@@ -148,6 +198,7 @@ def ParseDuration(string, calendar=False):
   Args:
     string: The ISO 8601 duration/period string to parse.
     calendar: Use duration units larger than hours if True.
+    default_suffix: Use this suffix if string is an unqualified int.
 
   Raises:
     DurationSyntaxError: Invalid duration syntax.
@@ -157,12 +208,18 @@ def ParseDuration(string, calendar=False):
     An iso_duration.Duration object for the given ISO 8601 duration/period
     string.
   """
+  if default_suffix:
+    try:
+      seconds = int(string)
+      string = '{}{}'.format(seconds, default_suffix)
+    except ValueError:
+      pass
   try:
     return iso_duration.Duration(calendar=calendar).Parse(string)
   except (AttributeError, OverflowError) as e:
-    raise DurationValueError(unicode(e))
+    raise DurationValueError(six.text_type(e))
   except ValueError as e:
-    raise DurationSyntaxError(unicode(e))
+    raise DurationSyntaxError(six.text_type(e))
 
 
 def GetDurationFromTimeDelta(delta, calendar=False):
@@ -210,7 +267,9 @@ def GetTimeZone(name):
     name = times_data.IANA_TO_WINDOWS.get(name, name)
     try:
       tzinfo = tzwin.tzwin(name)
+    # pytype: disable=name-error
     except WindowsError:  # pylint: disable=undefined-variable
+      # pytype: enable=name-error
       pass
   return tzinfo
 
@@ -257,7 +316,7 @@ def FormatDateTime(dt, fmt=None, tzinfo=None):
   while m:
     match = start + m.start()
     if start < match:
-      # Format the preceeding standard part.
+      # Format the preceding standard part.
       parts.append(encoding.Decode(_StrFtime(dt, fmt[start:match])))
 
     # The extensions only have one modifier char.
@@ -279,12 +338,18 @@ def FormatDateTime(dt, fmt=None, tzinfo=None):
       # Round the fractional part to n digits.
       val = _StrFtime(dt, std_fmt)
       if n and n < len(val):
-        round_format = '{{0:0{n}.0f}}'.format(n=n)
-        rounded = round_format.format(float(val) / 10 ** (len(val) - n))
-        if len(rounded) == n:
-          val = rounded
-        else:
-          val = val[:n]
+        # Explicitly avoiding implementation dependent floating point rounding
+        # diffs.
+        v = int(val[:n])  # The rounded value.
+        f = int(val[n])  # The first digit after the rounded value.
+        if f >= 5:
+          # Round up.
+          v += 1
+        zero_fill_format = '{{0:0{n}d}}'.format(n=n)
+        val = zero_fill_format.format(v)
+        if len(val) > n:
+          # All 9's rounded up by 1 overflowed width. Keep the unrounded value.
+          val = zero_fill_format.format(v - 1)
     elif spec == 's':
       # datetime.strftime('%s') botches tz aware dt!
       val = GetTimeStampFromDateTime(dt)
@@ -324,7 +389,7 @@ class _TzInfoOrOffsetGetter(object):
   def Get(self, name, offset):
     """Returns the tzinfo for name or offset.
 
-    Used by dateutil.parser.parser() to convert timezone names and offsets.
+    Used by dateutil.parser.parse() to convert timezone names and offsets.
 
     Args:
       name: A timezone name or None to use offset. If offset is also None then
@@ -346,6 +411,19 @@ class _TzInfoOrOffsetGetter(object):
     return self._timezone_was_specified
 
 
+def _SplitTzFromDate(string):
+  """Returns (prefix,tzinfo) if string has a trailing tz, else (None,None)."""
+  try:
+    match = re.match(r'(.*[\d\s])([^\d\s]+)$', string)
+  except TypeError:
+    return None, None
+  if match:
+    tzinfo = GetTimeZone(match.group(2))
+    if tzinfo:
+      return match.group(1), tzinfo
+  return None, None
+
+
 def ParseDateTime(string, fmt=None, tzinfo=LOCAL):
   """Parses a date/time string and returns a datetime.datetime object.
 
@@ -372,10 +450,10 @@ def ParseDateTime(string, fmt=None, tzinfo=LOCAL):
 
   # Use tzgetter to determine if string contains an explicit timezone name or
   # offset.
+  defaults = GetDateTimeDefaults(tzinfo=tzinfo)
   tzgetter = _TzInfoOrOffsetGetter()
+
   try:
-    # Check if it's a datetime string.
-    defaults = GetDateTimeDefaults(tzinfo=tzinfo)
     dt = parser.parse(string, tzinfos=tzgetter.Get, default=defaults)
     if tzinfo and not tzgetter.timezone_was_specified:
       # The string had no timezone name or offset => localize dt to tzinfo.
@@ -383,16 +461,29 @@ def ParseDateTime(string, fmt=None, tzinfo=LOCAL):
       dt = dt.replace(tzinfo=tzinfo)
     return dt
   except OverflowError as e:
-    exc = DateTimeValueError
+    exc = DateTimeValueError(six.text_type(e))
   except (AttributeError, ValueError, TypeError) as e:
-    exc = DateTimeSyntaxError
+    exc = DateTimeSyntaxError(six.text_type(e))
+    if not tzgetter.timezone_was_specified:
+      # Good ole parser.parse() has a tzinfos kwarg that it sometimes ignores.
+      # Compensate here when the string ends with a tz.
+      prefix, explicit_tzinfo = _SplitTzFromDate(string)
+      if explicit_tzinfo:
+        try:
+          dt = parser.parse(prefix, default=defaults)
+        except OverflowError as e:
+          exc = DateTimeValueError(six.text_type(e))
+        except (AttributeError, ValueError, TypeError) as e:
+          exc = DateTimeSyntaxError(six.text_type(e))
+        else:
+          return dt.replace(tzinfo=explicit_tzinfo)
 
   try:
-    # Check if its an iso_duration string.
+    # Check if it's an iso_duration string.
     return ParseDuration(string).GetRelativeDateTime(Now(tzinfo=tzinfo))
   except Error:
     # Not a duration - reraise the datetime parse error.
-    raise exc(unicode(e))
+    raise exc
 
 
 def GetDateTimeFromTimeStamp(timestamp, tzinfo=LOCAL):
@@ -412,7 +503,7 @@ def GetDateTimeFromTimeStamp(timestamp, tzinfo=LOCAL):
   try:
     return datetime.datetime.fromtimestamp(timestamp, tzinfo)
   except ValueError as e:
-    raise DateTimeValueError(unicode(e))
+    raise DateTimeValueError(six.text_type(e))
 
 
 def GetTimeStampFromDateTime(dt, tzinfo=LOCAL):
@@ -428,7 +519,7 @@ def GetTimeStampFromDateTime(dt, tzinfo=LOCAL):
   if not dt.tzinfo and tzinfo:
     dt = dt.replace(tzinfo=tzinfo)
   delta = dt - datetime.datetime.fromtimestamp(0, UTC)
-  return iso_duration.GetTotalSecondsFromTimeDelta(delta)
+  return delta.total_seconds()
 
 
 def LocalizeDateTime(dt, tzinfo=LOCAL):
@@ -487,3 +578,41 @@ def TzOffset(offset, name=None):
     A tzinfo for offset seconds east of UTC.
   """
   return tz.tzoffset(name, offset * 60)  # tz.tzoffset needs seconds east of UTC
+
+
+class Weekday(enum.Enum):
+  """Represents a day of the week."""
+
+  MONDAY = 0
+  TUESDAY = 1
+  WEDNESDAY = 2
+  THURSDAY = 3
+  FRIDAY = 4
+  SATURDAY = 5
+  SUNDAY = 6
+
+  @classmethod
+  def Get(cls, day):
+    day = day.upper()
+    value = getattr(cls, day, None)
+    if not value:
+      raise KeyError('[{}] is not a valid Weekday'.format(day))
+    return value
+
+
+def GetWeekdayInTimezone(dt, weekday, tzinfo=LOCAL):
+  """Returns the Weekday for dt in the timezone specified by tzinfo.
+
+  Args:
+    dt: The datetime object that represents the time on weekday.
+    weekday: The day of the week specified as a Weekday enum.
+    tzinfo: The timezone in which to get the new day of the week in.
+
+  Returns:
+    A Weekday that corresponds to dt and weekday pair localized to the timezone
+    specified by dt.
+  """
+  localized_dt = LocalizeDateTime(dt, tzinfo)
+  localized_weekday_offset = dt.weekday() - localized_dt.weekday()
+  localized_weekday_index = (weekday.value - localized_weekday_offset) % 7
+  return Weekday(localized_weekday_index)
