@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +15,9 @@
 
 """The gcloud app group."""
 
-import sys
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from googlecloudsdk.calliope import base
 
@@ -68,6 +71,9 @@ DETAILED_HELP = {
                     base.ReleaseTrack.BETA,
                     base.ReleaseTrack.GA)
 class AppengineGA(base.Group):
-  pass
+
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
 
 AppengineGA.detailed_help = DETAILED_HELP
