@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """`gcloud app repair` command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.app import appengine_api_client
 from googlecloudsdk.calliope import base
 from googlecloudsdk.core.console import progress_tracker
@@ -36,7 +41,7 @@ class Repair(base.Command):
   }
 
   def Run(self, args):
-    api_client = appengine_api_client.GetApiClient()
+    api_client = appengine_api_client.AppengineApiClient.GetApiClient('v1beta')
 
     with progress_tracker.ProgressTracker(
         'Repairing the app [{0}]'.format(api_client.project)):
