@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,11 +15,14 @@
 
 """The Stop command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.app import appengine_api_client
 from googlecloudsdk.api_lib.app import operations_util
 from googlecloudsdk.api_lib.app import version_util
 from googlecloudsdk.calliope import base
-from googlecloudsdk.calliope import exceptions as calliope_exceptions
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core.console import console_io
@@ -73,7 +77,7 @@ class Stop(base.Command):
     # TODO(b/36057452): This fails with "module/version does not exist" even
     # when it exists if the scaling mode is set to auto.  It would be good
     # to improve that error message.
-    api_client = appengine_api_client.GetApiClient()
+    api_client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
     services = api_client.ListServices()
     versions = version_util.GetMatchingVersions(
         api_client.ListVersions(services),
@@ -84,16 +88,15 @@ class Stop(base.Command):
       resource_printer.Print(versions, fmt, out=log.status)
       console_io.PromptContinue(cancel_on_no=True)
     else:
-      log.warn('No matching versions found.')
+      log.warning('No matching versions found.')
 
     errors = []
-    for version in sorted(versions):
+    for version in sorted(versions, key=str):
       try:
         with progress_tracker.ProgressTracker('Stopping [{0}]'.format(version)):
-          api_client.StopVersion(version.service, version.id, block=True)
-      except (calliope_exceptions.HttpException,
-              operations_util.OperationError,
-              operations_util.OperationTimeoutError) as err:
+          operations_util.CallAndCollectOpErrors(
+              api_client.StopVersion, version.service, version.id, block=True)
+      except operations_util.MiscOperationError as err:
         errors.append(str(err))
     if errors:
       raise VersionsStopError('\n\n'.join(errors))
