@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Command to list the available accounts."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.auth import auth_util
 from googlecloudsdk.core import log
@@ -26,10 +31,19 @@ class List(base.ListCommand):
   Lists accounts whose credentials have been obtained using `gcloud init`,
   `gcloud auth login` and `gcloud auth activate-service-account`, and shows
   which account is active. The active account is used by gcloud and other Cloud
-  SDK tools to access Google Cloud Platform.
+  SDK tools to access Google Cloud Platform. While there is no limit on the
+  number of accounts with stored credentials, there is only one active account.
 
   ## EXAMPLES
 
+  To set an existing account to be the current active account, run:
+
+    $ gcloud config set core/account your-email-account@gmail.com
+
+  If you don't have an existing account, create one using:
+
+    $ gcloud init
+
   To list the active account name:
 
     $ gcloud auth list --filter=status:ACTIVE --format="value(account)"
