@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,9 +14,16 @@
 # limitations under the License.
 """The main command group for bigtable."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
 class BigtableV2(base.Group):
   """Manage your Cloud Bigtable storage."""
+
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
