@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +15,18 @@
 
 """Command for adding a backend to a backend service."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import encoding
 
-from googlecloudsdk.api_lib.compute import backend_services_utils
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.compute import flags as compute_flags
 from googlecloudsdk.command_lib.compute.backend_services import backend_flags
+from googlecloudsdk.command_lib.compute.backend_services import backend_services_utils
 from googlecloudsdk.command_lib.compute.backend_services import flags
 
 
@@ -85,11 +90,17 @@ class AddBackend(base.UpdateCommand):
                 backendServiceResource=replacement,
                 project=backend_service_ref.project))
 
-  def _CreateBackendMessage(self, client, group_uri, balancing_mode, args):
+  def _GetGroupRef(self, args, resources, client):
+    return flags.MULTISCOPE_INSTANCE_GROUP_ARG.ResolveAsResource(
+        args,
+        resources,
+        scope_lister=compute_flags.GetDefaultScopeLister(client))
+
+  def _CreateBackendMessage(self, messages, group_uri, balancing_mode, args):
     """Create a backend message.
 
     Args:
-      client: The compute client.
+      messages: The avalible API proto messages.
       group_uri: String. The backend instance group uri.
       balancing_mode: Backend.BalancingModeValueValuesEnum. The backend load
         balancing mode.
@@ -100,8 +111,8 @@ class AddBackend(base.UpdateCommand):
       arguments.
     """
 
-    backend_services_utils.ValidateBalancingModeArgs(client.messages, args)
-    return client.messages.Backend(
+    backend_services_utils.ValidateBalancingModeArgs(messages, args)
+    return messages.Backend(
         balancingMode=balancing_mode,
         capacityScaler=args.capacity_scaler,
         description=args.description,
@@ -115,16 +126,13 @@ class AddBackend(base.UpdateCommand):
   def _Modify(self, client, resources, backend_service_ref, args, existing):
     replacement = encoding.CopyProtoMessage(existing)
 
-    group_ref = flags.MULTISCOPE_INSTANCE_GROUP_ARG.ResolveAsResource(
-        args,
-        resources,
-        scope_lister=compute_flags.GetDefaultScopeLister(client))
-
+    group_ref = self._GetGroupRef(args, resources, client)
     group_uri = group_ref.SelfLink()
 
     for backend in existing.backends:
       if group_uri == backend.group:
-        if group_ref.Collection() == 'compute.instanceGroups':
+        if (group_ref.Collection() == 'compute.instanceGroups'
+            or group_ref.Collection() == 'compute.networkEndpointGroups'):
           scope = 'zone'
         elif group_ref.Collection() == 'compute.regionInstanceGroups':
           scope = 'region'
@@ -141,8 +149,8 @@ class AddBackend(base.UpdateCommand):
     else:
       balancing_mode = None
 
-    backend = self._CreateBackendMessage(client, group_uri, balancing_mode,
-                                         args)
+    backend = self._CreateBackendMessage(client.messages, group_uri,
+                                         balancing_mode, args)
 
     replacement.backends.append(backend)
     return replacement
@@ -176,11 +184,12 @@ class AddBackendBeta(AddBackend):
   backend is a group of tasks that can handle requests sent to a
   backend service. Currently, the group of tasks can be one or
   more Google Compute Engine virtual machine instances grouped
-  together using an instance group.
+  together using an instance group or network endpoint group.
 
-  Traffic is first spread evenly across all virtual machines in
-  the group. When the group is full, traffic is sent to the next
-  nearest group(s) that still have remaining capacity.
+  Traffic is first spread evenly across all virtual machines or
+  network endpoints in the group. When the group is full, traffic
+  is sent to the next nearest group(s) that still have remaining
+  capacity.
 
   To modify the parameters of a backend after it has been added
   to the backend service, use
@@ -191,13 +200,42 @@ class AddBackendBeta(AddBackend):
   @staticmethod
   def Args(parser):
     flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.AddArgument(parser)
+    flags.AddInstanceGroupAndNetworkEndpointGroupArgs(parser, 'add to')
     backend_flags.AddDescription(parser)
-    flags.MULTISCOPE_INSTANCE_GROUP_ARG.AddArgument(
-        parser, operation_type='add to the backend service')
-    backend_flags.AddBalancingMode(parser)
-    backend_flags.AddCapacityLimits(parser)
+    backend_flags.AddBalancingMode(parser, supports_neg=True)
+    backend_flags.AddCapacityLimits(parser, supports_neg=True)
     backend_flags.AddCapacityScalar(parser)
 
+  def _GetGroupRef(self, args, resources, client):
+    if args.instance_group:
+      return flags.MULTISCOPE_INSTANCE_GROUP_ARG.ResolveAsResource(
+          args,
+          resources,
+          scope_lister=compute_flags.GetDefaultScopeLister(client))
+    if args.network_endpoint_group:
+      return flags.NETWORK_ENDPOINT_GROUP_ARG.ResolveAsResource(
+          args,
+          resources,
+          scope_lister=compute_flags.GetDefaultScopeLister(client))
+
+  def _CreateBackendMessage(self, messages, group_uri, balancing_mode, args):
+    """Overrides."""
+
+    backend_services_utils.ValidateBalancingModeArgs(
+        messages, args, supports_neg=True)
+    return messages.Backend(
+        balancingMode=balancing_mode,
+        capacityScaler=args.capacity_scaler,
+        description=args.description,
+        group=group_uri,
+        maxRate=args.max_rate,
+        maxRatePerInstance=args.max_rate_per_instance,
+        maxRatePerEndpoint=args.max_rate_per_endpoint,
+        maxUtilization=args.max_utilization,
+        maxConnections=args.max_connections,
+        maxConnectionsPerInstance=args.max_connections_per_instance,
+        maxConnectionsPerEndpoint=args.max_connections_per_endpoint)
+
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class AddBackendAlpha(AddBackendBeta):
@@ -207,11 +245,12 @@ class AddBackendAlpha(AddBackendBeta):
   backend is a group of tasks that can handle requests sent to a
   backend service. Currently, the group of tasks can be one or
   more Google Compute Engine virtual machine instances grouped
-  together using an instance group.
+  together using an instance group or network endpoint group.
 
-  Traffic is first spread evenly across all virtual machines in
-  the group. When the group is full, traffic is sent to the next
-  nearest group(s) that still have remaining capacity.
+  Traffic is first spread evenly across all virtual machines or
+  network endpoints in the group. When the group is full, traffic
+  is sent to the next nearest group(s) that still have remaining
+  capacity.
 
   To modify the parameters of a backend after it has been added
   to the backend service, use
@@ -222,9 +261,28 @@ class AddBackendAlpha(AddBackendBeta):
   @staticmethod
   def Args(parser):
     flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.AddArgument(parser)
+    flags.AddInstanceGroupAndNetworkEndpointGroupArgs(parser, 'add to')
     backend_flags.AddDescription(parser)
-    flags.MULTISCOPE_INSTANCE_GROUP_ARG.AddArgument(
-        parser, operation_type='add to the backend service')
-    backend_flags.AddBalancingMode(parser)
-    backend_flags.AddCapacityLimits(parser)
+    backend_flags.AddBalancingMode(parser, supports_neg=True)
+    backend_flags.AddCapacityLimits(parser, supports_neg=True)
     backend_flags.AddCapacityScalar(parser)
+    backend_flags.AddFailover(parser, default=None)
+
+  def _CreateBackendMessage(self, messages, group_uri, balancing_mode, args):
+    """Overrides."""
+
+    backend_services_utils.ValidateBalancingModeArgs(
+        messages, args, supports_neg=True)
+    return messages.Backend(
+        balancingMode=balancing_mode,
+        capacityScaler=args.capacity_scaler,
+        description=args.description,
+        group=group_uri,
+        maxRate=args.max_rate,
+        maxRatePerInstance=args.max_rate_per_instance,
+        maxRatePerEndpoint=args.max_rate_per_endpoint,
+        maxUtilization=args.max_utilization,
+        maxConnections=args.max_connections,
+        maxConnectionsPerInstance=args.max_connections_per_instance,
+        maxConnectionsPerEndpoint=args.max_connections_per_endpoint,
+        failover=args.failover)
