@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,24 +14,27 @@
 # limitations under the License.
 """Command for modifying backend services."""
 
-import cStringIO
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+import io
 from apitools.base.protorpclite import messages
 from apitools.base.py import encoding
 
-from googlecloudsdk.api_lib.compute import backend_services_utils
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import property_selector
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.calliope import exceptions as calliope_exceptions
 from googlecloudsdk.command_lib.compute import flags as compute_flags
+from googlecloudsdk.command_lib.compute.backend_services import backend_services_utils
 from googlecloudsdk.command_lib.compute.backend_services import flags
 from googlecloudsdk.core import resources
+from googlecloudsdk.core import yaml
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util import edit
 
-import yaml
-
 
 class InvalidResourceError(exceptions.ToolException):
   # Normally we'd want to subclass core.exceptions.Error, but base_classes.Edit
@@ -155,7 +159,7 @@ class Edit(base.Command):
       yield resource
 
   def BuildFileContents(self, args, client, original_record, modifiable_record):
-    buf = cStringIO.StringIO()
+    buf = io.StringIO()
     for line in base_classes.HELP.splitlines():
       buf.write('#')
       if line:
@@ -191,13 +195,10 @@ class Edit(base.Command):
                                                     original_record,
                                                     modifiable_record, args)
         break
-      except (ValueError, yaml.error.YAMLError,
+      except (ValueError, yaml.YAMLParseError,
               messages.ValidationError,
               calliope_exceptions.ToolException) as e:
-        if isinstance(e, ValueError):
-          message = e.message
-        else:
-          message = str(e)
+        message = getattr(e, 'message', str(e))
 
         if isinstance(e, calliope_exceptions.ToolException):
           problem_type = 'applying'
