@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Command for removing a backend from a backend service."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import encoding
 
 from googlecloudsdk.api_lib.compute import base_classes
@@ -40,8 +45,8 @@ class RemoveBackend(base.UpdateCommand):
 
   @classmethod
   def Args(cls, parser):
-    cls._BACKEND_SERVICE_ARG.AddArgument(parser)
-    cls._INSTANCE_GROUP_ARG.AddArgument(
+    flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.AddArgument(parser)
+    flags.MULTISCOPE_INSTANCE_GROUP_ARG.AddArgument(
         parser, operation_type='remove from the backend service')
 
   def GetGetRequest(self, client, backend_service_ref):
@@ -74,14 +79,16 @@ class RemoveBackend(base.UpdateCommand):
                 backendServiceResource=replacement,
                 project=backend_service_ref.project))
 
-  def Modify(self, client, resources, backend_service_ref, args, existing):
-    replacement = encoding.CopyProtoMessage(existing)
-
-    group_ref = RemoveBackend._INSTANCE_GROUP_ARG.ResolveAsResource(
+  def _GetGroupRef(self, args, resources, client):
+    return flags.MULTISCOPE_INSTANCE_GROUP_ARG.ResolveAsResource(
         args,
         resources,
         scope_lister=compute_flags.GetDefaultScopeLister(client))
 
+  def Modify(self, client, resources, backend_service_ref, args, existing):
+    replacement = encoding.CopyProtoMessage(existing)
+
+    group_ref = self._GetGroupRef(args, resources, client)
     group_uri = group_ref.SelfLink()
 
     backend_idx = None
@@ -112,10 +119,11 @@ class RemoveBackend(base.UpdateCommand):
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
     client = holder.client
 
-    backend_service_ref = self._BACKEND_SERVICE_ARG.ResolveAsResource(
-        args,
-        holder.resources,
-        scope_lister=compute_flags.GetDefaultScopeLister(client))
+    backend_service_ref = (
+        flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.ResolveAsResource(
+            args,
+            holder.resources,
+            scope_lister=compute_flags.GetDefaultScopeLister(client)))
     get_request = self.GetGetRequest(client, backend_service_ref)
 
     objects = client.MakeRequests([get_request])
@@ -127,7 +135,7 @@ class RemoveBackend(base.UpdateCommand):
         [self.GetSetRequest(client, backend_service_ref, new_object)])
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
 class RemoveBackendBeta(RemoveBackend):
   """Remove a backend from a backend service.
 
@@ -140,28 +148,19 @@ class RemoveBackendBeta(RemoveBackend):
   backend-services edit'.
   """
 
-  @classmethod
-  def Args(cls, parser):
-    cls._BACKEND_SERVICE_ARG.AddArgument(parser)
-    cls._INSTANCE_GROUP_ARG.AddArgument(
-        parser, operation_type='remove from the backend service')
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class RemoveBackendAlpha(RemoveBackendBeta):
-  """Remove a backend from a backend service.
-
-  *{command}* is used to remove a backend from a backend
-  service.
-
-  Before removing a backend, it is a good idea to "drain" the
-  backend first. A backend can be drained by setting its
-  capacity scaler to zero through 'gcloud compute
-  backend-services edit'.
-  """
+  def _GetGroupRef(self, args, resources, client):
+    if args.instance_group:
+      return flags.MULTISCOPE_INSTANCE_GROUP_ARG.ResolveAsResource(
+          args,
+          resources,
+          scope_lister=compute_flags.GetDefaultScopeLister(client))
+    if args.network_endpoint_group:
+      return flags.NETWORK_ENDPOINT_GROUP_ARG.ResolveAsResource(
+          args,
+          resources,
+          scope_lister=compute_flags.GetDefaultScopeLister(client))
 
   @classmethod
   def Args(cls, parser):
-    cls._BACKEND_SERVICE_ARG.AddArgument(parser)
-    cls._INSTANCE_GROUP_ARG.AddArgument(
-        parser, operation_type='remove from the backend service')
+    flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.AddArgument(parser)
+    flags.AddInstanceGroupAndNetworkEndpointGroupArgs(parser, 'remove from')
