@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,15 +14,20 @@
 # limitations under the License.
 """Command for deleting disks."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute.disks import flags as disks_flags
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class Delete(base.DeleteCommand):
   """Delete Google Compute Engine persistent disks.
 
@@ -34,6 +40,7 @@ class Delete(base.DeleteCommand):
   def Args(parser):
     Delete.disks_arg = disks_flags.MakeDiskArg(plural=True)
     Delete.disks_arg.AddArgument(parser, operation_type='delete')
+    parser.display_info.AddCacheUpdater(completers.DisksCompleter)
 
   def _GetCommonScopeNameForRefs(self, refs):
     """Gets common scope for references."""
@@ -91,6 +98,21 @@ class Delete(base.DeleteCommand):
     return holder.client.MakeRequests(requests)
 
 
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class BetaDelete(Delete):
+  """Delete Google Compute Engine persistent disks.
+
+  *{command}* deletes one or more Google Compute Engine
+  persistent disks. Disks can be deleted only if they are not
+  being used by any virtual machine instances.
+  """
+
+  @staticmethod
+  def Args(parser):
+    Delete.disks_arg = disks_flags.MakeDiskArgZonalOrRegional(plural=True)
+    Delete.disks_arg.AddArgument(parser, operation_type='delete')
+
+
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class AlphaDelete(Delete):
   """Delete Google Compute Engine persistent disks.
