@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Command for describing disks."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute.disks import flags as disks_flags
@@ -23,7 +28,7 @@ def _CommonArgs(parser):
   Describe.disk_arg.AddArgument(parser, operation_type='describe')
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class Describe(base.DescribeCommand):
   """Describe a Google Compute Engine disk."""
 
@@ -52,6 +57,16 @@ class Describe(base.DescribeCommand):
     return service.Get(request_type(**disk_ref.AsDict()))
 
 
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class DescribeBeta(Describe):
+  """Describe a Google Compute Engine disk."""
+
+  @staticmethod
+  def Args(parser):
+    Describe.disk_arg = disks_flags.MakeDiskArgZonalOrRegional(plural=False)
+    _CommonArgs(parser)
+
+
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class DescribeAlpha(Describe):
   """Describe a Google Compute Engine disk."""
@@ -69,5 +84,3 @@ Describe.detailed_help = {
         Engine disk in a project.
         """,
 }
-
-DescribeAlpha.detailed_help = Describe.detailed_help
