@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,10 +14,15 @@
 # limitations under the License.
 """Command for moving disks."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute.operations import poller
 from googlecloudsdk.api_lib.util import waiter
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute.disks import flags as disks_flags
 from googlecloudsdk.core import log
@@ -35,7 +41,7 @@ class Move(base.SilentCommand):
     parser.add_argument(
         '--destination-zone',
         help='The zone to move the disk to.',
-        completion_resource='compute.zones',
+        completer=completers.ZonesCompleter,
         required=True)
 
     base.ASYNC_FLAG.AddToParser(parser)
@@ -77,7 +83,7 @@ class Move(base.SilentCommand):
       log.UpdatedResource(
           operation_ref,
           kind='disk {0}'.format(target_disk.Name()),
-          async=True,
+          is_async=True,
           details='Run the [gcloud compute operations describe] command '
                   'to check the status of this operation.'
       )
