@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,13 +14,17 @@
 # limitations under the License.
 """Command for labels update to images."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute.operations import poller
 from googlecloudsdk.api_lib.util import waiter
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute.images import flags as images_flags
-from googlecloudsdk.command_lib.util import labels_util
+from googlecloudsdk.command_lib.util.args import labels_util
 
 
 @base.ReleaseTracks(
@@ -62,18 +67,15 @@ class Update(base.UpdateCommand):
         args, holder.resources,
         scope_lister=flags.GetDefaultScopeLister(holder.client))
 
-    update_labels, remove_labels = labels_util.GetAndValidateOpsFromArgs(args)
+    labels_diff = labels_util.GetAndValidateOpsFromArgs(args)
 
     image = client.images.Get(
         messages.ComputeImagesGetRequest(**image_ref.AsDict()))
 
-    replacement = labels_util.UpdateLabels(
-        image.labels,
-        messages.GlobalSetLabelsRequest.LabelsValue,
-        update_labels=update_labels,
-        remove_labels=remove_labels)
+    labels_update = labels_diff.Apply(
+        messages.GlobalSetLabelsRequest.LabelsValue, image.labels)
 
-    if not replacement:
+    if not labels_update.needs_update:
       return image
 
     request = messages.ComputeImagesSetLabelsRequest(
@@ -82,7 +84,7 @@ class Update(base.UpdateCommand):
         globalSetLabelsRequest=
         messages.GlobalSetLabelsRequest(
             labelFingerprint=image.labelFingerprint,
-            labels=replacement))
+            labels=labels_update.labels))
 
     operation = client.images.SetLabels(request)
     operation_ref = holder.resources.Parse(
