@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Command for abandoning instances owned by a managed instance group."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import instance_groups_utils
 from googlecloudsdk.calliope import arg_parsers
@@ -66,7 +71,7 @@ class AbandonInstances(base.Command):
                   instances=instances),
               project=igm_ref.project,
               zone=igm_ref.zone), field_name)
-    else:
+    elif igm_ref.Collection() == 'compute.regionInstanceGroupManagers':
       field_name = 'regionInstanceGroupManagersAbandonInstancesRequest'
       service = client.apitools_client.regionInstanceGroupManagers
       requests = instance_groups_utils.SplitInstancesInRequest(
@@ -78,6 +83,9 @@ class AbandonInstances(base.Command):
                   instances=instances),
               project=igm_ref.project,
               region=igm_ref.region,), field_name)
+    else:
+      raise ValueError('Unknown reference type {0}'.format(
+          igm_ref.Collection()))
 
     requests = instance_groups_utils.GenerateRequestTuples(
         service, 'AbandonInstances', requests)
