@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Command for deleting managed instance group."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import managed_instance_groups_utils
 from googlecloudsdk.api_lib.compute import path_simplifier
@@ -23,6 +28,9 @@ from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute.instance_groups import flags as instance_groups_flags
 from googlecloudsdk.core import properties
+from googlecloudsdk.core.console import progress_tracker
+from googlecloudsdk.core.util import text
+from six.moves import zip
 
 
 class Delete(base.DeleteCommand):
@@ -45,7 +53,7 @@ class Delete(base.DeleteCommand):
     Returns:
       Messages, which will be sent to delete autoscalers.
     """
-    mig_requests = zip(*mig_requests)[2] if mig_requests else []
+    mig_requests = list(zip(*mig_requests))[2] if mig_requests else []
     zone_migs = [(request.instanceGroupManager, 'zone',
                   managed_instance_groups_utils.CreateZoneRef(
                       holder.resources, request)) for request in mig_requests
@@ -55,8 +63,8 @@ class Delete(base.DeleteCommand):
                         holder.resources, request)) for request in mig_requests
                    if hasattr(request, 'region') and request.region is not None]
 
-    zones = zip(*zone_migs)[2] if zone_migs else []
-    regions = zip(*region_migs)[2] if region_migs else []
+    zones = list(zip(*zone_migs))[2] if zone_migs else []
+    regions = list(zip(*region_migs))[2] if region_migs else []
 
     client = holder.client.apitools_client
     messages = client.MESSAGES_MODULE
@@ -136,18 +144,32 @@ class Delete(base.DeleteCommand):
     requests = list(self._CreateDeleteRequests(
         holder.client.apitools_client, igm_refs))
 
+    resources = []
     # Delete autoscalers first.
     errors = []
-    resources = holder.client.MakeRequests(
-        self._GenerateAutoscalerDeleteRequests(
-            holder, project, mig_requests=requests),
-        errors)
-    if errors:
-      utils.RaiseToolException(errors)
+    autoscaler_delete_requests = self._GenerateAutoscalerDeleteRequests(
+        holder, project, mig_requests=requests)
+    if autoscaler_delete_requests:
+      with progress_tracker.ProgressTracker(
+          'Deleting ' + text.Pluralize(
+              len(autoscaler_delete_requests), 'autoscaler'),
+          autotick=False,
+      ) as tracker:
+        resources = holder.client.MakeRequests(
+            autoscaler_delete_requests,
+            errors,
+            progress_tracker=tracker)
+      if errors:
+        utils.RaiseToolException(errors)
 
     # Now delete instance group managers.
     errors = []
-    resources += holder.client.MakeRequests(requests, errors)
+    with progress_tracker.ProgressTracker(
+        'Deleting ' + text.Pluralize(len(requests), 'Managed Instance Group'),
+        autotick=False,
+    ) as tracker:
+      resources += holder.client.MakeRequests(
+          requests, errors, progress_tracker=tracker)
     if errors:
       utils.RaiseToolException(errors)
     return resources
