@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,6 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Command for listing named ports in instance groups."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import instance_groups_utils
 from googlecloudsdk.calliope import base
@@ -24,12 +30,10 @@ from googlecloudsdk.core import properties
 class GetNamedPorts(base.ListCommand):
   """Implements get-named-ports command, alpha, and beta versions."""
 
-  def DeprecatedFormat(self, unused_args):
-    return 'table(name, port)'
-
   @staticmethod
   def Args(parser):
     instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_ARG.AddArgument(parser)
+    parser.display_info.AddFormat('table(name, port)')
 
   def Run(self, args):
     """Retrieves response with named ports."""
