@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,9 +14,20 @@
 # limitations under the License.
 """Commands for reading and manipulating instances."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
+@base.CommandSuggestion('connect-to-serial-port',
+                        'compute connect-to-serial-port')
+@base.CommandSuggestion('copy-files', 'compute copy-files')
+@base.CommandSuggestion('reset-windows-password',
+                        'compute reset-windows-password')
+@base.CommandSuggestion('scp', 'compute scp')
+@base.CommandSuggestion('ssh', 'compute ssh')
 class Instances(base.Group):
   """Read and manipulate Google Compute Engine virtual machine instances."""
 
