@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Command for deleting access configs from virtual machine instances."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import constants
 from googlecloudsdk.calliope import arg_parsers
@@ -22,11 +27,22 @@ from googlecloudsdk.command_lib.compute.instances import flags
 
 
 class DeleteAccessConfig(base.SilentCommand):
-  """Delete an access configuration from a virtual machine network interface.
+  """Delete an access configuration from a virtual machine network interface."""
 
-  *{command}* is used to delete access configurations from network
-  interfaces of Google Compute Engine virtual machines.
-  """
+  detailed_help = {
+      'DESCRIPTION': """\
+        *{command}* is used to delete access configurations from network
+        interfaces of Google Compute Engine virtual machines. Access
+        configurations allow you to assign a public, external IP to a virtual
+        machine.
+      """,
+      'EXAMPLES': """\
+        To remove the externally accessible IP from a virtual machine named
+        ``example-instance'' in zone ``us-central1-a'', run:
+
+          $ {command} example-instance --zone us-central1-a
+      """,
+  }
 
   @staticmethod
   def Args(parser):
