@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Command for setting minimum CPU platform for virtual machine instances."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute.operations import poller
 from googlecloudsdk.api_lib.util import waiter
@@ -64,7 +69,7 @@ class SetMinCpuPlatform(base.UpdateCommand):
       log.UpdatedResource(
           operation_ref,
           kind='gce instance [{0}]'.format(instance_ref.Name()),
-          async=True,
+          is_async=True,
           details='Use [gcloud compute operations describe] command '
                   'to check the status of this operation.'
       )
