@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +15,12 @@
 
 """Command for setting scheduling for virtual machine instances."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
+from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute.instances import flags
 
@@ -26,18 +32,18 @@ class SetSchedulingInstances(base.SilentCommand):
   Engine virtual machines.
   """
 
-  @staticmethod
-  def Args(parser):
+  @classmethod
+  def Args(cls, parser):
     parser.add_argument(
         '--restart-on-failure',
-        action='store_true',
-        default=None,  # Tri-valued: None => don't change the setting.
+        action=arg_parsers.StoreTrueFalseAction,
         help="""\
         The instances will be restarted if they are terminated by Compute
         Engine.  This does not affect terminations performed by the user.
         """)
 
-    flags.AddMaintenancePolicyArgs(parser)
+    is_alpha = cls.ReleaseTrack() in [base.ReleaseTrack.ALPHA]
+    flags.AddMaintenancePolicyArgs(parser, deprecate=is_alpha)
     flags.INSTANCE_ARG.AddArgument(parser)
 
   def Run(self, args):
