@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,13 +14,18 @@
 # limitations under the License.
 """Commands for reading and manipulating network peerings."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
 class Peerings(base.Group):
   """List, create, and delete Google Compute Engine network peerings."""
 
 
-Peerings.detailed_help = {
-    'brief': 'List, create, and delete Google Compute Engine network peerings',
-}
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class PeeringsAlpha(base.Group):
+  """List, create, delete, and update Google Compute Engine network peerings."""
