@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,15 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Command for creating network peerings."""
-from apitools.base.py import encoding
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.compute import base_classes
-from googlecloudsdk.api_lib.compute import networks_utils
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.compute.networks.peerings import flags
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
 
 
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
 class Create(base.Command):
   """Create a Google Compute Engine network peering."""
 
@@ -78,7 +83,42 @@ class Create(base.Command):
             peerNetwork=peer_network_ref.RelativeName()),
         project=properties.VALUES.core.project.GetOrFail())
 
-    response = client.MakeRequests([(client.apitools_client.networks,
-                                     'AddPeering', request)])
+    return client.MakeRequests([(client.apitools_client.networks, 'AddPeering',
+                                 request)])
 
-    return networks_utils.AddMode([encoding.MessageToDict(m) for m in response])
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class CreateAlpha(Create):
+  """Create a Google Compute Engine network peering."""
+
+  @staticmethod
+  def Args(parser):
+    super(CreateAlpha, CreateAlpha).Args(parser)
+    flags.AddImportCustomRoutesFlag(parser)
+    flags.AddExportCustomRoutesFlag(parser)
+
+  def Run(self, args):
+    """Issues the request necessary for adding the peering."""
+    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+    client = holder.client
+
+    peer_network_ref = resources.REGISTRY.Parse(
+        args.peer_network,
+        params={
+            'project':
+                args.peer_project or properties.VALUES.core.project.GetOrFail
+        },
+        collection='compute.networks')
+
+    request = client.messages.ComputeNetworksAddPeeringRequest(
+        network=args.network,
+        networksAddPeeringRequest=client.messages.NetworksAddPeeringRequest(
+            autoCreateRoutes=args.auto_create_routes,
+            name=args.name,
+            peerNetwork=peer_network_ref.RelativeName(),
+            exportCustomRoutes=args.export_custom_routes,
+            importCustomRoutes=args.import_custom_routes),
+        project=properties.VALUES.core.project.GetOrFail())
+
+    return client.MakeRequests([(client.apitools_client.networks, 'AddPeering',
+                                 request)])
