@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,14 +12,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """Command for modifying the properties of a subnetwork."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
+from googlecloudsdk.api_lib.compute import subnets_utils
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute.networks.subnets import flags
 
 
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class Update(base.UpdateCommand):
   """Updates properties of an existing Google Compute Engine subnetwork."""
 
@@ -34,33 +40,50 @@ class Update(base.UpdateCommand):
     cls.SUBNETWORK_ARG = flags.SubnetworkArgument()
     cls.SUBNETWORK_ARG.AddArgument(parser, operation_type='update')
 
-    parser.add_argument(
-        '--enable-private-ip-google-access',
-        action='store_true',
-        default=None,  # Tri-valued, None => do not change.
-        help=('Enable/disable access to Google Cloud APIs from this subnet for '
-              'instances without a public ip address.'))
+    flags.AddUpdateArgs(parser)
 
   def Run(self, args):
     """Issues requests necessary to update Subnetworks."""
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
     client = holder.client
-
-    request_list = []
     subnet_ref = self.SUBNETWORK_ARG.ResolveAsResource(args, holder.resources)
 
-    if args.enable_private_ip_google_access is not None:
-      google_access = (
-          client.messages.SubnetworksSetPrivateIpGoogleAccessRequest())
-      google_access.privateIpGoogleAccess = args.enable_private_ip_google_access
+    return subnets_utils.MakeSubnetworkUpdateRequest(
+        client,
+        subnet_ref,
+        enable_private_ip_google_access=args.enable_private_ip_google_access,
+        add_secondary_ranges=args.add_secondary_ranges,
+        remove_secondary_ranges=args.remove_secondary_ranges,
+        enable_flow_logs=args.enable_flow_logs)
 
-      google_access_request = (
-          client.messages.ComputeSubnetworksSetPrivateIpGoogleAccessRequest(
-              project=subnet_ref.project,
-              region=subnet_ref.region,
-              subnetwork=subnet_ref.Name(),
-              subnetworksSetPrivateIpGoogleAccessRequest=google_access))
-      request_list.append((client.apitools_client.subnetworks,
-                           'SetPrivateIpGoogleAccess', google_access_request))
 
-    return client.MakeRequests(request_list)
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class UpdateAlpha(base.UpdateCommand):
+  """Updates properties of an existing Google Compute Engine subnetwork."""
+
+  @classmethod
+  def Args(cls, parser):
+    """The command argument handler."""
+    cls.SUBNETWORK_ARG = flags.SubnetworkArgument()
+    cls.SUBNETWORK_ARG.AddArgument(parser, operation_type='update')
+
+    flags.AddUpdateArgs(parser, include_alpha=True)
+
+  def Run(self, args):
+    """Issues requests necessary to update Subnetworks."""
+    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+    client = holder.client
+    subnet_ref = self.SUBNETWORK_ARG.ResolveAsResource(args, holder.resources)
+
+    return subnets_utils.MakeSubnetworkUpdateRequest(
+        client,
+        subnet_ref,
+        enable_private_ip_google_access=args.enable_private_ip_google_access,
+        add_secondary_ranges=args.add_secondary_ranges,
+        remove_secondary_ranges=args.remove_secondary_ranges,
+        enable_flow_logs=args.enable_flow_logs,
+        aggregation_interval=args.aggregation_interval,
+        flow_sampling=args.flow_sampling,
+        metadata=args.metadata,
+        set_role_active=getattr(args, 'role', None) == 'ACTIVE',
+        drain_timeout_seconds=args.drain_timeout)
