@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """rolling-updates start command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.compute import rolling_updates_util as updater_util
@@ -63,18 +68,16 @@ class Start(base.Command):
         Specifies minimum amount of time we will spend on updating single
         instance, measuring at the start of the first update action (currently
         only 'RECREATE' call). If actual instance update takes less time we will
-        simply sleep before proceeding with next instance. Valid units for this
-        flag are ``s'' for seconds, ``m'' for minutes, ``h'' for hours and
-        ``d'' for days. If no unit is specified, seconds is assumed.
+        simply sleep before proceeding with next instance.
+        See $ gcloud topic datetimes for information on duration formats.
         """)
     parser.add_argument(
         '--instance-startup-timeout', type=arg_parsers.Duration(),
         help="""\
         Maximum amount of time we will wait after finishing all steps until
         instance is in *RUNNING* state. If this deadline is exceeded instance
-        update is considered as failed. Valid units for this flag are ``s'' for
-        seconds, ``m'' for minutes, ``h'' for hours and ``d'' for days. If no
-        unit is specified, seconds is assumed.
+        update is considered as failed.
+        See $ gcloud topic datetimes for information on duration formats.
         """)
     parser.add_argument(
         '--max-num-failed-instances', type=int,
@@ -96,7 +99,7 @@ class Start(base.Command):
     Raises:
       HttpException: An http error response was received while executing api
           request.
-      ToolException: An error other than http error occured while executing
+      ToolException: An error other than http error occurred while executing
           the command.
     """
     client = updater_util.GetApiClientInstance()
