@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,7 +14,9 @@
 # limitations under the License.
 """Command for adding a BGP peer to a Google Compute Engine router."""
 
-from apitools.base.py import encoding
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute.operations import poller
@@ -23,64 +26,10 @@ from googlecloudsdk.command_lib.compute.routers import flags
 from googlecloudsdk.command_lib.compute.routers import router_utils
 from googlecloudsdk.core import log
 from googlecloudsdk.core import resources
+import six
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class AddBgpPeer(base.UpdateCommand):
-  """Add a BGP peer to a Google Compute Engine router.
-
-  *{command}* is used to add a BGP peer to a Google Compute Engine router.
-  """
-
-  ROUTER_ARG = None
-
-  @classmethod
-  def Args(cls, parser):
-    cls.ROUTER_ARG = flags.RouterArgument()
-    cls.ROUTER_ARG.AddArgument(parser, operation_type='update')
-    flags.AddBgpPeerArgs(parser, for_add_bgp_peer=True)
-
-  def GetGetRequest(self, client, router_ref):
-    return (client.apitools_client.routers, 'Get',
-            client.messages.ComputeRoutersGetRequest(
-                router=router_ref.Name(),
-                region=router_ref.region,
-                project=router_ref.project))
-
-  def GetSetRequest(self, client, router_ref, replacement):
-    return (client.apitools_client.routers, 'Update',
-            client.messages.ComputeRoutersUpdateRequest(
-                router=router_ref.Name(),
-                routerResource=replacement,
-                region=router_ref.region,
-                project=router_ref.project))
-
-  def Modify(self, client, args, existing):
-    replacement = encoding.CopyProtoMessage(existing)
-
-    peer = _CreateBgpPeer(client.messages, args)
-    replacement.bgpPeers.append(peer)
-
-    return replacement
-
-  def Run(self, args):
-    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
-    client = holder.client
-
-    router_ref = self.ROUTER_ARG.ResolveAsResource(args, holder.resources)
-    get_request = self.GetGetRequest(client, router_ref)
-
-    # There is only one response because one request is made
-    router = client.MakeRequests([get_request])[0]
-
-    modified_router = self.Modify(client, args, router)
-
-    return client.MakeRequests(
-        [self.GetSetRequest(client, router_ref, modified_router)])
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class AddBgpPeerAlpha(base.UpdateCommand):
   """Add a BGP peer to a Google Compute Engine router."""
 
   ROUTER_ARG = None
@@ -105,19 +54,19 @@ class AddBgpPeerAlpha(base.UpdateCommand):
     request_type = messages.ComputeRoutersGetRequest
     replacement = service.Get(request_type(**router_ref.AsDict()))
 
-    peer = _CreateBgpPeer(messages, args)
+    peer = _CreateBgpPeerMessage(messages, args)
 
     if router_utils.HasReplaceAdvertisementFlags(args):
-      mode, groups, prefixes = router_utils.ParseAdvertisements(
+      mode, groups, ranges = router_utils.ParseAdvertisements(
           messages=messages, resource_class=messages.RouterBgpPeer, args=args)
 
       attrs = {
           'advertiseMode': mode,
           'advertisedGroups': groups,
-          'advertisedPrefixs': prefixes,
+          'advertisedIpRanges': ranges,
       }
 
-      for attr, value in attrs.iteritems():
+      for attr, value in six.iteritems(attrs):
         if value is not None:
           setattr(peer, attr, value)
 
@@ -143,7 +92,7 @@ class AddBgpPeerAlpha(base.UpdateCommand):
           operation_ref,
           kind='router [{0}] to add peer [{1}]'.format(router_ref.Name(),
                                                        peer.name),
-          async=True,
+          is_async=True,
           details='Run the [gcloud compute operations describe] command '
           'to check the status of this operation.')
       return result
@@ -162,7 +111,7 @@ class AddBgpPeerAlpha(base.UpdateCommand):
                               peer.name, router_ref.Name()))
 
 
-def _CreateBgpPeer(messages, args):
+def _CreateBgpPeerMessage(messages, args):
   """Creates a BGP peer with base attributes based on flag arguments."""
 
   return messages.RouterBgpPeer(
