@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,11 +14,17 @@
 # limitations under the License.
 
 """Implements the command for SSHing into an instance."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import argparse
 import sys
 
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute import ssh_utils
@@ -28,82 +35,9 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core.util import retry
 
 
-def _Args(parser):
-  """Argument parsing for ssh, including hook for remote completion."""
-  ssh_utils.BaseSSHCLIHelper.Args(parser)
-
-  parser.add_argument(
-      '--command',
-      help="""\
-      A command to run on the virtual machine.
-
-      Runs the command on the target instance and then exits.
-      """)
-
-  parser.add_argument(
-      '--ssh-flag',
-      action='append',
-      help="""\
-      Additional flags to be passed to *ssh(1)*. It is recommended that flags
-      be passed using an assignment operator and quotes. This flag will
-      replace occurences of ``%USER%'' and ``%INSTANCE%'' with their
-      dereferenced values. Example:
-
-        $ {command} example-instance --zone us-central1-a --ssh-flag="-vvv" --ssh-flag="-L 80:%INSTANCE%:80"
-
-      is equivalent to passing the flags ``--vvv'' and ``-L
-      80:162.222.181.197:80'' to *ssh(1)* if the external IP address of
-      'example-instance' is 162.222.181.197.
-      """)
-
-  parser.add_argument(
-      '--container',
-      help="""\
-          The name or ID of a container inside of the virtual machine instance
-          to connect to. This only applies to virtual machines that are using
-          a Google Container-Optimized virtual machine image. For more
-          information, see [](https://cloud.google.com/compute/docs/containers)
-          """)
-
-  parser.add_argument(
-      'user_host',
-      completion_resource='compute.instances',
-      metavar='[USER@]INSTANCE',
-      help="""\
-      Specifies the instance to SSH into.
-
-      ``USER'' specifies the username with which to SSH. If omitted,
-      $USER from the environment is selected.
-
-      ``INSTANCE'' specifies the name of the virtual machine instance to SSH
-      into.
-      """)
-
-  parser.add_argument(
-      'ssh_args',
-      nargs=argparse.REMAINDER,
-      help="""\
-          Flags and positionals passed to the underlying ssh implementation.
-          """,
-      example="""\
-        $ {command} example-instance --zone us-central1-a -- -vvv -L 80:%INSTANCE%:80
-      """)
-
-  flags.AddZoneFlag(
-      parser,
-      resource_type='instance',
-      operation_type='connect to')
-
-
-@base.ReleaseTracks(base.ReleaseTrack.GA)
-class SshGA(base.Command):
+class Ssh(base.Command):
   """SSH into a virtual machine instance."""
 
-  def __init__(self, *args, **kwargs):
-    super(SshGA, self).__init__(*args, **kwargs)
-    self._use_account_service = False
-    self._use_internal_ip = False
-
   @staticmethod
   def Args(parser):
     """Set up arguments for this command.
@@ -111,19 +45,94 @@ class SshGA(base.Command):
     Args:
       parser: An argparse.ArgumentParser.
     """
-    _Args(parser)
+    ssh_utils.BaseSSHCLIHelper.Args(parser)
+
+    parser.add_argument(
+        '--command',
+        help="""\
+        A command to run on the virtual machine.
+
+        Runs the command on the target instance and then exits.
+        """)
+
+    parser.add_argument(
+        '--ssh-flag',
+        action='append',
+        help="""\
+        Additional flags to be passed to *ssh(1)*. It is recommended that flags
+        be passed using an assignment operator and quotes. This flag will
+        replace occurences of ``%USER%'' and ``%INSTANCE%'' with their
+        dereferenced values. Example:
+
+          $ {command} example-instance --zone us-central1-a --ssh-flag="-vvv" --ssh-flag="-L 80:%INSTANCE%:80"
+
+        is equivalent to passing the flags ``--vvv'' and ``-L
+        80:162.222.181.197:80'' to *ssh(1)* if the external IP address of
+        'example-instance' is 162.222.181.197.
+        """)
+
+    parser.add_argument(
+        '--container',
+        help="""\
+            The name or ID of a container inside of the virtual machine instance
+            to connect to. This only applies to virtual machines that are using
+            a Google Container-Optimized virtual machine image. For more
+            information, see [](https://cloud.google.com/compute/docs/containers).
+            """)
+
+    parser.add_argument(
+        'user_host',
+        completer=completers.InstancesCompleter,
+        metavar='[USER@]INSTANCE',
+        help="""\
+        Specifies the instance to SSH into.
+
+        ``USER'' specifies the username with which to SSH. If omitted,
+        $USER from the environment is selected.
+
+        ``INSTANCE'' specifies the name of the virtual machine instance to SSH
+        into.
+        """)
+
+    parser.add_argument(
+        'ssh_args',
+        nargs=argparse.REMAINDER,
+        help="""\
+            Flags and positionals passed to the underlying ssh implementation.
+            """,
+        example="""\
+          $ {command} example-instance --zone us-central1-a -- -vvv -L 80:%INSTANCE%:80
+        """)
+
+    parser.add_argument(
+        '--internal-ip',
+        default=False,
+        action='store_true',
+        help="""\
+          Connect to instances using their internal IP addresses rather than their
+          external IP addresses. Use this to connect from one instance to another
+          on the same VPC network, over a VPN connection, or between two peered
+          VPC networks.
+
+          For this connection to work, you must configure your networks and
+          firewall to allow SSH connections to the internal IP address of
+          the instance to which you want to connect.
+
+          To learn how to use this flag, see
+          [](https://cloud.google.com/compute/docs/instances/connecting-advanced#sshbetweeninstances).
+          """)
+
+    flags.AddZoneFlag(
+        parser, resource_type='instance', operation_type='connect to')
 
   def Run(self, args):
     """See ssh_utils.BaseSSHCLICommand.Run."""
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
-    cua_holder = base_classes.ComputeUserAccountsApiHolder(self.ReleaseTrack())
     client = holder.client
 
     ssh_helper = ssh_utils.BaseSSHCLIHelper()
     ssh_helper.Run(args)
-    user, instance_name = ssh_utils.GetUserAndInstance(
-        args.user_host, self._use_account_service,
-        client.apitools_client.http)
+    user, instance_name = ssh_utils.GetUserAndInstance(args.user_host)
     instance_ref = instance_flags.SSH_INSTANCE_RESOLVER.ResolveResources(
         [instance_name], compute_scope.ScopeEnum.ZONE, args.zone,
         holder.resources,
@@ -133,10 +142,10 @@ class SshGA(base.Command):
     if args.plain:
       use_oslogin = False
     else:
-      user, use_oslogin = ssh_helper.CheckForOsloginAndGetUser(
-          instance, project, user,
-          self.ReleaseTrack(), client.apitools_client.http)
-    if self._use_internal_ip:
+      public_key = ssh_helper.keys.GetPublicKey().ToEntry(include_comment=True)
+      user, use_oslogin = ssh.CheckForOsloginAndGetUser(
+          instance, project, user, public_key, self.ReleaseTrack())
+    if args.internal_ip:
       ip_address = ssh_utils.GetInternalIPAddress(instance)
     else:
       ip_address = ssh_utils.GetExternalIPAddress(instance)
@@ -150,17 +159,9 @@ class SshGA(base.Command):
       options = ssh_helper.GetConfig(ssh_utils.HostKeyAlias(instance),
                                      args.strict_host_key_checking)
 
-    extra_flags = []
+    extra_flags = ssh.ParseAndSubstituteSSHFlags(args, remote, ip_address)
     remainder = []
 
-    if args.ssh_flag:
-      for flag in args.ssh_flag:
-        for flag_part in flag.split():  # We want grouping here
-          dereferenced_flag = (
-              flag_part.replace('%USER%', remote.user)
-              .replace('%INSTANCE%', ip_address))
-          extra_flags.append(dereferenced_flag)
-
     if args.ssh_args:
       remainder.extend(args.ssh_args)
 
@@ -181,9 +182,7 @@ class SshGA(base.Command):
       keys_newly_added = False
     else:
       keys_newly_added = ssh_helper.EnsureSSHKeyExists(
-          client, cua_holder.client,
-          remote.user, instance, project,
-          use_account_service=self._use_account_service)
+          client, remote.user, instance, project)
 
     if keys_newly_added:
       poller = ssh.SSHPoller(
@@ -197,8 +196,8 @@ class SshGA(base.Command):
       except retry.WaitException:
         raise ssh_utils.NetworkError()
 
-    if self._use_internal_ip:
-      ssh_helper.PreliminarylyVerifyInstance(instance.id, remote, identity_file,
+    if args.internal_ip:
+      ssh_helper.PreliminarilyVerifyInstance(instance.id, remote, identity_file,
                                              options)
 
     return_code = cmd.Run(ssh_helper.env, force_connect=True)
@@ -208,66 +207,7 @@ class SshGA(base.Command):
       sys.exit(return_code)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
-class SshBeta(SshGA):
-  """SSH into a virtual machine instance."""
-
-  def __init__(self, *args, **kwargs):
-    super(SshBeta, self).__init__(*args, **kwargs)
-    self._use_account_service = True
-
-  @staticmethod
-  def Args(parser):
-    """Set up arguments for this command.
-
-    Args:
-      parser: An argparse.ArgumentParser.
-    """
-    _Args(parser)
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class SshAlpha(SshBeta):
-  """SSH into a virtual machine instance."""
-
-  @staticmethod
-  def Args(parser):
-    """Set up arguments for this command.
-
-    Args:
-      parser: An argparse.ArgumentParser.
-    """
-    _Args(parser)
-    parser.add_argument(
-        '--internal-ip', default=False, action='store_true',
-        help="""\
-        Connect to instances using their private IP addresses. By default,
-        gcloud attempts to establish a ssh connection only if the specified
-        instance has a public IP address. When this flag is present, gcloud
-        attempts to connect to the instance using the IP of the internal network
-        instance even if the instance has a public IP.
-
-        For the connection to work, your network must already be configured to
-        allow you to establish connections using the instance's private
-        network IP.
-
-        The same IP can appear in two internal networks and can result in an
-        incorrect connection. For example, instance-1 in network-1 has the same
-        internal IP as instance-2 in network-2. You want to connect to
-        instance-1 but your network configuration results in opening a
-        connection to instance-2. gcloud will verify that the instance it
-        connected to reports the same id as the instance you requested
-        connection to. This only helps with catching network configuration
-        mistakes and is not meant as protection against any kind of attack.
-        `errors gcloud` requires the instance to have curl tool available.""")
-
-  def Run(self, args):
-    """See SshGA.Run."""
-    self._use_internal_ip = args.internal_ip
-    super(SshAlpha, self).Run(args)
-
-
-def DetailedHelp(version):
+def DetailedHelp():
   """Construct help text based on the command release track."""
   detailed_help = {
       'brief': 'SSH into a virtual machine instance',
@@ -276,7 +216,19 @@ def DetailedHelp(version):
         takes care of authentication and the translation of the
         instance name into an IP address.
 
-        This command ensures that the user's public SSH key is present
+        Note, this command does not work when connecting to Windows VMs. To
+        connect to a Windows instance using a command-line method, refer to this
+        guide: https://cloud.google.com/compute/docs/instances/connecting-to-instance#windows_cli
+
+        The default network comes preconfigured to allow ssh access to
+        all VMs. If the default network was edited, or if not using the
+        default network, you may need to explicitly enable ssh access by adding
+        a firewall-rule:
+
+          $ gcloud compute firewall-rules create --network=NETWORK \
+            default-allow-ssh --allow tcp:22
+
+        {command} ensures that the user's public SSH key is present
         in the project's metadata. If the user does not have a public
         SSH key, one is generated using *ssh-keygen(1)* (if the `--quiet`
         flag is given, the generated key will have an empty passphrase).
@@ -291,27 +243,13 @@ def DetailedHelp(version):
 
           $ {command} example-instance --zone us-central1-a --command "ps -ejH"
 
-        If you are using the Google container virtual machine image, you
-        can SSH into one of your containers with:
+        If you are using the Google Container-Optimized virtual machine image,
+        you can SSH into one of your containers with:
 
           $ {command} example-instance --zone us-central1-a --container CONTAINER
         """,
   }
-  if version == 'BETA':
-    detailed_help['DESCRIPTION'] = """\
-        *{command}* is a thin wrapper around the *ssh(1)* command that
-        takes care of authentication and the translation of the
-        instance name into an IP address.
-
-        This command uses the Compute Accounts API to ensure that the user's
-        public SSH key is availibe to the VM. This form of key management
-        will only work with VMs configured to work with the Compute Accounts
-        API. If the user does not have a public SSH key, one is generated using
-        *ssh-keygen(1)* (if `the --quiet` flag is given, the generated key will
-        have an empty passphrase).
-
-        """
   return detailed_help
 
-SshGA.detailed_help = DetailedHelp('GA')
-SshBeta.detailed_help = DetailedHelp('BETA')
+
+Ssh.detailed_help = DetailedHelp()
