@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,13 +14,18 @@
 # limitations under the License.
 """Commands for reading and manipulating SSL certificates."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
 class SslCertificates(base.Group):
-  """List, create, and delete Google Compute Engine SSL certificates."""
+  """List, create, and delete Google Compute Engine SSL certificates.
 
+  List, create and delete Google Compute Engine SSL certificates that can be
+  used to configure a target HTTPS proxy. For more information, see:
 
-SslCertificates.detailed_help = {
-    'brief': 'List, create, and delete Google Compute Engine SSL certificates',
-}
+  [](https://cloud.google.com/load-balancing/docs/ssl-certificates)
+  """
