@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Command for creating target VPN Gateways."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute import flags as compute_flags
@@ -50,6 +55,8 @@ class Create(base.CreateCommand):
         '--description',
         help='An optional, textual description for the target VPN Gateway.')
 
+    parser.display_info.AddCacheUpdater(flags.TargetVpnGatewaysCompleter)
+
   def Run(self, args):
     """Issues API requests to construct Target VPN Gateways.
 
