@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,8 +14,11 @@
 # limitations under the License.
 """Command for modifying URL maps."""
 
-import cStringIO
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
+import io
 from apitools.base.protorpclite import messages
 from apitools.base.py import encoding
 
@@ -22,14 +26,13 @@ from googlecloudsdk.api_lib.compute import base_classes
 from googlecloudsdk.api_lib.compute import property_selector
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
-from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute.url_maps import flags
+from googlecloudsdk.command_lib.compute.url_maps import url_maps_utils
 from googlecloudsdk.core import resources
+from googlecloudsdk.core import yaml
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util import edit
 
-import yaml
-
 
 class InvalidResourceError(exceptions.ToolException):
   # Normally we'd want to subclass core.exceptions.Error, but base_classes.Edit
@@ -139,13 +142,10 @@ class EditGA(base.Command):
             holder, url_map_ref, file_contents, original_object,
             original_record, modifiable_record, args)
         break
-      except (ValueError, yaml.error.YAMLError,
+      except (ValueError, yaml.YAMLParseError,
               messages.ValidationError,
               exceptions.ToolException) as e:
-        if isinstance(e, ValueError):
-          message = e.message
-        else:
-          message = str(e)
+        message = getattr(e, 'message', str(e))
 
         if isinstance(e, exceptions.ToolException):
           problem_type = 'applying'
@@ -161,7 +161,7 @@ class EditGA(base.Command):
     return resource_list
 
   def BuildFileContents(self, args, client, modifiable_record, original_record):
-    buf = cStringIO.StringIO()
+    buf = io.StringIO()
     for line in base_classes.HELP.splitlines():
       buf.write('#')
       if line:
@@ -267,6 +267,8 @@ class EditGA(base.Command):
       return NormalizeReference
 
     allowed_collections = ['compute.backendServices', 'compute.backendBuckets']
+    if self.TRACK == 'alpha':
+      allowed_collections += ['compute.regionBackendServices']
     return [
         ('defaultService', MakeReferenceNormalizer(
             'defaultService', allowed_collections)),
@@ -279,10 +281,25 @@ class EditGA(base.Command):
     ]
 
   def GetGetRequest(self, client, url_map_ref):
+    if url_maps_utils.IsRegionalUrlMapRef(url_map_ref):
+      return (client.apitools_client.regionUrlMaps, 'Get',
+              client.messages.ComputeRegionUrlMapsGetRequest(
+                  urlMap=url_map_ref.Name(),
+                  project=url_map_ref.project,
+                  region=url_map_ref.region))
+
     return (client.apitools_client.urlMaps, 'Get',
             client.messages.ComputeUrlMapsGetRequest(**url_map_ref.AsDict()))
 
   def GetSetRequest(self, client, url_map_ref, replacement):
+    if url_maps_utils.IsRegionalUrlMapRef(url_map_ref):
+      return (client.apitools_client.regionUrlMaps, 'Update',
+              client.messages.ComputeRegionUrlMapsUpdateRequest(
+                  urlMap=url_map_ref.Name(),
+                  urlMapResource=replacement,
+                  project=url_map_ref.project,
+                  region=url_map_ref.region))
+
     return (client.apitools_client.urlMaps, 'Update',
             client.messages.ComputeUrlMapsUpdateRequest(
                 urlMapResource=replacement, **url_map_ref.AsDict()))
@@ -301,6 +318,11 @@ class EditAlpha(EditBeta):
 
   TRACK = 'alpha'
 
+  @classmethod
+  def Args(cls, parser):
+    cls.URL_MAP_ARG = flags.UrlMapArgument(include_alpha=True)
+    cls.URL_MAP_ARG.AddArgument(parser)
+
 
 EditGA.detailed_help = {
     'brief': 'Modify URL maps',
