@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Command for creating VPN tunnels."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import argparse
 import re
 
@@ -34,7 +39,7 @@ class DeprecatedArgumentException(exceptions.ToolException):
 
   def __init__(self, arg, msg):
     super(DeprecatedArgumentException, self).__init__(
-        u'{0} is deprecated. {1}'.format(arg, msg))
+        '{0} is deprecated. {1}'.format(arg, msg))
 
 
 def ValidateSimpleSharedSecret(possible_secret):
@@ -113,7 +118,8 @@ class CreateGA(base.CreateCommand):
     parser.add_argument(
         '--ike-networks',
         type=arg_parsers.ArgList(min_length=1),
-        help=argparse.SUPPRESS)
+        hidden=True,
+        help='THIS ARGUMENT NEEDS HELP TEXT.')
 
     parser.add_argument(
         '--local-traffic-selector',
@@ -144,6 +150,8 @@ class CreateGA(base.CreateCommand):
         '--router',
         help='The Router to use for dynamic routing.')
 
+    parser.display_info.AddCacheUpdater(flags.VpnTunnelsCompleter)
+
   def Run(self, args):
     """Issues API requests to construct VPN Tunnels."""
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
