@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,23 +13,40 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Command for listing VPN tunnels."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.compute import base_classes
+from googlecloudsdk.api_lib.compute import lister
+from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.compute.vpn_tunnels import flags
 
 
-class List(base_classes.RegionalLister):
+class List(base.ListCommand):
   """List VPN tunnels."""
 
   # Placeholder to indicate that a detailed_help field exists and should
   # be set outside the class definition.
   detailed_help = None
 
-  @property
-  def service(self):
-    return self.compute.vpnTunnels
+  @staticmethod
+  def Args(parser):
+    parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
+    lister.AddRegionsArg(parser)
+    parser.display_info.AddCacheUpdater(flags.VpnTunnelsCompleter)
 
-  @property
-  def resource_type(self):
-    return 'vpnTunnels'
+  def Run(self, args):
+    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+    client = holder.client
+
+    request_data = lister.ParseRegionalFlags(args, holder.resources)
+
+    list_implementation = lister.RegionalLister(
+        client, client.apitools_client.vpnTunnels)
+
+    return lister.Invoke(request_data, list_implementation)
 
 
 List.detailed_help = base_classes.GetRegionalListerHelp('VPN tunnels')
