@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Command to set properties."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions as c_exc
 from googlecloudsdk.command_lib.config import completers
@@ -20,6 +25,8 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.configurations import named_configs
 
+import six
+
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class GetValue(base.Command):
@@ -53,11 +60,13 @@ class GetValue(base.Command):
         completer=completers.PropertiesCompleter,
         help='The property to be fetched. Note that `SECTION/` is optional'
         ' while referring to properties in the core section.')
+    parser.display_info.AddFormat('value(.)')
 
   def Run(self, args):
     config_name = named_configs.ConfigurationStore.ActiveConfig().name
-    log.status.write('Your active configuration is: [{0}]\n\n'.format(
-        config_name))
+    if config_name != 'default':
+      log.status.write('Your active configuration is: [{0}]\n'.format(
+          config_name))
 
     section, prop = properties.ParsePropertyString(args.property)
     if not prop:
@@ -76,10 +85,7 @@ class GetValue(base.Command):
         log.err.Print('(unset)')
     except properties.InvalidValueError as e:
       # Writing warning to stderr but returning invalid value as is
-      log.warn(str(e))
+      log.warning(six.text_type(e))
       value = properties.VALUES.Section(section).Property(prop).Get(
           validate=False)
     return value
-
-  def DeprecatedFormat(self, args):
-    return 'value(.)'
