@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,34 +14,30 @@
 # limitations under the License.
 """Stream-logs command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
 from googlecloudsdk.api_lib.cloudbuild import logs as cb_logs
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.container.builds import flags
 from googlecloudsdk.core import properties
 from googlecloudsdk.core import resources
 
+DEPRECATED_WARNING_MESSAGE = """\
+This command is deprecated and will be removed on or after 2018-10-31. Please
+use `gcloud builds log` instead."""
 
+
+@base.Deprecate(is_removed=False, warning=DEPRECATED_WARNING_MESSAGE)
 @base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class Log(base.Command):
   """Stream the logs for a build."""
 
   @staticmethod
   def Args(parser):
-    """Register flags for this command.
-
-    Args:
-      parser: An argparse.ArgumentParser-like object. It is mocked out in order
-          to capture some information, but behaves like an ArgumentParser.
-    """
-    parser.add_argument(
-        'build',
-        completion_resource='cloudbuild.projects.builds',
-        list_command_path='container builds list --uri',
-        help=('The build whose logs shall be printed. The ID of the build is '
-              'printed at the end of the build submission process, or in the '
-              'ID column when listing builds.'),
-    )
+    flags.AddBuildArg(parser, intro='The build whose logs shall be printed.')
     parser.add_argument(
         '--stream',
         help=('If a build is ongoing, stream the logs to stdout until '
