@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,10 +14,14 @@
 # limitations under the License.
 
 """Delete cluster command."""
-import argparse
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from apitools.base.py import exceptions as apitools_exceptions
 
+from googlecloudsdk.api_lib.container import kubeconfig as kconfig
 from googlecloudsdk.api_lib.container import util
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
@@ -27,7 +32,22 @@ from googlecloudsdk.core.console import console_io
 
 
 class Delete(base.DeleteCommand):
-  """Delete an existing cluster for running containers."""
+  """Delete an existing cluster for running containers.
+
+  When you delete a cluster, the following resources are deleted:
+
+  - The master resources
+  - All of the node instances in the cluster
+  - Any Pods that are running on those instances
+  - Any firewalls and routes created by Kubernetes Engine at the time of cluster
+    creation
+  - Data stored in host hostPath and emptyDir volumes
+
+  The following resources are not deleted:
+
+  - External load balancers created by the cluster
+  - Persistent disk volumes
+  """
 
   @staticmethod
   def Args(parser):
@@ -46,7 +66,8 @@ class Delete(base.DeleteCommand):
         '--timeout',
         type=int,
         default=1800,
-        help=argparse.SUPPRESS)
+        hidden=True,
+        help='THIS ARGUMENT NEEDS HELP TEXT.')
     flags.AddAsyncFlag(parser)
 
   def Run(self, args):
@@ -60,12 +81,11 @@ class Delete(base.DeleteCommand):
       Some value that we want to have printed later.
     """
     adapter = self.context['api_adapter']
-
+    location_get = self.context['location_get']
+    location = location_get(args)
     cluster_refs = []
     for name in args.names:
-      cluster_refs.append(
-          adapter.ParseCluster(name,
-                               getattr(args, 'region', None)))
+      cluster_refs.append(adapter.ParseCluster(name, location))
     console_io.PromptContinue(
         message=util.ConstructList(
             'The following clusters will be deleted.',
@@ -80,9 +100,6 @@ class Delete(base.DeleteCommand):
     # Issue all deletes first
     for cluster_ref in cluster_refs:
       try:
-        # Make sure it exists (will raise appropriate error if not)
-        adapter.GetCluster(cluster_ref)
-
         op_ref = adapter.DeleteCluster(cluster_ref)
         operations.append((op_ref, cluster_ref))
       except apitools_exceptions.HttpError as error:
@@ -99,9 +116,13 @@ class Delete(base.DeleteCommand):
               'Deleting cluster {0}'.format(cluster_ref.clusterId),
               timeout_s=args.timeout)
           # Purge cached config files
-          util.ClusterConfig.Purge(cluster_ref.clusterId,
-                                   adapter.Zone(cluster_ref),
-                                   cluster_ref.projectId)
+          try:
+            util.ClusterConfig.Purge(cluster_ref.clusterId,
+                                     adapter.Zone(cluster_ref),
+                                     cluster_ref.projectId)
+          except kconfig.MissingEnvVarError as error:
+            log.warning(error)
+
           if properties.VALUES.container.cluster.Get() == cluster_ref.clusterId:
             properties.PersistProperty(
                 properties.VALUES.container.cluster, None)
