@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,13 +15,17 @@
 
 """List clusters command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import exceptions as apitools_exceptions
 
-from googlecloudsdk.api_lib.container import constants
 from googlecloudsdk.api_lib.container import transforms
 from googlecloudsdk.api_lib.container import util
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.command_lib.container import constants
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from surface.container.clusters.upgrade import UpgradeHelpText
@@ -45,18 +50,9 @@ class List(base.ListCommand):
       Some value that we want to have printed later.
     """
     adapter = self.context['api_adapter']
-
+    location_get = self.context['location_get']
+    location = location_get(args, ignore_property=True, required=False)
     project = properties.VALUES.core.project.Get(required=True)
-    if getattr(args, 'region', None):
-      location = adapter.registry.Parse(args.region,
-                                        params={'project': project},
-                                        collection='compute.regions').region
-    elif getattr(args, 'zone', None):
-      location = adapter.registry.Parse(args.zone,
-                                        params={'project': project},
-                                        collection='compute.zones').zone
-    else:
-      location = None
 
     def sort_key(cluster):
       return (cluster.zone, cluster.name)
@@ -75,11 +71,16 @@ class List(base.ListCommand):
       expiring = False
       self._upgrade_hint = ''
       self._expire_warning = ''
+      self._degraded_warning = ''
       vv = VersionVerifier()
       for c in clusters.clusters:
         time_left = transforms.ParseExpireTime(c.expireTime)
         if time_left and time_left.days < constants.EXPIRE_WARNING_DAYS:
           expiring = True
+        if adapter.IsDegraded(c):
+          self._degraded_warning += constants.DEGRADED_WARNING.format(
+              cluster_name=c.name,
+              cluster_degraded_warning=adapter.GetDegradedWarning(c))
         if c.enableKubernetesAlpha:
           # Don't print upgrade hints for alpha clusters, they aren't
           # upgradeable.
@@ -116,5 +117,6 @@ class List(base.ListCommand):
     if self._upgrade_hint:
       log.status.Print(self._upgrade_hint)
     if self._expire_warning:
-      log.warn(self._expire_warning)
-
+      log.warning(self._expire_warning)
+    if self._degraded_warning:
+      log.warning(self._degraded_warning)
