@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,9 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Resize cluster command."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from apitools.base.py import exceptions as apitools_exceptions
+
+from googlecloudsdk.api_lib.container import util
 from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.container import flags
 from googlecloudsdk.core import log
+from googlecloudsdk.core.console import console_attr
 from googlecloudsdk.core.console import console_io
 
 
@@ -47,34 +58,48 @@ class Resize(base.Command):
 
     """
     adapter = self.context['api_adapter']
-    cluster_ref = adapter.ParseCluster(args.name,
-                                       getattr(args, 'region', None))
-    cluster = adapter.GetCluster(cluster_ref)
-    pool = adapter.FindNodePool(cluster, args.node_pool)
+    location_get = self.context['location_get']
+    location = location_get(args)
+    cluster_ref = adapter.ParseCluster(args.name, location)
+    cluster = None
+    cluster_name = args.name
+    pool_name = args.node_pool if args.node_pool else 'default-pool'
+    try:
+      # Attempt to get cluster for better prompts and to validate args.
+      # Error is a warning but not fatal. Should only exit with a failure on
+      # ResizeNodePool(...) below.
+      cluster = adapter.GetCluster(cluster_ref)
+      cluster_name = cluster.name
+    except (exceptions.HttpException,
+            apitools_exceptions.HttpForbiddenError,
+            util.Error) as error:
+      log.warning(('Problem loading details of cluster to resize:\n\n{}\n\n'
+                   'You can still attempt to resize the cluster.\n')
+                  .format(console_attr.SafeText(error)))
+
+    if cluster is not None:
+      pool_name = adapter.FindNodePool(cluster, args.node_pool).name
+
     console_io.PromptContinue(
         message=('Pool [{pool}] for [{cluster_name}] will be resized to '
-                 '{new_size}.').format(pool=pool.name,
-                                       cluster_name=cluster.name,
-                                       new_size=args.size),
+                 '{new_size}.')
+        .format(pool=pool_name, cluster_name=cluster_name, new_size=args.size),
         throw_if_unattended=True,
         cancel_on_no=True)
-    ops = []
-    for ig in pool.instanceGroupUrls:
-      igm = adapter.ParseIGM(ig)
-      op_ref = adapter.ResizeCluster(igm.project, igm.zone,
-                                     igm.instanceGroupManager, args.size)
-      ops.append(op_ref)
+
+    op_ref = adapter.ResizeNodePool(cluster_ref, pool_name, args.size)
 
     if not args.async:
-      adapter.WaitForComputeOperations(
-          cluster_ref.projectId, cluster.zone, [op.name for op in ops],
-          'Resizing {0}'.format(cluster_ref.clusterId))
+      adapter.WaitForOperation(op_ref,
+                               'Resizing {0}'.format(cluster_ref.clusterId))
     log.UpdatedResource(cluster_ref)
 
 
 Resize.detailed_help = {
-    'brief': 'Resizes an existing cluster for running containers.',
-    'DESCRIPTION': """
+    'brief':
+        'Resizes an existing cluster for running containers.',
+    'DESCRIPTION':
+        """
         Resize an existing cluster to a provided size.
 
 If you have multiple node pools, you must specify which node pool to resize by
@@ -87,10 +112,10 @@ Existing pods are not moved onto the new instances,
 but new pods (such as those created by resizing a replication controller)
 will be scheduled onto the new instances.
 
-When decreasing a cluster, the pods that are scheduled on the instances being
-removed will be killed. If your pods are being managed by a replication
-controller, the controller will attempt to reschedule them onto the remaining
-instances. If your pods are not managed by a replication controller,
+When decreasing a cluster, the nodes are drained. As a result, the pods running
+on these nodes are gracefully terminated. If your pods are being managed by a
+workload controller, the controller will attempt to reschedule them onto the
+remaining instances. If your pods are not managed by a workload controller,
 they will not be restarted.
 Note that when resizing down, instances running pods and instances without pods
 are not differentiated. Resize will pick instances to remove at random.
