@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """Upgrade cluster command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.container import api_adapter
@@ -23,6 +28,8 @@ from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.container import container_command_util
 from googlecloudsdk.command_lib.container import flags
 from googlecloudsdk.core import log
+from googlecloudsdk.core import properties
+from googlecloudsdk.core.console import console_attr
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util.semver import SemVer
 
@@ -69,6 +76,19 @@ class VersionVerifier(object):
       return self.UPGRADE_AVAILABLE
 
 
+def ParseUpgradeOptionsBase(args):
+  """Parses the flags provided with the cluster upgrade command."""
+  return api_adapter.UpdateClusterOptions(
+      version=args.cluster_version,
+      update_master=args.master,
+      update_nodes=(not args.master),
+      node_pool=args.node_pool,
+      image_type=args.image_type,
+      image=args.image,
+      image_project=args.image_project,
+      concurrent_node_count=getattr(args, 'concurrent_node_count', None))
+
+
 def _Args(parser):
   """Register flags for this command.
 
@@ -85,9 +105,9 @@ def _Args(parser):
       help="""\
 The Kubernetes release version to which to upgrade the cluster's nodes.
 
-If provided, the --cluster-version must be no greater than the cluster
-master's minor version (x.*X*.x), and must be a latest patch version
-(x.x.*X*).
+If desired cluster version is omitted, *node* upgrades default to the current
+*master* version and *master* upgrades default to the default cluster version,
+which can be found in the server config.
 
 You can find the list of allowed versions for upgrades by running:
 
@@ -99,11 +119,20 @@ You can find the list of allowed versions for upgrades by running:
   parser.add_argument(
       '--master',
       help='Upgrade the cluster\'s master to the latest version of Kubernetes'
-      ' supported on Container Engine. Nodes cannot be upgraded at the same'
+      ' supported on Kubernetes Engine. Nodes cannot be upgraded at the same'
       ' time as the master.',
       action='store_true')
+  # Timeout in seconds for the operation, default 2700 seconds (45 minutes)
+  parser.add_argument(
+      '--timeout',
+      type=int,
+      default=2700,
+      hidden=True,
+      help='Timeout (seconds) for waiting on the operation to complete.')
   flags.AddAsyncFlag(parser)
   flags.AddImageTypeFlag(parser, 'cluster/node pool')
+  flags.AddImageFlag(parser, hidden=True)
+  flags.AddImageProjectFlag(parser, hidden=True)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
@@ -114,6 +143,9 @@ class Upgrade(base.Command):
   def Args(parser):
     _Args(parser)
 
+  def ParseUpgradeOptions(self, args):
+    return ParseUpgradeOptionsBase(args)
+
   def Run(self, args):
     """This is what gets called when the user runs this command.
 
@@ -125,31 +157,48 @@ class Upgrade(base.Command):
       Some value that we want to have printed later.
     """
     adapter = self.context['api_adapter']
+    location_get = self.context['location_get']
+    location = location_get(args)
+    cluster_ref = adapter.ParseCluster(args.name, location)
+    project_id = properties.VALUES.core.project.Get(required=True)
+    concurrent_node_count = getattr(args, 'concurrent_node_count', None)
 
-    cluster_ref = adapter.ParseCluster(args.name,
-                                       getattr(args, 'region', None))
+    try:
+      cluster = adapter.GetCluster(cluster_ref)
+    except (exceptions.HttpException,
+            apitools_exceptions.HttpForbiddenError,
+            util.Error) as error:
+      log.warning(('Problem loading details of cluster to upgrade:\n\n{}\n\n'
+                   'You can still attempt to upgrade the cluster.\n')
+                  .format(console_attr.SafeText(error)))
+      cluster = None
 
-    # Make sure it exists (will raise appropriate error if not)
-    cluster = adapter.GetCluster(cluster_ref)
-
-    options = api_adapter.UpdateClusterOptions(
-        version=args.cluster_version,
-        update_master=args.master,
-        update_nodes=(not args.master),
-        node_pool=args.node_pool,
-        image_type=args.image_type)
+    try:
+      server_conf = adapter.GetServerConfig(project_id, location)
+    except (exceptions.HttpException,
+            apitools_exceptions.HttpForbiddenError,
+            util.Error) as error:
+      log.warning(('Problem loading server config:\n\n{}\n\n'
+                   'You can still attempt to upgrade the cluster.\n')
+                  .format(console_attr.SafeText(error)))
+      server_conf = None
 
     upgrade_message = container_command_util.ClusterUpgradeMessage(
-        cluster,
+        name=args.name,
+        server_conf=server_conf,
+        cluster=cluster,
         master=args.master,
-        node_pool=args.node_pool,
-        new_version=options.version)
+        node_pool_name=args.node_pool,
+        new_version=args.cluster_version,
+        concurrent_node_count=concurrent_node_count)
 
     console_io.PromptContinue(
         message=upgrade_message,
         throw_if_unattended=True,
         cancel_on_no=True)
 
+    options = self.ParseUpgradeOptions(args)
+
     try:
       op_ref = adapter.UpdateCluster(cluster_ref, options)
     except apitools_exceptions.HttpError as error:
@@ -157,7 +206,8 @@ class Upgrade(base.Command):
 
     if not args.async:
       adapter.WaitForOperation(
-          op_ref, 'Upgrading {0}'.format(cluster_ref.clusterId))
+          op_ref, 'Upgrading {0}'.format(cluster_ref.clusterId),
+          timeout_s=args.timeout)
 
       log.UpdatedResource(cluster_ref)
 
@@ -165,9 +215,13 @@ Upgrade.detailed_help = {
     'DESCRIPTION': """\
       Upgrades the Kubernetes version of an existing container cluster.
 
-      This command upgrades the Kubernetes version of the *nodes* of a cluster.
-      The Kubernetes version of the cluster's *master* is periodically upgraded
-      automatically as new releases are available.
+      This command upgrades the Kubernetes version of the *nodes* or *master* of
+      a cluster. Note that the Kubernetes version of the cluster's *master* is
+      also periodically upgraded automatically as new releases are available.
+
+      If desired cluster version is omitted, *node* upgrades default to the
+      current *master* version and *master* upgrades default to the default
+      cluster version, which can be found in the server config.
 
       *By running this command, all of the cluster's nodes will be deleted and*
       *recreated one at a time.* While persistent Kubernetes resources, such as
@@ -186,10 +240,14 @@ Upgrade.detailed_help = {
 
         $ {command} <cluster>
 
-      Upgrade the nodes of <cluster> to Kubernetes version x.y.z.
+      Upgrade the nodes of <cluster> to Kubernetes version x.y.z:
 
         $ {command} <cluster> --cluster-version "x.y.z"
-    """,
+
+      Upgrade the master of <cluster> to the default cluster version:
+
+        $ {command} <cluster> --master"
+""",
 }
 
 
@@ -209,3 +267,11 @@ class UpgradeAlpha(Upgrade):
   @staticmethod
   def Args(parser):
     _Args(parser)
+    flags.AddConcurrentNodeCountFlag(parser)
+    flags.AddSecurityProfileForUpgradeFlags(parser)
+
+  def ParseUpgradeOptions(self, args):
+    ops = ParseUpgradeOptionsBase(args)
+    ops.security_profile = args.security_profile
+    ops.security_profile_runtime_rules = args.security_profile_runtime_rules
+    return ops
