@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,15 +15,24 @@
 
 """Get Server Config."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.container import container_command_util
 from googlecloudsdk.command_lib.container import flags
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class GetServerConfig(base.Command):
-  """Get Container Engine server config."""
+  """Get Kubernetes Engine server config."""
+
+  def __init__(self, *args, **kwargs):
+    super(GetServerConfig, self).__init__(*args, **kwargs)
+    self.location_get = container_command_util.GetZoneOrRegion
 
   @staticmethod
   def Args(parser):
@@ -33,31 +43,34 @@ class GetServerConfig(base.Command):
         which you can register arguments.  See the public argparse documentation
         for its capabilities.
     """
-    flags.AddZoneFlag(parser)
+    flags.AddZoneAndRegionFlags(parser)
 
   def Run(self, args):
     adapter = self.context['api_adapter']
 
     project_id = properties.VALUES.core.project.Get(required=True)
-    location = getattr(args, 'region', None)
-    if not location:
-      location = properties.VALUES.compute.zone.Get(required=True)
+    location = self.location_get(args)
 
-    log.status.Print('Fetching server config for {zone}'.format(zone=location))
+    log.status.Print('Fetching server config for {location}'.format(
+        location=location))
     return adapter.GetServerConfig(project_id, location)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class GetServerConfigAlpha(GetServerConfig):
-  """Get Container Engine server config."""
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
+class GetServerConfigAlphaBeta(GetServerConfig):
+  """Get Kubernetes Engine server config."""
 
-  @staticmethod
-  def Args(parser):
-    """Add arguments to the parser.
+  def Filter(self, context, args):
+    """Modify the context that will be given to this group's commands when run.
 
     Args:
-      parser: argparse.ArgumentParser, This is a standard argparser parser with
-        which you can register arguments.  See the public argparse documentation
-        for its capabilities.
+      context: {str:object}, A set of key-value pairs that can be used for
+          common initialization among commands.
+      args: argparse.Namespace: The same namespace given to the corresponding
+          .Run() invocation.
+
+    Returns:
+      The refined command context.
     """
-    flags.AddZoneAndRegionFlags(parser, region_hidden=True)
+    context['location_get'] = container_command_util.GetZoneOrRegion
+    return context
