@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,13 +15,19 @@
 
 """The command group for cloud container operations."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.container import container_command_util
 from googlecloudsdk.command_lib.container import flags
+from googlecloudsdk.command_lib.container import messages
+from googlecloudsdk.core import log
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
 class NodePools(base.Group):
-  """Create and delete operations for Google Container Engine node pools."""
+  """Create and delete operations for Google Kubernetes Engine node pools."""
 
   @staticmethod
   def Args(parser):
@@ -31,7 +38,7 @@ class NodePools(base.Group):
         which you can register arguments.  See the public argparse documentation
         for its capabilities.
     """
-    flags.AddZoneFlag(parser)
+    flags.AddZoneAndRegionFlags(parser)
 
   def Filter(self, context, args):
     """Modify the context that will be given to this group's commands when run.
@@ -45,20 +52,5 @@ class NodePools(base.Group):
     Returns:
       The refined command context.
     """
+    context['location_get'] = container_command_util.GetZoneOrRegion
     return context
-
-
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class NodePoolsAlpha(NodePools):
-  """Create and delete operations for Google Container Engine node pools."""
-
-  @staticmethod
-  def Args(parser):
-    """Add arguments to the parser.
-
-    Args:
-      parser: argparse.ArgumentParser, This is a standard argparser parser with
-        which you can register arguments.  See the public argparse documentation
-        for its capabilities.
-    """
-    flags.AddZoneAndRegionFlags(parser, region_hidden=True)
