@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,25 +14,30 @@
 # limitations under the License.
 """Create node pool command."""
 
-import argparse
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from apitools.base.py import exceptions as apitools_exceptions
 
-from googlecloudsdk.api_lib.compute import constants as compute_constants
+from googlecloudsdk.api_lib.compute import metadata_utils
+from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.api_lib.container import api_adapter
 from googlecloudsdk.api_lib.container import util
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.command_lib.container import constants
 from googlecloudsdk.command_lib.container import flags
 from googlecloudsdk.command_lib.container import messages
 from googlecloudsdk.core import log
+from googlecloudsdk.core import properties
 
 DETAILED_HELP = {
     'DESCRIPTION':
         """\
         *{command}* facilitates the creation of a node pool in a Google
-        Container Engine cluster. A variety of options exists to customize the
+        Kubernetes Engine cluster. A variety of options exists to customize the
         node configuration and the number of nodes created.
         """,
     'EXAMPLES':
@@ -60,59 +66,25 @@ def _Args(parser):
   """
   flags.AddNodePoolNameArg(parser, 'The name of the node pool to create.')
   flags.AddNodePoolClusterFlag(parser, 'The cluster to add the node pool to.')
-  parser.add_argument(
-      '--enable-cloud-endpoints',
-      action='store_true',
-      default=True,
-      help='Automatically enable Google Cloud Endpoints to take advantage of '
-      'API management features.')
   # Timeout in seconds for operation
   parser.add_argument(
-      '--timeout', type=int, default=1800, help=argparse.SUPPRESS)
+      '--timeout', type=int, default=1800, hidden=True,
+      help='THIS ARGUMENT NEEDS HELP TEXT.')
   parser.add_argument(
       '--num-nodes',
       type=int,
       help='The number of nodes in the node pool in each of the '
       'cluster\'s zones.',
       default=3)
-  parser.add_argument(
-      '--machine-type',
-      '-m',
-      help='The type of machine to use for nodes. Defaults to n1-standard-1')
+  flags.AddMachineTypeFlag(parser)
   parser.add_argument(
       '--disk-size',
-      type=int,
-      help='Size in GB for node VM boot disks. Defaults to 100GB.')
-  parser.add_argument(
-      '--scopes',
-      type=arg_parsers.ArgList(min_length=1),
-      metavar='SCOPE',
-      help="""\
-Specifies scopes for the node instances. The project's default
-service account is used. Examples:
-
-  $ {{command}} node-pool-1 --cluster=example-cluster --scopes https://www.googleapis.com/auth/devstorage.read_only
-
-  $ {{command}} node-pool-1 --cluster=example-cluster --scopes bigquery,storage-rw,compute-ro
-
-Multiple SCOPEs can specified, separated by commas. The scopes
-necessary for the cluster to function properly (compute-rw, storage-ro),
-are always added, even if not explicitly specified.
-
-SCOPE can be either the full URI of the scope or an alias.
-Available aliases are:
-
-[options="header",format="csv",grid="none",frame="none"]
-|========
-Alias,URI
-{aliases}
-|========
-
-{scope_deprecation_msg}
-""".format(
-    aliases=compute_constants.ScopesForHelp(),
-    scope_deprecation_msg=compute_constants.DEPRECATED_SCOPES_MESSAGES))
+      type=arg_parsers.BinarySize(lower_bound='10GB'),
+      help='Size for node VM boot disks. Defaults to 100GB.')
   flags.AddImageTypeFlag(parser, 'node pool')
+  flags.AddImageFlag(parser, hidden=True)
+  flags.AddImageProjectFlag(parser, hidden=True)
+  flags.AddImageFamilyFlag(parser, hidden=True)
   flags.AddNodeLabelsFlag(parser, for_node_pool=True)
   flags.AddTagsFlag(parser, """\
 Applies the given Compute Engine tags (comma separated) on all nodes in the new
@@ -125,29 +97,56 @@ on the Compute Engine API instance object and can be used in firewall rules.
 See https://cloud.google.com/sdk/gcloud/reference/compute/firewall-rules/create
 for examples.
 """)
-  flags.AddDiskTypeFlag(parser, suppressed=True)
+  flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True)
   parser.display_info.AddFormat(util.NODEPOOLS_FORMAT)
+  flags.AddNodeVersionFlag(parser)
+  flags.AddAcceleratorArgs(parser)
+  flags.AddDiskTypeFlag(parser)
+  flags.AddMetadataFlags(parser)
 
 
 def ParseCreateNodePoolOptionsBase(args):
+  """Parses the flags provided with the node pool creation command."""
+  if (args.IsSpecified('enable_cloud_endpoints') and
+      properties.VALUES.container.new_scopes_behavior.GetBool()):
+    raise util.Error('Flag --[no-]enable-cloud-endpoints is not allowed if '
+                     'property container/ new_scopes_behavior is set to true.')
+  if args.IsSpecified('enable_autorepair'):
+    enable_autorepair = args.enable_autorepair
+  else:
+    # Node pools using COS support auto repairs, enable it for them by default.
+    # Other node pools using (Ubuntu, custom images) don't support node auto
+    # repairs, attempting to enable autorepair for them will result in API call
+    # failing so don't do it.
+    enable_autorepair = ((args.image_type or '').lower() in ['', 'cos'])
+  metadata = metadata_utils.ConstructMetadataDict(args.metadata,
+                                                  args.metadata_from_file)
   return api_adapter.CreateNodePoolOptions(
+      accelerators=args.accelerator,
       machine_type=args.machine_type,
-      disk_size_gb=args.disk_size,
+      disk_size_gb=utils.BytesToGb(args.disk_size),
       scopes=args.scopes,
+      node_version=args.node_version,
       enable_cloud_endpoints=args.enable_cloud_endpoints,
       num_nodes=args.num_nodes,
       local_ssd_count=args.local_ssd_count,
       tags=args.tags,
       node_labels=args.node_labels,
+      node_taints=args.node_taints,
       enable_autoscaling=args.enable_autoscaling,
       max_nodes=args.max_nodes,
+      min_cpu_platform=args.min_cpu_platform,
       min_nodes=args.min_nodes,
       image_type=args.image_type,
+      image=args.image,
+      image_project=args.image_project,
+      image_family=args.image_family,
       preemptible=args.preemptible,
-      enable_autorepair=args.enable_autorepair,
+      enable_autorepair=enable_autorepair,
       enable_autoupgrade=args.enable_autoupgrade,
       service_account=args.service_account,
-      disk_type=args.disk_type)
+      disk_type=args.disk_type,
+      metadata=metadata)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
@@ -157,12 +156,13 @@ class Create(base.CreateCommand):
   @staticmethod
   def Args(parser):
     _Args(parser)
-    flags.AddClusterAutoscalingFlags(parser, hidden=True)
-    flags.AddLocalSSDFlag(parser, suppressed=True)
-    flags.AddPreemptibleFlag(parser, for_node_pool=True, suppressed=True)
-    flags.AddEnableAutoRepairFlag(parser, for_node_pool=True, suppressed=True)
-    flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True, suppressed=True)
-    flags.AddServiceAccountFlag(parser, suppressed=True)
+    flags.AddClusterAutoscalingFlags(parser)
+    flags.AddLocalSSDFlag(parser)
+    flags.AddPreemptibleFlag(parser, for_node_pool=True)
+    flags.AddEnableAutoRepairFlag(parser, for_node_pool=True, for_create=True)
+    flags.AddMinCpuPlatformFlag(parser, for_node_pool=True)
+    flags.AddNodeTaintsFlag(parser, for_node_pool=True)
+    flags.AddDeprecatedNodePoolNodeIdentityFlags(parser)
 
   def ParseCreateNodePoolOptions(self, args):
     return ParseCreateNodePoolOptionsBase(args)
@@ -181,13 +181,11 @@ class Create(base.CreateCommand):
       util.Error, if creation failed.
     """
     adapter = self.context['api_adapter']
-    if not args.scopes:
-      args.scopes = []
+    location_get = self.context['location_get']
+    location = location_get(args)
 
     try:
-      if not args.scopes:
-        args.scopes = []
-      pool_ref = adapter.ParseNodePool(args.name, getattr(args, 'region', None))
+      pool_ref = adapter.ParseNodePool(args.name, location)
       options = self.ParseCreateNodePoolOptions(args)
 
       if options.enable_autorepair is not None:
@@ -200,6 +198,9 @@ class Create(base.CreateCommand):
             messages.AutoUpdateUpgradeRepairMessage(options.enable_autoupgrade,
                                                     'autoupgrade'))
 
+      if options.accelerators is not None:
+        log.status.Print(constants.KUBERNETES_GPU_LIMITATION_MSG)
+
       operation_ref = adapter.CreateNodePool(pool_ref, options)
 
       adapter.WaitForOperation(
@@ -221,12 +222,24 @@ class CreateBeta(Create):
   @staticmethod
   def Args(parser):
     _Args(parser)
-    flags.AddClusterAutoscalingFlags(parser, hidden=True)
+    flags.AddClusterAutoscalingFlags(parser)
     flags.AddLocalSSDFlag(parser)
     flags.AddPreemptibleFlag(parser, for_node_pool=True)
-    flags.AddEnableAutoRepairFlag(parser, for_node_pool=True)
-    flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True)
-    flags.AddServiceAccountFlag(parser)
+    flags.AddEnableAutoRepairFlag(parser, for_node_pool=True, for_create=True)
+    flags.AddMinCpuPlatformFlag(parser, for_node_pool=True)
+    flags.AddWorkloadMetadataFromNodeFlag(parser)
+    flags.AddNodeTaintsFlag(parser, for_node_pool=True)
+    flags.AddNodePoolNodeIdentityFlags(parser)
+    flags.AddNodePoolAutoprovisioningFlag(parser, hidden=True)
+    flags.AddMaxPodsPerNodeFlag(parser, for_node_pool=True)
+
+  def ParseCreateNodePoolOptions(self, args):
+    ops = ParseCreateNodePoolOptionsBase(args)
+    ops.workload_metadata_from_node = args.workload_metadata_from_node
+    ops.new_scopes_behavior = True
+    ops.enable_autoprovisioning = args.enable_autoprovisioning
+    ops.max_pods_per_node = args.max_pods_per_node
+    return ops
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@@ -235,19 +248,30 @@ class CreateAlpha(Create):
 
   def ParseCreateNodePoolOptions(self, args):
     ops = ParseCreateNodePoolOptionsBase(args)
-    ops.accelerators = args.accelerator
+    ops.workload_metadata_from_node = args.workload_metadata_from_node
+    ops.enable_autoprovisioning = args.enable_autoprovisioning
+    ops.new_scopes_behavior = True
+    ops.local_ssd_volume_configs = args.local_ssd_volumes
+    ops.max_pods_per_node = args.max_pods_per_node
+    ops.sandbox = args.sandbox
+    ops.node_group = args.node_group
     return ops
 
   @staticmethod
   def Args(parser):
     _Args(parser)
     flags.AddClusterAutoscalingFlags(parser)
-    flags.AddLocalSSDFlag(parser)
+    flags.AddNodePoolAutoprovisioningFlag(parser, hidden=True)
+    flags.AddLocalSSDAndLocalSSDVolumeConfigsFlag(parser, for_node_pool=True)
     flags.AddPreemptibleFlag(parser, for_node_pool=True)
-    flags.AddEnableAutoRepairFlag(parser, for_node_pool=True)
-    flags.AddEnableAutoUpgradeFlag(parser, for_node_pool=True)
-    flags.AddServiceAccountFlag(parser)
-    flags.AddAcceleratorArgs(parser)
+    flags.AddEnableAutoRepairFlag(parser, for_node_pool=True, for_create=True)
+    flags.AddMinCpuPlatformFlag(parser, for_node_pool=True)
+    flags.AddWorkloadMetadataFromNodeFlag(parser)
+    flags.AddNodeTaintsFlag(parser, for_node_pool=True)
+    flags.AddNodePoolNodeIdentityFlags(parser)
+    flags.AddMaxPodsPerNodeFlag(parser, for_node_pool=True)
+    flags.AddSandboxFlag(parser, hidden=True)
+    flags.AddNodeGroupFlag(parser)
 
 
 Create.detailed_help = DETAILED_HELP
