@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +15,9 @@
 
 """Rollback node pool command."""
 
-import argparse
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from apitools.base.py import exceptions as apitools_exceptions
 
@@ -47,7 +50,8 @@ class Rollback(base.Command):
         '--timeout',
         type=int,
         default=1800,  # Seconds
-        help=argparse.SUPPRESS)
+        hidden=True,
+        help='THIS ARGUMENT NEEDS HELP TEXT.')
 
   def Run(self, args):
     """This is what gets called when the user runs this command.
@@ -63,8 +67,10 @@ class Rollback(base.Command):
       util.Error, if rollback failed.
     """
     adapter = self.context['api_adapter']
+    location_get = self.context['location_get']
+    location = location_get(args)
 
-    pool_ref = adapter.ParseNodePool(args.name, getattr(args, 'region', None))
+    pool_ref = adapter.ParseNodePool(args.name, location)
 
     console_io.PromptContinue(
         message=
