@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,11 @@
 # limitations under the License.
 
 """List operations command."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.container import util
@@ -47,19 +53,10 @@ class List(base.ListCommand):
       Some value that we want to have printed later.
     """
     adapter = self.context['api_adapter']
+    location_get = self.context['location_get']
 
     project_id = properties.VALUES.core.project.GetOrFail()
-
-    if getattr(args, 'region', None):
-      location = adapter.registry.Parse(args.region,
-                                        params={'project': project_id},
-                                        collection='compute.regions').region
-    elif getattr(args, 'zone', None):
-      location = adapter.registry.Parse(args.zone,
-                                        params={'project': project_id},
-                                        collection='compute.zones').zone
-    else:
-      location = None
+    location = location_get(args, ignore_property=True, required=False)
 
     try:
       return adapter.ListOperations(project_id, location).operations
