@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,6 +16,10 @@
 """Implementation of gcloud dataflow jobs drain command.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.dataflow import apis
 from googlecloudsdk.api_lib.util import exceptions
 from googlecloudsdk.calliope import base
@@ -43,9 +48,12 @@ class Drain(base.Command):
     Args:
       args: all the arguments that were provided to this command invocation.
     """
-    for job_ref in job_utils.ExtractJobRefs(args.jobs):
+    for job_ref in job_utils.ExtractJobRefs(args):
       try:
-        apis.Jobs.Drain(job_ref.jobId)
+        apis.Jobs.Drain(
+            job_ref.jobId,
+            project_id=job_ref.projectId,
+            region_id=job_ref.location)
         log.status.Print('Started draining job [{0}]'.format(job_ref.jobId))
       except exceptions.HttpException as error:
         log.status.Print('Failed to drain job [{0}]: {1}'.format(
