@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,9 +15,16 @@
 
 """The command group for cloud dataproc operations."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import base
+from googlecloudsdk.core import properties
 
 
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class Operations(base.Group):
   """View and manage Google Cloud Dataproc operations.
 
@@ -40,3 +48,12 @@ class Operations(base.Group):
 
     $ {command} delete operation_id
   """
+
+  @classmethod
+  def Args(cls, parser):
+    region_prop = properties.VALUES.dataproc.region
+    parser.add_argument(
+        '--region',
+        help=region_prop.help_text,
+        # Don't set default, because it would override users' property setting.
+        action=actions.StoreProperty(region_prop))
