@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,11 @@
 # limitations under the License.
 
 """List operation command."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import json
 
 from apitools.base.py import list_pager
@@ -93,7 +99,6 @@ class List(base.ListCommand):
     filter_dict = dict()
     if args.state_filter:
       filter_dict[STATE_MATCHER_FILTER] = STATE_MATCHER_MAP[args.state_filter]
-    # TODO(b/32669485) Get full flag test coverage.
     if args.cluster:
       filter_dict[CLUSTER_NAME_FILTER] = args.cluster
 
