@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,10 +15,15 @@
 
 """The gen_repo_info_file command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import json
 import os
 
 from googlecloudsdk.calliope import base
+from googlecloudsdk.core import exceptions as core_exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core.util import files
 from googlecloudsdk.third_party.appengine.tools import context_util
@@ -26,18 +32,17 @@ from googlecloudsdk.third_party.appengine.tools import context_util
 class GenRepoInfoFile(base.Command):
   """Generates repository information files for the Stackdriver Debugger.
 
-  The generated files contain opaque information representing which source
+  The generated file contains opaque information representing which source
   revision the application was built at, and which repository this revision
   will be pushed to.
   """
 
   detailed_help = {
       'DESCRIPTION': """\
-          This command generates two files, {old_name} and
-          {contexts_filename}, containing information on the source revision
-          and remote repository associated with the given source directory.
-          """.format(old_name=context_util.CONTEXT_FILENAME,
-                     contexts_filename=context_util.EXT_CONTEXT_FILENAME),
+          This command generates a file named {name} containing information on
+          the source revision and remote repository associated with the given
+          source directory.
+          """.format(name=context_util.CONTEXT_FILENAME),
       'EXAMPLES': """\
           To generate repository information files for your app,
           from your source directory run:
@@ -55,20 +60,25 @@ class GenRepoInfoFile(base.Command):
     parser.add_argument(
         '--output-directory',
         default='.',
-        help='The directory in which to create the source context files. ')
+        help='The directory in which to create the source context file. ')
 
   def Run(self, args):
-    contexts = context_util.CalculateExtendedSourceContexts(
-        args.source_directory)
+    try:
+      contexts = context_util.CalculateExtendedSourceContexts(
+          args.source_directory)
+    except context_util.GenerateSourceContextError as e:
+      # This is a usage error. Wrap it with core_exceptions.Error to report
+      # it properly (i.e., as an error instead of a crash).
+      raise core_exceptions.Error(e)
 
-    # First create the old-style source-context.json file
+    # Create the source-context.json file.
     output_file = context_util.CONTEXT_FILENAME
 
     output_directory = args.output_directory
     output_file = os.path.join(output_directory, output_file)
 
     if context_util.HasPendingChanges(args.source_directory):
-      log.warn(
+      log.warning(
           'There are uncommitted changes in directory [{0}].\n'
           'The generated source context files will not reflect the current '
           'state of your source code.\n'
@@ -76,11 +86,6 @@ class GenRepoInfoFile(base.Command):
           .format(args.source_directory))
     best_context = context_util.BestSourceContext(contexts)
     files.MakeDir(output_directory)
-    with open(output_file, 'w') as f:
-      json.dump(best_context, f, indent=2, sort_keys=True)
+    files.WriteFileContents(
+        output_file, json.dumps(best_context, indent=2, sort_keys=True))
 
-    # Create the new source-contexts.json file.
-    with open(
-        os.path.join(output_directory,
-                     context_util.EXT_CONTEXT_FILENAME), 'w') as f:
-      json.dump(contexts, f, indent=2, sort_keys=True)
