@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,13 +15,19 @@
 
 """deployments describe command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.deployment_manager import dm_api_util
 from googlecloudsdk.api_lib.deployment_manager import dm_base
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
+from googlecloudsdk.command_lib.deployment_manager import alpha_flags
 from googlecloudsdk.command_lib.deployment_manager import flags
+from googlecloudsdk.core import properties
 
 
 class _Results(object):
@@ -32,6 +39,7 @@ class _Results(object):
     self.outputs = outputs
 
 
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 @dm_base.UseDmApi(dm_base.DmApiVersion.V2)
 class Describe(base.DescribeCommand, dm_base.DmCommand):
   """Provide information about a deployment.
@@ -48,13 +56,15 @@ class Describe(base.DescribeCommand, dm_base.DmCommand):
   }
 
   @staticmethod
-  def Args(parser):
+  def Args(parser, version=base.ReleaseTrack.GA):
     """Args is called by calliope to gather arguments for this command.
 
     Args:
       parser: An argparse parser that you can use to add arguments that go
           on the command line after this command. Positional arguments are
           allowed.
+      version: The version this tool is running as. base.ReleaseTrack.GA
+          is the default.
     """
     flags.AddDeploymentNameFlag(parser)
     parser.display_info.AddFormat("""
@@ -63,10 +73,10 @@ class Describe(base.DescribeCommand, dm_base.DmCommand):
                 insertTime, manifest.basename(), labels, operation.operationType,
                 operation.name, operation.progress, operation.status,
                 operation.user, operation.endTime, operation.startTime,
-                operation.error, update)',
+                operation.error, operation.warnings, update)',
                 resources:format='table(
                   name:label=NAME,
-                  type:label=TYPE,
+                  type:wrap:label=TYPE,
                   update.state.yesno(no="COMPLETED"),
                   update.intent)',
               outputs:format='table(
@@ -89,10 +99,15 @@ class Describe(base.DescribeCommand, dm_base.DmCommand):
       HttpException: An http error response was received while executing api
           request.
     """
+    deployment_ref = self.resources.Parse(
+        args.deployment_name,
+        params={'project': properties.VALUES.core.project.GetOrFail},
+        collection='deploymentmanager.deployments')
     try:
       deployment = self.client.deployments.Get(
           self.messages.DeploymentmanagerDeploymentsGetRequest(
-              project=dm_base.GetProject(), deployment=args.deployment_name))
+              project=dm_base.GetProject(),
+              deployment=deployment_ref.deployment))
     except apitools_exceptions.HttpError as error:
       raise exceptions.HttpException(error, dm_api_util.HTTP_ERROR_FORMAT)
 
@@ -101,6 +116,11 @@ class Describe(base.DescribeCommand, dm_base.DmCommand):
           self.messages.DeploymentmanagerResourcesListRequest(
               project=dm_base.GetProject(), deployment=deployment.name))
       resources = response.resources
+      if self.ReleaseTrack() is base.ReleaseTrack.ALPHA:
+        dm_api_util.UpdateActionResourceIntent(resources)
+        if (not args.IsSpecified('format')) and (deployment.update):
+          args.format = (
+              alpha_flags.PREVIEWED_DEPLOYMENT_AND_RESOURCES_AND_OUTPUTS_FORMAT)
     except apitools_exceptions.HttpError:
       # Couldn't get resources, skip adding them to the table.
       resources = None
@@ -113,7 +133,7 @@ class Describe(base.DescribeCommand, dm_base.DmCommand):
       manifest_response = self.client.manifests.Get(
           self.messages.DeploymentmanagerManifestsGetRequest(
               project=dm_base.GetProject(),
-              deployment=args.deployment_name,
+              deployment=deployment_ref.deployment,
               manifest=manifest,
           )
       )
@@ -122,3 +142,30 @@ class Describe(base.DescribeCommand, dm_base.DmCommand):
         outputs = dm_api_util.FlattenLayoutOutputs(manifest_response.layout)
 
     return _Results(deployment, resources, outputs)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+@dm_base.UseDmApi(dm_base.DmApiVersion.ALPHA)
+class DescribeAlpha(Describe):
+  """Provide information about a deployment.
+
+  This command prints out all available details about a deployment.
+  """
+
+  @staticmethod
+  def Args(parser):
+    Describe.Args(parser, version=base.ReleaseTrack.ALPHA)
+    parser.display_info.AddFormat(
+        alpha_flags.DEPLOYMENT_AND_RESOURCES_AND_OUTPUTS_FORMAT)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class DescribeBeta(Describe):
+  """Provide information about a deployment.
+
+  This command prints out all available details about a deployment.
+  """
+
+  @staticmethod
+  def Args(parser):
+    Describe.Args(parser, version=base.ReleaseTrack.BETA)
