@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """types list command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import collections
 
 from apitools.base.py import list_pager
@@ -123,6 +128,8 @@ class ListALPHA(base.ListCommand, dm_base.DmCommand):
 
   @staticmethod
   def Args(parser):
+    # TODO(b/79993361): support URI flag after resolving the bug.
+    base.URI_FLAG.RemoveFromParser(parser)
     parser.add_argument('--provider', help='Type provider name.')
     parser.add_argument('--provider-project',
                         help='Project id with types you want to see.')
@@ -214,6 +221,7 @@ class ListALPHA(base.ListCommand, dm_base.DmCommand):
             yield {'types': types,
                    'provider': project + '/' + type_provider}
         except api_exceptions.HttpException as error:
+          self.exit_code = 1
           yield {'types': [],
                  'provider': project + '/' + type_provider,
                  'error': error.message}
