@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """The gcloud dns command group."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
@@ -47,3 +52,7 @@ class DNS(base.Group):
 
     $ {command} project-info describe --help
   """
+
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
