@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """gcloud dns managed-zones list command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import list_pager
 from googlecloudsdk.api_lib.dns import util
 from googlecloudsdk.api_lib.util import apis
@@ -84,10 +89,10 @@ class ListBeta(base.ListCommand):
   @staticmethod
   def Args(parser):
     parser.display_info.AddFormat('table(name, dnsName, description)')
-    parser.display_info.AddUriFunc(_GetUriFunction('v2beta1'))
+    parser.display_info.AddUriFunc(_GetUriFunction('v1beta2'))
 
   def Run(self, args):
-    dns_client = apis.GetClientInstance('dns', 'v2beta1')
+    dns_client = apis.GetClientInstance('dns', 'v1beta2')
 
     project_id = properties.VALUES.core.project.GetOrFail()
 
