@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """gcloud dns record-sets transaction add command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.dns import transaction_util as trans_util
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.dns import flags
@@ -59,7 +64,7 @@ class Add(base.Command):
     # If in the future there are differences between API version, do NOT use
     # this patter of checking ReleaseTrack. Break this into multiple classes.
     if self.ReleaseTrack() == base.ReleaseTrack.BETA:
-      api_version = 'v2beta1'
+      api_version = 'v1beta2'
 
     with trans_util.TransactionFile(args.transaction_file) as trans_file:
       change = trans_util.ChangeFromYamlFile(
