@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,15 +14,14 @@
 # limitations under the License.
 """The gcloud domains group."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
-from googlecloudsdk.core import exceptions
-from googlecloudsdk.core import log
-from googlecloudsdk.core import properties
-from googlecloudsdk.core import resources
-from googlecloudsdk.core.util import platforms
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
 class Domains(base.Group):
   """Base class for gcloud domains command group."""
 
@@ -41,3 +41,7 @@ class Domains(base.Group):
             $ {command} list-user-verified
           """
   }
+
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
