@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """The `domains list-user-verified` command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.app.api import appengine_domains_api_client as api_client
 from googlecloudsdk.calliope import base
 
@@ -27,20 +32,17 @@ class ListUserVerified(base.Command):
           """\
           To list domains that have been verified by the current user, run:
 
-            $ {command} list-user-verified
+            $ {command}
 
           Use the {parent_command} verify command to verify additional
           domains.
           """,
   }
 
-  def Run(self, args):
-    client = api_client.AppengineDomainsApiClient.GetApiClient()
-    return client.ListVerifiedDomains()
+  @staticmethod
+  def Args(parser):
+    parser.display_info.AddFormat('table(id:sort=1)')
 
-  def DeprecatedFormat(self, args):
-    return """
-            table(
-              id:sort=1
-            )
-          """
+  def Run(self, args):
+    client = api_client.GetApiClientForTrack(self.ReleaseTrack())
+    return client.ListVerifiedDomains()
