@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """gcloud bigtable emulator start command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.emulators import bigtable_util
@@ -40,7 +45,8 @@ class Start(base.Command):
         '--host-port',
         required=False,
         type=lambda arg: arg_parsers.HostPort.Parse(arg, ipv6_enabled=True),
-        help='The host:port to which the emulator should be bound.')
+        help='The host:port to which the emulator should be bound. The default '
+        'value is localhost:8086.')
 
   # Override
   def Run(self, args):
