@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """The main command group for 'gcloud firebase'."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
@@ -25,3 +30,6 @@ class Firebase(base.Group):
       $ {command} test --help
   """
 
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
