@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,25 +13,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""'functions describe' command."""
+"""Displays details of a Google Cloud Function."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.functions import util
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.functions import flags
-from googlecloudsdk.core import properties
-from googlecloudsdk.core import resources
 
 
 class Describe(base.DescribeCommand):
-  """Show description of a function."""
+  """Display details of a Google Cloud Function."""
 
   @staticmethod
   def Args(parser):
     """Register flags for this command."""
-    parser.add_argument(
-        'name', help='The name of the function to describe.',
-        type=util.ValidateFunctionNameOrRaise)
-    flags.AddRegionFlag(parser)
+    flags.AddFunctionResourceArg(parser, 'to describe')
 
   @util.CatchHTTPErrorRaiseHTTPException
   def Run(self, args):
@@ -45,13 +45,7 @@ class Describe(base.DescribeCommand):
     """
     client = util.GetApiClientInstance()
     messages = client.MESSAGES_MODULE
-
-    function_ref = resources.REGISTRY.Parse(
-        args.name, params={
-            'projectsId': properties.VALUES.core.project.GetOrFail,
-            'locationsId': properties.VALUES.functions.region.GetOrFail},
-        collection='cloudfunctions.projects.locations.functions')
-
+    function_ref = args.CONCEPTS.name.Parse()
     return client.projects_locations_functions.Get(
         messages.CloudfunctionsProjectsLocationsFunctionsGetRequest(
             name=function_ref.RelativeName()))
