@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,11 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""The command group for accessing Google Cloud Functions regions."""
+"""List regions available to Google Cloud Functions."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from googlecloudsdk.calliope import base
 
 
 class Regions(base.Group):
-  """Provides information about Google Cloud Functions regions."""
+  """List regions available to Google Cloud Functions."""
   pass
