@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,6 +23,10 @@ for this command tree.  You can implement methods in this class to override some
 of the default behavior.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
@@ -29,3 +34,7 @@ from googlecloudsdk.calliope import base
 class Genomics(base.Group):
   """Manage Genomics resources using version 1 of the API."""
 
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
+
