@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,6 +17,10 @@
 """Implementation of gcloud genomics datasets add-iam-policy-binding.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.genomics import genomics_util
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.genomics import completers
@@ -38,7 +43,8 @@ class AddIamPolicyBinding(base.Command):
     parser.add_argument('id', type=str,
                         help='The ID of the dataset.')
     iam_util.AddArgsForAddIamPolicyBinding(
-        parser, completer=completers.GenomicsIamRolesCompleter)
+        parser,
+        role_completer=completers.GenomicsIamRolesCompleter)
 
   def Run(self, args):
     apitools_client = genomics_util.GetGenomicsClient()
