@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,11 @@
 
 """Implementation of gcloud genomics variantsets describe.
 """
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.genomics import genomics_util
 from googlecloudsdk.calliope import base
 
@@ -26,6 +32,7 @@ class Describe(base.DescribeCommand):
   def Args(parser):
     """Register flags for this command."""
     parser.add_argument('id', help='The ID of the variant set to describe.')
+    parser.display_info.AddFormat('json')
 
   def Run(self, args):
     """This is what gets called when the user runs this command.
@@ -47,6 +54,3 @@ class Describe(base.DescribeCommand):
         variantSetId=args.id,)
 
     return apitools_client.variantsets.Get(get_request)
-
-  def DeprecatedFormat(self, unused_args):
-    return 'json'
