@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,16 +12,22 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for adding IAM policy bindings for service accounts."""
 
-import httplib
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
+from googlecloudsdk.api_lib.iam import util
 from googlecloudsdk.api_lib.util import http_retry
-from googlecloudsdk.command_lib.iam import base_classes
+from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.iam import iam_util
 
+import six.moves.http_client
 
-class AddIamPolicyBinding(base_classes.BaseIamCommand):
+
+class AddIamPolicyBinding(base.Command):
   """Add an IAM policy binding to a service account.
 
   This command adds a policy binding to the IAM policy of a service account,
@@ -28,7 +35,10 @@ class AddIamPolicyBinding(base_classes.BaseIamCommand):
   """
 
   detailed_help = iam_util.GetDetailedHelpForAddIamPolicyBinding(
-      'service account', 'my-iam-account@somedomain.com')
+      'service account',
+      'my-iam-account@somedomain.com')
+  detailed_help['DESCRIPTION'] += '\n\n' + (
+      iam_util.GetHintForServiceAccountResource('add iam policy bindings to'))
 
   @staticmethod
   def Args(parser):
@@ -37,17 +47,18 @@ class AddIamPolicyBinding(base_classes.BaseIamCommand):
         action='whose policy to add bindings to')
     iam_util.AddArgsForAddIamPolicyBinding(parser)
 
-  @http_retry.RetryOnHttpStatus(httplib.CONFLICT)
+  @http_retry.RetryOnHttpStatus(six.moves.http_client.CONFLICT)
   def Run(self, args):
-    policy = self.iam_client.projects_serviceAccounts.GetIamPolicy(
-        self.messages.IamProjectsServiceAccountsGetIamPolicyRequest(
+    client, messages = util.GetClientAndMessages()
+    policy = client.projects_serviceAccounts.GetIamPolicy(
+        messages.IamProjectsServiceAccountsGetIamPolicyRequest(
             resource=iam_util.EmailToAccountResourceName(args.service_account)))
 
-    iam_util.AddBindingToIamPolicy(self.messages.Binding, policy, args.member,
+    iam_util.AddBindingToIamPolicy(messages.Binding, policy, args.member,
                                    args.role)
 
-    return self.iam_client.projects_serviceAccounts.SetIamPolicy(
-        self.messages.IamProjectsServiceAccountsSetIamPolicyRequest(
+    return client.projects_serviceAccounts.SetIamPolicy(
+        messages.IamProjectsServiceAccountsSetIamPolicyRequest(
             resource=iam_util.EmailToAccountResourceName(args.service_account),
-            setIamPolicyRequest=self.messages.SetIamPolicyRequest(
+            setIamPolicyRequest=messages.SetIamPolicyRequest(
                 policy=policy)))
