@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,17 +12,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for getting service accounts."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 import textwrap
 
+from googlecloudsdk.api_lib.iam import util
 from googlecloudsdk.calliope import base
-from googlecloudsdk.command_lib.iam import base_classes
 from googlecloudsdk.command_lib.iam import iam_util
 
 
-class Describe(base_classes.BaseIamCommand, base.DescribeCommand):
+class Describe(base.DescribeCommand):
   """Show metadata for a service account from a project."""
 
   detailed_help = {
@@ -47,6 +52,7 @@ class Describe(base_classes.BaseIamCommand, base.DescribeCommand):
 
   def Run(self, args):
     # TODO(b/25212870): use resource parsing.
-    return self.iam_client.projects_serviceAccounts.Get(
-        self.messages.IamProjectsServiceAccountsGetRequest(
+    client, messages = util.GetClientAndMessages()
+    return client.projects_serviceAccounts.Get(
+        messages.IamProjectsServiceAccountsGetRequest(
             name=iam_util.EmailToAccountResourceName(args.service_account)))
