@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,17 +12,22 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command to create private keys for service accounts."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 import textwrap
 
-from googlecloudsdk.command_lib.iam import base_classes
+from googlecloudsdk.api_lib.iam import util
+from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.core import log
 
 
-class Create(base_classes.BaseIamCommand):
+class Create(base.Command):
   """Create a private key for a service account."""
 
   detailed_help = {
@@ -56,16 +62,18 @@ class Create(base_classes.BaseIamCommand):
                         'be written.')
 
   def Run(self, args):
-    result = self.iam_client.projects_serviceAccounts_keys.Create(
-        self.messages.IamProjectsServiceAccountsKeysCreateRequest(
+    client, messages = util.GetClientAndMessages()
+    result = client.projects_serviceAccounts_keys.Create(
+        messages.IamProjectsServiceAccountsKeysCreateRequest(
             name=iam_util.EmailToAccountResourceName(args.iam_account),
             createServiceAccountKeyRequest=
-            self.messages.CreateServiceAccountKeyRequest(
+            messages.CreateServiceAccountKeyRequest(
                 privateKeyType=iam_util.KeyTypeToCreateKeyType(
                     iam_util.KeyTypeFromString(args.key_file_type)))))
 
     # Only the creating user has access. Set file permission to "-rw-------".
-    self.WriteFile(args.output, result.privateKeyData, make_private=True)
+    log.WriteToFileOrStdout(
+        args.output, content=result.privateKeyData, binary=True, private=True)
     log.status.Print(
         'created key [{0}] of type [{1}] as [{2}] for [{3}]'.format(
             iam_util.GetKeyIdFromResourceName(result.name),
