@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,18 +12,23 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for to list all of a project's service accounts."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import list_pager
 
+from googlecloudsdk.api_lib.iam import exceptions
+from googlecloudsdk.api_lib.iam import util
 from googlecloudsdk.calliope import base
-from googlecloudsdk.calliope import exceptions
-from googlecloudsdk.command_lib.iam import base_classes
 from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.core import properties
 
 
-class List(base_classes.BaseIamCommand, base.ListCommand):
+class List(base.ListCommand):
   """List all of a project's service accounts."""
 
   @staticmethod
@@ -35,12 +41,13 @@ class List(base_classes.BaseIamCommand, base.ListCommand):
   def Run(self, args):
     if args.limit is not None:
       if args.limit < 1:
-        raise exceptions.ToolException('Limit size must be >=1')
+        raise exceptions.InvalidArgumentException('Limit size must be >=1')
 
     project = properties.VALUES.core.project.Get(required=True)
+    client, messages = util.GetClientAndMessages()
     for item in list_pager.YieldFromList(
-        self.iam_client.projects_serviceAccounts,
-        self.messages.IamProjectsServiceAccountsListRequest(
+        client.projects_serviceAccounts,
+        messages.IamProjectsServiceAccountsListRequest(
             name=iam_util.ProjectToProjectResourceName(project)),
         field='accounts',
         limit=args.limit,
