@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,16 +12,22 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for removing IAM policies for service accounts."""
 
-import httplib
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
+from googlecloudsdk.api_lib.iam import util
 from googlecloudsdk.api_lib.util import http_retry
-from googlecloudsdk.command_lib.iam import base_classes
+from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.iam import iam_util
 
+import six.moves.http_client
 
-class RemoveIamPolicyBinding(base_classes.BaseIamCommand):
+
+class RemoveIamPolicyBinding(base.Command):
   """Remove an IAM policy binding from a service account.
 
   This command removes a policy binding to the IAM policy of a service account,
@@ -29,6 +36,9 @@ class RemoveIamPolicyBinding(base_classes.BaseIamCommand):
 
   detailed_help = iam_util.GetDetailedHelpForRemoveIamPolicyBinding(
       'service account', 'my-iam-account@somedomain.com')
+  detailed_help['DESCRIPTION'] += '\n\n' + (
+      iam_util.GetHintForServiceAccountResource(
+          'remove a policy binding from'))
 
   @staticmethod
   def Args(parser):
@@ -37,16 +47,17 @@ class RemoveIamPolicyBinding(base_classes.BaseIamCommand):
         action='to remove the policy binding from')
     iam_util.AddArgsForRemoveIamPolicyBinding(parser)
 
-  @http_retry.RetryOnHttpStatus(httplib.CONFLICT)
+  @http_retry.RetryOnHttpStatus(six.moves.http_client.CONFLICT)
   def Run(self, args):
-    policy = self.iam_client.projects_serviceAccounts.GetIamPolicy(
-        self.messages.IamProjectsServiceAccountsGetIamPolicyRequest(
+    client, messages = util.GetClientAndMessages()
+    policy = client.projects_serviceAccounts.GetIamPolicy(
+        messages.IamProjectsServiceAccountsGetIamPolicyRequest(
             resource=iam_util.EmailToAccountResourceName(args.service_account)))
 
     iam_util.RemoveBindingFromIamPolicy(policy, args.member, args.role)
 
-    return self.iam_client.projects_serviceAccounts.SetIamPolicy(
-        self.messages.IamProjectsServiceAccountsSetIamPolicyRequest(
+    return client.projects_serviceAccounts.SetIamPolicy(
+        messages.IamProjectsServiceAccountsSetIamPolicyRequest(
             resource=iam_util.EmailToAccountResourceName(args.service_account),
-            setIamPolicyRequest=self.messages.SetIamPolicyRequest(
+            setIamPolicyRequest=messages.SetIamPolicyRequest(
                 policy=policy)))
