@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,13 +12,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """Command for setting IAM policies for service accounts."""
 
-from googlecloudsdk.command_lib.iam import base_classes
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+from googlecloudsdk.api_lib.iam import util
+from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.iam import iam_util
 
 
-class SetIamPolicy(base_classes.BaseIamCommand):
+class SetIamPolicy(base.Command):
   """Set the IAM policy for a service account.
 
   This command replaces the existing IAM policy for a service account, given
@@ -32,6 +39,9 @@ class SetIamPolicy(base_classes.BaseIamCommand):
 
   detailed_help = iam_util.GetDetailedHelpForSetIamPolicy(
       'service account', 'my-iam-account@somedomain.com')
+  detailed_help['DESCRIPTION'] += '\n\n' + (
+      iam_util.GetHintForServiceAccountResource(
+          'set the iam policy of'))
 
   @staticmethod
   def Args(parser):
@@ -45,12 +55,13 @@ class SetIamPolicy(base_classes.BaseIamCommand):
         'containing a valid policy.')
 
   def Run(self, args):
-    policy = iam_util.ParsePolicyFile(args.policy_file, self.messages.Policy)
+    client, messages = util.GetClientAndMessages()
+    policy = iam_util.ParsePolicyFile(args.policy_file, messages.Policy)
 
-    result = self.iam_client.projects_serviceAccounts.SetIamPolicy(
-        self.messages.IamProjectsServiceAccountsSetIamPolicyRequest(
+    result = client.projects_serviceAccounts.SetIamPolicy(
+        messages.IamProjectsServiceAccountsSetIamPolicyRequest(
             resource=iam_util.EmailToAccountResourceName(args.service_account),
-            setIamPolicyRequest=self.messages.SetIamPolicyRequest(
+            setIamPolicyRequest=messages.SetIamPolicyRequest(
                 policy=policy)))
     iam_util.LogSetIamPolicy(args.service_account, 'service account')
     return result
