@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Command to add IAM policy binding for a key."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.cloudkms import iam
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.iam import iam_util
@@ -46,9 +51,10 @@ class AddIamPolicyBinding(base.Command):
 
   @staticmethod
   def Args(parser):
-    flags.AddCryptoKeyArgument(parser, 'whose IAM policy to modify')
+    flags.AddKeyResourceArgument(parser, 'whose IAM policy to modify')
     iam_util.AddArgsForAddIamPolicyBinding(
-        parser, completer=completers.CryptoKeysIamRolesCompleter)
+        parser,
+        role_completer=completers.CryptoKeysIamRolesCompleter)
 
   def Run(self, args):
     crypto_key_ref = flags.ParseCryptoKeyName(args)
