@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Command to remove a policy binding from a key."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.cloudkms import iam
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.iam import iam_util
@@ -45,9 +50,11 @@ class RemoveIamPolicyBinding(base.Command):
 
   @staticmethod
   def Args(parser):
-    flags.AddCryptoKeyArgument(parser, 'from which to remove a policy binding')
+    flags.AddKeyResourceArgument(parser,
+                                 'from which to remove a policy binding')
     iam_util.AddArgsForRemoveIamPolicyBinding(
-        parser, completer=completers.CryptoKeysIamRolesCompleter)
+        parser,
+        role_completer=completers.CryptoKeysIamRolesCompleter)
 
   def Run(self, args):
     return iam.RemovePolicyBindingFromCryptoKey(
