@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Remove a rotation schedule."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.kms import flags
@@ -35,8 +40,8 @@ class RemoveRotationSchedule(base.UpdateCommand):
 
   @staticmethod
   def Args(parser):
-    flags.AddCryptoKeyArgument(parser,
-                               'from which to clear the rotation schedule')
+    flags.AddKeyResourceArgument(parser,
+                                 'from which to clear the rotation schedule')
 
   def Run(self, args):
     client = cloudkms_base.GetClientInstance()
