@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +15,21 @@
 
 """A command that validates gcloud flags according to Cloud SDK CLI Style."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
+import io
 import os
-import StringIO
 
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import usage_text
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
+from googlecloudsdk.core.util import files
+
+import six
 
 
 class UnknownCheckException(Exception):
@@ -141,8 +149,18 @@ class BadListsChecker(Checker):
             error_message=(
                 'flag [{flg}] has nargs={nargs}'.format(
                     flg=flag.option_strings[0],
-                    nargs=repr(flag.nargs)))))
-      if isinstance(flag.type, arg_parsers.ArgType):
+                    nargs="'{}'".format(six.text_type(flag.nargs))))))
+      if isinstance(flag.type, arg_parsers.ArgDict):
+        if not (flag.metavar or flag.type.spec):
+          self._issues.append(
+              LintError(
+                  name=BadListsChecker.name,
+                  command=cmd_or_group,
+                  error_message=(
+                      ('dict flag [{flg}] has no metavar and type.spec'
+                       ' (at least one needed)'
+                      ).format(flg=flag.option_strings[0]))))
+      elif isinstance(flag.type, arg_parsers.ArgList):
         if not flag.metavar:
           self._issues.append(LintError(
               name=BadListsChecker.name,
@@ -166,9 +184,9 @@ def _GetWhitelistedCommandVocabulary():
 
   vocabulary_file = os.path.join(os.path.dirname(__file__),
                                  'gcloud_command_vocabulary.txt')
-  with open(vocabulary_file, 'r') as f:
-    return set(line for line in f.read().split('\n')
-               if not line.startswith('#'))
+  return set(
+      line for line in files.ReadFileContents(vocabulary_file).split('\n')
+      if not line.startswith('#'))
 
 
 class VocabularyChecker(Checker):
@@ -208,7 +226,7 @@ def _WalkGroupTree(group):
 
   yield group
 
-  for sub_group in group.groups.itervalues():
+  for sub_group in six.itervalues(group.groups):
     for value in _WalkGroupTree(sub_group):
       yield value
 
@@ -227,7 +245,7 @@ class Linter(object):
     for group in _WalkGroupTree(group_root):
       for check in self._checks:
         check.ForEveryGroup(group)
-      for command in group.commands.itervalues():
+      for command in six.itervalues(group.commands):
         for check in self._checks:
           check.ForEveryCommand(command)
 
@@ -246,7 +264,7 @@ _LINT_CHECKS = [
 
 
 def _FormatCheckList(check_list):
-  buf = StringIO.StringIO()
+  buf = io.StringIO()
   for check in check_list:
     usage_text.WrapWithPrefix(
         check.name, check.description, 20, 78, '  ', writer=buf)
