@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,17 +14,25 @@
 # limitations under the License.
 """The `gcloud meta test` command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
 import os
 import signal
 import sys
 import time
 
 from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope import parser_completer
 from googlecloudsdk.calliope import parser_errors
 from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import execution_utils
+from googlecloudsdk.core import module_util
 from googlecloudsdk.core.console import console_io
+from googlecloudsdk.core.console import progress_tracker
 
 
 class Test(base.Command):
@@ -62,6 +71,17 @@ class Test(base.Command):
         action='store_true',
         help=('Call console_io.IsInteractive(heuristic=True) and exit 0 '
               'if the return value is True, 1 if False.'))
+    scenarios.add_argument(
+        '--prompt-completer',
+        metavar='MODULE_PATH',
+        help=('Call console_io.PromptResponse() with a MODULE_PATH completer '
+              'and print the response on the standard output.'))
+    scenarios.add_argument(
+        '--progress-tracker',
+        metavar='SECONDS',
+        type=float,
+        default=0.0,
+        help='Run the progress tracker for SECONDS seconds and exit.')
     scenarios.add_argument(
         '--sleep',
         metavar='SECONDS',
@@ -74,8 +94,7 @@ class Test(base.Command):
         help='Trigger an exception that is not caught.')
 
   def _RunArgumenterrorOutsideArgparse(self, args):
-    raise parser_errors.RequiredArgumentError(
-        'Argument required exception.', argument='--some-flag')
+    raise parser_errors.RequiredError(argument='--some-flag')
 
   def _RunCoreException(self, args):
     raise exceptions.Error('Some core exception.')
@@ -85,7 +104,7 @@ class Test(base.Command):
     execution_utils.Exec(['bash', args.exec_file])
 
   def _RunIsInteractive(self, args):
-    sys.exit(int(console_io.IsInteractive(heuristic=True)))
+    sys.exit(int(not console_io.IsInteractive(heuristic=True)))
 
   def _RunInterrupt(self, args):
     try:
@@ -98,6 +117,22 @@ class Test(base.Command):
       os.kill(os.getpid(), signal.SIGINT)
     raise exceptions.Error('SIGINT delivery failed.')
 
+  def _RunPromptCompleter(self, args):
+    completer_class = module_util.ImportModule(args.prompt_completer)
+    choices = parser_completer.ArgumentCompleter(completer_class, args)
+    response = console_io.PromptResponse('Complete this: ', choices=choices)
+    print(response)
+
+  def _RunProgressTracker(self, args):
+    start_time = time.time()
+    def message_callback():
+      remaining_time = args.progress_tracker - (time.time() - start_time)
+      return '{0:.1f}s remaining'.format(remaining_time)
+    with progress_tracker.ProgressTracker(
+        message='This is a progress tracker.',
+        detail_message_callback=message_callback):
+      time.sleep(args.progress_tracker)
+
   def _RunSleep(self, args):
     time.sleep(args.sleep)
 
@@ -115,6 +150,10 @@ class Test(base.Command):
       self._RunInterrupt(args)
     elif args.is_interactive:
       self._RunIsInteractive(args)
+    elif args.prompt_completer:
+      self._RunPromptCompleter(args)
+    elif args.progress_tracker:
+      self._RunProgressTracker(args)
     elif args.sleep:
       self._RunSleep(args)
     elif args.uncaught_exception:
