@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,6 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """ml-engine local train command."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import os
 
 from googlecloudsdk.calliope import base
@@ -88,10 +94,10 @@ class RunLocal(base.Command):
           user_args=user_args)
     else:
       if args.parameter_server_count:
-        log.warn(_BAD_FLAGS_WARNING_MESSAGE.format(
+        log.warning(_BAD_FLAGS_WARNING_MESSAGE.format(
             flag='--parameter-server-count'))
       if args.worker_count:
-        log.warn(_BAD_FLAGS_WARNING_MESSAGE.format(flag='--worker-count'))
+        log.warning(_BAD_FLAGS_WARNING_MESSAGE.format(flag='--worker-count'))
       retval = local_train.MakeProcess(args.module_name,
                                        package_root,
                                        args=user_args,
