@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,17 +13,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """ml-engine models create command."""
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.ml_engine import models
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.ml_engine import flags
 from googlecloudsdk.command_lib.ml_engine import models_util
+from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import log
 
 
 def _AddCreateArgs(parser):
   """Get arguments for the `ml-engine models create` command."""
   flags.GetModelName().AddToParser(parser)
+  flags.GetDescriptionFlag('model').AddToParser(parser)
   parser.add_argument(
       '--regions',
       metavar='REGION',
@@ -37,6 +45,7 @@ Will soon be required, but defaults to 'us-central1' for now.
       '--enable-logging',
       action='store_true',
       help=('If set, enables StackDriver Logging for online prediction.'))
+  labels_util.AddCreateLabelsFlags(parser)
 
 
 class Create(base.CreateCommand):
@@ -47,7 +56,11 @@ class Create(base.CreateCommand):
     _AddCreateArgs(parser)
 
   def Run(self, args):
-    model = models_util.Create(models.ModelsClient(), args.model,
+    models_client = models.ModelsClient()
+    labels = models_util.ParseCreateLabels(models_client, args)
+    model = models_util.Create(models_client, args.model,
                                regions=args.regions,
-                               enable_logging=args.enable_logging)
+                               enable_logging=args.enable_logging,
+                               labels=labels,
+                               description=args.description)
     log.CreatedResource(model.name, kind='ml engine model')
