@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +15,10 @@
 
 """Command to remove IAM policy binding for a resource."""
 
-import httplib
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 
 from googlecloudsdk.api_lib.cloudresourcemanager import projects_api
 from googlecloudsdk.api_lib.util import http_retry
@@ -23,8 +27,10 @@ from googlecloudsdk.command_lib.iam import iam_util
 from googlecloudsdk.command_lib.projects import flags
 from googlecloudsdk.command_lib.projects import util as command_lib_util
 from googlecloudsdk.command_lib.resource_manager import completers
+import six.moves.http_client
 
 
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class RemoveIamPolicyBinding(base.Command):
   """Remove IAM policy binding for a project.
 
@@ -39,9 +45,10 @@ class RemoveIamPolicyBinding(base.Command):
   def Args(parser):
     flags.GetProjectFlag('remove IAM policy binding from').AddToParser(parser)
     iam_util.AddArgsForRemoveIamPolicyBinding(
-        parser, completer=completers.ProjectsIamRolesCompleter)
+        parser,
+        role_completer=completers.ProjectsIamRolesCompleter)
 
-  @http_retry.RetryOnHttpStatus(httplib.CONFLICT)
+  @http_retry.RetryOnHttpStatus(six.moves.http_client.CONFLICT)
   def Run(self, args):
     project_ref = command_lib_util.ParseProject(args.id)
     return projects_api.RemoveIamPolicyBinding(project_ref,
