@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,17 +15,14 @@
 
 """The main command group for Cloud Pub/Sub snapshots."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
-@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
 class Snapshots(base.Group):
-  """These features are part of an invite-only release of the Cloud Pub/Sub API.
-
-  Manage Cloud Pub/Sub snapshots.
-  These features are part of an invitation-only release of the underlying
-  Cloud Pub/Sub API. These commands will generate errors unless you have access
-  to this API. This restriction should be relaxed in the near future. Please
-  contact cloud-pubsub@google.com with any questions in the meantime.
-  """
+  """Manage Cloud Pub/Sub snapshots."""
   pass
