@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """The super-group for the Cloud Resource Manager CLI."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
@@ -22,3 +27,7 @@ class ResourceManager(base.Group):
 
   Commands to query and update your Cloud Organizations, Folders, and Projects.
   """
+
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
