@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """The configs variables describe command."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.runtime_config import util
 from googlecloudsdk.command_lib.runtime_config import base_commands
 
@@ -35,4 +40,5 @@ class Describe(base_commands.VariableRetrieverCommand):
 
   def Run(self, args):
     result = super(Describe, self).Run(args)
-    return util.FormatVariable(result)
+    # Describe always returns the value.
+    return util.FormatVariable(result, True)
