@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,9 +14,13 @@
 # limitations under the License.
 """Command for spanner databases delete."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.spanner import databases
 from googlecloudsdk.calliope import base
-from googlecloudsdk.command_lib.spanner import flags
+from googlecloudsdk.command_lib.spanner import resource_args
 from googlecloudsdk.core.console import console_io
 
 
@@ -24,17 +29,8 @@ class Delete(base.DeleteCommand):
 
   @staticmethod
   def Args(parser):
-    """Args is called by calliope to gather arguments for this command.
-
-    Please add arguments in alphabetical order except for no- or a clear-
-    pair for that argument which can follow the argument itself.
-    Args:
-      parser: An argparse parser that you can use to add arguments that go
-          on the command line after this command. Positional arguments are
-          allowed.
-    """
-    flags.Instance(positional=False).AddToParser(parser)
-    flags.Database().AddToParser(parser)
+    """See base class."""
+    resource_args.AddDatabaseResourceArg(parser, 'to delete')
 
   def Run(self, args):
     """This is what gets called when the user runs this command.
@@ -46,8 +42,9 @@ class Delete(base.DeleteCommand):
     Returns:
       Some value that we want to have printed later.
     """
+    database_ref = args.CONCEPTS.database.Parse()
     console_io.PromptContinue(
-        message='Delete database [{1}] in instance [{0}]. Are you sure?'.format(
-            args.instance, args.database),
+        'You are about to delete database: [{}]'.format(database_ref.Name()),
+        throw_if_unattended=True,
         cancel_on_no=True)
-    return databases.Delete(args.instance, args.database)
+    return databases.Delete(database_ref)
