@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,20 +19,13 @@ The fact that this is a directory with
 an __init__.py in it makes it a command group. The methods written below will
 all be called by calliope (though they are all optional).
 """
-import argparse
-import os
-import re
 
-from googlecloudsdk.api_lib.util import apis
-from googlecloudsdk.calliope import actions
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
-from googlecloudsdk.calliope import exceptions
-from googlecloudsdk.core import properties
 
-_ACTIVE_VERSIONS = [
-    'v1beta3',
-    'v1beta4',
-]
 
 DETAILED_HELP = {
     'DESCRIPTION': """\
@@ -49,29 +43,10 @@ DETAILED_HELP = {
 }
 
 
-def _Args(parser):
-  parser.add_argument(
-      '--api-version',
-      help=argparse.SUPPRESS,
-      choices=_ACTIVE_VERSIONS,
-      action=actions.StoreProperty(
-          properties.VALUES.api_endpoint_overrides.sql))
-
-
-@base.ReleaseTracks(base.ReleaseTrack.GA)
 class SQL(base.Group):
-  """Manage Cloud SQL databases."""
-
-  @staticmethod
-  def Args(parser):
-    _Args(parser)
-
-
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
-class SQLBeta(base.Group):
   """Create and manage Google Cloud SQL databases."""
   detailed_help = DETAILED_HELP
 
-  @staticmethod
-  def Args(parser):
-    _Args(parser)
+  def Filter(self, context, args):
+    del context, args
+    base.DisableUserProjectQuota()
