@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Creates a backup of a Cloud SQL instance."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.sql import api_util
 from googlecloudsdk.api_lib.sql import operations
 from googlecloudsdk.api_lib.sql import validate
@@ -22,7 +27,8 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.ALPHA)
 class CreateBackup(base.CreateCommand):
   """Creates a backup of a Cloud SQL instance."""
 
@@ -35,13 +41,14 @@ class CreateBackup(base.CreateCommand):
           on the command line after this command. Positional arguments are
           allowed.
     """
-    flags.INSTANCE_FLAG.AddToParser(parser)
+    flags.AddInstance(parser)
     parser.add_argument(
         '--description', help='A friendly description of the backup.')
     parser.add_argument(
         '--async',
         action='store_true',
         help='Do not wait for the operation to complete.')
+    parser.display_info.AddCacheUpdater(None)
 
   def Run(self, args):
     """Restores a backup of a Cloud SQL instance.
@@ -53,11 +60,6 @@ class CreateBackup(base.CreateCommand):
     Returns:
       A dict object representing the operations resource describing the
       restoreBackup operation if the restoreBackup was successful.
-    Raises:
-      HttpException: A http error response was received while executing api
-          request.
-      ToolException: An error other than http error occured while executing the
-          command.
     """
 
     client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
