@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,6 +18,10 @@ Lists all backups associated with a given instance and configuration
 in the reverse chronological order of the enqueued time.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import list_pager
 
 from googlecloudsdk.api_lib.sql import api_util
@@ -26,7 +31,8 @@ from googlecloudsdk.command_lib.sql import flags
 from googlecloudsdk.core import properties
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.ALPHA)
 class List(base.ListCommand):
   """Lists all backups associated with a given instance.
 
@@ -36,7 +42,7 @@ class List(base.ListCommand):
 
   @staticmethod
   def Args(parser):
-    flags.INSTANCE_FLAG.AddToParser(parser)
+    flags.AddInstance(parser)
     parser.display_info.AddFormat("""
       table(
         id,
@@ -45,6 +51,7 @@ class List(base.ListCommand):
         status
       )
     """)
+    parser.display_info.AddCacheUpdater(None)
 
   def Run(self, args):
     """Lists all backups associated with a given instance.
@@ -56,11 +63,6 @@ class List(base.ListCommand):
     Returns:
       A dict object that has the list of backup run resources if the command ran
       successfully.
-    Raises:
-      HttpException: A http error response was received while executing api
-          request.
-      ToolException: An error other than http error occured while executing the
-          command.
     """
 
     client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
