@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,20 +13,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Creates a new Cloud SQL instance."""
-import argparse
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
 
 from apitools.base.py import exceptions as apitools_exceptions
 
-from googlecloudsdk.api_lib.sql import api_util
-from googlecloudsdk.api_lib.sql import instances
+from googlecloudsdk.api_lib.sql import api_util as common_api_util
 from googlecloudsdk.api_lib.sql import operations
 from googlecloudsdk.api_lib.sql import validate
-from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.sql import flags
+from googlecloudsdk.command_lib.sql import instances as command_util
 from googlecloudsdk.command_lib.sql import validate as command_validate
-from googlecloudsdk.command_lib.util import labels_util
+from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.console import console_io
@@ -37,86 +40,28 @@ def AddBaseArgs(parser):
   """Declare flag and positional arguments for this command parser."""
   # TODO(b/35705305): move common flags to command_lib.sql.flags
   base.ASYNC_FLAG.AddToParser(parser)
-  parser.display_info.AddFormat(flags.INSTANCES_FORMAT_BETA)
-  parser.add_argument(
-      '--activation-policy',
-      required=False,
-      choices=['ALWAYS', 'NEVER', 'ON_DEMAND'],
-      default=None,
-      help=('The activation policy for this instance. This specifies when '
-            'the instance should be activated and is applicable only when '
-            'the instance state is RUNNABLE. More information on activation '
-            'policies can be found here: '
-            'https://cloud.google.com/sql/faq#activation_policy'))
-  parser.add_argument(
-      '--assign-ip',
-      required=False,
-      action='store_true',
-      default=None,  # Tri-valued: None => don't change the setting.
-      help='Specified if the instance must be assigned an IP address.')
-  parser.add_argument(
-      '--authorized-gae-apps',
-      type=arg_parsers.ArgList(min_length=1),
-      metavar='APP',
-      required=False,
-      default=[],
-      help=('First Generation instances only. List of IDs for App Engine '
-            'applications running in the Standard environment that can '
-            'access this instance.'))
-  parser.add_argument(
-      '--authorized-networks',
-      type=arg_parsers.ArgList(min_length=1),
-      metavar='NETWORK',
-      required=False,
-      default=[],
-      help=('The list of external networks that are allowed to connect to '
-            'the instance. Specified in CIDR notation, also known as '
-            '\'slash\' notation (e.g. 192.168.100.0/24).'))
+  parser.display_info.AddFormat(flags.GetInstanceListFormat())
+  flags.AddActivationPolicy(parser)
+  flags.AddAssignIp(parser, show_negated_in_help=False)
+  flags.AddAuthorizedGAEApps(parser)
+  flags.AddAuthorizedNetworks(parser)
+  flags.AddAvailabilityType(parser)
   parser.add_argument(
       '--backup',
       required=False,
       action='store_true',
       default=True,
       help='Enables daily backup.')
-  parser.add_argument(
-      '--backup-start-time',
-      required=False,
-      help=('The start time of daily backups, specified in the 24 hour '
-            'format - HH:MM, in the UTC timezone.'))
-  parser.add_argument(
-      '--cpu',
-      type=int,
-      required=False,
-      help=('A whole number value indicating how many cores are desired in '
-            'the machine. Both --cpu and --memory must be specified if a '
-            'custom machine type is desired, and the --tier flag must be '
-            'omitted.'))
-  parser.add_argument(
-      '--database-flags',
-      type=arg_parsers.ArgDict(min_length=1),
-      metavar='FLAG=VALUE',
-      required=False,
-      help=('A comma-separated list of database flags to set on the '
-            'instance. Use an equals sign to separate flag name and value. '
-            'Flags without values, like skip_grant_tables, can be written '
-            'out without a value after, e.g., `skip_grant_tables=`. Use '
-            'on/off for booleans. View the Instance Resource API for allowed '
-            'flags. (e.g., `--database-flags max_allowed_packet=55555,'
-            'skip_grant_tables=,log_output=1`)'))
+  flags.AddBackupStartTime(parser)
+  flags.AddCPU(parser)
+  flags.AddDatabaseFlags(parser)
   parser.add_argument(
       '--database-version',
       required=False,
-      default='MYSQL_5_6',
       choices=['MYSQL_5_5', 'MYSQL_5_6', 'MYSQL_5_7', 'POSTGRES_9_6'],
-      help='The database engine type and version.')
-  parser.add_argument(
-      '--enable-bin-log',
-      required=False,
-      action='store_true',
-      default=None,  # Tri-valued: None => don't change the setting.
-      help=(
-          'Specified if binary log should be enabled. If backup '
-          'configuration is disabled, binary log must be disabled as well.'))
+      help=('The database engine type and version. If left unspecified, the '
+            'API defaults will be used.'))
+  flags.AddEnableBinLog(parser, show_negated_in_help=False)
   parser.add_argument(
       '--failover-replica-name',
       required=False,
@@ -127,6 +72,7 @@ def AddBaseArgs(parser):
       help=('First Generation instances only. The App Engine app this '
             'instance should follow. It must be in the same region as '
             'the instance.'))
+  # TODO(b/73362466): Add `--zone` and deprecate `--gce-zone`.
   parser.add_argument(
       '--gce-zone',
       required=False,
@@ -136,45 +82,16 @@ def AddBaseArgs(parser):
       'instance',
       type=command_validate.InstanceNameRegexpValidator(),
       help='Cloud SQL instance ID.')
-  parser.add_argument(
-      '--maintenance-release-channel',
-      choices={
-          'production': 'Production updates are stable and recommended '
-                        'for applications in production.',
-          'preview': 'Preview updates release prior to production '
-                     'updates. You may wish to use the preview channel '
-                     'for dev/test applications so that you can preview '
-                     'their compatibility with your application prior '
-                     'to the production release.'
-      },
-      type=str.lower,
-      help="Which channel's updates to apply during the maintenance window.")
-  parser.add_argument(
-      '--maintenance-window-day',
-      choices=arg_parsers.DayOfWeek.DAYS,
-      type=arg_parsers.DayOfWeek.Parse,
-      help='Day of week for maintenance window, in UTC time zone.')
-  parser.add_argument(
-      '--maintenance-window-hour',
-      type=arg_parsers.BoundedInt(lower_bound=0, upper_bound=23),
-      help='Hour of day for maintenance window, in UTC time zone.')
+  flags.AddMaintenanceReleaseChannel(parser)
+  flags.AddMaintenanceWindowDay(parser)
+  flags.AddMaintenanceWindowHour(parser)
   parser.add_argument(
       '--master-instance-name',
       required=False,
       help=('Name of the instance which will act as master in the '
             'replication setup. The newly created instance will be a read '
             'replica of the specified master instance.'))
-  parser.add_argument(
-      '--memory',
-      type=arg_parsers.BinarySize(),
-      required=False,
-      help=('A whole number value indicating how much memory is desired in '
-            'the machine. A size unit should be provided (eg. 3072MiB or '
-            '9GiB) - if no units are specified, GiB is assumed. Both --cpu '
-            'and --memory must be specified if a custom machine type is '
-            'desired, and the --tier flag must be omitted.'))
-  parser.add_argument(
-      '--on-premises-host-port', required=False, help=argparse.SUPPRESS)
+  flags.AddMemory(parser)
   parser.add_argument(
       '--pricing-plan',
       '-p',
@@ -184,6 +101,7 @@ def AddBaseArgs(parser):
       help=('First Generation instances only. The pricing plan for this '
             'instance.'))
   # TODO(b/31989340): add remote completion
+  # TODO(b/73362466): Make zone and region mutually exclusive.
   parser.add_argument(
       '--region',
       required=False,
@@ -195,43 +113,21 @@ def AddBaseArgs(parser):
       '--replica-type',
       choices=['READ', 'FAILOVER'],
       help='The type of replica to create.')
-  parser.add_argument(
-      '--replication',
-      required=False,
-      choices=['SYNCHRONOUS', 'ASYNCHRONOUS'],
-      default=None,
-      help='The type of replication this instance uses.')
+  flags.AddReplication(parser)
   parser.add_argument(
       '--require-ssl',
       required=False,
       action='store_true',
-      default=None,  # Tri-valued: None => don't change the setting.
-      help='Specified if users connecting over IP must use SSL.')
-  parser.add_argument(
-      '--storage-auto-increase',
-      action='store_true',
       default=None,
-      help=('Storage size can be increased, but it cannot be decreased; '
-            'storage increases are permanent for the life of the instance. '
-            'With this setting enabled, a spike in storage requirements '
-            'can result in permanently increased storage costs for your '
-            'instance. However, if an instance runs out of available space, '
-            'it can result in the instance going offline, dropping existing '
-            'connections.'))
-  parser.add_argument(
-      '--storage-size',
-      type=arg_parsers.BinarySize(
-          lower_bound='10GB',
-          upper_bound='10230GB',
-          suggested_binary_size_scales=['GB']),
-      help=('Amount of storage allocated to the instance. Must be an integer '
-            'number of GB between 10GB and 10230GB inclusive.'))
+      help='Specified if users connecting over IP must use SSL.')
+  flags.AddStorageAutoIncrease(parser)
+  flags.AddStorageSize(parser)
   parser.add_argument(
       '--storage-type',
       required=False,
       choices=['SSD', 'HDD'],
       default=None,
-      help='The storage type for the instance.')
+      help='The storage type for the instance. The default is SSD.')
   parser.add_argument(
       '--tier',
       '-t',
@@ -239,16 +135,26 @@ def AddBaseArgs(parser):
       help=('The tier for this instance. For Second Generation instances, '
             'TIER is the instance\'s machine type (e.g., db-n1-standard-1). '
             'For PostgreSQL instances, only shared-core machine types '
-            '(e.g., db-f1-micro) apply. A complete list of tiers is '
-            'available here: https://cloud.google.com/sql/pricing.'))
+            '(e.g., db-f1-micro) apply. The default tier is db-n1-standard-1. '
+            'A complete list of tiers is available here: '
+            'https://cloud.google.com/sql/pricing'))
 
 
-def RunBaseCreateCommand(args):
+def AddBetaArgs(parser):
+  """Declare beta flag and positional arguments for this command parser."""
+  flags.AddExternalMasterGroup(parser)
+  flags.AddInstanceResizeLimit(parser)
+  flags.AddNetwork(parser)
+  labels_util.AddCreateLabelsFlags(parser)
+
+
+def RunBaseCreateCommand(args, release_track):
   """Creates a new Cloud SQL instance.
 
   Args:
     args: argparse.Namespace, The arguments that this command was invoked
         with.
+    release_track: base.ReleaseTrack, the release track that this was run under.
 
   Returns:
     A dict object representing the operations resource describing the create
@@ -256,10 +162,8 @@ def RunBaseCreateCommand(args):
   Raises:
     HttpException: A http error response was received while executing api
         request.
-    ToolException: An error other than http error occured while executing the
-        command.
   """
-  client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
+  client = common_api_util.SqlClient(common_api_util.API_VERSION_DEFAULT)
   sql_client = client.sql_client
   sql_messages = client.sql_messages
 
@@ -268,14 +172,49 @@ def RunBaseCreateCommand(args):
       args.instance,
       params={'project': properties.VALUES.core.project.GetOrFail},
       collection='sql.instances')
-  instance_resource = instances.InstancesV1Beta4.ConstructInstanceFromArgs(
-      sql_messages, args, instance_ref=instance_ref)
+
+  # Get the region, tier, and database version from the master if these fields
+  # are not specified.
+  # TODO(b/64266672): Remove once API does not require these fields.
+  if args.IsSpecified('master_instance_name'):
+    master_instance_ref = client.resource_parser.Parse(
+        args.master_instance_name,
+        params={'project': properties.VALUES.core.project.GetOrFail},
+        collection='sql.instances')
+    try:
+      master_instance_resource = sql_client.instances.Get(
+          sql_messages.SqlInstancesGetRequest(
+              project=instance_ref.project,
+              instance=master_instance_ref.instance))
+    except apitools_exceptions.HttpError as error:
+      # TODO(b/64292220): Remove once API gives helpful error message.
+      log.debug('operation : %s', str(master_instance_ref))
+      exc = exceptions.HttpException(error)
+      if resource_property.Get(exc.payload.content,
+                               resource_lex.ParseKey('error.errors[0].reason'),
+                               None) == 'notAuthorized':
+        msg = ('You are either not authorized to access the master instance or '
+               'it does not exist.')
+        raise exceptions.HttpException(msg)
+      raise
+    if not args.IsSpecified('region'):
+      args.region = master_instance_resource.region
+    if not args.IsSpecified('database_version'):
+      args.database_version = master_instance_resource.databaseVersion
+    if not args.IsSpecified('tier') and master_instance_resource.settings:
+      args.tier = master_instance_resource.settings.tier
+
+  instance_resource = (
+      command_util.InstancesV1Beta4.ConstructCreateInstanceFromArgs(
+          sql_messages,
+          args,
+          instance_ref=instance_ref,
+          release_track=release_track))
 
   if args.pricing_plan == 'PACKAGE':
-    if not console_io.PromptContinue(
+    console_io.PromptContinue(
         'Charges will begin accruing immediately. Really create Cloud '
-        'SQL instance?'):
-      raise exceptions.ToolException('canceled by the user.')
+        'SQL instance?', cancel_on_no=True)
 
   operation_ref = None
   try:
@@ -321,7 +260,7 @@ class Create(base.Command):
   """Creates a new Cloud SQL instance."""
 
   def Run(self, args):
-    return RunBaseCreateCommand(args)
+    return RunBaseCreateCommand(args, self.ReleaseTrack())
 
   @staticmethod
   def Args(parser):
@@ -329,16 +268,15 @@ class Create(base.Command):
     AddBaseArgs(parser)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
 class CreateBeta(base.Command):
   """Creates a new Cloud SQL instance."""
 
   def Run(self, args):
-    return RunBaseCreateCommand(args)
+    return RunBaseCreateCommand(args, self.ReleaseTrack())
 
   @staticmethod
   def Args(parser):
     """Args is called by calliope to gather arguments for this command."""
     AddBaseArgs(parser)
-    flags.INSTANCE_RESIZE_LIMIT_FLAG.AddToParser(parser)
-    labels_util.AddCreateLabelsFlags(parser)
+    AddBetaArgs(parser)
