@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Deletes a Cloud SQL instance."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import exceptions
 
 from googlecloudsdk.api_lib.sql import api_util
@@ -22,11 +27,11 @@ from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.sql import flags
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
-from googlecloudsdk.core import remote_completion
 from googlecloudsdk.core.console import console_io
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.ALPHA)
 class Delete(base.Command):
   """Deletes a Cloud SQL instance."""
 
@@ -55,11 +60,6 @@ class Delete(base.Command):
     Returns:
       A dict object representing the operations resource describing the delete
       operation if the delete was successful.
-    Raises:
-      HttpException: A http error response was received while executing api
-          request.
-      ToolException: An error other than http error occured while executing the
-          command.
     """
     client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
     sql_client = client.sql_client
@@ -93,8 +93,6 @@ class Delete(base.Command):
           sql_client, operation_ref, 'Deleting Cloud SQL instance')
 
       log.DeletedResource(instance_ref)
-      cache = remote_completion.RemoteCompletion()
-      cache.DeleteFromCache(instance_ref.SelfLink())
 
     except exceptions.HttpError:
       log.debug('operation : %s', str(operation_ref))
