@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,6 +18,10 @@ Lists all instance operations that have been performed on the given
 Cloud SQL instance.
 """
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from apitools.base.py import list_pager
 
 from googlecloudsdk.api_lib.sql import api_util
@@ -26,14 +31,16 @@ from googlecloudsdk.command_lib.sql import flags
 from googlecloudsdk.core import properties
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.ALPHA)
 class List(base.ListCommand):
   """Lists all instance operations for the given Cloud SQL instance."""
 
   @staticmethod
   def Args(parser):
-    flags.INSTANCE_FLAG.AddToParser(parser)
+    flags.AddInstance(parser)
     parser.display_info.AddFormat(flags.OPERATION_FORMAT_BETA)
+    parser.display_info.AddCacheUpdater(None)
 
   def Run(self, args):
     """Lists all instance operations that have been performed on an instance.
@@ -45,11 +52,6 @@ class List(base.ListCommand):
     Returns:
       A dict object that has the list of operation resources if the command ran
       successfully.
-    Raises:
-      HttpException: A http error response was received while executing api
-          request.
-      ToolException: An error other than http error occured while executing the
-          command.
     """
     client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
     sql_client = client.sql_client
