@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,12 +14,16 @@
 # limitations under the License.
 """Deletes an SSL certificate for a Cloud SQL instance."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.sql import api_util
 from googlecloudsdk.api_lib.sql import cert
+from googlecloudsdk.api_lib.sql import exceptions
 from googlecloudsdk.api_lib.sql import operations
 from googlecloudsdk.api_lib.sql import validate
 from googlecloudsdk.calliope import base
-from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.sql import flags
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
@@ -41,10 +46,11 @@ class _BaseDelete(object):
     parser.add_argument(
         'common_name',
         help='User supplied name. Constrained to ```[a-zA-Z.-_ ]+```.')
-    flags.INSTANCE_FLAG.AddToParser(parser)
+    flags.AddInstance(parser)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.ALPHA)
 class Delete(_BaseDelete, base.Command):
   """Deletes an SSL certificate for a Cloud SQL instance."""
 
@@ -59,10 +65,7 @@ class Delete(_BaseDelete, base.Command):
       A dict object representing the operations resource describing the delete
       operation if the api request was successful.
     Raises:
-      HttpException: A http error response was received while executing api
-          request.
-      ToolException: An error other than http error occured while executing the
-          command.
+      ResourceNotFoundError: The ssl cert could not be found for the instance.
     """
     client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
     sql_client = client.sql_client
@@ -88,7 +91,7 @@ class Delete(_BaseDelete, base.Command):
                                        client.resource_parser, instance_ref,
                                        args.common_name)
     if not cert_ref:
-      raise exceptions.ToolException(
+      raise exceptions.ResourceNotFoundError(
           'no ssl cert named [{name}] for instance [{instance}]'.format(
               name=args.common_name, instance=instance_ref))
 
