@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2013 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Retrieves information about an SSL cert for a Cloud SQL instance."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.sql import api_util
 from googlecloudsdk.api_lib.sql import cert
 from googlecloudsdk.api_lib.sql import validate
@@ -36,7 +41,7 @@ class _BaseGet(object):
     parser.add_argument(
         'common_name',
         help='User supplied name. Constrained to ```[a-zA-Z.-_ ]+```.')
-    flags.INSTANCE_FLAG.AddToParser(parser)
+    flags.AddInstance(parser)
 
   def Run(self, args):
     """Retrieves information about an SSL cert for a Cloud SQL instance.
@@ -48,11 +53,6 @@ class _BaseGet(object):
     Returns:
       A dict object representing the sslCerts resource if the api request was
       successful.
-    Raises:
-      HttpException: A http error response was received while executing api
-          request.
-      ToolException: An error other than http error occured while executing the
-          command.
     """
     client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
     sql_client = client.sql_client
@@ -69,7 +69,8 @@ class _BaseGet(object):
                                 args.common_name)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA,
+                    base.ReleaseTrack.ALPHA)
 class Get(_BaseGet, base.DescribeCommand):
   """Retrieves information about an SSL cert for a Cloud SQL instance."""
   pass
