@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2014 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +15,10 @@
 
 """gcloud supplementary help topic command group."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import base
 
 
@@ -28,7 +33,7 @@ class Topic(base.Group):
 
     $ gcloud topic [TOPIC_NAME]
 
-  For a lits of available topics run:
+  For a list of available topics run:
 
     $ gcloud topic --help
   """
