@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*- #
 # Copyright 2016 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +14,10 @@
 # limitations under the License.
 """Script to regenerate samples with latest client generator."""
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import unicode_literals
+
 import argparse
 import logging
 import os
@@ -20,7 +25,8 @@ import pprint
 import sys
 
 from googlecloudsdk.api_lib.regen import generate
-import yaml
+from googlecloudsdk.core import yaml
+import six
 
 
 class Error(Exception):
@@ -65,8 +71,7 @@ def main(argv=None):
         format='%(asctime)s %(filename)s:%(lineno)d %(message)s',
         level=getattr(logging, args.log_level))
 
-  with open(args.config, 'r') as stream:
-    config = yaml.load(stream)
+  config = yaml.load_path(args.config)
 
   logging.debug('Config %s', pprint.pformat(config))
 
@@ -96,13 +101,14 @@ def main(argv=None):
                                  config_file=args.config))
       regenerate_list = [(api_name, api_version, api_config)]
     else:
-      regenerate_list = [(api_name, api_version, api_config)
-                         for api_version, api_config in api_section.iteritems()]
+      regenerate_list = [
+          (api_name, api_version, api_config)
+          for api_version, api_config in six.iteritems(api_section)]
   else:
     regenerate_list = [
         (api_name, api_version, api_config)
-        for api_name, api_version_config in config['apis'].iteritems()
-        for api_version, api_config in api_version_config.iteritems()
+        for api_name, api_version_config in six.iteritems(config['apis'])
+        for api_version, api_config in six.iteritems(api_version_config)
     ]
 
   for api_name, api_version, api_config in sorted(regenerate_list):
