@@ -38,13 +38,13 @@ import traceback
 # so boto requests would not include gsutil/version# in the UserAgent string.
 import boto
 import gslib
-# TODO: gsutil-beta: Cloud SDK scans for this string and performs
-# substitution; ensure this works with both apitools and boto.
+from gslib.utils import system_util
+
 boto.UserAgent += ' gsutil/%s (%s)' % (gslib.VERSION, sys.platform)
-if os.environ.get('CLOUDSDK_WRAPPER') == '1':
+if system_util.InvokedViaCloudSdk():
   boto.UserAgent += ' google-cloud-sdk'
-  if os.environ.get('CLOUDSDK_VERSION'):
-    boto.UserAgent += '/%s' % os.environ.get('CLOUDSDK_VERSION')
+  if system_util.CloudSdkVersion():
+    boto.UserAgent += '/%s' % system_util.CloudSdkVersion()
 # pylint: disable=g-import-not-at-top
 # This module also imports boto, and will override the UserAgent global variable
 # if imported above.
@@ -57,6 +57,7 @@ else:
 # pylint: disable=g-bad-import-order
 import httplib2
 import oauth2client
+from google_reauth import reauth_creds
 from gslib import wildcard_iterator
 from gslib.cloud_api import AccessDeniedException
 from gslib.cloud_api import ArgumentException
@@ -68,16 +69,8 @@ import gslib.exception
 from gslib.exception import CommandException
 from gslib.exception import ControlCException
 import apitools.base.py.exceptions as apitools_exceptions
-from gslib.util import CreateLock
-from gslib.util import DEBUGLEVEL_DUMP_REQUESTS
-from gslib.util import DEBUGLEVEL_DUMP_REQUESTS_AND_PAYLOADS
-from gslib.util import GetBotoConfigFileList
-from gslib.util import GetCertsFile
-from gslib.util import GetCleanupFiles
-from gslib.util import GetGsutilClientIdAndSecret
-from gslib.util import GsutilStreamHandler
-from gslib.util import ProxyInfoFromEnvironmentVar
-from gslib.util import UTF8
+from gslib.utils import boto_util
+from gslib.utils import constants
 from gslib.sig_handling import GetCaughtSignals
 from gslib.sig_handling import InitializeSignalHandling
 from gslib.sig_handling import RegisterSignalHandler
@@ -122,20 +115,29 @@ HTTP_WARNING = """
 ***************************** WARNING *****************************
 """.lstrip()
 
-debug = 0
+debug_level = 0
 test_exception_traces = False
 
 
 # pylint: disable=unused-argument
 def _CleanupSignalHandler(signal_num, cur_stack_frame):
-  """Cleans up if process is killed with SIGINT, SIGQUIT or SIGTERM."""
+  """Cleans up if process is killed with SIGINT, SIGQUIT or SIGTERM.
+
+  Note that this method is called after main() has been called, so it has
+  access to all the modules imported at the start of main().
+
+  Args:
+    signal_num: Unused, but required in the method signature.
+    cur_stack_frame: Unused, but required in the method signature.
+  """
   _Cleanup()
-  if gslib.util.CheckMultiprocessingAvailableAndInit().is_available:
+  if (gslib.utils.parallelism_framework_util
+      .CheckMultiprocessingAvailableAndInit().is_available):
     gslib.command.TeardownMultiprocessingProcesses()
 
 
 def _Cleanup():
-  for fname in GetCleanupFiles():
+  for fname in boto_util.GetCleanupFiles():
     try:
       os.unlink(fname)
     except:  # pylint: disable=bare-except
@@ -151,14 +153,14 @@ def _OutputAndExit(message, exception=None):
     message: Message to print to stderr.
     exception: The exception that caused gsutil to fail.
   """
-  if debug >= DEBUGLEVEL_DUMP_REQUESTS or test_exception_traces:
+  if debug_level >= constants.DEBUGLEVEL_DUMP_REQUESTS or test_exception_traces:
     stack_trace = traceback.format_exc()
     err = ('DEBUG: Exception stack trace:\n    %s\n%s\n' %
            (re.sub('\\n', '\n    ', stack_trace), message))
   else:
     err = '%s\n' % message
   try:
-    sys.stderr.write(err.encode(UTF8))
+    sys.stderr.write(err.encode(constants.UTF8))
   except UnicodeDecodeError:
     # Can happen when outputting invalid Unicode filenames.
     sys.stderr.write(err)
@@ -183,12 +185,12 @@ class GsutilFormatter(logging.Formatter):
     return super(GsutilFormatter, self).formatTime(record, datefmt=datefmt)
 
 
-def _ConfigureLogging(level=logging.INFO):
+def _ConfigureRootLogger(level=logging.INFO):
   """Similar to logging.basicConfig() except it always adds a handler."""
   log_format = '%(levelname)s %(asctime)s %(filename)s] %(message)s'
   date_format = '%m%d %H:%M:%S.%f'
   formatter = GsutilFormatter(fmt=log_format, datefmt=date_format)
-  handler = GsutilStreamHandler()
+  handler = logging.StreamHandler()
   handler.setFormatter(formatter)
   root_logger = logging.getLogger()
   root_logger.addHandler(handler)
@@ -202,15 +204,13 @@ def main():
   # pylint: disable=redefined-outer-name,g-import-not-at-top
   import gslib.boto_translation
   import gslib.command
-  import gslib.util
-  from gslib.util import BOTO_IS_SECURE
-  from gslib.util import CERTIFICATE_VALIDATION_ENABLED
+  import gslib.utils.parallelism_framework_util
   # pylint: disable=unused-variable
   from gcs_oauth2_boto_plugin import oauth2_client
   from apitools.base.py import credentials_lib
   # pylint: enable=unused-variable
-  from gslib.util import CheckMultiprocessingAvailableAndInit
-  if CheckMultiprocessingAvailableAndInit().is_available:
+  if (gslib.utils.parallelism_framework_util
+      .CheckMultiprocessingAvailableAndInit().is_available):
     # These setup methods must be called, and, on Windows, they can only be
     # called from within an "if __name__ == '__main__':" block.
     gslib.command.InitializeMultiprocessingVariables()
@@ -218,24 +218,29 @@ def main():
   else:
     gslib.command.InitializeThreadingVariables()
 
-  # This needs to be done after gslib.util.InitializeMultiprocessingVariables(),
-  # since otherwise we can't call gslib.util.CreateLock.
+  # This needs to be done after InitializeMultiprocessingVariables(), since
+  # otherwise we can't call CreateLock.
   try:
     # pylint: disable=unused-import,g-import-not-at-top
     import gcs_oauth2_boto_plugin
-    gsutil_client_id, gsutil_client_secret = GetGsutilClientIdAndSecret()
+    gsutil_client_id, gsutil_client_secret = (
+        system_util.GetGsutilClientIdAndSecret())
     gcs_oauth2_boto_plugin.oauth2_helper.SetFallbackClientIdAndSecret(
         gsutil_client_id, gsutil_client_secret)
-    gcs_oauth2_boto_plugin.oauth2_helper.SetLock(CreateLock())
-    credentials_lib.SetCredentialsCacheFileLock(CreateLock())
+    gcs_oauth2_boto_plugin.oauth2_helper.SetLock(
+        gslib.utils.parallelism_framework_util.CreateLock())
+    credentials_lib.SetCredentialsCacheFileLock(
+        gslib.utils.parallelism_framework_util.CreateLock())
   except ImportError:
     pass
 
-  global debug
+  global debug_level
   global test_exception_traces
 
-  if not (2, 6) <= sys.version_info[:3] < (3,):
-    raise CommandException('gsutil requires python 2.6 or 2.7.')
+  if not (2, 7) <= sys.version_info[:3] < (3,):
+    raise CommandException('gsutil requires python 2.7.')
+
+  boto_util.MonkeyPatchBoto()
 
   # In gsutil 4.0 and beyond, we don't use the boto library for the JSON
   # API. However, we still store gsutil configuration data in the .boto
@@ -243,7 +248,7 @@ def main():
   # Many users have a .boto configuration file from previous versions, and it
   # is useful to have all of the configuration for gsutil stored in one place.
   command_runner = CommandRunner()
-  if not BOTO_IS_SECURE:
+  if not boto_util.BOTO_IS_SECURE:
     raise CommandException('\n'.join(textwrap.wrap(
         'Your boto configuration has is_secure = False. Gsutil cannot be '
         'run this way, for security reasons.')))
@@ -252,10 +257,11 @@ def main():
   parallel_operations = False
   quiet = False
   version = False
-  debug = 0
+  debug_level = 0
   trace_token = None
   perf_trace_token = None
   test_exception_traces = False
+  user_project = None
 
   # If user enters no commands just print the usage info.
   if len(sys.argv) == 1:
@@ -268,14 +274,15 @@ def main():
       boto.config.add_section('Boto')
     boto.config.setbool('Boto', 'https_validate_certificates', True)
 
-  gslib.util.configured_certs_file = gslib.util.ConfigureCertsFile()
+  boto_util.configured_certs_file = (
+      boto_util.ConfigureCertsFile())
   for signal_num in GetCaughtSignals():
     RegisterSignalHandler(signal_num, _CleanupSignalHandler)
-  GetCertsFile()
+  boto_util.GetCertsFile()
 
   try:
     try:
-      opts, args = getopt.getopt(sys.argv[1:], 'dDvo:h:mq',
+      opts, args = getopt.getopt(sys.argv[1:], 'dDvo:h:u:mq',
                                  ['debug', 'detailedDebug', 'version', 'option',
                                   'help', 'header', 'multithreaded', 'quiet',
                                   'testexceptiontraces', 'trace-token=',
@@ -285,16 +292,16 @@ def main():
     for o, a in opts:
       if o in ('-d', '--debug'):
         # Also causes boto to include httplib header output.
-        debug = DEBUGLEVEL_DUMP_REQUESTS
+        debug_level = constants.DEBUGLEVEL_DUMP_REQUESTS
       elif o in ('-D', '--detailedDebug'):
         # We use debug level 3 to ask gsutil code to output more detailed
         # debug output. This is a bit of a hack since it overloads the same
         # flag that was originally implemented for boto use. And we use -DD
         # to ask for really detailed debugging (i.e., including HTTP payload).
-        if debug == DEBUGLEVEL_DUMP_REQUESTS:
-          debug = DEBUGLEVEL_DUMP_REQUESTS_AND_PAYLOADS
+        if debug_level == constants.DEBUGLEVEL_DUMP_REQUESTS:
+          debug_level = constants.DEBUGLEVEL_DUMP_REQUESTS_AND_PAYLOADS
         else:
-          debug = DEBUGLEVEL_DUMP_REQUESTS
+          debug_level = constants.DEBUGLEVEL_DUMP_REQUESTS
       elif o in ('-?', '--help'):
         _OutputUsageAndExit(command_runner)
       elif o in ('-h', '--header'):
@@ -306,6 +313,8 @@ def main():
         parallel_operations = True
       elif o in ('-q', '--quiet'):
         quiet = True
+      elif o == '-u':
+        user_project = a
       elif o in ('-v', '--version'):
         version = True
       elif o == '--perf-trace-token':
@@ -316,7 +325,7 @@ def main():
         test_exception_traces = True
         # Avoid printing extra warnings to stderr regarding long retries by
         # setting the threshold very high.
-        gslib.util.LONG_RETRY_WARN_SEC = 3600
+        constants.LONG_RETRY_WARN_SEC = 3600
       elif o in ('-o', '--option'):
         (opt_section_name, _, opt_value) = a.partition('=')
         if not opt_section_name:
@@ -328,36 +337,52 @@ def main():
           boto.config.add_section(opt_section)
         boto.config.set(opt_section, opt_name, opt_value)
     metrics.LogCommandParams(global_opts=opts)
-    httplib2.debuglevel = debug
+    httplib2.debuglevel = debug_level
     if trace_token:
       sys.stderr.write(TRACE_WARNING)
-    if debug >= DEBUGLEVEL_DUMP_REQUESTS:
+    if debug_level >= constants.DEBUGLEVEL_DUMP_REQUESTS:
       sys.stderr.write(DEBUG_WARNING)
-      _ConfigureLogging(level=logging.DEBUG)
+      _ConfigureRootLogger(level=logging.DEBUG)
       command_runner.RunNamedCommand('ver', ['-l'])
+
       config_items = []
-      try:
-        config_items.extend(boto.config.items('Boto'))
-        config_items.extend(boto.config.items('GSUtil'))
-      except ConfigParser.NoSectionError:
-        pass
+      for config_section in ('Boto', 'GSUtil'):
+        try:
+          config_items.extend(boto.config.items(config_section))
+        except ConfigParser.NoSectionError:
+          pass
       for i in xrange(len(config_items)):
         config_item_key = config_items[i][0]
         if config_item_key in CONFIG_KEYS_TO_REDACT:
           config_items[i] = (config_item_key, 'REDACTED')
       sys.stderr.write('Command being run: %s\n' % ' '.join(sys.argv))
-      sys.stderr.write('config_file_list: %s\n' % GetBotoConfigFileList())
+      sys.stderr.write(
+          'config_file_list: %s\n' % boto_util.GetBotoConfigFileList())
       sys.stderr.write('config: %s\n' % str(config_items))
-    elif quiet:
-      _ConfigureLogging(level=logging.WARNING)
-    else:
-      _ConfigureLogging(level=logging.INFO)
-      # oauth2client uses info logging in places that would better
+    else:  # Non-debug log level.
+      root_logger_level = logging.WARNING if quiet else logging.INFO
+      # oauth2client uses INFO and WARNING logging in places that would better
       # correspond to gsutil's debug logging (e.g., when refreshing
-      # access tokens).
-      oauth2client.client.logger.setLevel(logging.WARNING)
+      # access tokens), so we bump the threshold one level higher where
+      # appropriate. These log levels work for regular- and quiet-level logging.
+      oa2c_logger_level = logging.WARNING
+      oa2c_multiprocess_file_storage_logger_level = logging.ERROR
 
-    if not CERTIFICATE_VALIDATION_ENABLED:
+      _ConfigureRootLogger(level=root_logger_level)
+      oauth2client.client.logger.setLevel(oa2c_logger_level)
+      oauth2client.contrib.multiprocess_file_storage.logger.setLevel(
+          oa2c_multiprocess_file_storage_logger_level)
+      # pylint: disable=protected-access
+      oauth2client.transport._LOGGER.setLevel(oa2c_logger_level)
+      reauth_creds._LOGGER.setLevel(oa2c_logger_level)
+      # pylint: enable=protected-access
+
+    # TODO(reauth): Fix once reauth pins to pyu2f version newer than 0.1.3.
+    # Fixes pyu2f v0.1.3 bug.
+    import six  # pylint: disable=g-import-not-at-top
+    six.input = six.moves.input
+
+    if not boto_util.CERTIFICATE_VALIDATION_ENABLED:
       sys.stderr.write(HTTP_WARNING)
 
     if version:
@@ -369,32 +394,18 @@ def main():
 
     _CheckAndWarnForProxyDifferences()
 
-    if not test_exception_traces:
-      # Disable warning for tests, as it interferes with test stderr parsing.
-      _CheckAndWarnForPython26()
-
     if os.environ.get('_ARGCOMPLETE', '0') == '1':
       return _PerformTabCompletion(command_runner)
 
     return _RunNamedCommandAndHandleExceptions(
         command_runner, command_name, args=args[1:], headers=headers,
-        debug_level=debug, trace_token=trace_token,
+        debug_level=debug_level, trace_token=trace_token,
         parallel_operations=parallel_operations,
-        perf_trace_token=perf_trace_token)
+        perf_trace_token=perf_trace_token, user_project=user_project)
   finally:
     _Cleanup()
 
 
-def _CheckAndWarnForPython26():
-  if (2, 6) == sys.version_info[:2]:
-    sys.stderr.write('\n'.join(textwrap.wrap(
-        'Warning: You are running Python 2.6, which stopped receiving '
-        'security patches as of October 2013. gsutil will stop supporting '
-        'Python 2.6 on September 1, 2016. Please update your Python '
-        'installation to 2.7 to ensure compatibility with future gsutil '
-        'versions.\n')))
-
-
 def _CheckAndWarnForProxyDifferences():
   # If there are both boto config and environment variable config present for
   # proxies, unset the environment variable and warn if it differs.
@@ -403,7 +414,7 @@ def _CheckAndWarnForProxyDifferences():
     for proxy_env_var in ['http_proxy', 'https_proxy', 'HTTPS_PROXY']:
       if proxy_env_var in os.environ and os.environ[proxy_env_var]:
         differing_values = []
-        proxy_info = ProxyInfoFromEnvironmentVar(proxy_env_var)
+        proxy_info = boto_util.ProxyInfoFromEnvironmentVar(proxy_env_var)
         if proxy_info.proxy_host != boto.config.get('Boto', 'proxy', None):
           differing_values.append(
               'Boto proxy host: "%s" differs from %s proxy host: "%s"' %
@@ -462,7 +473,7 @@ def _HandleControlC(signal_num, cur_stack_frame):
     signal_num: Signal that was caught.
     cur_stack_frame: Unused.
   """
-  if debug >= 2:
+  if debug_level >= 2:
     stack_trace = ''.join(traceback.format_list(traceback.extract_stack()))
     _OutputAndExit(
         'DEBUG: Caught CTRL-C (signal %d) - Exception stack trace:\n'
@@ -519,9 +530,7 @@ def _CheckAndHandleCredentialException(e, args):
   # Provide detail to users who have no boto config file (who might previously
   # have been using gsutil only for accessing publicly readable buckets and
   # objects).
-  # pylint: disable=g-import-not-at-top
-  from gslib.util import HasConfiguredCredentials
-  if (not HasConfiguredCredentials() and
+  if (not boto_util.HasConfiguredCredentials() and
       not boto.config.get_value('Tests', 'bypass_anonymous_access_warning',
                                 False)):
     # The check above allows tests to assert that we get a particular,
@@ -529,7 +538,7 @@ def _CheckAndHandleCredentialException(e, args):
     # when there are no configured credentials. This allows tests to
     # simulate a second user without permissions, without actually requiring
     # two separate configured users.
-    if os.environ.get('CLOUDSDK_WRAPPER') == '1':
+    if system_util.InvokedViaCloudSdk():
       message = '\n'.join(textwrap.wrap(
           'You are attempting to access protected data with no configured '
           'credentials. Please visit '
@@ -555,26 +564,27 @@ def _CheckAndHandleCredentialException(e, args):
 
 def _RunNamedCommandAndHandleExceptions(
     command_runner, command_name, args=None, headers=None, debug_level=0,
-    trace_token=None, parallel_operations=False, perf_trace_token=None):
+    trace_token=None, parallel_operations=False, perf_trace_token=None,
+    user_project=None):
   """Runs the command and handles common exceptions."""
+  # Note that this method is run at the end of main() and thus has access to
+  # all of the modules imported there.
   # pylint: disable=g-import-not-at-top
-  from gslib.util import GetConfigFilePaths
-  from gslib.util import IS_WINDOWS
-  from gslib.util import IsRunningInteractively
   try:
     # Catch ^C so we can print a brief message instead of the normal Python
     # stack trace. Register as a final signal handler because this handler kills
     # the main gsutil process (so it must run last).
     RegisterSignalHandler(signal.SIGINT, _HandleControlC, is_final_handler=True)
     # Catch ^\ so we can force a breakpoint in a running gsutil.
-    if not IS_WINDOWS:
+    if not system_util.IS_WINDOWS:
       RegisterSignalHandler(signal.SIGQUIT, _HandleSigQuit)
 
     return command_runner.RunNamedCommand(command_name, args, headers,
                                           debug_level, trace_token,
                                           parallel_operations,
                                           perf_trace_token=perf_trace_token,
-                                          collect_analytics=True)
+                                          collect_analytics=True,
+                                          user_project=user_project)
   except AttributeError as e:
     if str(e).find('secret_access_key') != -1:
       _OutputAndExit(
@@ -596,8 +606,9 @@ def _RunNamedCommandAndHandleExceptions(
   except OSError as e:
     _OutputAndExit(message='OSError: %s.' % e.strerror, exception=e)
   except IOError as e:
-    if (e.errno == errno.EPIPE or (IS_WINDOWS and e.errno == errno.EINVAL)
-        and not IsRunningInteractively()):
+    if (e.errno == errno.EPIPE
+        or (system_util.IS_WINDOWS and e.errno == errno.EINVAL)
+        and not system_util.IsRunningInteractively()):
       # If we get a pipe error, this just means that the pipe to stdout or
       # stderr is broken. This can happen if the user pipes gsutil to a command
       # that doesn't use the entire output stream. Instead of raising an error,
@@ -626,7 +637,7 @@ def _RunNamedCommandAndHandleExceptions(
   except ServiceException as e:
     _OutputAndExit(message=e, exception=e)
   except oauth2client.client.HttpAccessTokenRefreshError as e:
-    if os.environ.get('CLOUDSDK_WRAPPER') == '1':
+    if system_util.InvokedViaCloudSdk():
       _OutputAndExit('Your credentials are invalid. '
                      'Please run\n$ gcloud auth login',
                      exception=e)
@@ -673,8 +684,8 @@ def _RunNamedCommandAndHandleExceptions(
         'pasted the ENTIRE authorization code (including any numeric prefix '
         "e.g. '4/')." % e)), exception=e)
   except Exception as e:  # pylint: disable=broad-except
-    if GetConfigFilePaths():
-      config_paths = ', '.join(GetConfigFilePaths())
+    if boto_util.GetConfigFilePaths():
+      config_paths = ', '.join(boto_util.GetConfigFilePaths())
     else:
       config_paths = 'no config found'
     # Check for two types of errors related to service accounts. These errors
@@ -711,8 +722,7 @@ def _PerformTabCompletion(command_runner):
         ' not found.\nCause: %s' % e,
         exception=e)
   parser = argparse.ArgumentParser(add_help=False)
-  subparsers = parser.add_subparsers()
-  command_runner.ConfigureCommandArgumentParsers(subparsers)
+  command_runner.ConfigureCommandArgumentParsers(parser)
   argcomplete.autocomplete(parser, exit_method=sys.exit)
 
   return 0
