@@ -28,7 +28,7 @@ class CloudApi(object):
 
   def __init__(self, bucket_storage_uri_class, logger, status_queue,
                provider=None, debug=0, trace_token=None,
-               perf_trace_token=None):
+               perf_trace_token=None, user_project=None):
     """Performs necessary setup for interacting with the cloud storage provider.
 
     Args:
@@ -42,6 +42,7 @@ class CloudApi(object):
       trace_token: Google internal trace token to pass to the API
                    implementation (string).
       perf_trace_token: Performance trace token to use when making API calls.
+      user_project: Project to be billed for this request.
     """
     self.bucket_storage_uri_class = bucket_storage_uri_class
     self.logger = logger
@@ -50,6 +51,7 @@ class CloudApi(object):
     self.debug = debug
     self.trace_token = trace_token
     self.perf_trace_token = perf_trace_token
+    self.user_project = user_project
 
   def GetBucket(self, bucket_name, provider=None, fields=None):
     """Gets Bucket metadata.
@@ -156,6 +158,23 @@ class CloudApi(object):
     """
     raise NotImplementedError('PatchBucket must be overloaded')
 
+  def LockRetentionPolicy(self, bucket_name, metageneration, provider=None):
+    """Locks the Retention Policy on the bucket.
+
+    Args:
+      bucket_name: Name of bucket to update.
+      metageneration: Bucket metageneration to use as a precondition.
+      provider: Cloud storage provider to connect to. If not present,
+                class-wide default is used.
+
+    Raises:
+      ServiceException for errors interacting with cloud storage providers.
+
+    Returns:
+      None
+    """
+    raise NotImplementedError('LockRetentionPolicy must be overloaded')
+
   def CreateBucket(self, bucket_name, project_id=None, metadata=None,
                    provider=None, fields=None):
     """Creates a new bucket with the specified metadata.
@@ -341,6 +360,8 @@ class CloudApi(object):
     ONE_SHOT = 'oneshot'
     RESUMABLE = 'resumable'
 
+  # TODO: Change {de,en}cryption_tuple field names, as they don't actually
+  # accept tuples.
   def GetObjectMedia(self, bucket_name, object_name, download_stream,
                      provider=None, generation=None, object_size=None,
                      compressed_encoding=False,
@@ -375,8 +396,8 @@ class CloudApi(object):
                  update(bytes) and digest() using that algorithm.
                  Implementation can set the digester value to None to indicate
                  bytes were not successfully digested on-the-fly.
-      decryption_tuple: Optional CryptoTuple for decrypting an encrypted
-          object.
+      decryption_tuple: Optional utils.encryption_helper.CryptoKeyWrapper for
+                        decrypting an encrypted object.
 
     Raises:
       ArgumentException for errors during input validation.
@@ -390,7 +411,8 @@ class CloudApi(object):
 
   def UploadObject(self, upload_stream, object_metadata, canned_acl=None,
                    size=None, preconditions=None, progress_callback=None,
-                   encryption_tuple=None, provider=None, fields=None):
+                   encryption_tuple=None, provider=None, fields=None,
+                   gzip_encoded=False):
     """Uploads object data and metadata.
 
     Args:
@@ -404,11 +426,12 @@ class CloudApi(object):
       progress_callback: Optional callback function for progress notifications.
                          Receives calls with arguments
                          (bytes_transferred, total_size).
-      encryption_tuple: Optional CryptoTuple for encrypting the uploaded
-          object.
+      encryption_tuple: Optional utils.encryption_helper.CryptoKeyWrapper for
+                        encrypting the uploaded object.
       provider: Cloud storage provider to connect to.  If not present,
                 class-wide default is used.
       fields: If present, return only these Object metadata fields.
+      gzip_encoded: Whether to use gzip transport encoding for the upload.
 
     Raises:
       ArgumentException for errors during input validation.
@@ -422,7 +445,7 @@ class CloudApi(object):
   def UploadObjectStreaming(self, upload_stream, object_metadata,
                             canned_acl=None, preconditions=None,
                             progress_callback=None, encryption_tuple=None,
-                            provider=None, fields=None):
+                            provider=None, fields=None, gzip_encoded=False):
     """Uploads object data and metadata.
 
     Args:
@@ -436,11 +459,12 @@ class CloudApi(object):
                          Receives calls with arguments
                          (bytes_transferred, total_size), but fills in only
                          bytes_transferred.
-      encryption_tuple: Optional CryptoTuple for encrypting the uploaded
-          object.
+      encryption_tuple: Optional utils.encryption_helper.CryptoKeyWrapper for
+                        encrypting the uploaded object.
       provider: Cloud storage provider to connect to.  If not present,
                 class-wide default is used.
       fields: If present, return only these Object metadata fields.
+      gzip_encoded: Whether to use gzip transport encoding for the upload.
 
     Raises:
       ArgumentException for errors during input validation.
@@ -455,7 +479,7 @@ class CloudApi(object):
       self, upload_stream, object_metadata, canned_acl=None,
       size=None, preconditions=None, serialization_data=None,
       tracker_callback=None, progress_callback=None, encryption_tuple=None,
-      provider=None, fields=None):
+      provider=None, fields=None, gzip_encoded=False):
     """Uploads object data and metadata using a resumable upload strategy.
 
     Args:
@@ -476,12 +500,13 @@ class CloudApi(object):
       progress_callback: Optional callback function for progress notifications.
                          Receives calls with arguments
                          (bytes_transferred, total_size).
-      encryption_tuple: Optional CryptoTuple for encrypting the uploaded
-          object.
+      encryption_tuple: Optional utils.encryption_helper.CryptoKeyWrapper for
+                        encrypting the uploaded object.
       provider: Cloud storage provider to connect to.  If not present,
                 class-wide default is used.
       fields: If present, return only these Object metadata fields when the
               upload is complete.
+      gzip_encoded: Whether to use gzip transport encoding for the upload.
 
     Raises:
       ArgumentException for errors during input validation.
@@ -492,6 +517,8 @@ class CloudApi(object):
     """
     raise NotImplementedError('UploadObjectResumable must be overloaded')
 
+  # TODO: Change {de,en}cryption_tuple field names, as they don't actually
+  # accept tuples.
   def CopyObject(self, src_obj_metadata, dst_obj_metadata, src_generation=None,
                  canned_acl=None, preconditions=None, progress_callback=None,
                  max_bytes_per_call=None, encryption_tuple=None,
@@ -512,11 +539,12 @@ class CloudApi(object):
                          (bytes_transferred, total_size).
       max_bytes_per_call: Integer describing maximum number of bytes
                           to rewrite per service call.
-      encryption_tuple: Optional CryptoTuple for encrypting the destination
-          object.
-      decryption_tuple: Optional CryptoTuple for decrypting the source
-          object. If supplied without encryption_tuple, destination object will
-          be written without customer-supplied encryption.
+      encryption_tuple: Optional utils.encryption_helper.CryptoKeyWrapper for
+                        encrypting the destination object.
+      decryption_tuple: Optional utils.encryption_helper.CryptoKeyWrapper for
+                        decrypting the source object. If supplied without
+                        encryption_tuple, destination object will be written
+                        without customer-supplied encryption.
       provider: Cloud storage provider to connect to.  If not present,
                 class-wide default is used.
       fields: If present, return only these Object metadata fields.
@@ -541,8 +569,9 @@ class CloudApi(object):
       dst_obj_metadata: Metadata for the destination object including bucket
                         and object name.
       preconditions: Destination object preconditions for the request.
-      encryption_tuple: Optional CryptoTuple for decrypting source objects
-          and encrypting the destination object.
+      encryption_tuple: Optional utils.encryption_helper.CryptoKeyWrapper for
+                        decrypting source objects and encrypting the destination
+                        object.
       provider: Cloud storage provider to connect to.  If not present,
                 class-wide default is used.
       fields: If present, return only these Object metadata fields.
@@ -618,6 +647,21 @@ class CloudApi(object):
     """
     raise NotImplementedError('StopChannel must be overloaded')
 
+  def ListChannels(self, bucket_name, provider=None):
+    """Lists object change notifications for a bucket.
+
+    Args:
+      bucket_name: Bucket containing the objects
+
+    Raises:
+      ArgumentException for errors during input validation.
+      ServiceException for errors interacting with cloud storage providers.
+
+    Returns:
+      None.
+    """
+    raise NotImplementedError('ListChannels must be overloaded')
+
   def GetProjectServiceAccount(self, project_number, provider=None):
     """Get the GCS-owned service account representing this project.
 
@@ -707,19 +751,6 @@ class CloudApi(object):
     raise NotImplementedError('ListNotificationConfig must be overloaded')
 
 
-class CryptoTuple(object):
-  """Class describing an encryption/decryption key for cloud API requests."""
-
-  def __init__(self, crypto_key):
-    """Initialize the CryptoTuple.
-
-    Args:
-      crypto_key: Base64-encoded string of encryption key.
-    """
-    self.crypto_key = crypto_key
-    self.crypto_alg = 'AES256'  # Only currently supported encryption algorithm.
-
-
 class Preconditions(object):
   """Preconditions class for specifying preconditions to cloud API requests."""
 
