@@ -38,24 +38,23 @@ import gslib.commands
 from gslib.cs_api_map import ApiSelector
 from gslib.cs_api_map import GsutilApiClassMapFactory
 from gslib.cs_api_map import GsutilApiMapFactory
+from gslib.discard_messages_queue import DiscardMessagesQueue
 from gslib.exception import CommandException
 from gslib.gcs_json_api import GcsJsonApi
 from gslib.no_op_credentials import NoOpCredentials
 from gslib.tab_complete import MakeCompleter
-from gslib.util import CheckMultiprocessingAvailableAndInit
-from gslib.util import CompareVersions
-from gslib.util import DiscardMessagesQueue
-from gslib.util import GetGsutilVersionModifiedTime
-from gslib.util import GSUTIL_PUB_TARBALL
-from gslib.util import InsistAsciiHeader
-from gslib.util import InsistAsciiHeaderValue
-from gslib.util import IsCustomMetadataHeader
-from gslib.util import IsRunningInteractively
-from gslib.util import LAST_CHECKED_FOR_GSUTIL_UPDATE_TIMESTAMP_FILE
-from gslib.util import LookUpGsutilVersion
-from gslib.util import RELEASE_NOTES_URL
-from gslib.util import SECONDS_PER_DAY
-from gslib.util import UTF8
+from gslib.utils import boto_util
+from gslib.utils import system_util
+from gslib.utils.constants import GSUTIL_PUB_TARBALL
+from gslib.utils.constants import RELEASE_NOTES_URL
+from gslib.utils.constants import UTF8
+from gslib.utils.metadata_util import IsCustomMetadataHeader
+from gslib.utils.parallelism_framework_util import CheckMultiprocessingAvailableAndInit
+from gslib.utils.text_util import CompareVersions
+from gslib.utils.text_util import InsistAsciiHeader
+from gslib.utils.text_util import InsistAsciiHeaderValue
+from gslib.utils.unit_util import SECONDS_PER_DAY
+from gslib.utils.update_util import LookUpGsutilVersion
 
 
 def HandleHeaderCoding(headers):
@@ -160,31 +159,62 @@ class CommandRunner(object):
         command_map[command_name_aliases] = command
     return command_map
 
+  def _GetTabCompleteLogger(self):
+    """Returns a logger for tab completion."""
+    return CreateGsutilLogger('tab_complete')
+
   def _ConfigureCommandArgumentParserArguments(
-      self, parser, arguments, gsutil_api):
-    """Configures an argument parser with the given arguments.
+      self, parser, subcommands_or_arguments, gsutil_api):
+    """Creates parsers recursively for potentially nested subcommands.
 
     Args:
       parser: argparse parser object.
-      arguments: array of CommandArgument objects.
+      subcommands_or_arguments: list of CommandArgument objects, or recursive
+          dict with subcommand names as keys.
       gsutil_api: gsutil Cloud API instance to use.
+
     Raises:
       RuntimeError: if argument is configured with unsupported completer
+      TypeError: if subcommands_or_arguments is not a dict or list
+
     """
-    for command_argument in arguments:
-      action = parser.add_argument(
-          *command_argument.args, **command_argument.kwargs)
-      if command_argument.completer:
-        action.completer = MakeCompleter(command_argument.completer, gsutil_api)
+    logger = self._GetTabCompleteLogger()
 
-  def ConfigureCommandArgumentParsers(self, subparsers):
-    """Configures argparse arguments and argcomplete completers for commands.
+    def HandleList():
+      for command_argument in subcommands_or_arguments:
+        action = parser.add_argument(
+            *command_argument.args, **command_argument.kwargs)
+        if command_argument.completer:
+          action.completer = MakeCompleter(
+              command_argument.completer, gsutil_api)
 
-    Args:
-      subparsers: argparse object that can be used to add parsers for
-                  subcommands (called just 'commands' in gsutil)
+    def HandleDict():
+      subparsers = parser.add_subparsers()
+      for subcommand_name, subcommand_value in subcommands_or_arguments.items():
+        cur_subcommand_parser = subparsers.add_parser(
+            subcommand_name, add_help=False)
+        logger.info(
+            'Constructing argument parsers for {}'.format(subcommand_name))
+        self._ConfigureCommandArgumentParserArguments(
+            cur_subcommand_parser, subcommand_value, gsutil_api)
+
+    if isinstance(subcommands_or_arguments, list):
+      HandleList()
+    elif isinstance(subcommands_or_arguments, dict):
+      HandleDict()
+    else:
+      error_format = ('subcommands_or_arguments {} should be list or dict, '
+                      'found type {}')
+      raise TypeError(
+          error_format.format(
+              subcommands_or_arguments, type(subcommands_or_arguments)))
+
+  def GetGsutilApiForTabComplete(self):
+    """Builds and returns a gsutil_api based off gsutil_api_class_map_factory.
+
+    Returns:
+      the gsutil_api instance
     """
-
     # This should match the support map for the "ls" command.
     support_map = {
         'gs': [ApiSelector.XML, ApiSelector.JSON],
@@ -197,30 +227,54 @@ class CommandRunner(object):
     gsutil_api_map = GsutilApiMapFactory.GetApiMap(
         self.gsutil_api_class_map_factory, support_map, default_map)
 
-    logger = CreateGsutilLogger('tab_complete')
     gsutil_api = CloudApiDelegator(
         self.bucket_storage_uri_class, gsutil_api_map,
-        logger, DiscardMessagesQueue(), debug=0)
+        self._GetTabCompleteLogger(), DiscardMessagesQueue(), debug=0)
+    return gsutil_api
 
-    for command in set(self.command_map.values()):
-      command_parser = subparsers.add_parser(
-          command.command_spec.command_name, add_help=False)
-      if isinstance(command.command_spec.argparse_arguments, dict):
-        subcommand_parsers = command_parser.add_subparsers()
-        subcommand_argument_dict = command.command_spec.argparse_arguments
-        for subcommand, arguments in subcommand_argument_dict.iteritems():
-          subcommand_parser = subcommand_parsers.add_parser(
-              subcommand, add_help=False)
-          self._ConfigureCommandArgumentParserArguments(
-              subcommand_parser, arguments, gsutil_api)
-      else:
-        self._ConfigureCommandArgumentParserArguments(
-            command_parser, command.command_spec.argparse_arguments, gsutil_api)
+  def ConfigureCommandArgumentParsers(self, main_parser):
+    """Configures argparse arguments and argcomplete completers for commands.
+
+    Args:
+      main_parser: argparse object that can be called to get subparsers to add
+      subcommands (called just 'commands' in gsutil)
+    """
+    gsutil_api = self.GetGsutilApiForTabComplete()
+
+    # build a dict mapping from command name to the argparse arguments.
+    # this dict has values with either a recursive dictionary or a list of
+    # CommandArgument objects.
+    command_to_argparse_arguments = {
+        command.command_spec.command_name:
+            command.command_spec.argparse_arguments for command in
+        self.command_map.values()}
+
+    # At this point command_to_argparse_arguments looks like
+    # {
+    #   'retention': {
+    #     'set' : `set arguments array`
+    #     'get' : `set arguments array`
+    #     ...
+    #     'event': {
+    #       'set': `event set arguments array`
+    #       'release': `event release arguments array`
+    #     }
+    #   },
+    #   ... other commands here
+    # }
+    #
+    # Which will be passed into the helper and called recursively on the items
+    # in the dict, with the base case being the arguments arrays, where the
+    # arguments are added to the subparser for the lowest level command.
+
+    self._ConfigureCommandArgumentParserArguments(
+        main_parser, command_to_argparse_arguments, gsutil_api)
 
   def RunNamedCommand(self, command_name, args=None, headers=None, debug=0,
                       trace_token=None, parallel_operations=False,
                       skip_update_check=False, logging_filters=None,
                       do_shutdown=True, perf_trace_token=None,
+                      user_project=None,
                       collect_analytics=False):
     """Runs the named command.
 
@@ -239,6 +293,7 @@ class CommandRunner(object):
       do_shutdown: Stop all parallelism framework workers iff this is True.
       perf_trace_token: Performance measurement trace token to pass to the
           underlying API.
+      user_project: The project to bill this request to.
       collect_analytics: Set to True to collect an analytics metric logging this
           command.
 
@@ -256,7 +311,7 @@ class CommandRunner(object):
       args = ['-n']
 
       # Check for opt-in analytics.
-      if IsRunningInteractively() and collect_analytics:
+      if system_util.IsRunningInteractively() and collect_analytics:
         metrics.CheckAndMaybePromptForAnalyticsEnabling()
 
     if not args:
@@ -303,7 +358,7 @@ class CommandRunner(object):
         self, args, headers, debug, trace_token, parallel_operations,
         self.bucket_storage_uri_class, self.gsutil_api_class_map_factory,
         logging_filters, command_alias_used=command_name,
-        perf_trace_token=perf_trace_token)
+        perf_trace_token=perf_trace_token, user_project=user_project)
 
     # Log the command name, command alias, and sub-options after being parsed by
     # RunCommand and the command constructor. For commands with subcommands and
@@ -362,11 +417,13 @@ class CommandRunner(object):
     #   gcloud components update)
     logger = logging.getLogger()
     gs_host = boto.config.get('Credentials', 'gs_host', None)
-    if (not IsRunningInteractively()
+    gs_host_is_not_default = (
+        gs_host != boto.gs.connection.GSConnection.DefaultHost)
+    if (not system_util.IsRunningInteractively()
         or command_name in ('config', 'update', 'ver', 'version')
         or not logger.isEnabledFor(logging.INFO)
-        or gs_host
-        or os.environ.get('CLOUDSDK_WRAPPER') == '1'):
+        or gs_host_is_not_default
+        or system_util.InvokedViaCloudSdk()):
       return False
 
     software_update_check_period = boto.config.getint(
@@ -376,17 +433,20 @@ class CommandRunner(object):
     if software_update_check_period == 0:
       return False
 
+    last_checked_for_gsutil_update_timestamp_file = (
+        boto_util.GetLastCheckedForGsutilUpdateTimestampFile())
+
     cur_ts = int(time.time())
-    if not os.path.isfile(LAST_CHECKED_FOR_GSUTIL_UPDATE_TIMESTAMP_FILE):
+    if not os.path.isfile(last_checked_for_gsutil_update_timestamp_file):
       # Set last_checked_ts from date of VERSION file, so if the user installed
       # an old copy of gsutil it will get noticed (and an update offered) the
       # first time they try to run it.
-      last_checked_ts = GetGsutilVersionModifiedTime()
-      with open(LAST_CHECKED_FOR_GSUTIL_UPDATE_TIMESTAMP_FILE, 'w') as f:
+      last_checked_ts = gslib.GetGsutilVersionModifiedTime()
+      with open(last_checked_for_gsutil_update_timestamp_file, 'w') as f:
         f.write(str(last_checked_ts))
     else:
       try:
-        with open(LAST_CHECKED_FOR_GSUTIL_UPDATE_TIMESTAMP_FILE, 'r') as f:
+        with open(last_checked_for_gsutil_update_timestamp_file, 'r') as f:
           last_checked_ts = int(f.readline())
       except (TypeError, ValueError):
         return False
@@ -400,7 +460,7 @@ class CommandRunner(object):
                               credentials=NoOpCredentials(), debug=debug)
 
       cur_ver = LookUpGsutilVersion(gsutil_api, GSUTIL_PUB_TARBALL)
-      with open(LAST_CHECKED_FOR_GSUTIL_UPDATE_TIMESTAMP_FILE, 'w') as f:
+      with open(last_checked_for_gsutil_update_timestamp_file, 'w') as f:
         f.write(str(cur_ts))
       (g, m) = CompareVersions(cur_ver, gslib.VERSION)
       if m:
