@@ -32,6 +32,8 @@ import webbrowser
 
 import boto
 from boto.provider import Provider
+from httplib2 import ServerNotFoundError
+from oauth2client.client import HAS_CRYPTO
 
 import gslib
 from gslib.command import Command
@@ -40,18 +42,14 @@ from gslib.commands.compose import MAX_COMPONENT_COUNT
 from gslib.cred_types import CredTypes
 from gslib.exception import AbortException
 from gslib.exception import CommandException
-from gslib.hashing_helper import CHECK_HASH_ALWAYS
-from gslib.hashing_helper import CHECK_HASH_IF_FAST_ELSE_FAIL
-from gslib.hashing_helper import CHECK_HASH_IF_FAST_ELSE_SKIP
-from gslib.hashing_helper import CHECK_HASH_NEVER
 from gslib.metrics import CheckAndMaybePromptForAnalyticsEnabling
 from gslib.sig_handling import RegisterSignalHandler
-from gslib.util import IS_WINDOWS
-from gslib.util import RESUMABLE_THRESHOLD_B
-
-from httplib2 import ServerNotFoundError
-from oauth2client.client import HAS_CRYPTO
-
+from gslib.utils import constants
+from gslib.utils import system_util
+from gslib.utils.hashing_helper import CHECK_HASH_ALWAYS
+from gslib.utils.hashing_helper import CHECK_HASH_IF_FAST_ELSE_FAIL
+from gslib.utils.hashing_helper import CHECK_HASH_IF_FAST_ELSE_SKIP
+from gslib.utils.hashing_helper import CHECK_HASH_NEVER
 
 _SYNOPSIS = """
   gsutil [-D] config [-a] [-b] [-e] [-f] [-n] [-o <file>] [-r] [-s <scope>] [-w]
@@ -63,7 +61,16 @@ _DETAILED_HELP_TEXT = ("""
 
 
 <B>DESCRIPTION</B>
-  The gsutil config command obtains access credentials for Google Cloud
+  The ``gsutil config`` command applies to users who have installed gsutil as a
+  standalone tool. If you installed gsutil via the Cloud SDK, ``gsutil config``
+  will fail unless you are specifically using the ``-a`` flag or have configured
+  gcloud to not pass its managed credentials to gsutil (via the command ``gcloud
+  config set pass_credentials_to_gsutil false``). For all other use cases, Cloud
+  SDK users should use the ``gcloud auth`` group of commands instead, which will
+  configure OAuth2 credentials that gcloud implicitly passes to gsutil at
+  runtime.
+
+  The ``gsutil config`` command obtains access credentials for Google Cloud
   Storage and writes a boto/gsutil configuration file containing the obtained
   credentials along with a number of other configuration-controllable values.
 
@@ -71,41 +78,67 @@ _DETAILED_HELP_TEXT = ("""
   to ~/.boto (i.e., the file .boto under the user's home directory). If the
   default file already exists, an attempt is made to rename the existing file
   to ~/.boto.bak; if that attempt fails the command will exit. A different
-  destination file can be specified with the -o option (see OPTIONS).
+  destination file can be specified with the ``-o`` option (see OPTIONS).
 
   Because the boto configuration file contains your credentials you should
   keep its file permissions set so no one but you has read access. (The file
-  is created read-only when you run gsutil config.)
+  is created read-only when you run ``gsutil config``.)
 
 
 <B>CREDENTIALS</B>
-  By default gsutil config obtains OAuth2 credentials, and writes them
-  to the [Credentials] section of the configuration file. The -r, -w,
-  -f options (see OPTIONS below) cause gsutil config to request a token
-  with restricted scope; the resulting token will be restricted to read-only
-  operations, read-write operations, or all operations (including acl get/set,
-  defacl get/set, and logging get/'set on'/'set off' operations). In
-  addition, -s <scope> can be used to request additional (non-Google-Storage)
-  scopes.
+  By default ``gsutil config`` obtains OAuth2 credentials and writes them to the
+  [Credentials] section of the configuration file. Unless otherwise specified,
+  it requests a token allowing full control of resources in several services,
+  e.g. Cloud Storage, Cloud KMS (used for the 'kms' command), and Cloud Pub/Sub
+  (used for the 'notification' command). To request a token with more limited
+  scopes, you can specify additional options (see the OPTIONS section below for
+  the full list). Some examples include:
+
+  Create a token with read-only access for storage resources:
+
+    gsutil config -r
+
+  Create a token with read-write access for storage resources:
+
+    gsutil config -w
+
+  Create a token with full-control access for storage resources:
+
+    gsutil config -f
+
+  In addition, ``-s <scope>`` can be specified multiple times to request
+  additional scopes, where ``<scope>`` is specified using the full URL of the
+  desired scope as listed on
+  https://developers.google.com/identity/protocols/googlescopes.
 
   If you want to use credentials based on access key and secret (the older
   authentication method before OAuth2 was supported) instead of OAuth2,
-  see help about the -a option in the OPTIONS section.
+  see help about the ``-a`` option in the OPTIONS section.
 
   If you wish to use gsutil with other providers (or to copy data back and
   forth between multiple providers) you can edit their credentials into the
-  [Credentials] section after creating the initial configuration file.
+  [Credentials] section after creating the initial configuration file. See the
+  list of settings below for supported settings.
 
 
 <B>CONFIGURING SERVICE ACCOUNT CREDENTIALS</B>
-  You can configure credentials for service accounts using the gsutil config -e
-  option. Service accounts are useful for authenticating on behalf of a service
-  or application (as opposed to a user).
+  Service accounts are useful for authenticating on behalf of a service or
+  application (as opposed to a user). You can configure credentials for service
+  accounts using the ``-e`` option:
 
-  When you run gsutil config -e, you will be prompted for your service account
-  email address and the path to your private key file. To get this data,
-  follow the instructions on
-  `Service Accounts <https://cloud.google.com/storage/docs/authentication#generating-a-private-key>`_.
+    gsutil config -e
+
+  Note that if you are using gsutil through the Cloud SDK, you should instead
+  activate your service account via the ``gcloud auth activate-service-account``
+  command.
+
+  When you run ``gsutil config -e``, you will be prompted for the path to your
+  private key file and, if not using a JSON key file, your service account
+  email address and key file password. To get this data, follow the instructions
+  on `Service Accounts <https://cloud.google.com/storage/docs/authentication#generating-a-private-key>`_.
+  Using this information, gsutil populates the "gs_service_key_file" attribute,
+  along with "gs_service_client_id" and "gs_service_key_file_password" if not
+  using a JSON key file.
 
   Note that your service account will NOT be considered an Owner for the
   purposes of API access (see "gsutil help creds" for more information about
@@ -141,7 +174,7 @@ _DETAILED_HELP_TEXT = ("""
   [Boto], [GSUtil], and [OAuth2]. If you edit the file, make sure to edit the
   appropriate section (discussed below), and to be careful not to mis-edit
   any of the setting names (like "gs_access_key_id") and not to remove the
-  section delimiters (like "[Credentials]").
+  section delimiters (like [Credentials]).
 
 
 <B>ADDITIONAL CONFIGURATION-CONTROLLABLE FEATURES</B>
@@ -170,6 +203,9 @@ _DETAILED_HELP_TEXT = ("""
       gs_oauth2_refresh_token
       gs_port
       gs_secret_access_key
+      gs_service_client_id
+      gs_service_key_file
+      gs_service_key_file_password
       s3_host
       s3_port
 
@@ -180,6 +216,7 @@ _DETAILED_HELP_TEXT = ("""
       proxy_pass
       proxy_rdns
       http_socket_timeout
+      ca_certificates_file
       https_validate_certificates
       debug
       max_retry_delay
@@ -197,6 +234,7 @@ _DETAILED_HELP_TEXT = ("""
       disable_analytics_prompt
       encryption_key
       json_api_version
+      max_upload_compression_buffer_size
       parallel_composite_upload_component_size
       parallel_composite_upload_threshold
       sliced_object_download_component_size
@@ -228,16 +266,14 @@ _DETAILED_HELP_TEXT = ("""
 <B>UPDATING TO THE LATEST CONFIGURATION FILE</B>
   We add new configuration controllable features to the boto configuration file
   over time, but most gsutil users create a configuration file once and then
-  keep it for a long time, so new features aren't apparent when you update
-  to a newer version of gsutil. If you want to get the latest configuration
-  file (which includes all the latest settings and documentation about each)
-  you can rename your current file (e.g., to '.boto_old'), run gsutil config,
-  and then edit any configuration settings you wanted from your old file
-  into the newly created file. Note, however, that if you're using OAuth2
-  credentials and you go back through the OAuth2 configuration dialog it will
-  invalidate your previous OAuth2 credentials.
-
-  If no explicit scope option is given, -f (full control) is assumed by default.
+  keep it for a long time, so new features aren't apparent when you update to a
+  newer version of gsutil. If you want to get the latest configuration file
+  (which includes all the latest settings and documentation about each) you can
+  rename your current file (e.g., to '.boto_old'), run ``gsutil config``, and
+  then edit any configuration settings you wanted from your old file into the
+  newly created file. Note, however, that if you're using OAuth2 credentials and
+  you go back through the OAuth2 configuration dialog it will invalidate your
+  previous OAuth2 credentials.
 
 
 <B>OPTIONS</B>
@@ -245,28 +281,36 @@ _DETAILED_HELP_TEXT = ("""
               authentication method before OAuth2 was supported) instead of
               obtaining an OAuth2 token.
 
-  -b          Causes gsutil config to launch a browser to obtain OAuth2 approval
-              and the project ID instead of showing the URL for each and asking
-              the user to open the browser. This will probably not work as
-              expected if you are running gsutil from an ssh window, or using
-              gsutil on Windows.
+  -b          Causes ``gsutil config`` to launch a browser to obtain OAuth2
+              approval and the project ID instead of showing the URL for each
+              and asking the user to open the browser. This will probably not
+              work as expected if you are running gsutil from an ssh window, or
+              using gsutil on Windows.
 
   -e          Prompt for service account credentials. This option requires that
-              -a is not set.
+              ``-a`` is not set.
 
-  -f          Request token with full-control access (default).
+  -f          Request token with full control (devstorage.full_control scope).
+              Note that this does not provide non-storage scopes, such as those
+              needed to edit Pub/Sub and KMS resources (used with the
+              'notification' and 'kms' commands).
 
   -n          Write the configuration file without authentication configured.
-              This flag is mutually exlusive with all flags other than -o.
+              This flag is mutually exlusive with all flags other than ``-o``.
 
   -o <file>   Write the configuration to <file> instead of ~/.boto.
-              Use '-' for stdout.
+              Use ``-`` for stdout.
 
-  -r          Request token restricted to read-only access.
+  -r          Request token with read-only access (devstorage.read_only scope).
 
-  -s <scope>  Request additional OAuth2 <scope>.
+  --reauth    Request token with reauth access (accounts.reauth scope).
 
-  -w          Request token restricted to read-write access.
+  -s <scope>  Request a specific OAuth2 <scope> instead of the default(s). This
+              option may be repeated to request multiple scopes, and may be used
+              in conjuction with other flags that request a specific scope.
+
+  -w          Request token with read-write access
+              (devstorage.read_write scope).
 """)
 
 
@@ -277,11 +321,6 @@ except ImportError:
 
 GOOG_CLOUD_CONSOLE_URI = 'https://cloud.google.com/console#/project'
 
-SCOPE_CLOUD_PLATFORM = 'https://www.googleapis.com/auth/cloud-platform'
-SCOPE_FULL_CONTROL = 'https://www.googleapis.com/auth/devstorage.full_control'
-SCOPE_READ_WRITE = 'https://www.googleapis.com/auth/devstorage.read_write'
-SCOPE_READ_ONLY = 'https://www.googleapis.com/auth/devstorage.read_only'
-
 CONFIG_PRELUDE_CONTENT = """
 # This file contains credentials and other configuration information needed
 # by the boto library, used by gsutil. You can edit this file (e.g., to add
@@ -323,6 +362,12 @@ DEFAULT_SLICED_OBJECT_DOWNLOAD_THRESHOLD = '150M'
 DEFAULT_SLICED_OBJECT_DOWNLOAD_COMPONENT_SIZE = '200M'
 DEFAULT_SLICED_OBJECT_DOWNLOAD_MAX_COMPONENTS = 4
 
+# Compressed transport encoded uploads buffer chunks of compressed data. When
+# running many uploads in parallel, compression may consume more memory than
+# available. This restricts the number of compressed transport encoded uploads
+# running in parallel such that they don't consume more memory than set here.
+DEFAULT_MAX_UPLOAD_COMPRESSION_BUFFER_SIZE = '2G'
+
 CONFIG_BOTO_SECTION_CONTENT = """
 [Boto]
 
@@ -471,19 +516,33 @@ software_update_check_period = 0
 # Note: Parallel composite uploads should not be used with NEARLINE or COLDLINE
 # storage class buckets, as doing this incurs an early deletion charge for
 # each component object.
+#
+# Note: Parallel composite uploads are not enabled with Cloud KMS encrypted
+# objects as a source or destination, as composition with KMS objects is not yet
+# supported.
+
 #parallel_composite_upload_threshold = %(parallel_composite_upload_threshold)s
 #parallel_composite_upload_component_size = %(parallel_composite_upload_component_size)s
 
 # 'sliced_object_download_threshold' and
 # 'sliced_object_download_component_size' have analogous functionality to
 # their respective parallel_composite_upload config values.
-# 'sliced_object_download_max_components' specifies the maximum number of 
+# 'sliced_object_download_max_components' specifies the maximum number of
 # slices to be used when performing a sliced object download. It is not
 # restricted by MAX_COMPONENT_COUNT.
 #sliced_object_download_threshold = %(sliced_object_download_threshold)s
 #sliced_object_download_component_size = %(sliced_object_download_component_size)s
 #sliced_object_download_max_components = %(sliced_object_download_max_components)s
 
+# Compressed transport encoded uploads buffer chunks of compressed data. When
+# running a composite upload and/or many uploads in parallel, compression may
+# consume more memory than available. This setting restricts the number of
+# compressed transport encoded uploads running in parallel such that they
+# don't consume more memory than set here. This is 2GiB by default.
+# Values can be provided either in bytes or as human-readable values
+# (e.g., "2G" to represent 2 gibibytes)
+#max_upload_compression_buffer_size = %(max_upload_compression_buffer_size)s
+
 # 'task_estimation_threshold' controls how many files or objects gsutil
 # processes before it attempts to estimate the total work that will be
 # performed by the command. Estimation makes extra directory listing or API
@@ -492,8 +551,8 @@ software_update_check_period = 0
 # listing calls; to disable it entirely, set this value to 0.
 #task_estimation_threshold=%(task_estimation_threshold)s
 
-# 'use_magicfile' specifies if the 'file --mime-type <filename>' command should
-# be used to guess content types instead of the default filename extension-based
+# 'use_magicfile' specifies if the 'file --mime <filename>' command should be
+# used to guess content types instead of the default filename extension-based
 # mechanism. Available on UNIX and MacOS (and possibly on Windows, if you're
 # running Cygwin or some other package that provides implementations of
 # UNIX-like commands). When available and enabled use_magicfile should be more
@@ -565,7 +624,7 @@ content_language = en
        'hash_fast_else_skip': CHECK_HASH_IF_FAST_ELSE_SKIP,
        'hash_always': CHECK_HASH_ALWAYS,
        'hash_never': CHECK_HASH_NEVER,
-       'resumable_threshold': RESUMABLE_THRESHOLD_B,
+       'resumable_threshold': constants.RESUMABLE_THRESHOLD_B,
        'parallel_process_count': DEFAULT_PARALLEL_PROCESS_COUNT,
        'parallel_thread_count': DEFAULT_PARALLEL_THREAD_COUNT,
        'parallel_composite_upload_threshold': (
@@ -579,7 +638,9 @@ content_language = en
        'sliced_object_download_max_components': (
            DEFAULT_SLICED_OBJECT_DOWNLOAD_MAX_COMPONENTS),
        'max_component_count': MAX_COMPONENT_COUNT,
-       'task_estimation_threshold': DEFAULT_TASK_ESTIMATION_THRESHOLD}
+       'task_estimation_threshold': DEFAULT_TASK_ESTIMATION_THRESHOLD,
+       'max_upload_compression_buffer_size': (
+           DEFAULT_MAX_UPLOAD_COMPRESSION_BUFFER_SIZE)}
 
 CONFIG_OAUTH2_CONFIG_CONTENT = """
 [OAuth2]
@@ -613,7 +674,7 @@ CONFIG_OAUTH2_CONFIG_CONTENT = """
 # authorization provider being used. Primarily useful for tool developers.
 #provider_label = Google
 #provider_authorization_uri = https://accounts.google.com/o/oauth2/auth
-#provider_token_uri = https://accounts.google.com/o/oauth2/token
+#provider_token_uri = https://oauth2.googleapis.com/token
 
 # 'oauth2_refresh_retries' controls the number of retry attempts made when
 # rate limiting errors occur for OAuth2 requests to retrieve an access token.
@@ -639,7 +700,8 @@ class ConfigCommand(Command):
       usage_synopsis=_SYNOPSIS,
       min_args=0,
       max_args=0,
-      supported_sub_args='habefnwrs:o:',
+      supported_sub_args='abefhno:rs:w',
+      supported_private_args=['reauth'],
       file_url_ok=False,
       provider_url_ok=False,
       urls_start_arg=0,
@@ -694,7 +756,7 @@ class ConfigCommand(Command):
     Args:
       file_path: The name of the private key file.
     """
-    if IS_WINDOWS:
+    if system_util.IS_WINDOWS:
       # For Windows, this check doesn't work (it actually just checks whether
       # the file is read-only). Since Windows files have a complicated ACL
       # system, this check doesn't make much sense on Windows anyway, so we
@@ -793,8 +855,12 @@ class ConfigCommand(Command):
         '# To use a proxy, edit and uncomment the proxy and proxy_port lines.\n'
         '# If you need a user/password with this proxy, edit and uncomment\n'
         '# those lines as well. If your organization also disallows DNS\n'
-        '# lookups by client machines set proxy_rdns = True\n'
-        '# If proxy_host and proxy_port are not specified in this file and\n'
+        '# lookups by client machines, set proxy_rdns to True (the default).\n'
+        '# If you have installed gsutil through the Cloud SDK and have \n'
+        '# configured proxy settings in gcloud, those proxy settings will \n'
+        '# override any other options (including those set here, along with \n'
+        '# any settings in proxy-related environment variables). Otherwise, \n'
+        '# if proxy_host and proxy_port are not specified in this file and\n'
         '# one of the OS environment variables http_proxy, https_proxy, or\n'
         '# HTTPS_PROXY is defined, gsutil will use the proxy server specified\n'
         '# in these environment variables, in order of precedence according\n'
@@ -818,7 +884,7 @@ class ConfigCommand(Command):
 
   # pylint: disable=dangerous-default-value,too-many-statements
   def _WriteBotoConfigFile(self, config_file, launch_browser=True,
-                           oauth2_scopes=[SCOPE_CLOUD_PLATFORM],
+                           oauth2_scopes=[constants.Scopes.CLOUD_PLATFORM],
                            cred_type=CredTypes.OAUTH2_USER_ACCOUNT,
                            configure_auth=True):
     """Creates a boto config file interactively.
@@ -965,7 +1031,7 @@ class ConfigCommand(Command):
             'edit and uncomment the\n# following line:\n'
             '#gs_oauth2_refresh_token = <your OAuth2 refresh token>\n\n')
     else:
-      if os.environ.get('CLOUDSDK_WRAPPER') == '1':
+      if system_util.InvokedViaCloudSdk():
         config_file.write(
             '# Google OAuth2 credentials are managed by the Cloud SDK and\n'
             '# do not need to be present in this file.\n')
@@ -1025,7 +1091,7 @@ class ConfigCommand(Command):
 
     # Write the config file GSUtil section that includes the default
     # project ID input from the user.
-    if not os.environ.get('CLOUDSDK_WRAPPER'):
+    if not system_util.InvokedViaCloudSdk():
       if launch_browser:
         sys.stdout.write(
             'Attempting to launch a browser to open the Google Cloud Console '
@@ -1104,17 +1170,19 @@ class ConfigCommand(Command):
         cred_type = CredTypes.OAUTH2_SERVICE_ACCOUNT
         has_e = True
       elif opt == '-f':
-        scopes.append(SCOPE_FULL_CONTROL)
+        scopes.append(constants.Scopes.FULL_CONTROL)
       elif opt == '-n':
         configure_auth = False
       elif opt == '-o':
         output_file_name = opt_arg
       elif opt == '-r':
-        scopes.append(SCOPE_READ_ONLY)
+        scopes.append(constants.Scopes.READ_ONLY)
+      elif opt == '--reauth':
+        scopes.append(constants.Scopes.REAUTH)
       elif opt == '-s':
         scopes.append(opt_arg)
       elif opt == '-w':
-        scopes.append(SCOPE_READ_WRITE)
+        scopes.append(constants.Scopes.READ_WRITE)
       else:
         self.RaiseInvalidArgumentException()
 
@@ -1123,16 +1191,18 @@ class ConfigCommand(Command):
                              '"gsutil help config" for more information.')
 
     if not configure_auth and (has_a or has_e or scopes or launch_browser):
-      raise CommandException('The -a, -b, -e, -f, -s, and -w flags cannot be '
-                             'specified with the -n flag. Please see '
-                             '"gsutil help config" for more information.')
+      raise CommandException(
+          'The -a, -b, -e, -f, -r, --reauth, -s, and -w flags cannot be '
+          'specified with the -n flag. Please see "gsutil help config" for '
+          'more information.')
 
     # Don't allow users to configure Oauth2 (any option other than -a and -n)
     # when running in the Cloud SDK, unless they have the Cloud SDK configured
     # not to pass credentials to gsutil.
-    if (os.environ.get('CLOUDSDK_WRAPPER') == '1' and
-        os.environ.get('CLOUDSDK_CORE_PASS_CREDENTIALS_TO_GSUTIL') == '1' and
-        not has_a and configure_auth):
+    if (system_util.InvokedViaCloudSdk() and
+        system_util.CloudSdkCredPassingEnabled() and
+        not has_a and
+        configure_auth):
       raise CommandException('\n'.join([
           'OAuth2 is the preferred authentication mechanism with the Cloud '
           'SDK.',
@@ -1149,7 +1219,7 @@ class ConfigCommand(Command):
               initial_indent='- ', subsequent_indent='  ')),
       ]))
 
-    if os.environ.get('CLOUDSDK_WRAPPER') == '1' and has_a:
+    if system_util.InvokedViaCloudSdk() and has_a:
       sys.stderr.write('\n'.join(textwrap.wrap(
           'This command will configure HMAC credentials, but gsutil will use '
           'OAuth2 credentials from the Cloud SDK by default. To make sure '
@@ -1157,7 +1227,8 @@ class ConfigCommand(Command):
           'pass_credentials_to_gsutil false".')) + '\n\n')
 
     if not scopes:
-      scopes.append(SCOPE_CLOUD_PLATFORM)
+      scopes.append(constants.Scopes.CLOUD_PLATFORM)
+      scopes.append(constants.Scopes.REAUTH)
 
     default_config_path_bak = None
     if not output_file_name:
