@@ -17,6 +17,7 @@ from __future__ import absolute_import
 
 import itertools
 import json
+import re
 
 from apitools.base.protorpclite import protojson
 from apitools.base.protorpclite.messages import DecodeError
@@ -29,21 +30,21 @@ from gslib.command_argument import CommandArgument
 from gslib.cs_api_map import ApiSelector
 from gslib.exception import CommandException
 from gslib.help_provider import CreateHelpText
-from gslib.iamhelpers import BindingStringToTuple
-from gslib.iamhelpers import BindingsTuple
-from gslib.iamhelpers import DeserializeBindingsTuple
-from gslib.iamhelpers import IsEqualBindings
-from gslib.iamhelpers import PatchBindings
-from gslib.iamhelpers import SerializeBindingsTuple
 from gslib.metrics import LogCommandParams
 from gslib.name_expansion import NameExpansionIterator
 from gslib.name_expansion import SeekAheadNameExpansionIterator
 from gslib.plurality_checkable_iterator import PluralityCheckableIterator
 from gslib.storage_url import StorageUrlFromString
 from gslib.third_party.storage_apitools import storage_v1_messages as apitools_messages
-from gslib.util import GetCloudApiInstance
-from gslib.util import NO_MAX
-from gslib.util import Retry
+from gslib.utils.cloud_api_helper import GetCloudApiInstance
+from gslib.utils.constants import NO_MAX
+from gslib.utils.iam_helper import BindingStringToTuple
+from gslib.utils.iam_helper import BindingsTuple
+from gslib.utils.iam_helper import DeserializeBindingsTuple
+from gslib.utils.iam_helper import IsEqualBindings
+from gslib.utils.iam_helper import PatchBindings
+from gslib.utils.iam_helper import SerializeBindingsTuple
+from gslib.utils.retry_util import Retry
 
 _SET_SYNOPSIS = """
   gsutil iam set [-afRr] [-e <etag>] file url ...
@@ -54,7 +55,7 @@ _GET_SYNOPSIS = """
 """
 
 _CH_SYNOPSIS = """
-  gsutil iam ch [-fRr] binding ...
+  gsutil iam ch [-fRr] binding ... url
 
   where each binding is of the form:
 
@@ -62,6 +63,11 @@ _CH_SYNOPSIS = """
       [-d] ("allUsers"|"allAuthenticatedUsers"):role[,...]
       -d ("user"|"serviceAccount"|"domain"|"group"):id
       -d ("allUsers"|"allAuthenticatedUsers")
+
+Note: The gsutil iam command disallows setting/changing to bindings using roles
+for project conveneience groups (projectOwner, projectEditor, projectViewer),
+because it goes against the principle of least privilege.
+
 """
 
 _GET_DESCRIPTION = """
@@ -185,6 +191,8 @@ _get_help_text = CreateHelpText(_GET_SYNOPSIS, _GET_DESCRIPTION)
 _set_help_text = CreateHelpText(_SET_SYNOPSIS, _SET_DESCRIPTION)
 _ch_help_text = CreateHelpText(_CH_SYNOPSIS, _CH_DESCRIPTION)
 
+STORAGE_URI_REGEX = re.compile(r'[a-z]+://[a-z].*')
+
 
 def _PatchIamWrapper(cls, iter_result, thread_state):
   (serialized_bindings_tuples, expansion_result) = iter_result
@@ -422,24 +430,21 @@ class IamCommand(Command):
     # expecting to come across the -r, -f flags here.
     it = iter(self.args)
     for token in it:
+      if STORAGE_URI_REGEX.match(token):
+        patterns.append(token)
+        break
       if token == '-d':
         patch_bindings_tuples.append(
             BindingStringToTuple(False, it.next()))
       else:
-        try:
-          patch_bindings_tuples.append(
-              BindingStringToTuple(True, token)
-          )
-        # All following arguments are urls.
-        except (ArgumentException, CommandException):
-          patterns.append(token)
-          for token in it:
-            patterns.append(token)
-
-    # We must have some bindings to process, else this is pointless.
+        patch_bindings_tuples.append(BindingStringToTuple(True, token))
     if not patch_bindings_tuples:
       raise CommandException('Must specify at least one binding.')
 
+    # All following arguments are urls.
+    for token in it:
+      patterns.append(token)
+
     self.everything_set_okay = True
     threaded_wildcards = []
     for pattern in patterns:
@@ -476,10 +481,6 @@ class IamCommand(Command):
           threaded_wildcards, self.recursion_requested,
           all_versions=self.all_versions)
 
-      # N.B.: Python2.6 support means we can't use a partial function here to
-      # curry the bindings tuples into the wrapper function. We instead pass
-      # the bindings along by zipping them with each name_expansion_iterator
-      # result. See http://bugs.python.org/issue5228.
       serialized_bindings_tuples_it = itertools.repeat(
           [SerializeBindingsTuple(t) for t in patch_bindings_tuples])
       self.Apply(
@@ -582,8 +583,6 @@ class IamCommand(Command):
           threaded_wildcards, self.recursion_requested,
           all_versions=self.all_versions)
 
-      # We cannot curry policy along due to a Python2.6 bug; see comments in
-      # IamCommand._PatchIam for more information.
       policy_it = itertools.repeat(protojson.encode_message(policy))
       self.Apply(
           _SetIamWrapper,
