@@ -18,26 +18,24 @@ from __future__ import absolute_import
 
 import re
 
-from gslib.boto_translation import S3_DELETE_MARKER_GUID
 from gslib.cloud_api import NotFoundException
 from gslib.command import Command
 from gslib.command_argument import CommandArgument
 from gslib.cs_api_map import ApiSelector
 from gslib.exception import CommandException
-from gslib.ls_helper import ENCRYPTED_FIELDS
-from gslib.ls_helper import LsHelper
-from gslib.ls_helper import UNENCRYPTED_FULL_LISTING_FIELDS
 from gslib.storage_url import ContainsWildcard
 from gslib.storage_url import StorageUrlFromString
-from gslib.translation_helper import AclTranslation
-from gslib.translation_helper import LabelTranslation
-from gslib.util import InsistAscii
-from gslib.util import ListingStyle
-from gslib.util import MakeHumanReadable
-from gslib.util import NO_MAX
-from gslib.util import PrintFullInfoAboutObject
-from gslib.util import UTF8
-
+from gslib.utils.constants import NO_MAX
+from gslib.utils.constants import S3_DELETE_MARKER_GUID
+from gslib.utils.constants import UTF8
+from gslib.utils.ls_helper import ENCRYPTED_FIELDS
+from gslib.utils.ls_helper import LsHelper
+from gslib.utils.ls_helper import PrintFullInfoAboutObject
+from gslib.utils.ls_helper import UNENCRYPTED_FULL_LISTING_FIELDS
+from gslib.utils.text_util import InsistAscii
+from gslib.utils.translation_helper import AclTranslation
+from gslib.utils.translation_helper import LabelTranslation
+from gslib.utils.unit_util import MakeHumanReadable
 
 # Regex that assists with converting JSON timestamp to ls-style output.
 # This excludes timestamp fractional seconds, for example:
@@ -128,8 +126,8 @@ _DETAILED_HELP_TEXT = ("""
   will print the object size, creation time stamp, and name of each matching
   object, along with the total count and sum of sizes of all matching objects:
 
-       2276224  2012-03-02T19:25:17Z  gs://bucket/obj1
-       3914624  2012-03-02T19:30:27Z  gs://bucket/obj2
+       2276224  2017-03-02T19:25:17Z  gs://bucket/obj1
+       3914624  2017-03-02T19:30:27Z  gs://bucket/obj2
     TOTAL: 2 objects, 6190848 bytes (5.9 MiB)
 
   Note that the total listed in parentheses above is in mebibytes (or gibibytes,
@@ -149,30 +147,38 @@ _DETAILED_HELP_TEXT = ("""
   will print something like:
 
     gs://bucket/obj1:
-            Creation time:                    Fri, 21 Oct 2016 19:25:17 GMT
-            Update time:                      Fri, 21 Oct 2016 21:17:59 GMT
-            Storage class update time:        Fri, 21 Oct 2016 22:12:32 GMT
-            Size:                             2276224
-            Cache-Control:                    private, max-age=0
-            Content-Type:                     application/x-executable
-            ETag:                             5ca6796417570a586723b7344afffc81
+            Creation time:                    Fri, 26 May 2017 22:55:44 GMT
+            Update time:                      Tue, 18 Jul 2017 12:31:18 GMT
+            Storage class:                    MULTI_REGIONAL
+            Content-Length:                   60183
+            Content-Type:                     image/jpeg
+            Hash (crc32c):                    zlUhtg==
+            Hash (md5):                       Bv86IAzFzrD1Z2io/c7yqA==
+            ETag:                             5ca67960a586723b7344afffc81
             Generation:                       1378862725952000
             Metageneration:                   1
-            ACL:
-    [
+            ACL:                              [
       {
-        "entity": "group-00b4903a97163d99003117abe64d292561d2b4074fc90ce5c0e35ac45f66ad70",
-        "entityId": "00b4903a97163d99003117abe64d292561d2b4074fc90ce5c0e35ac45f66ad70",
+        "entity": "project-owners-867484910061",
+        "projectTeam": {
+          "projectNumber": "867484910061",
+          "team": "owners"
+        },
+        "role": "OWNER"
+      },
+      {
+        "email": "jane@gmail.com",
+        "entity": "user-jane@gmail.com",
         "role": "OWNER"
       }
     ]
-    TOTAL: 1 objects, 2276224 bytes (2.17 MiB)
+    TOTAL: 1 objects, 60183 bytes (58.77 KiB)
 
-  Note that some fields above (time updated, storage class update time) are
-  not available with the (non-default) XML API.
+  Note that results may contain additional fields, such as custom metadata or
+  a storage class update time, if they are applicable to the object.
 
-  Also note that the Storage class update time field does not display unless it
-  differs from Creation time.
+  Also note that some fields, such as update time, are not available with the
+  (non-default) XML API.
 
   See also "gsutil help acl" for getting a more readable version of the ACL.
 
@@ -188,31 +194,39 @@ _DETAILED_HELP_TEXT = ("""
     gs://bucket/ :
             Storage class:                MULTI_REGIONAL
             Location constraint:          US
-            Versioning enabled:           True
+            Versioning enabled:           False
             Logging configuration:        None
             Website configuration:        None
-            CORS configuration:           Present
+            CORS configuration:           None
             Lifecycle configuration:      None
+            Requester Pays enabled:       True
             Labels:                       None
-            Time created:                 Fri, 21 Oct 2016 19:25:17 GMT
-            Time updated:                 Fri, 21 Oct 2016 21:17:59 GMT
+            Default KMS key:              None
+            Time created:                 Thu, 14 Jan 2016 19:25:17 GMT
+            Time updated:                 Thu, 08 Jun 2017 21:17:59 GMT
             Metageneration:               1
             ACL:
-    [
-      {
-        "entity": "group-00b4903a97163d99003117abe64d292561d2b4074fc90ce5c0e35ac45f66ad70",
-        "entityId": "00b4903a97163d99003117abe64d292561d2b4074fc90ce5c0e35ac45f66ad70",
-        "role": "OWNER"
-      }
-    ]
+              [
+                {
+                  "entity": "project-owners-867489160491",
+                  "projectTeam": {
+                    "projectNumber": "867489160491",
+                    "team": "owners"
+                  },
+                  "role": "OWNER"
+                }
+              ]
             Default ACL:
-    [
-      {
-        "entity": "group-00b4903a97163d99003117abe64d292561d2b4074fc90ce5c0e35ac45f66ad70",
-        "entityId": "00b4903a97163d99003117abe64d292561d2b4074fc90ce5c0e35ac45f66ad70",
-        "role": "OWNER"
-      }
-    ]
+              [
+                {
+                  "entity": "project-owners-867489160491",
+                  "projectTeam": {
+                    "projectNumber": "867489160491",
+                    "team": "owners"
+                  },
+                  "role": "OWNER"
+                }
+              ]
 
   Note that some fields above (time created, time updated, metageneration) are
   not available with the (non-default) XML API.
@@ -253,6 +267,13 @@ _DETAILED_HELP_TEXT = ("""
 """)
 
 
+class ListingStyle(object):
+  """Enum class for specifying listing style."""
+  SHORT = 'SHORT'
+  LONG = 'LONG'
+  LONG_LONG = 'LONG_LONG'
+
+
 class LsCommand(Command):
   """Implementation of gsutil ls command."""
 
@@ -314,11 +335,19 @@ class LsCommand(Command):
     fields['logging_config'] = 'Present' if bucket.logging else 'None'
     fields['cors_config'] = 'Present' if bucket.cors else 'None'
     fields['lifecycle_config'] = 'Present' if bucket.lifecycle else 'None'
+    fields['requester_pays'] = bucket.billing and bucket.billing.requesterPays
+    if bucket.retentionPolicy:
+      fields['retention_policy'] = 'Present'
     if bucket.labels:
       fields['labels'] = LabelTranslation.JsonFromMessage(
           bucket.labels, pretty_print=True)
     else:
       fields['labels'] = 'None'
+    if bucket.encryption and bucket.encryption.defaultKmsKeyName:
+      fields['default_kms_key'] = bucket.encryption.defaultKmsKeyName
+    else:
+      fields['default_kms_key'] = 'None'
+    fields['encryption_config'] = 'Present' if bucket.encryption else 'None'
     # Fields not available in all APIs (e.g. the XML API)
     if bucket.metageneration:
       fields['metageneration'] = bucket.metageneration
@@ -327,6 +356,8 @@ class LsCommand(Command):
           '%a, %d %b %Y %H:%M:%S GMT')
     if bucket.updated:
       fields['updated'] = bucket.updated.strftime('%a, %d %b %Y %H:%M:%S GMT')
+    if bucket.defaultEventBasedHold:
+      fields['default_eventbased_hold'] = bucket.defaultEventBasedHold
 
     # For field values that are multiline, add indenting to make it look
     # prettier.
@@ -346,12 +377,19 @@ class LsCommand(Command):
     metageneration_line = ''
     time_created_line = ''
     time_updated_line = ''
+    default_eventbased_hold_line = ''
+    retention_policy_line = ''
     if 'metageneration' in fields:
       metageneration_line = '\tMetageneration:\t\t\t{metageneration}\n'
     if 'time_created' in fields:
       time_created_line = '\tTime created:\t\t\t{time_created}\n'
     if 'updated' in fields:
       time_updated_line = '\tTime updated:\t\t\t{updated}\n'
+    if 'default_eventbased_hold' in fields:
+      default_eventbased_hold_line = (
+          '\tDefault Event-Based Hold:\t{default_eventbased_hold}\n')
+    if 'retention_policy' in fields:
+      retention_policy_line = '\tRetention Policy:\t\t{retention_policy}\n'
 
     print(('{bucket} :\n'
            '\tStorage class:\t\t\t{storage_class}\n'
@@ -361,7 +399,11 @@ class LsCommand(Command):
            '\tWebsite configuration:\t\t{website_config}\n'
            '\tCORS configuration: \t\t{cors_config}\n'
            '\tLifecycle configuration:\t{lifecycle_config}\n'
+           '\tRequester Pays enabled:\t\t{requester_pays}\n' +
+           retention_policy_line +
+           default_eventbased_hold_line +
            '\tLabels:\t\t\t\t{labels}\n' +
+           '\tDefault KMS key:\t\t{default_kms_key}\n' +
            time_created_line +
            time_updated_line +
            metageneration_line +
@@ -467,13 +509,17 @@ class LsCommand(Command):
         bucket_fields = ['id']
       elif listing_style == ListingStyle.LONG_LONG:
         bucket_fields = ['acl',
+                         'billing',
                          'cors',
                          'defaultObjectAcl',
+                         'encryption',
                          'labels',
                          'location',
                          'logging',
                          'lifecycle',
                          'metageneration',
+                         'retentionPolicy',
+                         'defaultEventBasedHold',
                          'storageClass',
                          'timeCreated',
                          'updated',
@@ -509,11 +555,12 @@ class LsCommand(Command):
 
         if listing_style == ListingStyle.SHORT:
           # ls helper by default readies us for a short listing.
-          ls_helper = LsHelper(self.WildcardIterator, self.logger,
-                               all_versions=self.all_versions,
-                               print_bucket_header_func=print_bucket_header,
-                               should_recurse=self.recursion_requested,
-                               list_subdir_contents=self.list_subdir_contents)
+          listing_helper = LsHelper(
+              self.WildcardIterator, self.logger,
+              all_versions=self.all_versions,
+              print_bucket_header_func=print_bucket_header,
+              should_recurse=self.recursion_requested,
+              list_subdir_contents=self.list_subdir_contents)
         elif listing_style == ListingStyle.LONG:
           bucket_listing_fields = ['name', 'timeCreated', 'updated', 'size']
           if self.all_versions:
@@ -521,31 +568,34 @@ class LsCommand(Command):
           if self.include_etag:
             bucket_listing_fields.append('etag')
 
-          ls_helper = LsHelper(self.WildcardIterator, self.logger,
-                               print_object_func=self._PrintLongListing,
-                               print_dir_func=_PrintPrefixLong,
-                               print_bucket_header_func=print_bucket_header,
-                               all_versions=self.all_versions,
-                               should_recurse=self.recursion_requested,
-                               fields=bucket_listing_fields,
-                               list_subdir_contents=self.list_subdir_contents)
+          listing_helper = LsHelper(
+              self.WildcardIterator, self.logger,
+              print_object_func=self._PrintLongListing,
+              print_dir_func=_PrintPrefixLong,
+              print_bucket_header_func=print_bucket_header,
+              all_versions=self.all_versions,
+              should_recurse=self.recursion_requested,
+              fields=bucket_listing_fields,
+              list_subdir_contents=self.list_subdir_contents)
 
         elif listing_style == ListingStyle.LONG_LONG:
           # List all fields
           bucket_listing_fields = (UNENCRYPTED_FULL_LISTING_FIELDS +
                                    ENCRYPTED_FIELDS)
-          ls_helper = LsHelper(self.WildcardIterator, self.logger,
-                               print_object_func=PrintFullInfoAboutObject,
-                               print_dir_func=_PrintPrefixLong,
-                               print_bucket_header_func=print_bucket_header,
-                               all_versions=self.all_versions,
-                               should_recurse=self.recursion_requested,
-                               fields=bucket_listing_fields,
-                               list_subdir_contents=self.list_subdir_contents)
+          listing_helper = LsHelper(
+              self.WildcardIterator, self.logger,
+              print_object_func=PrintFullInfoAboutObject,
+              print_dir_func=_PrintPrefixLong,
+              print_bucket_header_func=print_bucket_header,
+              all_versions=self.all_versions,
+              should_recurse=self.recursion_requested,
+              fields=bucket_listing_fields,
+              list_subdir_contents=self.list_subdir_contents)
         else:
           raise CommandException('Unknown listing style: %s' % listing_style)
 
-        exp_dirs, exp_objs, exp_bytes = ls_helper.ExpandUrlAndPrint(storage_url)
+        exp_dirs, exp_objs, exp_bytes = (
+            listing_helper.ExpandUrlAndPrint(storage_url))
         if storage_url.IsObject() and exp_objs == 0 and exp_dirs == 0:
           got_nomatch_errors = True
         total_bytes += exp_bytes
