@@ -22,8 +22,10 @@ import re
 import time
 import uuid
 
+from datetime import datetime
 from gslib import metrics
 from gslib.cloud_api import AccessDeniedException
+from gslib.cloud_api import BadRequestException
 from gslib.cloud_api import NotFoundException
 from gslib.cloud_api import PublishPermissionDeniedException
 from gslib.command import Command
@@ -36,7 +38,7 @@ from gslib.project_id import PopulateProjectId
 from gslib.pubsub_api import PubsubApi
 from gslib.storage_url import StorageUrlFromString
 from gslib.third_party.pubsub_apitools.pubsub_v1_messages import Binding
-
+from gslib.utils import copy_helper
 
 # Cloud Pub/Sub commands
 
@@ -76,14 +78,19 @@ _LIST_DESCRIPTION = """
   given bucket. The listed name of each notification config can be used with
   the delete sub-command to delete that specific notification config.
 
-  No object change notifications will be listed. Only Cloud Pub/Sub notification
-  subscription configs will be listed.
+  For listing Object Change Notifications instead of Cloud Pub/Sub notification
+  subscription configs, add a -o flag.
 
 <B>LIST EXAMPLES</B>
   Fetch the list of notification configs for the bucket example-bucket:
 
     gsutil notification list gs://example-bucket
 
+  The same as above, but for Object Change Notifications instead of Cloud
+  Pub/Sub notification subscription configs.
+
+    gsutil notification list -o gs://example-bucket
+
   Fetch the notification configs in all buckets matching a wildcard:
 
     gsutil notification list gs://example-*
@@ -141,7 +148,7 @@ _CREATE_DESCRIPTION = """
   You can create multiple notification configurations for a bucket, but their
   triggers cannot overlap such that a single event could send multiple
   notifications. Attempting to create a notification configuration that
-  overlaps with an exisitng notification configuration results in an error.
+  overlaps with an existing notification configuration results in an error.
 
 <B>CREATE EXAMPLES</B>
   Begin sending notifications of all changes to the bucket example-bucket
@@ -164,7 +171,7 @@ _CREATE_DESCRIPTION = """
   Create a notification config that will only send an event when a new object
   has been created:
 
-    gsutil notification create -f json -t OBJECT_FINALIZE gs://example-bucket
+    gsutil notification create -f json -e OBJECT_FINALIZE gs://example-bucket
 
   Create a topic and notification config that will only send an event when
   an object beginning with "photos/" is affected:
@@ -221,6 +228,20 @@ _CREATE_DESCRIPTION = """
             not specified, this command will choose a topic whose project is
             your default project and whose ID is the same as the Cloud Storage
             bucket name.
+
+<B>NEXT STEPS</B>
+  Once the create command has succeeded, Cloud Storage will publish a message to
+  the specified Cloud Pub/Sub topic when eligible changes occur. In order to
+  receive these messages, you must create a Pub/Sub subscription for your
+  Pub/Sub topic. To learn more about creating Pub/Sub subscriptions, see `the
+  Pub/Sub Subscriber Overview <https://cloud.google.com/pubsub/docs/subscriber>`_.
+
+  You can create a simple Pub/Sub subscription using the ``gcloud`` command-line
+  tool. For example, to create a new subscription on the topic "myNewTopic" and
+  attempt to pull messages from it, you could run:
+
+    gcloud beta pubsub subscriptions create --topic myNewTopic testSubscription
+    gcloud beta pubsub subscriptions pull --auto-ack testSubscription
 """
 
 _WATCHBUCKET_DESCRIPTION = """
@@ -230,8 +251,8 @@ _WATCHBUCKET_DESCRIPTION = """
 
   The app_url parameter must be an HTTPS URL to an application that will be
   notified of changes to any object in the bucket. The URL endpoint must be
-  a verified domain on your project. See
-  `Notification Authorization <https://cloud.google.com/storage/docs/object-change-notification#_Authorization>`_
+  a verified domain on your project. See `Notification Authorization
+  <https://cloud.google.com/storage/docs/object-change-notification#_Authorization>`_
   for details.
 
   The optional id parameter can be used to assign a unique identifier to the
@@ -285,8 +306,9 @@ _DESCRIPTION = """
   Storage integration with Google Cloud Pub/Sub.
 """ + _CREATE_DESCRIPTION + _LIST_DESCRIPTION + _DELETE_DESCRIPTION + """
 <B>OBJECT CHANGE NOTIFICATIONS</B>
-  For more information on the Object Change Notification feature, please see:
-  https://cloud.google.com/storage/docs/object-change-notification
+  For more information on the Object Change Notification feature, please see
+  `the Object Change Notification docs
+  <https://cloud.google.com/storage/docs/object-change-notification>`_.
 
   The "watchbucket" and "stopchannel" sub-commands enable and disable Object
   Change Notifications.
@@ -299,9 +321,12 @@ _DESCRIPTION = """
   will then see a notification for each of these components being created and
   deleted. If this is a concern for you, note that parallel composite uploads
   can be disabled by setting "parallel_composite_upload_threshold = 0" in your
-  boto config file.
+  boto config file. Alternately, your subscriber code can filter out gsutil's
+  parallel composite uploads by ignoring any notification about objects whose
+  names contain (but do not start with) the following string:
+    "{composite_namespace}".
 
-"""
+""".format(composite_namespace=copy_helper.PARALLEL_UPLOAD_TEMP_NAMESPACE)
 
 
 NOTIFICATION_AUTHORIZATION_FAILED_MESSAGE = """
@@ -316,9 +341,9 @@ which is not authorized for your project. Please ensure that you are using
 Service Account authentication and that the Service Account's project is
 authorized for the application URL. Notification endpoint URLs must also be
 whitelisted in your Cloud Console project. To do that, the domain must also be
-verified using Google Webmaster Tools. For instructions, please see:
-
-  https://cloud.google.com/storage/docs/object-change-notification#_Authorization
+verified using Google Webmaster Tools. For instructions, please see
+`Notification Authorization
+<https://cloud.google.com/storage/docs/object-change-notification#_Authorization>`_.
 """
 
 _DETAILED_HELP_TEXT = CreateHelpText(_SYNOPSIS, _DESCRIPTION)
@@ -334,7 +359,6 @@ _watchbucket_help_text = (
 _stopchannel_help_text = (
     CreateHelpText(_STOPCHANNEL_SYNOPSIS, _STOPCHANNEL_DESCRIPTION))
 
-
 PAYLOAD_FORMAT_MAP = {
     'none': 'NONE',
     'json': 'JSON_API_V1'
@@ -365,7 +389,7 @@ class NotificationCommand(Command):
       usage_synopsis=_SYNOPSIS,
       min_args=2,
       max_args=NO_MAX,
-      supported_sub_args='i:t:m:t:o:f:e:p:s',
+      supported_sub_args='i:t:m:t:of:e:p:s',
       file_url_ok=False,
       provider_url_ok=False,
       urls_start_arg=1,
@@ -468,6 +492,29 @@ class NotificationCommand(Command):
 
     return 0
 
+  def _ListChannels(self, bucket_arg):
+    """Lists active channel watches on a bucket given in self.args."""
+    bucket_url = StorageUrlFromString(bucket_arg)
+    if not (bucket_url.IsBucket() and bucket_url.scheme == 'gs'):
+      raise CommandException(
+          'The %s command can only be used with gs:// bucket URLs.' %
+          self.command_name)
+    if not bucket_url.IsBucket():
+      raise CommandException('URL must name a bucket for the %s command.' %
+                             self.command_name)
+    channels = self.gsutil_api.ListChannels(
+          bucket_url.bucket_name, provider='gs').items
+    self.logger.info('Bucket %s has the following active Object Change Notifications:', bucket_url.bucket_name)
+    for idx, channel in enumerate(channels):
+      self.logger.info('\tNotification channel %d:', idx+1)
+      self.logger.info('\t\tChannel identifier: %s', channel.channel_id)
+      self.logger.info('\t\tResource identifier: %s', channel.resource_id)
+      self.logger.info('\t\tApplication URL: %s', channel.push_url)
+      self.logger.info('\t\tCreated by: %s', channel.subscriber_email)
+      self.logger.info('\t\tCreation time: %s', str(datetime.fromtimestamp(channel.creation_time_ms/1000)))
+
+    return 0
+
   def _Create(self):
     self.CheckArguments()
 
@@ -679,10 +726,14 @@ class NotificationCommand(Command):
 
   def _List(self):
     self.CheckArguments()
-
-    for bucket_name, notification in self._EnumerateNotificationsFromArgs(
-        accept_notification_configs=False):
-      self._PrintNotificationDetails(bucket_name, notification)
+    if self.sub_opts:
+      if '-o' in dict(self.sub_opts):
+        for bucket_name in self.args:
+          self._ListChannels(bucket_name) 
+    else:
+      for bucket_name, notification in self._EnumerateNotificationsFromArgs(
+          accept_notification_configs=False):
+        self._PrintNotificationDetails(bucket_name, notification)
     return 0
 
   def _PrintNotificationDetails(self, bucket, notification):
