@@ -42,7 +42,6 @@ import boto.gs.connection
 import gslib
 from gslib.cloud_api import NotFoundException
 from gslib.cloud_api import ServiceException
-from gslib.cloud_api_helper import GetDownloadSerializationData
 from gslib.command import Command
 from gslib.command import DummyArgChecker
 from gslib.command_argument import CommandArgument
@@ -50,27 +49,28 @@ from gslib.commands import config
 from gslib.cs_api_map import ApiSelector
 from gslib.exception import CommandException
 from gslib.file_part import FilePart
-from gslib.hashing_helper import CalculateB64EncodedMd5FromContents
 from gslib.storage_url import StorageUrlFromString
 from gslib.third_party.storage_apitools import storage_v1_messages as apitools_messages
-from gslib.util import CheckFreeSpace
-from gslib.util import DivideAndCeil
-from gslib.util import GetCloudApiInstance
-from gslib.util import GetDiskCounters
-from gslib.util import GetFileSize
-from gslib.util import GetMaxRetryDelay
-from gslib.util import HumanReadableToBytes
-from gslib.util import IS_LINUX
-from gslib.util import MakeBitsHumanReadable
-from gslib.util import MakeHumanReadable
-from gslib.util import Percentile
-from gslib.util import ResumableThreshold
+from gslib.utils.boto_util import GetMaxRetryDelay
+from gslib.utils.boto_util import ResumableThreshold
+from gslib.utils.cloud_api_helper import GetCloudApiInstance
+from gslib.utils.cloud_api_helper import GetDownloadSerializationData
+from gslib.utils.hashing_helper import CalculateB64EncodedMd5FromContents
+from gslib.utils.system_util import CheckFreeSpace
+from gslib.utils.system_util import GetDiskCounters
+from gslib.utils.system_util import GetFileSize
+from gslib.utils.system_util import IS_LINUX
+from gslib.utils.unit_util import DivideAndCeil
+from gslib.utils.unit_util import HumanReadableToBytes
+from gslib.utils.unit_util import MakeBitsHumanReadable
+from gslib.utils.unit_util import MakeHumanReadable
+from gslib.utils.unit_util import Percentile
 
 _SYNOPSIS = """
   gsutil perfdiag [-i in.json]
   gsutil perfdiag [-o out.json] [-n objects] [-c processes]
       [-k threads] [-p parallelism type] [-y slices] [-s size] [-d directory]
-      [-t tests] url...
+      [-t tests] [-j ratio] url...
 """
 
 _DETAILED_HELP_TEXT = ("""
@@ -203,6 +203,13 @@ _DETAILED_HELP_TEXT = ("""
   -i          Reads the JSON output file created using the -o command and prints
               a formatted description of the results.
 
+  -j          Applies gzip transport encoding and sets the target compression
+              ratio for the generated test files. This ratio can be an integer
+              between 0 and 100 (inclusive), with 0 generating a file with
+              uniform data, and 100 generating random data. When you specify
+              the -j option, files being uploaded are compressed in-memory and
+              on-the-wire only. See cp -j for specific semantics.
+
 
 <B>MEASURING AVAILABILITY</B>
   The perfdiag command ignores the boto num_retries configuration parameter.
@@ -218,10 +225,12 @@ _DETAILED_HELP_TEXT = ("""
 
 <B>NOTE</B>
   The perfdiag command collects system information. It collects your IP address,
-  executes DNS queries to Google servers and collects the results, and collects
-  network statistics information from the output of netstat -s. It will also
-  attempt to connect to your proxy server if you have one configured. None of
-  this information will be sent to Google unless you choose to send it.
+  executes DNS queries to Google servers and collects the results, collects
+  network statistics information from the output of netstat -s, and looks at the
+  BIOS product name string. It will also attempt to connect to your proxy server
+  if you have one configured, and will look up the location and storage class of
+  the bucket being used for performance testing. None of this information will
+  be sent to Google unless you choose to send it.
 """)
 
 FileDataTuple = namedtuple(
@@ -246,13 +255,13 @@ SliceDownloadTuple = namedtuple(
 # names are the same as documented in PerfDiagCommand.Upload.
 FanUploadTuple = namedtuple(
     'FanUploadTuple',
-    'need_to_slice file_name object_name use_file')
+    'need_to_slice file_name object_name use_file gzip_encoded')
 
 # Describes one slice in a sliced upload.
 # Field names are the same as documented in PerfDiagCommand.Upload.
 SliceUploadTuple = namedtuple(
     'SliceUploadTuple',
-    'file_name object_name use_file file_start file_size')
+    'file_name object_name use_file file_start file_size gzip_encoded')
 
 # Dict storing file_path:FileDataTuple for each temporary file used by
 # perfdiag. This data should be kept outside of the PerfDiagCommand class
@@ -311,9 +320,10 @@ def _UploadObject(cls, args, thread_state=None):
   gsutil_api = GetCloudApiInstance(cls, thread_state)
   if args.need_to_slice:
     cls.PerformSlicedUpload(args.file_name, args.object_name, args.use_file,
-                            gsutil_api)
+                            gsutil_api, gzip_encoded=args.gzip_encoded)
   else:
-    cls.Upload(args.file_name, args.object_name, gsutil_api, args.use_file)
+    cls.Upload(args.file_name, args.object_name, gsutil_api, args.use_file,
+               gzip_encoded=args.gzip_encoded)
 
 
 def _UploadSlice(cls, args, thread_state=None):
@@ -326,7 +336,7 @@ def _UploadSlice(cls, args, thread_state=None):
   """
   gsutil_api = GetCloudApiInstance(cls, thread_state)
   cls.Upload(args.file_name, args.object_name, gsutil_api, args.use_file,
-             args.file_start, args.file_size)
+             args.file_start, args.file_size, gzip_encoded=args.gzip_encoded)
 
 
 def _DeleteWrapper(cls, object_name, thread_state=None):
@@ -360,6 +370,37 @@ class DummyFile(object):
     pass
 
 
+def _GenerateFileData(fp, file_size=0, random_ratio=100,
+                      max_unique_random_bytes=5242883):
+  """Writes data into a file like object.
+
+  Args:
+    fp: A file like object to write the data to.
+    file_size: The amount of data to write to the file.
+    random_ratio: The percentage of randomly generated data to write. This can
+        be any number between 0 and 100 (inclusive), with 0 producing uniform
+        data, and 100 producing random data.
+    max_unique_random_bytes: The maximum number of bytes to generate
+                             pseudo-randomly before beginning to repeat
+                             bytes. The default was chosen as the next prime
+                             larger than 5 MiB.
+  """
+  # Normalize the ratio.
+  random_ratio /= 100.0
+  random_bytes = os.urandom(min(file_size, max_unique_random_bytes))
+  total_bytes_written = 0
+  while total_bytes_written < file_size:
+    num_bytes = min(max_unique_random_bytes, file_size - total_bytes_written)
+    # Calculate the amount of random and sequential data to write and
+    # resolve rounding errors by adjusting the random bytes to write.
+    num_bytes_seq = int(num_bytes * (1 - random_ratio))
+    num_bytes_random = num_bytes - num_bytes_seq
+
+    fp.write(random_bytes[:num_bytes_random])
+    fp.write(b'x' * num_bytes_seq)
+    total_bytes_written += num_bytes
+
+
 # Many functions in perfdiag re-define a temporary function based on a
 # variable from a loop, resulting in a false positive from the linter.
 # pylint: disable=cell-var-from-loop
@@ -373,7 +414,7 @@ class PerfDiagCommand(Command):
       usage_synopsis=_SYNOPSIS,
       min_args=0,
       max_args=1,
-      supported_sub_args='n:c:k:p:y:s:d:t:m:i:o:',
+      supported_sub_args='n:c:k:p:y:s:d:t:m:i:o:j:',
       file_url_ok=False,
       provider_url_ok=False,
       urls_start_arg=0,
@@ -441,10 +482,6 @@ class PerfDiagCommand(Command):
   # bottleneck at high throughput rates, so we increase it.
   KEY_BUFFER_SIZE = 16384
 
-  # The maximum number of bytes to generate pseudo-randomly before beginning
-  # to repeat bytes. This number was chosen as the next prime larger than 5 MiB.
-  MAX_UNIQUE_RANDOM_BYTES = 5242883
-
   # Maximum amount of time, in seconds, we will wait for object listings to
   # reflect what we expect in the listing tests.
   MAX_LISTING_WAIT_TIME = 60.0
@@ -484,7 +521,8 @@ class PerfDiagCommand(Command):
       self.logger.info('This is a large operation, and could take a while.')
 
   def _MakeTempFile(self, file_size=0, mem_metadata=False,
-                    mem_data=False, prefix='gsutil_test_file'):
+                    mem_data=False, prefix='gsutil_test_file',
+                    random_ratio=100):
     """Creates a temporary file of the given size and returns its path.
 
     Args:
@@ -495,6 +533,9 @@ class PerfDiagCommand(Command):
                 temp_file_dict[fpath].data
       prefix: The prefix to use for the temporary file. Defaults to
               gsutil_test_file.
+      random_ratio: The percentage of randomly generated data to write. This
+          can be any number between 0 and 100 (inclusive), with 0 producing
+          uniform data, and 100 producing random data.
 
     Returns:
       The file path of the created temporary file.
@@ -502,14 +543,7 @@ class PerfDiagCommand(Command):
     fd, fpath = tempfile.mkstemp(suffix='.bin', prefix=prefix,
                                  dir=self.directory, text=False)
     with os.fdopen(fd, 'wb') as fp:
-      random_bytes = os.urandom(min(file_size,
-                                    self.MAX_UNIQUE_RANDOM_BYTES))
-      total_bytes_written = 0
-      while total_bytes_written < file_size:
-        num_bytes = min(self.MAX_UNIQUE_RANDOM_BYTES,
-                        file_size - total_bytes_written)
-        fp.write(random_bytes[:num_bytes])
-        total_bytes_written += num_bytes
+      _GenerateFileData(fp, file_size, random_ratio)
 
     if mem_metadata or mem_data:
       with open(fpath, 'rb') as fp:
@@ -559,7 +593,8 @@ class PerfDiagCommand(Command):
       # instead of creating N objects, in order to avoid excessive memory usage.
       if self.diag_tests.intersection((self.RTHRU, self.WTHRU)):
         self.mem_thru_file_name = self._MakeTempFile(
-            self.thru_filesize, mem_metadata=True, mem_data=True)
+            self.thru_filesize, mem_metadata=True, mem_data=True,
+            random_ratio=self.gzip_compression_ratio)
         self.mem_thru_object_name = os.path.basename(self.mem_thru_file_name)
 
       # For tests that use disk I/O, it is necessary to create N objects in
@@ -577,8 +612,9 @@ class PerfDiagCommand(Command):
                               MakeHumanReadable(self.thru_filesize)))
           self._WarnIfLargeData()
           for _ in range(self.num_objects):
-            file_name = self._MakeTempFile(self.thru_filesize,
-                                           mem_metadata=True)
+            file_name = self._MakeTempFile(
+                self.thru_filesize, mem_metadata=True,
+                random_ratio=self.gzip_compression_ratio)
             self.thru_file_names.append(file_name)
             self.thru_object_names.append(os.path.basename(file_name))
         else:
@@ -803,7 +839,7 @@ class PerfDiagCommand(Command):
         process_count=self.processes, thread_count=self.threads)
 
   def PerformFannedUpload(self, need_to_slice, file_names, object_names,
-                          use_file):
+                          use_file, gzip_encoded=False):
     """Performs a parallel upload of multiple files using the fan strategy.
 
     The metadata for file_name should be present in temp_file_dict prior
@@ -817,18 +853,23 @@ class PerfDiagCommand(Command):
       object_names: A list, corresponding by by index to file_names, of object
                     names to upload data to.
       use_file: If true, use disk I/O, otherwise read upload data from memory.
+      gzip_encoded: Flag for if the file will be uploaded with the gzip
+                    transport encoding. If true, a lock is used to limit
+                    resource usage.
     """
     args = []
     for i in range(len(file_names)):
       args.append(FanUploadTuple(
-          need_to_slice, file_names[i], object_names[i], use_file))
+          need_to_slice, file_names[i], object_names[i], use_file,
+          gzip_encoded))
     self.Apply(
         _UploadObject, args, _PerfdiagExceptionHandler,
         ('total_requests', 'request_errors'), arg_checker=DummyArgChecker,
         parallel_operations_override=self.ParallelOverrideReason.PERFDIAG,
         process_count=self.processes, thread_count=self.threads)
 
-  def PerformSlicedUpload(self, file_name, object_name, use_file, gsutil_api):
+  def PerformSlicedUpload(self, file_name, object_name, use_file, gsutil_api,
+                          gzip_encoded=False):
     """Performs a parallel upload of a file using the slice strategy.
 
     The metadata for file_name should be present in temp_file_dict prior
@@ -840,6 +881,9 @@ class PerfDiagCommand(Command):
       object_name: The name of the object to upload to.
       use_file: If true, use disk I/O, otherwise read upload data from memory.
       gsutil_api: CloudApi instance to use for operations in this thread.
+      gzip_encoded: Flag for if the file will be uploaded with the gzip
+                    transport encoding. If true, a semaphore is used to limit
+                    resource usage.
     """
     # Divide the file into components.
     component_size = DivideAndCeil(self.thru_filesize, self.num_slices)
@@ -851,8 +895,9 @@ class PerfDiagCommand(Command):
       component_start = i * component_size
       component_size = min(component_size,
                            temp_file_dict[file_name].size - component_start)
-      args.append(SliceUploadTuple(file_name, component_object_names[i],
-                                   use_file, component_start, component_size))
+      args.append(SliceUploadTuple(
+          file_name, component_object_names[i], use_file, component_start,
+          component_size, gzip_encoded))
 
     # Upload the components in parallel.
     try:
@@ -980,10 +1025,13 @@ class PerfDiagCommand(Command):
          MakeHumanReadable(self.thru_filesize)))
     self._WarnIfLargeData()
 
-    self.results[test_name] = {'file_size': self.thru_filesize,
-                               'processes': self.processes,
-                               'threads': self.threads,
-                               'parallelism': self.parallel_strategy}
+    self.results[test_name] = {
+        'file_size': self.thru_filesize,
+        'processes': self.processes,
+        'threads': self.threads,
+        'parallelism': self.parallel_strategy,
+        'gzip_encoded_writes': self.gzip_encoded_writes,
+        'gzip_compression_ratio': self.gzip_compression_ratio}
 
     # Warmup the TCP connection.
     warmup_obj_name = os.path.basename(self.tcp_warmup_file)
@@ -1007,16 +1055,20 @@ class PerfDiagCommand(Command):
     t0 = time.time()
     if self.processes == 1 and self.threads == 1:
       for i in range(self.num_objects):
-        self.Upload(file_names[i], object_names[i], self.gsutil_api, use_file)
+        self.Upload(
+            file_names[i], object_names[i], self.gsutil_api, use_file,
+            gzip_encoded=self.gzip_encoded_writes)
     else:
       if self.parallel_strategy in (self.FAN, self.BOTH):
         need_to_slice = (self.parallel_strategy == self.BOTH)
-        self.PerformFannedUpload(need_to_slice, file_names, object_names,
-                                 use_file)
+        self.PerformFannedUpload(
+            need_to_slice, file_names, object_names, use_file,
+            gzip_encoded=self.gzip_encoded_writes)
       elif self.parallel_strategy == self.SLICE:
         for i in range(self.num_objects):
-          self.PerformSlicedUpload(file_names[i], object_names[i], use_file,
-                                   self.gsutil_api)
+          self.PerformSlicedUpload(
+              file_names[i], object_names[i], use_file,
+              self.gsutil_api, gzip_encoded=self.gzip_encoded_writes)
     t1 = time.time()
 
     time_took = t1 - t0
@@ -1045,7 +1097,7 @@ class PerfDiagCommand(Command):
                                  prefix=list_prefix)
       object_name = os.path.basename(fpath)
       list_objects.append(object_name)
-      args.append(FanUploadTuple(False, fpath, object_name, False))
+      args.append(FanUploadTuple(False, fpath, object_name, False, False))
       self.temporary_objects.add(object_name)
 
     # Add the objects to the bucket.
@@ -1131,7 +1183,7 @@ class PerfDiagCommand(Command):
     }
 
   def Upload(self, file_name, object_name, gsutil_api, use_file=False,
-             file_start=0, file_size=None):
+             file_start=0, file_size=None, gzip_encoded=False):
     """Performs an upload to the test bucket.
 
     The file is uploaded to the bucket referred to by self.bucket_url, and has
@@ -1147,7 +1199,9 @@ class PerfDiagCommand(Command):
                   (only should be specified for sliced uploads)
       file_size: The size of the file to upload.
                  (only should be specified for sliced uploads)
-
+      gzip_encoded: Flag for if the file will be uploaded with the gzip
+                    transport encoding. If true, a lock is used to limit
+                    resource usage.
     Returns:
       Uploaded Object Metadata.
     """
@@ -1170,13 +1224,24 @@ class PerfDiagCommand(Command):
         if file_size < ResumableThreshold():
           return gsutil_api.UploadObject(
               fp, upload_target, provider=self.provider, size=file_size,
-              fields=['name', 'mediaLink', 'size'])
+              fields=['name', 'mediaLink', 'size'],
+              gzip_encoded=gzip_encoded)
         else:
           return gsutil_api.UploadObjectResumable(
               fp, upload_target, provider=self.provider, size=file_size,
               fields=['name', 'mediaLink', 'size'],
-              tracker_callback=_DummyTrackerCallback)
-      return self._RunOperation(_InnerUpload)
+              tracker_callback=_DummyTrackerCallback,
+              gzip_encoded=gzip_encoded)
+
+      # Because perfdiag does not go through copy_helper, it misses the
+      # resource lock. Here, if the upload is compressed, the lock is applied
+      # manually.
+      def _InnerUploadLock():
+        with gslib.command.concurrent_compressed_upload_lock:
+          return _InnerUpload()
+
+      upload_func = _InnerUploadLock if gzip_encoded else _InnerUpload
+      return self._RunOperation(upload_func)
     finally:
       if fp:
         fp.close()
@@ -1343,9 +1408,11 @@ class PerfDiagCommand(Command):
     proxy_port = boto.config.getint('Boto', 'proxy_port', 0)
     sysinfo['using_proxy'] = bool(proxy_host)
 
-    if boto.config.get('Boto', 'proxy_rdns', False):
-      self.logger.info('DNS lookups are disallowed in this environment, so '
-                       'some information is not included in this perfdiag run.')
+    if boto.config.get('Boto', 'proxy_rdns', True if proxy_host else False):
+      self.logger.info(
+          'DNS lookups are disallowed in this environment, so some information '
+          'is not included in this perfdiag run. To allow local DNS lookups '
+          'while using a proxy, set proxy_rdns to False in your boto file.')
 
     # Get the local IP address from socket lib.
     try:
@@ -1370,6 +1437,32 @@ class PerfDiagCommand(Command):
     except (CommandException, OSError):
       sysinfo['googserv_route'] = ''
 
+    # Check the BIOS product name, to determine if we're running on a GCE
+    # instance.
+    # TODO: Add support for checking if we're on GCE from Windows, via WMI.
+    if IS_LINUX:
+      try:
+        with open('/sys/class/dmi/id/product_name', 'r') as fp:
+          sysinfo['on_gce'] = (fp.readline() == 'Google Compute Engine\n')
+      except OSError:
+        pass
+      # If we're running on GCE include details about the instance.
+      if sysinfo.get('on_gce', False):
+        hostname = socket.gethostname()
+        cmd = ['gcloud', 'compute', 'instances', 'list', '--filter=', hostname]
+        try:
+          sysinfo['gce_instance_info'] = self._Exec(cmd, return_output=True)
+        except (CommandException, OSError):
+          sysinfo['gce_instance_info'] = ''
+
+    # Record info about location and storage class of the bucket being used for
+    # performance testing.
+    bucket_info = self.gsutil_api.GetBucket(
+        self.bucket_url.bucket_name, fields=['location', 'storageClass'],
+        provider=self.bucket_url.scheme)
+    sysinfo['bucket_location'] = bucket_info.location
+    sysinfo['bucket_storageClass'] = bucket_info.storageClass
+
     # Try to determine the latency of a DNS lookup for the Google hostname
     # endpoint. Note: we don't piggyback on gethostbyname_ex below because
     # the _ex version requires an extra RTT.
@@ -1676,6 +1769,13 @@ class PerfDiagCommand(Command):
           print 'Measurement time: \n %s' % localdt.strftime(
               '%Y-%m-%d %I:%M:%S %p %Z')
 
+      if 'on_gce' in info:
+        print 'Running on GCE: \n  %s' % info['on_gce']
+        if info['on_gce']:
+          print ('GCE Instance:\n\t%s' %
+                 info['gce_instance_info'].replace('\n', '\n\t'))
+      print 'Bucket location: \n  %s' % info['bucket_location']
+      print 'Bucket storage class: \n  %s' % info['bucket_storageClass']
       print 'Google Server: \n  %s' % info['googserv_route']
       print ('Google Server IP Addresses: \n  %s' %
              ('\n  '.join(info['googserv_ips'])))
@@ -1834,6 +1934,9 @@ class PerfDiagCommand(Command):
     self.input_file = None
     # From -m.
     self.metadata_keys = {}
+    # From -j.
+    self.gzip_encoded_writes = False
+    self.gzip_compression_ratio = 100
 
     if self.sub_opts:
       for o, a in self.sub_opts:
@@ -1893,6 +1996,16 @@ class PerfDiagCommand(Command):
             raise CommandException("Could not decode input file (-i): '%s'." %
                                    a)
           return
+        if o == '-j':
+          self.gzip_encoded_writes = True
+          try:
+            self.gzip_compression_ratio = int(a)
+          except ValueError:
+            self.gzip_compression_ratio = -1
+          if (self.gzip_compression_ratio < 0 or
+              self.gzip_compression_ratio > 100):
+            raise CommandException(
+                'The -j parameter must be between 0 and 100 (inclusive).')
 
     # If parallelism is specified, default parallelism strategy to fan.
     if (self.processes > 1 or self.threads > 1) and not self.parallel_strategy:
@@ -1954,14 +2067,18 @@ class PerfDiagCommand(Command):
         'Number of threads: %d\n'
         'Parallelism strategy: %s\n'
         'Throughput file size: %s\n'
-        'Diagnostics to run: %s',
+        'Diagnostics to run: %s\n'
+        'Gzip compression ratio: %s\n'
+        'Gzip transport encoding writes: %s',
         self.num_objects,
         self.bucket_url,
         self.processes,
         self.threads,
         self.parallel_strategy,
         MakeHumanReadable(self.thru_filesize),
-        (', '.join(self.diag_tests)))
+        (', '.join(self.diag_tests)),
+        self.gzip_compression_ratio,
+        self.gzip_encoded_writes)
 
     try:
       self._SetUp()
