@@ -20,10 +20,12 @@ from collections import namedtuple
 import logging
 import os
 import subprocess
+import re
 import sys
 import tempfile
 import textwrap
 import time
+import traceback
 
 import gslib
 from gslib.cloud_api import ProjectIdException
@@ -32,25 +34,12 @@ from gslib.command import ResetFailureCount
 from gslib.exception import CommandException
 from gslib.project_id import PopulateProjectId
 import gslib.tests as tests
-from gslib.util import IS_WINDOWS
-from gslib.util import NO_MAX
+from gslib.tests.util import GetTestNames
+from gslib.tests.util import unittest
+from gslib.utils.constants import NO_MAX
+from gslib.utils.system_util import IS_WINDOWS
 
-
-# For Python 2.6, unittest2 is required to run the tests. If it's not available,
-# display an error if the test command is run instead of breaking the whole
-# program.
 # pylint: disable=g-import-not-at-top
-try:
-  from gslib.tests.util import GetTestNames
-  from gslib.tests.util import unittest
-except ImportError as e:
-  if 'unittest2' in str(e):
-    unittest = None
-    GetTestNames = None  # pylint: disable=invalid-name
-  else:
-    raise
-
-
 try:
   import coverage
 except ImportError:
@@ -464,10 +453,6 @@ class TestCommand(Command):
 
   def RunCommand(self):
     """Command entry point for the test command."""
-    if not unittest:
-      raise CommandException('On Python 2.6, the unittest2 module is required '
-                             'to run the gsutil tests.')
-
     failfast = False
     list_tests = False
     max_parallel_tests = _DEFAULT_TEST_PARALLEL_PROCESSES
@@ -533,10 +518,26 @@ class TestCommand(Command):
     loader = unittest.TestLoader()
 
     if commands_to_test:
-      try:
-        suite = loader.loadTestsFromNames(commands_to_test)
-      except (ImportError, AttributeError) as e:
-        raise CommandException('Invalid test argument name: %s' % e)
+      suite = unittest.TestSuite()
+      for command_name in commands_to_test:
+        try:
+          suite_for_current_command = loader.loadTestsFromName(command_name)
+          suite.addTests(suite_for_current_command)
+        except (ImportError, AttributeError) as e:
+          msg = ('Failed to import test code from file %s. TestLoader provided '
+                 'this error:\n\n%s' % (command_name, str(e)))
+
+          # Try to give a better error message; by default, unittest swallows
+          # ImportErrors and only shows that an import failed, not why. E.g.:
+          # "'module' object has no attribute 'test_cp'
+          try:
+            __import__(command_name)
+          except Exception as e:
+            stack_trace = traceback.format_exc()
+            err = re.sub('\\n', '\n    ', stack_trace)
+            msg += '\n\nAdditional traceback:\n\n%s' % (err)
+
+          raise CommandException(msg)
 
     if list_tests:
       test_names = GetTestNamesFromSuites(suite)
