@@ -20,7 +20,6 @@ import os
 import shutil
 import signal
 import stat
-import sys
 import tarfile
 import tempfile
 import textwrap
@@ -31,15 +30,14 @@ from gslib.cs_api_map import ApiSelector
 from gslib.exception import CommandException
 from gslib.metrics import CheckAndMaybePromptForAnalyticsEnabling
 from gslib.sig_handling import RegisterSignalHandler
-from gslib.util import CERTIFICATE_VALIDATION_ENABLED
-from gslib.util import CompareVersions
-from gslib.util import DisallowUpdateIfDataInGsutilDir
-from gslib.util import GetBotoConfigFileList
-from gslib.util import GSUTIL_PUB_TARBALL
-from gslib.util import IS_CYGWIN
-from gslib.util import IS_WINDOWS
-from gslib.util import LookUpGsutilVersion
-from gslib.util import RELEASE_NOTES_URL
+from gslib.utils import system_util
+from gslib.utils.boto_util import GetBotoConfigFileList
+from gslib.utils.boto_util import CERTIFICATE_VALIDATION_ENABLED
+from gslib.utils.constants import GSUTIL_PUB_TARBALL
+from gslib.utils.constants import RELEASE_NOTES_URL
+from gslib.utils.text_util import CompareVersions
+from gslib.utils.update_util import DisallowUpdateIfDataInGsutilDir
+from gslib.utils.update_util import LookUpGsutilVersion
 
 
 _SYNOPSIS = """
@@ -54,7 +52,8 @@ _DETAILED_HELP_TEXT = ("""
 <B>DESCRIPTION</B>
   The gsutil update command downloads the latest gsutil release, checks its
   version, and offers to let you update to it if it differs from the version
-  you're currently running.
+  you're currently running. Note that this functionality is not available if
+  you're using a gsutil installation from a package manager or the Cloud SDK.
 
   Once you say "Y" to the prompt of whether to install the update, the gsutil
   update command locates where the running copy of gsutil is installed,
@@ -120,7 +119,7 @@ class UpdateCommand(Command):
       subcommand_help_text={},
   )
 
-  def _ExplainIfSudoNeeded(self, tf, dirs_to_remove):
+  def _ExplainIfSudoNeeded(self, tf, dirs_to_remove, old_cwd):
     """Explains what to do if sudo needed to update gsutil software.
 
     Happens if gsutil was previously installed by a different user (typically if
@@ -129,12 +128,17 @@ class UpdateCommand(Command):
     Args:
       tf: Opened TarFile.
       dirs_to_remove: List of directories to remove.
+      old_cwd: Path to the working directory we should chdir back to if sudo is
+          needed. It's possible that we've chdir'd to a temp directory that's
+          been deleted, which can cause odd behavior (e.g. OSErrors when opening
+          the metrics subprocess). If this is not truthy, we won't attempt to
+          chdir back to this value.
 
     Raises:
       CommandException: if errors encountered.
     """
     # If running under Windows or Cygwin we don't need (or have) sudo.
-    if IS_CYGWIN or IS_WINDOWS:
+    if system_util.IS_CYGWIN or system_util.IS_WINDOWS:
       return
 
     user_id = os.getuid()
@@ -219,7 +223,7 @@ class UpdateCommand(Command):
         # happen because of Windows exclusive file locking, and the update
         # actually succeeds but just leaves the old versions around in the
         # user's temp dir.
-        if not IS_WINDOWS:
+        if not system_util.IS_WINDOWS:
           raise
     if old_cwd:
       try:
@@ -236,7 +240,7 @@ class UpdateCommand(Command):
           'tarball. If you installed gsutil via another method, use the same '
           'method to update it.')
 
-    if os.environ.get('CLOUDSDK_WRAPPER') == '1':
+    if system_util.InvokedViaCloudSdk():
       raise CommandException(
           'The update command is disabled for Cloud SDK installs. Please run '
           '"gcloud components update" to update it. Note: the Cloud SDK '
@@ -315,14 +319,6 @@ class UpdateCommand(Command):
 
     if not no_prompt:
       CheckAndMaybePromptForAnalyticsEnabling()
-      if (2, 6) == sys.version_info[:2]:
-        print('\n'.join(textwrap.wrap(
-            'WARNING: You are using Python 2.6, which gsutil will stop '
-            'supporting on September 1, 2016. If run gsutil update to a '
-            'version released after that date, you will need to upgrade your '
-            'system\'s Python installation to a supported Python version '
-            '(at the time of this writing, version 2.7), or else gsutil will '
-            'fail.\n')))
       (_, major) = CompareVersions(tarball_version, gslib.VERSION)
       if major:
         print('\n'.join(textwrap.wrap(
@@ -334,7 +330,7 @@ class UpdateCommand(Command):
       else:
         print('This command will update to the "%s" version of\ngsutil at %s'
               % (tarball_version, gslib.GSUTIL_DIR))
-    self._ExplainIfSudoNeeded(tf, dirs_to_remove)
+    self._ExplainIfSudoNeeded(tf, dirs_to_remove, old_cwd)
 
     if no_prompt:
       answer = 'y'
@@ -376,7 +372,7 @@ class UpdateCommand(Command):
     # here. Since enterprise mode is not not supported for Windows
     # users, we can skip this step when running on Windows, which
     # avoids the problem that Windows has no find or xargs command.
-    if not IS_WINDOWS:
+    if not system_util.IS_WINDOWS:
       # Make all files and dirs in updated area owner-RW and world-R, and make
       # all directories owner-RWX and world-RX.
       for dirname, subdirs, filenames in os.walk(new_dir):
