@@ -16,7 +16,6 @@
 
 import atexit
 from collections import defaultdict
-from collections import namedtuple
 from functools import wraps
 import logging
 import os
@@ -36,11 +35,9 @@ import boto
 
 from gslib import VERSION
 from gslib.metrics_tuple import Metric
-from gslib.util import CalculateThroughput
-from gslib.util import CreateDirIfNeeded
-from gslib.util import GetDiskCounters
-from gslib.util import HumanReadableToBytes
-from gslib.util import IS_LINUX
+from gslib.utils import system_util
+from gslib.utils.unit_util import CalculateThroughput
+from gslib.utils.unit_util import HumanReadableToBytes
 
 _GA_ENDPOINT = 'https://ssl.google-analytics.com/collect'
 
@@ -299,7 +296,7 @@ class MetricsCollector(object):
 
     # Non-testing cases involve checking the cloud SDK wrapper and the analytics
     # uuid file.
-    elif os.environ.get('CLOUDSDK_WRAPPER') == '1':
+    elif system_util.InvokedViaCloudSdk():
       cls._disabled_cache = not os.environ.get('GA_CID')
     elif os.path.exists(_UUID_FILE_PATH):
       with open(_UUID_FILE_PATH) as f:
@@ -411,7 +408,7 @@ class MetricsCollector(object):
 
     data = urllib.urlencode(sorted(params))
     self._metrics.append(Metric(endpoint=self.endpoint, method='POST',
-                                 body=data, user_agent=self.user_agent))
+                                body=data, user_agent=self.user_agent))
 
   # TODO: Collect CPU usage (Linux-only), latency to first byte, and slowest
   # thread process.
@@ -427,8 +424,8 @@ class MetricsCollector(object):
 
       # Store the disk stats at the beginning of the command so we can calculate
       # time spent on disk I/O.
-      if IS_LINUX:
-        self.disk_counters_start = GetDiskCounters()
+      if system_util.IS_LINUX:
+        self.disk_counters_start = system_util.GetDiskCounters()
 
       # True if using fan parallelism, when the user specifies the -m option.
       self.uses_fan = False
@@ -620,9 +617,9 @@ class MetricsCollector(object):
                                                     attr_name)
 
     # Calculate the disk stats again to calculate deltas of time spent on I/O.
-    if IS_LINUX:
+    if system_util.IS_LINUX:
       disk_start = self.perf_sum_params.disk_counters_start
-      disk_end = GetDiskCounters()
+      disk_end = system_util.GetDiskCounters()
       # Read and write time are the 5th and 6th elements of the stat tuple.
       custom_params[_GA_LABEL_MAP['Disk I/O Time']] = (
           sum([stat[4] + stat[5] for stat in disk_end.values()]) -
@@ -690,7 +687,8 @@ class MetricsCollector(object):
         category=_GA_PERFSUM_CATEGORY, action=action,
         execution_time=apply_execution_time, **custom_params)
 
-  def ReportMetrics(self, wait_for_report=False, log_level=None):
+  def ReportMetrics(
+      self, wait_for_report=False, log_level=None, log_file_path=None):
     """Reports the collected metrics using a separate async process.
 
     Args:
@@ -698,6 +696,10 @@ class MetricsCollector(object):
         subprocess to exit for testing purposes.
       log_level: int, The subprocess logger's level of debugging for testing
         purposes.
+      log_file_path: str, The file that the metrics_reporter module should
+        write its logs to. If not supplied, the metrics_reporter module will
+        use a predetermined default path. This parameter is intended for use
+        by tests that need to evaluate the contents of the file at this path.
     """
     self._CollectCommandAndErrorMetrics()
     self._CollectPerformanceSummaryMetric()
@@ -716,10 +718,16 @@ class MetricsCollector(object):
     logging.debug(self._metrics)
     self._metrics = []
 
+    if log_file_path is not None:
+      # If the path is not None, we'll need to surround the path with quotes
+      # so that the path is passed as a string to the metrics_reporter module.
+      log_file_path = '"%s"' % log_file_path
+
     reporting_code = ('from gslib.metrics_reporter import ReportMetrics; '
-                      'ReportMetrics("{0}", {1})').format(
+                      'ReportMetrics("{0}", {1}, log_file_path={2})').format(
                           temp_metrics_file.name,
-                          log_level).encode('string-escape')
+                          log_level,
+                          log_file_path).encode('string-escape')
     execution_args = [sys.executable, '-c', reporting_code]
     exec_env = os.environ.copy()
     exec_env['PYTHONPATH'] = os.pathsep.join(sys.path)
@@ -737,6 +745,11 @@ class MetricsCollector(object):
       # This can happen specifically if the Python executable moves between the
       # start of this process and now.
       self.logger.debug('Metrics reporting process failed to start.')
+      # Delete the tempfile that would normally be cleaned up in the subprocess.
+      try:
+        os.unlink(temp_metrics_file.name)
+      except:  # pylint: disable=bare-except
+        pass
 
 
 def CaptureAndLogException(func):
@@ -899,9 +912,9 @@ def CheckAndMaybePromptForAnalyticsEnabling():
   If the user agrees, generates a UUID file. Will not prompt if part of SDK.
   """
   disable_prompt = boto.config.get_value('GSUtil', 'disable_analytics_prompt')
-  if not os.path.exists(
-      _UUID_FILE_PATH) and not disable_prompt and not os.environ.get(
-          'CLOUDSDK_WRAPPER'):
+  if (not os.path.exists(_UUID_FILE_PATH) and
+      not disable_prompt and
+      not system_util.InvokedViaCloudSdk()):
     enable_analytics = raw_input('\n' + textwrap.fill(
         'gsutil developers rely on user feedback to make improvements to the '
         'tool. Would you like to send anonymous usage statistics to help '
@@ -910,7 +923,7 @@ def CheckAndMaybePromptForAnalyticsEnabling():
     text_to_write = _DISABLED_TEXT
     if enable_analytics.lower()[0] == 'y':
       text_to_write = uuid.uuid4().hex
-    CreateDirIfNeeded(os.path.dirname(_UUID_FILE_PATH))
+    system_util.CreateDirIfNeeded(os.path.dirname(_UUID_FILE_PATH))
     with open(_UUID_FILE_PATH, 'w') as f:
       f.write(text_to_write)
 
