@@ -47,13 +47,18 @@ class PluralityCheckableIterator(object):
     self.base_iterator = None
     self.head = []
     self.underlying_iter_empty = False
-    # Populate first 2 elems into head so we can check whether iterator has
-    # more than 1 item.
-    for _ in range(0, 2):
-      self._PopulateHead()
 
-  def _PopulateHead(self):
-    if not self.underlying_iter_empty:
+  def _PopulateHead(self, num_elements=1):
+    """Populates self.head from the underlying iterator.
+
+    Args:
+      num_elements: Populate until self.head contains this many
+          elements (or until the underlying iterator runs out).
+
+    Returns:
+      Number of elements at self.head after execution complete.
+    """
+    while not self.underlying_iter_empty and len(self.head) < num_elements:
       try:
         if not self.base_iterator:
           self.base_iterator = iter(self.orig_iterator)
@@ -67,20 +72,18 @@ class PluralityCheckableIterator(object):
         # Indicates we can no longer call next() on underlying iterator, but
         # there could still be elements left to iterate in head.
         self.underlying_iter_empty = True
-      except Exception, e:
+      except Exception, e:  # pylint: disable=broad-except
         # Buffer the exception and raise it when the element is accessed.
         # Also, preserve the original stack trace, as the stack trace from
         # within plurality_checkable_iterator.next is not very useful.
         self.head.append(('exception', e, sys.exc_info()[2]))
+    return len(self.head)
 
   def __iter__(self):
     return self
 
   def next(self):
-    # Backfill into head each time we pop an element so we can always check
-    # for emptiness and for HasPlurality().
-    while self.head:
-      self._PopulateHead()
+    if self._PopulateHead():
       item_tuple = self.head.pop(0)
       if item_tuple[0] == 'element':
         return item_tuple[1]
@@ -89,13 +92,15 @@ class PluralityCheckableIterator(object):
     raise StopIteration()
 
   def IsEmpty(self):
-    return not self.head
+    return not self._PopulateHead()
 
   def HasPlurality(self):
-    return len(self.head) > 1
+    # Populate 2 elements (if possible) into head so we can check whether
+    # iterator has more than 1 item remaining.
+    return self._PopulateHead(num_elements=2) > 1
 
   def PeekException(self):
     """Raises an exception if the first iterated element raised."""
-    if self.head and self.head[0][0] == 'exception':
+    if self._PopulateHead() and self.head[0][0] == 'exception':
       exception_tuple = self.head[0]
       raise exception_tuple[1].__class__, exception_tuple[1], exception_tuple[2]
