@@ -24,8 +24,10 @@ import time
 
 import gslib
 from gslib.cs_api_map import ApiSelector
+from gslib.project_id import PopulateProjectId
 import gslib.tests.testcase as testcase
 from gslib.tests.testcase.integration_testcase import SkipForS3
+from gslib.tests.testcase.integration_testcase import SkipForXML
 from gslib.tests.util import CaptureStdout
 from gslib.tests.util import ObjectToURI as suri
 from gslib.tests.util import SetBotoConfigForTest
@@ -54,12 +56,17 @@ from gslib.tests.util import TEST_ENCRYPTION_KEY4
 from gslib.tests.util import TEST_ENCRYPTION_KEY4_SHA256_B64
 from gslib.tests.util import unittest
 from gslib.third_party.storage_apitools import storage_v1_messages as apitools_messages
-from gslib.util import IS_WINDOWS
-from gslib.util import PrintFullInfoAboutObject
-from gslib.util import Retry
-from gslib.util import UTF8
+from gslib.utils.constants import UTF8
+from gslib.utils.ls_helper import PrintFullInfoAboutObject
+from gslib.utils.retry_util import Retry
+from gslib.utils.system_util import IS_WINDOWS
 import mock
 
+KMS_XML_SKIP_MSG = ('gsutil does not support KMS operations for S3 buckets, '
+                    'or listing KMS keys with the XML API.')
+BUCKET_LOCK_SKIP_MSG = ('gsutil does not support bucket lock operations for '
+                        'S3 buckets or listing retention policy with XML API.')
+
 
 class TestLsUnit(testcase.GsUtilUnitTestCase):
   """Unit tests for ls command."""
@@ -84,7 +91,7 @@ class TestLsUnit(testcase.GsUtilUnitTestCase):
     # Print out attributes of object message.
     with CaptureStdout() as output:
       PrintFullInfoAboutObject(obj_ref)
-    output = "\n".join(output)
+    output = '\n'.join(output)
 
     # Verify that no Storage class update time field displays since it's the
     # same as Creation time.
@@ -110,7 +117,7 @@ class TestLsUnit(testcase.GsUtilUnitTestCase):
     # Print out attributes of object message.
     with CaptureStdout() as output2:
       PrintFullInfoAboutObject(obj_ref2)
-    output2 = "\n".join(output2)
+    output2 = '\n'.join(output2)
 
     # Verify that Creation time and Storage class update time fields display and
     # are the same as the times set in the object message.
@@ -221,7 +228,7 @@ class TestLs(testcase.GsUtilIntegrationTestCase):
     common_prefix = posixpath.commonprefix([suri(bucket1_uri),
                                             suri(bucket2_uri)])
     self.assertTrue(common_prefix.startswith(
-        '%s://%sgsutil-test-test_bucket_list_wildcard-bucket-' %
+        '%s://%sgsutil-test-test-bucket-list-wildcard-bucket-' %
         (self.default_provider, random_prefix)))
     wildcard = '%s*' % common_prefix
 
@@ -275,7 +282,7 @@ class TestLs(testcase.GsUtilIntegrationTestCase):
     output_items = stdout.split()
     self.assertTrue(output_items[0].isdigit())
     # Throws exception if time string is not formatted correctly.
-    time_created = time.strptime(stdout.split()[1], '%Y-%m-%dT%H:%M:%SZ')
+    time.strptime(stdout.split()[1], '%Y-%m-%dT%H:%M:%SZ')
     self.assertEqual(output_items[2], suri(obj_uri))
 
   def test_one_object_with_L(self):
@@ -499,6 +506,35 @@ class TestLs(testcase.GsUtilIntegrationTestCase):
                             return_stdout=True)
     self.assertIn('Website configuration:\t\tNone', stdout)
 
+  @SkipForS3('S3 bucket configuration values are not supported via ls.')
+  @SkipForXML('Requester Pays is not supported for the XML API.')
+  def test_requesterpays(self):
+    """Tests listing a bucket with requester pays (billing) config."""
+    bucket_uri = self.CreateBucket()
+    bucket_suri = suri(bucket_uri)
+
+    # No requester pays configuration
+    stdout = self.RunGsUtil(['ls', '-lb', bucket_suri],
+                            return_stdout=True)
+    self.assertNotIn('Requester Pays enabled', stdout)
+
+    # Requester Pays configuration is absent by default
+    stdout = self.RunGsUtil(['ls', '-Lb', bucket_suri],
+                            return_stdout=True)
+    self.assertIn('Requester Pays enabled:\t\tNone', stdout)
+
+    # Initialize and check
+    self.RunGsUtil(['requesterpays', 'set', 'on', bucket_suri])
+    stdout = self.RunGsUtil(['ls', '-Lb', bucket_suri],
+                            return_stdout=True)
+    self.assertIn('Requester Pays enabled:\t\tTrue', stdout)
+
+    # Clear and check
+    self.RunGsUtil(['requesterpays', 'set', 'off', bucket_suri])
+    stdout = self.RunGsUtil(['ls', '-Lb', bucket_suri],
+                            return_stdout=True)
+    self.assertIn('Requester Pays enabled:\t\tFalse', stdout)
+
   def test_list_sizes(self):
     """Tests various size listing options."""
     bucket_uri = self.CreateBucket()
@@ -779,3 +815,90 @@ class TestLs(testcase.GsUtilIntegrationTestCase):
                             return_stderr=True)
     self.assertIn('Invalid non-ASCII', stderr)
 
+  def set_default_kms_key_on_bucket(self, bucket_uri):
+    # Make sure our keyRing and cryptoKey exist.
+    keyring_fqn = self.kms_api.CreateKeyRing(
+        PopulateProjectId(None), testcase.KmsTestingResources.KEYRING_NAME,
+        location=testcase.KmsTestingResources.KEYRING_LOCATION)
+    key_fqn = self.kms_api.CreateCryptoKey(
+        keyring_fqn, testcase.KmsTestingResources.CONSTANT_KEY_NAME)
+    # Make sure that the service account for the desired bucket's parent project
+    # is authorized to encrypt with the key above.
+    self.RunGsUtil(['kms', 'encryption', '-k', key_fqn, suri(bucket_uri)])
+    return key_fqn
+
+  @SkipForXML(KMS_XML_SKIP_MSG)
+  @SkipForS3(KMS_XML_SKIP_MSG)
+  def test_default_kms_key_listed_for_bucket(self):
+    bucket_uri = self.CreateBucket()
+
+    # Default KMS key is not set by default.
+    stdout = self.RunGsUtil(['ls', '-Lb', suri(bucket_uri)], return_stdout=True)
+    self.assertRegexpMatches(stdout, r'Default KMS key:\s+None')
+
+    # Default KMS key's name should be listed after being set on the bucket.
+    key_fqn = self.set_default_kms_key_on_bucket(bucket_uri)
+    stdout = self.RunGsUtil(['ls', '-Lb', suri(bucket_uri)], return_stdout=True)
+    self.assertRegexpMatches(stdout, r'Default KMS key:\s+%s' % key_fqn)
+
+  @SkipForXML(KMS_XML_SKIP_MSG)
+  @SkipForS3(KMS_XML_SKIP_MSG)
+  def test_kms_key_listed_for_kms_encrypted_object(self):
+    bucket_uri = self.CreateBucket()
+    key_fqn = self.set_default_kms_key_on_bucket(bucket_uri)
+    # Copy an object into our bucket and encrypt using the key from above.
+    obj_uri = self.CreateObject(
+        bucket_uri=bucket_uri, object_name='foo', contents='foo',
+        kms_key_name=key_fqn)
+
+    stdout = self.RunGsUtil(['ls', '-L', suri(obj_uri)], return_stdout=True)
+
+    self.assertRegexpMatches(stdout, r'KMS key:\s+%s' % key_fqn)
+
+  @SkipForXML(BUCKET_LOCK_SKIP_MSG)
+  @SkipForS3(BUCKET_LOCK_SKIP_MSG)
+  def test_list_retention_policy(self):
+    bucket_uri = self.CreateBucketWithRetentionPolicy(
+        retention_period_in_seconds=1)
+    stdout = self.RunGsUtil(['ls', '-Lb', suri(bucket_uri)], return_stdout=True)
+    self.assertRegexpMatches(stdout, r'Retention Policy\:\t*Present')
+    # Clearing Retention Policy on the bucket.
+    self.RunGsUtil(['retention', 'clear', suri(bucket_uri)])
+    stdout = self.RunGsUtil(['ls', '-Lb', suri(bucket_uri)], return_stdout=True)
+    self.assertNotRegexpMatches(stdout, r'Retention Policy:')
+
+  @SkipForXML(BUCKET_LOCK_SKIP_MSG)
+  @SkipForS3(BUCKET_LOCK_SKIP_MSG)
+  def test_list_default_event_based_hold(self):
+    bucket_uri = self.CreateBucket()
+    self.RunGsUtil(['retention', 'event-default', 'set', suri(bucket_uri)])
+    stdout = self.RunGsUtil(['ls', '-Lb', suri(bucket_uri)], return_stdout=True)
+    self.assertRegexpMatches(stdout, r'Default Event-Based Hold:\t* *True')
+    # Clearing the default Event-Based Hold on the bucket.
+    self.RunGsUtil(['retention', 'event-default', 'release', suri(bucket_uri)])
+    stdout = self.RunGsUtil(['ls', '-Lb', suri(bucket_uri)], return_stdout=True)
+    self.assertNotRegexpMatches(stdout, r'Default Event-Based Hold')
+
+  @SkipForXML(BUCKET_LOCK_SKIP_MSG)
+  @SkipForS3(BUCKET_LOCK_SKIP_MSG)
+  def test_list_temporary_hold(self):
+    object_uri = self.CreateObject(contents='content')
+    self.RunGsUtil(['retention', 'temp', 'set', suri(object_uri)])
+    stdout = self.RunGsUtil(['ls', '-L', suri(object_uri)], return_stdout=True)
+    self.assertRegexpMatches(stdout, r'Temporary Hold')
+    # Clearing the Temporary Hold on the object.
+    self.RunGsUtil(['retention', 'temp', 'release', suri(object_uri)])
+    stdout = self.RunGsUtil(['ls', '-L', suri(object_uri)], return_stdout=True)
+    self.assertNotRegexpMatches(stdout, r'Temporary Hold')
+
+  @SkipForXML(BUCKET_LOCK_SKIP_MSG)
+  @SkipForS3(BUCKET_LOCK_SKIP_MSG)
+  def test_list_event_based_hold(self):
+    object_uri = self.CreateObject(contents='content')
+    self.RunGsUtil(['retention', 'event', 'set', suri(object_uri)])
+    stdout = self.RunGsUtil(['ls', '-L', suri(object_uri)], return_stdout=True)
+    self.assertRegexpMatches(stdout, r'Event-Based Hold')
+    # Clearing the Event-Based Hold on the object.
+    self.RunGsUtil(['retention', 'event', 'release', suri(object_uri)])
+    stdout = self.RunGsUtil(['ls', '-L', suri(object_uri)], return_stdout=True)
+    self.assertNotRegexpMatches(stdout, r'Event-Based Hold')
