@@ -24,12 +24,13 @@ import shutil
 import tempfile
 
 import boto
-from gslib.posix_util import NA_ID
-from gslib.posix_util import NA_MODE
+from six import string_types
+
 import gslib.tests.util as util
 from gslib.tests.util import unittest
-from gslib.util import UTF8
-from six import string_types
+from gslib.utils.constants import UTF8
+from gslib.utils.posix_util import NA_ID
+from gslib.utils.posix_util import NA_MODE
 
 MAX_BUCKET_LENGTH = 63
 
@@ -87,22 +88,30 @@ class GsUtilTestCase(unittest.TestCase):
     """Creates a temporary name that is most-likely unique.
 
     Args:
-      kind: A string indicating what kind of test name this is.
-      prefix: Prefix string to be used in the temporary name.
+      kind (str): A string indicating what kind of test name this is.
+      prefix (str): Prefix string to be used in the temporary name.
 
     Returns:
-      The temporary name.
+      (str) The temporary name. If `kind` was "bucket", the temporary name may
+      have coerced this string, including the supplied `prefix`, such that it
+      contains only characters that are valid across all supported storage
+      providers (e.g. replacing "_" with "-", converting uppercase letters to
+      lowercase, etc.).
     """
     name = '%sgsutil-test-%s-%s' % (prefix, self.GetTestMethodName(), kind)
     name = name[:MAX_BUCKET_LENGTH-9]
     name = '%s-%s' % (name, self.MakeRandomTestString())
+    # As of March 2018, S3 no longer accepts underscores or uppercase letters in
+    # bucket names.
+    if kind == 'bucket':
+      name = util.MakeBucketNameValid(name)
     return name
 
   # TODO: Convert tests to use this for object names.
   def MakeTempUnicodeName(self, kind, prefix=''):
     return self.MakeTempName(kind, prefix=prefix) + '材'
 
-  def CreateTempDir(self, test_files=0):
+  def CreateTempDir(self, test_files=0, contents=None):
     """Creates a temporary directory on disk.
 
     The directory and all of its contents will be deleted after the test.
@@ -110,6 +119,7 @@ class GsUtilTestCase(unittest.TestCase):
     Args:
       test_files: The number of test files to place in the directory or a list
                   of test file names.
+      contents: The contents for each generated test file.
 
     Returns:
       The path to the new temporary directory.
@@ -121,7 +131,10 @@ class GsUtilTestCase(unittest.TestCase):
     except TypeError:
       test_files = [self.MakeTempName('file') for _ in range(test_files)]
     for i, name in enumerate(test_files):
-      self.CreateTempFile(tmpdir=tmpdir, file_name=name, contents='test %d' % i)
+      contents_file = contents
+      if contents_file is None:
+        contents_file = 'test %d' % i
+      self.CreateTempFile(tmpdir=tmpdir, file_name=name, contents=contents_file)
     return tmpdir
 
   def CreateTempFifo(self, tmpdir=None, file_name=None):
