@@ -18,12 +18,14 @@ from __future__ import absolute_import
 
 from contextlib import contextmanager
 import cStringIO
+import datetime
 import locale
 import logging
 import os
 import subprocess
 import sys
 import tempfile
+import time
 
 import boto
 from boto import config
@@ -33,17 +35,12 @@ from boto.storage_uri import BucketStorageUri
 
 import gslib
 from gslib.boto_translation import BotoTranslation
-from gslib.cloud_api import CryptoTuple
 from gslib.cloud_api import PreconditionException
 from gslib.cloud_api import Preconditions
-from gslib.encryption_helper import Base64Sha256FromBase64EncryptionKey
+from gslib.discard_messages_queue import DiscardMessagesQueue
+from gslib.exception import CommandException
 from gslib.gcs_json_api import GcsJsonApi
-from gslib.hashing_helper import Base64ToHexHash
-from gslib.posix_util import ATIME_ATTR
-from gslib.posix_util import GID_ATTR
-from gslib.posix_util import MODE_ATTR
-from gslib.posix_util import MTIME_ATTR
-from gslib.posix_util import UID_ATTR
+from gslib.kms_api import KmsApi
 from gslib.project_id import GOOG_PROJ_ID_HDR
 from gslib.project_id import PopulateProjectId
 from gslib.tests.testcase import base
@@ -55,12 +52,19 @@ from gslib.tests.util import SetEnvironmentForTest
 from gslib.tests.util import unittest
 from gslib.tests.util import USING_JSON_API
 import gslib.third_party.storage_apitools.storage_v1_messages as apitools_messages
-from gslib.util import CreateCustomMetadata
-from gslib.util import DiscardMessagesQueue
-from gslib.util import GetValueFromObjectCustomMetadata
-from gslib.util import IS_WINDOWS
-from gslib.util import Retry
-from gslib.util import UTF8
+from gslib.utils.constants import UTF8
+from gslib.utils.encryption_helper import Base64Sha256FromBase64EncryptionKey
+from gslib.utils.encryption_helper import CryptoKeyWrapperFromKey
+from gslib.utils.hashing_helper import Base64ToHexHash
+from gslib.utils.metadata_util import CreateCustomMetadata
+from gslib.utils.metadata_util import GetValueFromObjectCustomMetadata
+from gslib.utils.posix_util import ATIME_ATTR
+from gslib.utils.posix_util import GID_ATTR
+from gslib.utils.posix_util import MODE_ATTR
+from gslib.utils.posix_util import MTIME_ATTR
+from gslib.utils.posix_util import UID_ATTR
+from gslib.utils.system_util import IS_WINDOWS
+from gslib.utils.retry_util import Retry
 
 
 LOGGER = logging.getLogger('integration-test')
@@ -129,6 +133,7 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
                                DiscardMessagesQueue(), 'gs')
     self.xml_api = BotoTranslation(BucketStorageUri, logging.getLogger(),
                                    DiscardMessagesQueue, self.default_provider)
+    self.kms_api = KmsApi()
 
     self.multiregional_buckets = util.USE_MULTIREGIONAL_BUCKETS
 
@@ -167,12 +172,24 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
             else:
               k.delete()
           except StorageResponseError, e:
-            # This could happen if objects that have already been deleted are
-            # still showing up in the listing due to eventual consistency. In
-            # that case, we continue on until we've tried to deleted every
-            # object in the listing before raising the error on which to retry.
             if e.status == 404:
+              # This could happen if objects that have already been deleted are
+              # still showing up in the listing due to eventual consistency. In
+              # that case, we continue on until we've tried to deleted every
+              # obj in the listing before raising the error on which to retry.
               error = e
+            elif e.status == 403 and (e.error_code == 'ObjectUnderActiveHold' or
+                                      e.error_code == 'RetentionPolicyNotMet'):
+              # Object deletion fails if they are under active Temporary Hold,
+              # Event-Based hold or still under retention.
+              #
+              # We purposefully do not raise error in order to allow teardown
+              # to process all the objects in a bucket first. The retry logic on
+              # the teardown method will kick in when bucket deletion fails (due
+              # to bucket being non-empty) and retry deleting these objects
+              # and their associated buckets.
+              self._ClearHoldsOnObjectAndWaitForRetentionDuration(
+                  bucket_uri, k.name)
             else:
               raise
         if error:
@@ -181,6 +198,48 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
       bucket_uri.delete_bucket()
       self.bucket_uris.pop()
 
+  def _ClearHoldsOnObjectAndWaitForRetentionDuration(self, bucket_uri,
+                                                     object_name):
+    """Removes Holds on test objects and waits till retention duration is over.
+
+    This method makes sure that object is not under active Temporary Hold or
+    Release Hold. It also waits (up to 1 minute) till retention duration for the
+    object is over. This is necessary for cleanup, otherwise such test objects
+    cannot be deleted.
+
+    It's worth noting that tests should do their best to remove holds and wait
+    for objects' retention period on their own and this is just a fallback.
+    Additionally, Tests should not use retention duration longer than 1 minute,
+    preferably only few seconds in order to avoid lengthening test execution
+    time unnecessarily.
+
+    Args:
+      bucket_uri: bucket's uri.
+      object_name: object's name.
+    """
+    object_metadata = self.json_api.GetObjectMetadata(
+        bucket_uri.bucket_name,
+        object_name,
+        fields=['timeCreated', 'temporaryHold', 'eventBasedHold'])
+    object_uri = '{}{}'.format(bucket_uri, object_name)
+    if object_metadata.temporaryHold:
+      self.RunGsUtil(['retention', 'temp', 'release', object_uri])
+
+    if object_metadata.eventBasedHold:
+      self.RunGsUtil(['retention', 'event', 'release', object_uri])
+
+    retention_policy = self.json_api.GetBucket(
+        bucket_uri.bucket_name, fields=['retentionPolicy']).retentionPolicy
+    retention_period = (retention_policy.retentionPeriod
+                        if retention_policy is not None else 0)
+    # throwing exceptions for Retention durations larger than 60 seconds.
+    if retention_period <= 60:
+      time.sleep(retention_period)
+    else:
+      raise CommandException(('Retention duration is too large for bucket "{}".'
+                              ' Use shorter durations for Retention duration in'
+                              ' tests').format(bucket_uri))
+
   def _SetObjectCustomMetadataAttribute(self, provider, bucket_name,
                                         object_name, attr_name, attr_value):
     """Sets a custom metadata attribute for an object.
@@ -300,9 +359,14 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
     else:
       return _CheckBucket()
 
-  def AssertObjectUsesEncryptionKey(self, object_uri_str, encryption_key):
-    """Strongly consistent check that the correct encryption key is used."""
-    stdout = self.RunGsUtil(['stat', object_uri_str], return_stdout=True)
+  def AssertObjectUsesCSEK(self, object_uri_str, encryption_key):
+    """Strongly consistent check that the correct CSEK encryption key is used.
+
+    This check forces use of the JSON API, as encryption information is not
+    returned in object metadata via the XML API.
+    """
+    with SetBotoConfigForTest([('GSUtil', 'prefer_api', 'json')]):
+      stdout = self.RunGsUtil(['stat', object_uri_str], return_stdout=True)
     self.assertIn(
         Base64Sha256FromBase64EncryptionKey(encryption_key), stdout,
         'Object %s did not use expected encryption key with hash %s. '
@@ -310,12 +374,105 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
         (object_uri_str, Base64Sha256FromBase64EncryptionKey(encryption_key),
          stdout))
 
+  def AssertObjectUsesCMEK(self, object_uri_str, encryption_key):
+    """Strongly consistent check that the correct KMS encryption key is used.
+
+    This check forces use of the JSON API, as encryption information is not
+    returned in object metadata via the XML API.
+    """
+    with SetBotoConfigForTest([('GSUtil', 'prefer_api', 'json')]):
+      stdout = self.RunGsUtil(['stat', object_uri_str], return_stdout=True)
+    self.assertRegexpMatches(stdout, r'KMS key:\s+%s' % encryption_key)
+
   def AssertObjectUnencrypted(self, object_uri_str):
-    stdout = self.RunGsUtil(['stat', object_uri_str], return_stdout=True)
+    """Checks that no CSEK or CMEK attributes appear in `stat` output.
+
+    This check forces use of the JSON API, as encryption information is not
+    returned in object metadata via the XML API.
+    """
+    with SetBotoConfigForTest([('GSUtil', 'prefer_api', 'json')]):
+      stdout = self.RunGsUtil(['stat', object_uri_str], return_stdout=True)
     self.assertNotIn('Encryption key SHA256', stdout)
+    self.assertNotIn('KMS key', stdout)
+
+  def CreateBucketWithRetentionPolicy(self,
+                                      retention_period_in_seconds,
+                                      is_locked=None,
+                                      bucket_name=None):
+    """Creates a test bucket with Retention Policy.
+
+    The bucket and all of its contents will be deleted after the test.
+
+    Args:
+      retention_period_in_seconds: Retention duration in seconds
+      is_locked: Indicates whether Retention Policy should be locked
+                 on the bucket or not.
+      bucket_name: Create the bucket with this name. If not provided, a
+                   temporary test bucket name is constructed.
+
+    Returns:
+      StorageUri for the created bucket.
+    """
+    # Creating bucket with Retention Policy.
+    retention_policy = (apitools_messages.Bucket.RetentionPolicyValue(
+        retentionPeriod=retention_period_in_seconds))
+    bucket_uri = self.CreateBucket(bucket_name=bucket_name,
+                                   retention_policy=retention_policy,
+                                   prefer_json_api=True)
+
+    if is_locked:
+      # Locking Retention Policy
+      self.RunGsUtil(['retention', 'lock', suri(bucket_uri)], stdin='y')
+
+    # Verifying Retention Policy on the bucket.
+    self.VerifyRetentionPolicy(
+        bucket_uri,
+        expected_retention_period_in_seconds=retention_period_in_seconds,
+        expected_is_locked=is_locked)
+
+    return bucket_uri
+
+  def VerifyRetentionPolicy(self,
+                            bucket_uri,
+                            expected_retention_period_in_seconds=None,
+                            expected_is_locked=None):
+    """Verifies the Retention Policy on a bucket.
+
+    Args:
+      bucket_uri: Specifies the bucket.
+      expected_retention_period_in_seconds: Specifies the expected Retention
+                                            Period of the Retention Policy on
+                                            the bucket. Setting this field to
+                                            None, implies that no Retention
+                                            Policy should be present.
+      expected_is_locked: Indicates whether the Retention Policy should be
+                          locked or not.
+    """
+    actual_retention_policy = self.json_api.GetBucket(
+        bucket_uri.bucket_name, fields=['retentionPolicy']).retentionPolicy
+
+    if expected_retention_period_in_seconds is None:
+      self.assertEqual(actual_retention_policy, None)
+    else:
+      self.assertEqual(actual_retention_policy.retentionPeriod,
+                       expected_retention_period_in_seconds)
+      self.assertEqual(actual_retention_policy.isLocked, expected_is_locked)
+      # Verifying the effectiveTime of the Retention Policy:
+      #    since this is integration test and we don't have exact time of the
+      #    server. We just verify that the effective time is a timestamp within
+      #    last minute.
+      effective_time_in_seconds = self.DateTimeToSeconds(
+          actual_retention_policy.effectiveTime)
+      current_time_in_seconds = self.DateTimeToSeconds(datetime.datetime.now())
+      self.assertGreater(effective_time_in_seconds,
+                         current_time_in_seconds - 60)
+
+  def DateTimeToSeconds(self, datetime_obj):
+    return int(time.mktime(datetime_obj.timetuple()))
 
   def CreateBucket(self, bucket_name=None, test_objects=0, storage_class=None,
-                   provider=None, prefer_json_api=False):
+                   retention_policy=None, provider=None, prefer_json_api=False,
+                   versioning_enabled=False):
     """Creates a test bucket.
 
     The bucket and all of its contents will be deleted after the test.
@@ -326,8 +483,11 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
       test_objects: The number of objects that should be placed in the bucket.
                     Defaults to 0.
       storage_class: Storage class to use. If not provided we us standard.
+      retention_policy: Retention policy to be used on the bucket.
       provider: Provider to use - either "gs" (the default) or "s3".
-      prefer_json_api: If true, use the JSON creation functions where possible.
+      prefer_json_api: If True, use the JSON creation functions where possible.
+      versioning_enabled: If True, set the bucket's versioning attribute to
+          True.
 
     Returns:
       StorageUri for the created bucket.
@@ -341,15 +501,19 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
     else:
       location = 'us-central1'
 
+    if bucket_name:
+      bucket_name = util.MakeBucketNameValid(bucket_name)
+
     if prefer_json_api and provider == 'gs':
       json_bucket = self.CreateBucketJson(bucket_name=bucket_name,
                                           test_objects=test_objects,
                                           storage_class=storage_class,
-                                          location=location)
+                                          location=location,
+                                          versioning_enabled=versioning_enabled,
+                                          retention_policy=retention_policy)
       bucket_uri = boto.storage_uri(
           'gs://%s' % json_bucket.name.encode(UTF8).lower(),
           suppress_consec_slashes=False)
-      self.bucket_uris.append(bucket_uri)
       return bucket_uri
 
     bucket_name = bucket_name or self.MakeTempName('bucket')
@@ -389,6 +553,10 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
 
     _CreateBucketWithExponentialBackoff()
     self.bucket_uris.append(bucket_uri)
+
+    if versioning_enabled:
+      bucket_uri.configure_versioning(True)
+
     for i in range(test_objects):
       self.CreateObject(bucket_uri=bucket_uri,
                         object_name=self.MakeTempName('obj'),
@@ -409,15 +577,20 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
     Returns:
       StorageUri for the created bucket with versioning enabled.
     """
-    bucket_uri = self.CreateBucket(bucket_name=bucket_name,
-                                   test_objects=test_objects)
-    bucket_uri.configure_versioning(True)
+    # Note that we prefer the JSON API so that we don't require two separate
+    # steps to create and then set versioning on the bucket (as versioning
+    # propagation on an existing bucket is subject to eventual consistency).
+    bucket_uri = self.CreateBucket(
+        bucket_name=bucket_name,
+        test_objects=test_objects,
+        prefer_json_api=True,
+        versioning_enabled=True)
     return bucket_uri
 
   def CreateObject(self, bucket_uri=None, object_name=None, contents=None,
                    prefer_json_api=False, encryption_key=None, mode=None,
                    mtime=None, uid=None, gid=None, storage_class=None,
-                   gs_idempotent_generation=0):
+                   gs_idempotent_generation=0, kms_key_name=None):
     """Creates a test object.
 
     Args:
@@ -445,20 +618,26 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
           idempotently by supplying this generation number as a precondition
           and assuming the current object is correct on precondition failure.
           Defaults to 0 (new object); to disable, set to None.
+      kms_key_name: Fully-qualified name of the KMS key that should be used to
+          encrypt the object. Note that this is currently only valid for 'gs'
+          objects.
 
     Returns:
       A StorageUri for the created object.
     """
     bucket_uri = bucket_uri or self.CreateBucket()
 
-    if contents and bucket_uri.scheme == 'gs' and (prefer_json_api or
-                                                   encryption_key):
+    if (contents and
+        bucket_uri.scheme == 'gs' and
+        (prefer_json_api or encryption_key or kms_key_name)):
+
       object_name = object_name or self.MakeTempName('obj')
       json_object = self.CreateObjectJson(
           contents=contents, bucket_name=bucket_uri.bucket_name,
           object_name=object_name, encryption_key=encryption_key,
           mtime=mtime, storage_class=storage_class,
-          gs_idempotent_generation=gs_idempotent_generation)
+          gs_idempotent_generation=gs_idempotent_generation,
+          kms_key_name=kms_key_name)
       object_uri = bucket_uri.clone_replace_name(object_name)
       # pylint: disable=protected-access
       # Need to update the StorageUri with the correct values while
@@ -498,7 +677,9 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
     return key_uri
 
   def CreateBucketJson(self, bucket_name=None, test_objects=0,
-                       storage_class=None, location=None):
+                       storage_class=None, location=None,
+                       versioning_enabled=False,
+                       retention_policy=None):
     """Creates a test bucket using the JSON API.
 
     The bucket and all of its contents will be deleted after the test.
@@ -510,24 +691,33 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
                     Defaults to 0.
       storage_class: Storage class to use. If not provided we use standard.
       location: Location to use.
+      versioning_enabled: If True, set the bucket's versioning attribute to
+          True.
+      retention_policy: Retention policy to be used on the bucket.
 
     Returns:
       Apitools Bucket for the created bucket.
     """
-    bucket_name = bucket_name or self.MakeTempName('bucket')
+    bucket_name = util.MakeBucketNameValid(
+        bucket_name or self.MakeTempName('bucket'))
     bucket_metadata = apitools_messages.Bucket(name=bucket_name.lower())
     if storage_class:
       bucket_metadata.storageClass = storage_class
     if location:
       bucket_metadata.location = location
+    if versioning_enabled:
+      bucket_metadata.versioning = (
+          apitools_messages.Bucket.VersioningValue(enabled=True))
+    if retention_policy:
+      bucket_metadata.retentionPolicy = retention_policy
 
     # TODO: Add retry and exponential backoff.
-    bucket = self.json_api.CreateBucket(bucket_name.lower(),
+    bucket = self.json_api.CreateBucket(bucket_name,
                                         metadata=bucket_metadata)
     # Add bucket to list of buckets to be cleaned up.
     # TODO: Clean up JSON buckets using JSON API.
     self.bucket_uris.append(
-        boto.storage_uri('gs://%s' % (bucket_name.lower()),
+        boto.storage_uri('gs://%s' % bucket_name,
                          suppress_consec_slashes=False))
     for i in range(test_objects):
       self.CreateObjectJson(bucket_name=bucket_name,
@@ -537,13 +727,14 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
 
   def CreateObjectJson(self, contents, bucket_name=None, object_name=None,
                        encryption_key=None, mtime=None, storage_class=None,
-                       gs_idempotent_generation=None):
+                       gs_idempotent_generation=None, kms_key_name=None):
     """Creates a test object (GCS provider only) using the JSON API.
 
     Args:
       contents: The contents to write to the object.
       bucket_name: Name of bucket to place the object in. If not specified,
-          a new temporary bucket is created.
+          a new temporary bucket is created. Assumes the given bucket name is
+          valid.
       object_name: The name to use for the object. If not specified, a temporary
           test object name is constructed.
       encryption_key: AES256 encryption key to use when creating the object,
@@ -558,6 +749,9 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
           idempotently by supplying this generation number as a precondition
           and assuming the current object is correct on precondition failure.
           Defaults to 0 (new object); to disable, set to None.
+      kms_key_name: Fully-qualified name of the KMS key that should be used to
+          encrypt the object. Note that this is currently only valid for 'gs'
+          objects.
 
     Returns:
       An apitools Object for the created object.
@@ -574,15 +768,15 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
         metadata=custom_metadata,
         bucket=bucket_name,
         contentType='application/octet-stream',
-        storageClass=storage_class)
-    encryption_tuple = None
-    if encryption_key:
-      encryption_tuple = CryptoTuple(encryption_key)
+        storageClass=storage_class,
+        kmsKeyName=kms_key_name)
+    encryption_keywrapper = CryptoKeyWrapperFromKey(encryption_key)
     try:
-      return self.json_api.UploadObject(cStringIO.StringIO(contents),
-                                        object_metadata, provider='gs',
-                                        encryption_tuple=encryption_tuple,
-                                        preconditions=preconditions)
+      return self.json_api.UploadObject(
+          cStringIO.StringIO(contents),
+          object_metadata, provider='gs',
+          encryption_tuple=encryption_keywrapper,
+          preconditions=preconditions)
     except PreconditionException:
       if gs_idempotent_generation is None:
         raise
@@ -823,3 +1017,22 @@ class GsUtilIntegrationTestCase(base.GsUtilTestCase):
     """Perform a flat listing over bucket_url_string."""
     return self.RunGsUtil(['ls', suri(bucket_url_string, '**')],
                           return_stdout=True)
+
+
+class KmsTestingResources(object):
+  """Constants for KMS resource names to be used in integration testing."""
+  KEYRING_LOCATION = 'global'
+  # Since KeyRings and their child resources cannot be deleted, we minimize the
+  # number of resources created by using a hard-coded keyRing name.
+  KEYRING_NAME = 'keyring-for-gsutil-integration-tests'
+
+  # Used by tests where we don't need to alter the state of a cryptoKey and/or
+  # its IAM policy bindings once it's initialized the first time.
+  CONSTANT_KEY_NAME = 'key-for-gsutil-integration-tests'
+  CONSTANT_KEY_NAME2 = 'key-for-gsutil-integration-tests2'
+  # Pattern used for keys that should only be operated on by one tester at a
+  # time. Because multiple integration test invocations can run at the same
+  # time, we want to minimize the risk of them operating on each other's key,
+  # while also not creating too many one-time-use keys (as they cannot be
+  # deleted). Tests should fill in the %d entries with a digit between 0 and 9.
+  MUTABLE_KEY_NAME_TEMPLATE = 'cryptokey-for-gsutil-integration-tests-%d%d%d'
