@@ -31,57 +31,65 @@ import boto
 import crcmod
 from gslib.cloud_api import ResumableDownloadException
 from gslib.cloud_api import ResumableUploadException
-from gslib.encryption_helper import Base64Sha256FromBase64EncryptionKey
-from gslib.posix_util import GetDefaultMode
+from gslib.lazy_wrapper import LazyWrapper
 import gslib.tests as gslib_tests
-from gslib.util import IS_WINDOWS
-from gslib.util import LazyWrapper
-from gslib.util import MakeHumanReadable
-from gslib.util import UsingCrcmodExtension
-
-if not hasattr(unittest.TestCase, 'assertIsNone'):
-  # external dependency unittest2 required for Python <= 2.6
-  import unittest2 as unittest  # pylint: disable=g-import-not-at-top
+from gslib.utils.boto_util import UsingCrcmodExtension
+from gslib.utils.encryption_helper import Base64Sha256FromBase64EncryptionKey
+from gslib.utils.posix_util import GetDefaultMode
+from gslib.utils.system_util import IS_WINDOWS
+from gslib.utils.unit_util import MakeHumanReadable
 
 # pylint: disable=g-import-not-at-top, g-long-lambda
 if not IS_WINDOWS:
   import grp
   import pwd
 
-  USER_ID = os.getuid()
-  USER_NAME = LazyWrapper(lambda: (pwd.getpwuid(USER_ID).pw_name))
-  PRIMARY_GID = LazyWrapper(lambda: (pwd.getpwuid(USER_ID).pw_gid))
+  def GetInvalidGid():
+    # Get a list of all groups on the system which are not necessarily sorted by
+    # GID, then sort them and take the last element and add one to the GID to
+    # get a GID that is guaranteed not to be on the current system.
+    return sorted([group.gr_gid for group in grp.getgrall()])[-1] + 1
 
-  # Get a list of all groups on the system where the current username is listed
-  # as a member of the group in the gr_mem group attribute. Make this a list of
-  # all group IDs and cast as a set for more efficient lookup times.
-  USER_GROUPS = LazyWrapper(
-      lambda: set([PRIMARY_GID] +
-                  [g.gr_gid for g in grp.getgrall()
-                   if USER_NAME in g.gr_mem]))
-  # Select a group for the current user that is not the user's primary group. If
-  # the length of the user's groups is 1, then we must use the primary group.
-  # Otherwise put all of the user's groups (except the primary group) in a list,
-  # and use the first element. This guarantees us a group that is not the user's
-  # primary group (unless the user is only a member of one group).
-  NON_PRIMARY_GID = LazyWrapper(lambda: (PRIMARY_GID if len(USER_GROUPS) == 1
-                                         else [g for g in list(USER_GROUPS)
-                                               if g != PRIMARY_GID][0]))
+  def GetNonPrimaryGid():
+    # Select a group for the current user that is not the user's primary group.
+    # If the length of the user's groups is 1, then we must use the primary
+    # group.  Otherwise put all of the user's groups (except the primary group)
+    # in a list, and use the first element. This guarantees us a group that is
+    # not the user's primary group (unless the user is only a member of one
+    # group).
+    primary_gid = GetPrimaryGid()
+    user_groups = GetUserGroups()
+    if len(user_groups) == 1:
+      return primary_gid
+    return [g for g in list(user_groups) if g != primary_gid][0]
+
+  def GetPrimaryGid():
+    return os.getgid()
+
+  def GetUserGroups():
+    return set([GetPrimaryGid()] +
+               [g.gr_gid for g in grp.getgrall() if USER_NAME() in g.gr_mem])
 
   DEFAULT_MODE = int(GetDefaultMode(), 8)
-
-  # Get a list of all groups on the system which are not necessarily sorted by
-  # GID, then sort them and take the last element and add one to the GID to get
-  # a GID that is guaranteed not to be on the current system.
-  INVALID_GID = LazyWrapper(lambda: sorted([group.gr_gid for group
-                                            in grp.getgrall()])[-1] + 1)
-
+  USER_ID = os.getuid()
+  USER_NAME = LazyWrapper(lambda: pwd.getpwuid(USER_ID).pw_name)
   # Take the current user's UID and increment it by one, this counts as an
   # invalid UID, as the metric used is if the UID matches the current user's,
   # exactly.
   INVALID_UID = LazyWrapper(lambda: sorted([user.pw_uid for user
                                             in pwd.getpwall()])[-1] + 1)
 
+  # Note that because the system's GID mapping can change mid-test, tests that
+  # check for specific errors should always re-fetch these GID-related values,
+  # rather than reusing these LazyWrapper values.
+  INVALID_GID = LazyWrapper(lambda: GetInvalidGid())
+  NON_PRIMARY_GID = LazyWrapper(lambda: GetNonPrimaryGid())
+  PRIMARY_GID = LazyWrapper(lambda: GetPrimaryGid())
+  # Get a list of all groups on the system where the current username is listed
+  # as a member of the group in the gr_mem group attribute. Make this a list of
+  # all group IDs and cast as a set for more efficient lookup times.
+  USER_GROUPS = LazyWrapper(lambda: GetUserGroups())
+
 # 256-bit base64 encryption keys used for testing AES256 customer-supplied
 # encryption. These are public and open-source, so don't ever use them for
 # real data.
@@ -180,12 +188,14 @@ def _HasGSHost():
   return boto.config.get('Credentials', 'gs_host', None) is not None
 
 HAS_GS_HOST = _HasGSHost()
+HAS_NON_DEFAULT_GS_HOST = (
+    boto.gs.connection.GSConnection.DefaultHost != boto.config.get(
+        'Credentials', 'gs_host', None))
 
 
 def _HasGSPort():
   return boto.config.get('Credentials', 'gs_port', None) is not None
 
-
 HAS_GS_PORT = _HasGSPort()
 
 
@@ -315,6 +325,9 @@ class GSMockBucketStorageUri(mock_storage_service.MockBucketStorageUri):
     """Dummy implementation to allow parallel uploads with tests."""
     return self.new_key()
 
+  def get_location(self, headers=None):
+    return 'US'
+
 
 TEST_BOTO_REMOVE_SECTION = 'TestRemoveSection'
 
@@ -545,6 +558,20 @@ def GetTestNames():
   return names
 
 
+def MakeBucketNameValid(name):
+  """Returns a copy of the given name with any invalid characters replaced.
+
+  Args:
+    name (str): The bucket name to transform into a valid name.
+
+  Returns:
+    (str) The version of the bucket name containing only valid characters.
+  """
+  # Neither underscores nor uppercase letters are valid characters for a
+  # bucket name. Replace those with hyphens and lowercase characters.
+  return name.replace('_', '-').lower()
+
+
 @contextmanager
 def WorkingDirectory(new_working_directory):
   """Changes the working directory for the duration of a 'with' call.
